// G723Codec.cpp: implementation of the G723Codec class.
//
//////////////////////////////////////////////////////////////////////
#ifdef WIN32
#include <windows.h>
#else
#define NULL 0
#endif
#include "G723Codec.h"

extern "C" {
	#include "g723api.h"
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

G723Codec::G723Codec()
{
	// By default use best possible quality
	EncoderQuality = 2047;
	DecoderQuality = 1;
	// Initialize Codec Memory, as well as encode and decode process
	codecMemory = new char[sizeof(g723Mem)];
	if (codecMemory == NULL) 
	{
		return;
	}
	if (g723((g723Mem *)codecMemory, NULL, NULL, G723_INITIALIZE, 0) < 0) 
	{
		return;
	}
	if (g723((g723Mem *)codecMemory, NULL, NULL, G723_INIT_ENCODE, EncoderQuality) < 0) 
	{
		return;
	}
	if (g723((g723Mem *)codecMemory, NULL, NULL, G723_INIT_DECODE, DecoderQuality) < 0)
	{
		return;
	}
	SetRate63();
}

G723Codec::~G723Codec()
{
	delete [] codecMemory;
}

int
G723Codec::SetRate53()
{
	codecRate = G723RATE_53;
	return 0;
}

int
G723Codec::SetRate63()
{
	codecRate = G723RATE_63;
	return 0;
}

int
G723Codec::GetCodecRate()
{
	return codecRate;
}

int
G723Codec::Encode(void *input, int inputSizeBytes, void *output, int *outputSizeBytes)
{
	short *dataIn = (short *)input;
	char *dataOut = (char *)output;

	if (inputSizeBytes > 0) 
	{
		int minFrameSize;
		int encoder;
		if (codecRate == G723RATE_53)
		{
			minFrameSize = 20;
			encoder = G723_53_ENCODER;
		}
		else
		{
			minFrameSize = 24;
			encoder = G723_63_ENCODER;
		}

		if (inputSizeBytes % 480 != 0) 
		{	// input must be multiple of 30ms (480 bytes of pcm 16bit 8kHz Mono)
			*outputSizeBytes = 0;
			return -10;
		}
		if ((inputSizeBytes/480)*minFrameSize > *outputSizeBytes) 
		{
			*outputSizeBytes = 0;
			return -20;
		}
		else 
		{
			// convert each frame in input
			const int nFrames = inputSizeBytes / 480;
			for (int i = 0; i < nFrames; i++) 
			{
				char  *g723Data	= &dataOut[minFrameSize * i];
				short *pcmData	= &dataIn[240 * i];
				g723 ((g723Mem *)codecMemory, pcmData, g723Data, encoder, EncoderQuality);		
			}
			*outputSizeBytes = nFrames * minFrameSize;
		}
	}
	return 0;
}

int
G723Codec::Decode(void *input, int inputSizeBytes, void *output, int *outputSizeBytes)
{
	char  *dataIn	= (char  *)input;
	short *dataOut	= (short *)output;

	if (inputSizeBytes > 0) 
	{
		int minFrameSize;
		int decoder = G723_DECODER;
		if (codecRate == G723RATE_53)
		{
			minFrameSize = 20;
		}
		else
		{
			minFrameSize = 24;
		}

		if (inputSizeBytes % minFrameSize != 0) 
		{	// input must be multiple of 30ms (480 bytes of pcm 16bit 8kHz Mono)
			*outputSizeBytes = 0;
			return -10;
		}
		if ((inputSizeBytes/minFrameSize)*480 > *outputSizeBytes) 
		{
			*outputSizeBytes = 0;
			return -20;
		}
		else 
		{
			// convert each frame in input
			const int nFrames = inputSizeBytes / minFrameSize;
			for (int i = 0; i < nFrames; i++) 
			{
				char  *g723Data	= &dataIn[minFrameSize * i];
				short *pcmData	= &dataOut[240 * i];
				g723 ((g723Mem *)codecMemory, pcmData, g723Data, decoder, DecoderQuality);		
			}
			*outputSizeBytes = nFrames * 480;
		}
	}
	return 0;
}
