// G7112PCMTransformer.cpp: implementation of the G7112PCMTransformer class.
//
//////////////////////////////////////////////////////////////////////

#include "G7112PCMTransformer.h"
#include "AudioSampleManager.h"
#include <stdio.h>
#include "RTPPacket.h"
#include "WaveFormats.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

G7112PCMTransformer::G7112PCMTransformer()
{
	char subFacilityName[100];
	sprintf(subFacilityName, "G7112PCMTransformer:%x", this);
	tracer.SetSubFacilityName(subFacilityName);
	SetTraceLevel();
}

G7112PCMTransformer::~G7112PCMTransformer()
{
}

int
G7112PCMTransformer::SetTraceLevel()
{
#ifdef WIN32
	long SystemMask = 0;
	if ((SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "AllComponents", 0x0)) == 0)
	{
		SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "G7112PCMTransformer", 0x100000);
	}
	tracer.SetSystemMask(SystemMask);
#endif
	return 0;
}

int
G7112PCMTransformer::SetALaw()
{
	g711Codec.SetALaw();
	return 0;
}

int
G7112PCMTransformer::SetULaw()
{
	g711Codec.SetULaw();
	return 0;
}

int
G7112PCMTransformer::TransformAudioSamples(std::vector<std::pair<AudioSample *, AudioSource *> > &data, AudioSample **ppAudioSample)
{
        if(data.size() == 0) return 0;
	AudioSample *inSample = data[0].first;
	AudioSample *outSample;
	int result = 0;
	result = AudioSampleManager::GetInstance()->GetAudioSample(&outSample);

	if (outSample)
	{
		*ppAudioSample = outSample;
                WAVEFORMATEX wfmx = WaveFormat::GetWaveFormat(WaveFormat_PCM_16_8_1);
		outSample->SetFormat( wfmx);
		
		if (inSample->RTPHeader())
		{
			outSample->SetRTPHeader((inSample->RTPHeader())->CloneHeader());
		}

		if (inSample->DataSize() <= 0)
		{
			outSample->SetSilenceDuration(inSample->GetSilenceDuration());
		}
		else
		{
			int outputSize = outSample->BufferSize();
			result = g711Codec.Decode(inSample->Data(), inSample->DataSize(), outSample->Data(), &outputSize);
			outSample->SetDataSize(outputSize);
		}
	}
	else
	{
		result = -10;
	}
	return result;
}
