// G7232PCMTransformer.cpp: implementation of the G7232PCMTransformer class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "G7232PCMTransformer.h"
#include "AudioSampleManager.h"
#include "AudioSample.h"
#include "RTPPacket.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

G7232PCMTransformer::G7232PCMTransformer()
{
	char subFacilityName[100];
	sprintf(subFacilityName, "G7232PCMTransformer:%x", this);
	tracer.SetSubFacilityName(subFacilityName);
	SetTraceLevel();
	SetRate63();
}

G7232PCMTransformer::~G7232PCMTransformer()
{

}

int
G7232PCMTransformer::SetTraceLevel()
{
#ifdef WIN32
	long SystemMask = 0;
	if ((SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "AllComponents", 0x0)) == 0)
	{
		SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "G7232PCMTransformer", 0x100000);
	}
	tracer.SetSystemMask(SystemMask);
#endif
	return 0;
}

int
G7232PCMTransformer::SetRate53()
{
	return g723Codec.SetRate53();
}

int
G7232PCMTransformer::SetRate63()
{
	return g723Codec.SetRate63();
}

int
G7232PCMTransformer::TransformAudioSamples(std::vector<std::pair<AudioSample *, AudioSource *> > &data, AudioSample **ppAudioSample)
{
	AudioSample *inSample = data[0].first;
	AudioSample *outSample;
	int result = 0;
	result = AudioSampleManager::GetInstance()->GetAudioSample(&outSample);

	if (outSample)
	{
		*ppAudioSample = outSample;
                WAVEFORMATEX wfmx = WaveFormat::GetWaveFormat(WaveFormat_PCM_16_8_1);
		outSample->SetFormat(wfmx);
		
		if (inSample->RTPHeader())
		{
			outSample->SetRTPHeader((inSample->RTPHeader())->CloneHeader());
		}

		if (inSample->DataSize() <= 0)
		{
			outSample->SetSilenceDuration(inSample->GetSilenceDuration());
		}
		else
		{
			int outputSize = outSample->BufferSize();
			result = g723Codec.Decode(inSample->Data(), inSample->DataSize(), outSample->Data(), &outputSize);
			outSample->SetDataSize(outputSize);
		}
	}
	else
	{
		result = -10;
	}
	return result;
}
