// G729A2PCMTransformer.cpp: implementation of the G729A2PCMTransformer class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "G729A2PCMTransformer.h"
#include "AudioSampleManager.h"
#include "RTPPacket.h"

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

G729A2PCMTransformer::G729A2PCMTransformer()
{
	char subFacilityName[100];
	sprintf(subFacilityName, "G729A2PCMTransformer:%x", this);
	tracer.SetSubFacilityName(subFacilityName);
	SetTraceLevel();
}

G729A2PCMTransformer::~G729A2PCMTransformer()
{
}

int
G729A2PCMTransformer::SetTraceLevel()
{
#ifdef WIN32
	long SystemMask = 0;
	if ((SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "AllComponents", 0x0)) == 0)
	{
		SystemMask = GetRegKeyLong(HKEY_CURRENT_USER, "Software\\Cisco Systems\\MTC\\Tracing", "G729A2PCMTransformer", 0x100000);
	}
#endif
	return 0;
}

int
G729A2PCMTransformer::StartTransform()
{
	g729Codec.StartDecoder();
	return 0;
}

int
G729A2PCMTransformer::StopTransform()
{
	g729Codec.StopDecoder();
	return 0;
}

int
G729A2PCMTransformer::TransformAudioSamples(std::vector<std::pair<AudioSample *, AudioSource *> > &data, AudioSample **ppAudioSample)
{
	AudioSample *inSample = data[0].first;
	AudioSample *outSample;
	int result = 0;
	result = AudioSampleManager::GetInstance()->GetAudioSample(&outSample);

	if (outSample)
	{
		*ppAudioSample = outSample;
                WAVEFORMATEX wfmx = WaveFormat::GetWaveFormat(WaveFormat_PCM_16_8_1);
		outSample->SetFormat(wfmx);
		
		if (inSample->RTPHeader())
		{
			outSample->SetRTPHeader((inSample->RTPHeader())->CloneHeader());
		}

		if (inSample->DataSize() <= 0)
		{
			outSample->SetSilenceDuration(inSample->GetSilenceDuration());
		}
		else
		{
			int outputSize = outSample->BufferSize();
			result = g729Codec.Decode(inSample->Data(), inSample->DataSize(), outSample->Data(), &outputSize);
			outSample->SetDataSize(outputSize);
		}
	}
	else
	{
		result = -10;
	}
	return result;
}
