
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const IPJackHardware_cxx_Version =
    "$Id: IPJackHardware.cxx,v 1.45 2001/02/27 05:59:22 icahoon Exp $";
/* stdlib */
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <signal.h>

/* sockets */
#include <sys/types.h>
#include <sys/socket.h>
#include <VTime.hxx>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "VCondition.h"

/* error handling */
#include <errno.h>

#include "ixjuser.h"
#include "IPJackHardware.h"
#include "ReqSig.h"
#include "cpLog.h"

#define ULAW_PAYLOAD 240
#define ULAW_SAMPLE_RATE 240

#define LINEAR16_PAYLOAD 480
#define LINEAR16_SAMPLE_RATE 480

#define RESID_RTP_RATE 240
#define NETWORK_RTP_RATE 160

/** The following array provides mappings from MGCP encoded signals to
 ** the corresponding IPJackHardware signal.
 **/
HardwareSignal IPJackHardwareGenericSigOn[GenericReqSigMaxSignal] =
    {
        SignalNULL,  //GenericReqSigPatternDetected,
        SignalLocalRingbackStart,  //GenericReqSigRingbackTone,
        SignalNULL,  //GenericReqSigRingbackOnConnection,
        SignalNULL,  //GenericReqSigConfirmTone,
        SignalNULL,  //GenericReqSigNetworkCongestionTone,
        SignalNULL,  //GenericReqSigInterceptTone,
        SignalNULL,  //GenericReqSigPreemptionTone,
    };

HardwareSignal IPJackHardwareGenericSigOff[GenericReqSigMaxSignal] =
    {
        //GenericReqSigModemDetected,
        //GenericSigFaxToneDetected,
        //GenericReqSigLongDurationConnection,
        SignalNULL,  //GenericReqSigPatternDetected,
        SignalLocalRingbackStop,  //GenericReqSigRingbackTone,
        SignalNULL,  //GenericReqSigRingbackOnConnection,
        SignalNULL,  //GenericReqSigConfirmTone,
        SignalNULL,  //GenericReqSigNetworkCongestionTone,
        SignalNULL,  //GenericReqSigInterceptTone,
        SignalNULL,  //GenericReqSigPreemptionTone,
        //GenericSigReportFailure,
    };

HardwareSignal IPJackHardwareLineSigOn[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineReqSigAdsiDisplay,
        SignalNULL,  // LineReqSigVisualMsg,
        SignalNULL,  // LineReqSigAnswerTone,
        SignalBusyStart,  // LineReqSigBusyTone,
        SignalNULL,  // LineReqSigCallerId,
        SignalNULL,  // LineReqSigCallWaitTone,
        SignalDialToneStart,  // LineReqSigDialTone,
        SignalNULL,  // LineReqSigMsgWaitInd,
        SignalNULL,  // LineReqSigNetworkBusy,
        SignalRingStart,  // LineReqSigRinging,
        SignalNULL,  // LineReqSigDsnctRing0,
        SignalNULL,  // LineReqSigDsnctRing1,
        SignalNULL,  // LineReqSigDsnctRing2,
        SignalNULL,  // LineReqSigDsnctRing3,
        SignalNULL,  // LineReqSigDsnctRing4,
        SignalNULL,  // LineReqSigDsnctRing5,
        SignalNULL,  // LineReqSigDsnctRing6,
        SignalNULL,  // LineReqSigDsnctRing7,
        SignalNULL,  // LineReqSigRingSplash,
        SignalNULL,  // LineReqSigPromptTone,
        SignalNULL,  // LineReqSigErrorTone,
        SignalNULL,  // LineReqSigStutterTone,
        SignalNULL,  // LineReqSigAlertTone,
        SignalNULL,  // LineReqSigRecordWarnTone,
        SignalNULL,  // LineReqSigCallCardSrvTone,
        SignalNULL,  // LineReqSigOffhookWarnTone,
        SignalNULL,  // LineReqSigDsnctTonePattern,
    };

HardwareSignal IPJackHardwareLineSigOff[LineReqSigMaxSignal] =
    {
        SignalNULL,  // LineReqSigAdsiDisplay,
        SignalNULL,  // LineReqSigVisualMsg,
        SignalNULL,  // LineReqSigAnswerTone,
        SignalBusyStop,  // LineReqSigBusyTone,
        SignalNULL,  // LineReqSigCallerId,
        SignalNULL,  // LineReqSigCallWaitTone,
        SignalDialToneStop,  // LineReqSigDialTone,
        SignalNULL,  // LineReqSigMsgWaitInd,
        SignalNULL,  // LineReqSigNetworkBusy,
        SignalRingStop,  // LineReqSigRinging,
        SignalNULL,  // LineReqSigDsnctRing0,
        SignalNULL,  // LineReqSigDsnctRing1,
        SignalNULL,  // LineReqSigDsnctRing2,
        SignalNULL,  // LineReqSigDsnctRing3,
        SignalNULL,  // LineReqSigDsnctRing4,
        SignalNULL,  // LineReqSigDsnctRing5,
        SignalNULL,  // LineReqSigDsnctRing6,
        SignalNULL,  // LineReqSigDsnctRing7,
        SignalNULL,  // LineReqSigRingSplash,
        SignalNULL,  // LineReqSigPromptTone,
        SignalNULL,  // LineReqSigErrorTone,
        SignalNULL,  // LineReqSigStutterTone,
        SignalNULL,  // LineReqSigAlertTone,
        SignalNULL,  // LineReqSigRecordWarnTone,
        SignalNULL,  // LineReqSigCallCardSrvTone,
        SignalNULL,  // LineReqSigOffhookWarnTone,
        SignalNULL,  // LineReqSigDsnctTonePattern,
    };

HardwareSignal IPJackHardwareDtmfSigOn[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfReqSig0,
        SignalNULL,  //DtmfReqSig1,
        SignalNULL,  //DtmfReqSig2,
        SignalNULL,  //DtmfReqSig3,
        SignalNULL,  //DtmfReqSig4,
        SignalNULL,  //DtmfReqSig5,
        SignalNULL,  //DtmfReqSig6,
        SignalNULL,  //DtmfReqSig7,
        SignalNULL,  //DtmfReqSig8,
        SignalNULL,  //DtmfReqSig9,
        SignalNULL,  //DtmfReqSigHash,
        SignalNULL,  //DtmfReqSigStar,
        SignalNULL,  //DtmfReqSigA,
        SignalNULL,  //DtmfReqSigB,
        SignalNULL,  //DtmfReqSigC,
        SignalNULL,  //DtmfReqSigD
    };

HardwareSignal IPJackHardwareDtmfSigOff[DtmfReqSigMaxSignal] =
    {
        SignalNULL,  //DtmfReqSig0,
        SignalNULL,  //DtmfReqSig1,
        SignalNULL,  //DtmfReqSig2,
        SignalNULL,  //DtmfReqSig3,
        SignalNULL,  //DtmfReqSig4,
        SignalNULL,  //DtmfReqSig5,
        SignalNULL,  //DtmfReqSig6,
        SignalNULL,  //DtmfReqSig7,
        SignalNULL,  //DtmfReqSig8,
        SignalNULL,  //DtmfReqSig9,
        SignalNULL,  //DtmfReqSigHash,
        SignalNULL,  //DtmfReqSigStar,
        SignalNULL,  //DtmfReqSigA,
        SignalNULL,  //DtmfReqSigB,
        SignalNULL,  //DtmfReqSigC,
        SignalNULL,  //DtmfReqSigD
    };

IPJackHardware::IPJackHardware ( const char* deviceName,
                                 Fifo < HardwareMsg > * inputQ,
                                 Fifo < EndpointMsg > * outputQ )
        : audioActive(false),
        hasPlayed(false)
{
    // initialize queues
    endpointQ = outputQ;
    myQ = inputQ;

    // open hardware device

#ifdef __vxworks
    myFD = -1;
#else
    myFD = open(deviceName, O_RDWR);
#endif


    if (myFD < 0)
    {
        cerr << "Cannot open " << deviceName << endl;
        exit(1);
    }


    // close and reopen to make sure device is not in a funny state

    ioctl(myFD, IXJCTL_REC_STOP);
    ioctl(myFD, IXJCTL_PLAY_STOP);
    ioctl(myFD, IXJCTL_AEC_STOP);

    close(myFD);
    myFD = open(deviceName, O_RDWR);
    if (myFD < 0)
    {
        cerr << "Cannot open " << deviceName << endl;
        exit(1);
    }

    // initialize hookstate
    if (ioctl(myFD, IXJCTL_HOOKSTATE))
        hookStateOffhook = true;
    else
        hookStateOffhook = false;

    // set ring to 3 ring instead of the default of 5
    ioctl(myFD, IXJCTL_MAXRINGS, 3);


    // store the device name.

    myDeviceName = deviceName;

}  // end IPJackHardware::IPJackHardware()


IPJackHardware::~IPJackHardware (void)
{
    close(myFD);
} // end IPJackHardware::~IPJackHardware()


void* IPJackHardware::hardwareMain (void* parms)
{
    cerr << "IPJackHardware::hardwareMain\n";
    // process forever on behalf of null hardware
    fd_set readfds;
    struct timeval tv;
    int retval;
    int maxFd = 128;  // should be 0 then increment as needed when addToFdSet
    bool operational = true;

    while (operational)
    {
        // reset file descriptor
        FD_ZERO(&readfds);
        addToFdSet(&readfds);

        // block on select for asyncronous events, but poll to process
        // audio and signal requests from endpoints in message queue
        // times are selected based on the quicknet test files (see
        // contrib/quicknet).
        tv.tv_sec = 0;
        tv.tv_usec = 300;
        if ((retval = select(maxFd, &readfds, NULL, NULL, &tv)) < 0)
        {
            cerr << "select() returned with an error\n";
        }
        else
        {
            if (process(&readfds) < 0)
            {
                cerr << "hardware encountered an error\n";
                operational = false;
            }
        }

        // check for signal requests from endpoints
        while (myQ->size())
        {
            processEndpointMsg(myQ->getNext());
        }
    } // end while

    return 0;

} // end IPJackHardware::hardwareMain()


int IPJackHardware::process (fd_set* fd)
{
    // vusleep is needed to stabilize hookstate readings
    // this may no longer be needed... need to check the latest
    // quicknet driver.
    vusleep(0);  
    if (ioctl(myFD, IXJCTL_HOOKSTATE))
    {
        // user is offhook
        if (hookStateOffhook == false)
        {
            // report change from onhook to offhook
            hookStateOffhook = true;
            reportEvent(endpointQ, EventOffhook);
        }

        // check for DTMF digit
        if (ioctl(myFD, IXJCTL_DTMF_READY))
        {
            // new digit
            switch (ioctl(myFD, IXJCTL_GET_DTMF))
            {
                case 0x01:
                {
                    reportEvent(endpointQ, EventDigit1);
                }
                break;

                case 0x02:
                {
                    reportEvent(endpointQ, EventDigit2);
                }
                break;

                case 0x03:
                {
                    reportEvent(endpointQ, EventDigit3);
                }
                break;

                case 0x04:
                {
                    reportEvent(endpointQ, EventDigit4);
                }
                break;

                case 0x05:
                {
                    reportEvent(endpointQ, EventDigit5);
                }
                break;

                case 0x06:
                {
                    reportEvent(endpointQ, EventDigit6);
                }
                break;

                case 0x07:
                {
                    reportEvent(endpointQ, EventDigit7);
                }
                break;

                case 0x08:
                {
                    reportEvent(endpointQ, EventDigit8);
                }
                break;

                case 0x09:
                {
                    reportEvent(endpointQ, EventDigit9);
                }
                break;

                case 0x0A:
                {
                    reportEvent(endpointQ, EventDigitStar);
                }
                break;

                case 0x0B:
                {
                    reportEvent(endpointQ, EventDigit0);
                }
                break;

                case 0x0C:
                {
                    reportEvent(endpointQ, EventDigitHash);
                }
                break;

                default:
                {
                    cerr << "unrecognized DTMF digit\n";
                }
                break;
            } // end switch
        }

    } // end if offhook
    else
    {
        // user is onhook
        if (hookStateOffhook == true)
        {
            // report change from offhook to onhook
            reportEvent(endpointQ, EventOnhook);
            hookStateOffhook = false;
        }
    }

    if (audioActive)
    {
        if (!hasPlayed)
        {
            cerr << "Begin audio\n";
            hasPlayed = true;
        }
        processRTP();
    }
    return 0;
} // end IPJackHardware::process()


void
IPJackHardware::processRTP ()
{
    if (audioStack == 0)
    { 
        return ;
    }

    // Checks for packets from the network.
    if (audioStack)
    {
        inRtpPkt = audioStack->receive ();
    }

    // Writes packets to the device.
    if (inRtpPkt)
    {
        if (ioctl(myFD, IXJCTL_HOOKSTATE) == 0) return ;
        write (myFD, inRtpPkt->getPayloadLoc(), RESID_RTP_RATE);

        // need to delete it since RTP doesn't delete it any more.
        delete inRtpPkt;
        inRtpPkt = 0;
    }

    // Reads packets from the device.
    int cc;
    //if (outRtpPkt)
    {
        if (ioctl(myFD, IXJCTL_HOOKSTATE) == 0) return ;
        cc = read(myFD, outBufferPkt, RESID_RTP_RATE);
        if (cc != RESID_RTP_RATE)
        {
            cerr << "Wrong packet size on resid card: " << cc << endl;
            return ;
        }
    }

    // Transmits packets to the network.
    if (audioStack)
    {
        audioStack->transmitRaw (outBufferPkt, cc);
    }

    // Do RTCP stuff.
    if (audioStack)
    {
        audioStack->processRTCP();
    }
}


int IPJackHardware::addToFdSet (fd_set* fd)
{
    if (audioActive)
    {
        FD_SET((audioStack->getRtpRecv())->getSocketFD(), fd);
        FD_SET((audioStack->getRtcpRecv())->getSocketFD(), fd);
    }

    FD_SET(myFD, fd);

    return 0;
} // end IPJackHardware::addToFdSet()


int IPJackHardware::sendSignal (HardwareSignal signal, int endpointIndex)
{
    switch (signal)
    {
        case SignalDialToneStart:
        {
            cerr << "%%%   Start dial tone %%%\n";
            // TBD ioctl(myFD, IXJCTL_DIALTONE_START);
            // provide simulated dialtone by playing an audio file
            provideTone(IPJackDialTone);
        }
        break;

        case SignalDialToneStop:
        {
            cerr << "%%%   Stop all tones %%%\n";
            // TBD ioctl(myFD, IXJCTL_CPT_STOP);
        }

        break;

        case SignalRingStart:
        {
            cerr << "%%%   Start ringing   %%%\n";
            ioctl(myFD, IXJCTL_RING);
        }
        break;

        case SignalRingStop:
        {
            cerr << "%%%   Stop ringing   %%%\n";
            // no call to explicitly stop ringing is available at this time
            //ioctl(myFD, IXJCTL_RING_STOP);
        }

        break;

        case SignalLocalRingbackStart:
        {
            cerr << "%%%   Start local ringback   %%%\n";
            //ioctl(myFD, IXJCTL_RINGBACK);
            // provide simulated dialtone by playing an audio file
            provideTone(IPJackRingbackTone);
        }
        break;

        case SignalLocalRingbackStop:
        {
            //cerr << "%%%   Stop local ringback   %%%\n";
            //ioctl(myFD, IXJCTL_CPT_STOP);
        }

        break;

        case SignalBusyStart:
        {
            cerr << "%%%   Start busy start   %%%\n";
            ioctl(myFD, IXJCTL_BUSY);
        }
        break;

        case SignalBusyStop:
        {
            cerr << "%%%   Stop busy stop   %%%\n";
            ioctl(myFD, IXJCTL_CPT_STOP);
        }
        break;

        default:
        {
            cerr << "requested signal not supported by hardware\n";
            return -1;
        }
        break;

    }
    return 0;
} // end IPJackHardware::sendSignal()


int IPJackHardware::audioStart (const HardwareAudioRequest& request,
                                const int channelNum)
{
    cerr << "%%%   Establishing audio   %%%\n";
    cerr << "Listening on port: " << request.localPort << "\n";
    cerr << "Sending to host: " << request.remoteHost << "\n";
    cerr << "Sending to port: " << request.remotePort << "\n";

    // TBD, once stable, change asserts to checks which return errorcodes

    // create new audioStack for this audio session
    // 0 is rtpPayloadPCUM
    // last paramter, -1, disables jitter buffer
    audioStack = new RtpSession(request.remoteHost, request.remotePort,
                                request.localPort, request.remotePort + 1,
                                request.localPort + 1, rtpPayloadPCMU,
                                rtpPayloadPCMU, 5);

    // allocate RTP packet spaces
    inRtpPkt = 0;

#ifdef USE_LINEAR16
    audioStack->setApiFormat (rtpPayloadL16_mono, RESID_RTP_RATE);
    audioStack->setNetworkFormat (rtpPayloadL16_mono, NETWORK_RTP_RATE);
#else
    audioStack->setApiFormat (rtpPayloadPCMU, RESID_RTP_RATE);
    audioStack->setNetworkFormat (rtpPayloadPCMU, NETWORK_RTP_RATE);
#endif


    // mark audio as active
    audioActive = true;
    audioHalfActive = true;
    hasPlayed = false;


#ifdef USE_LINEAR16
    ioctl(myFD, IXJCTL_PLAY_CODEC, LINEAR16);
    ioctl(myFD, IXJCTL_REC_CODEC, LINEAR16);
#else
    ioctl(myFD, IXJCTL_PLAY_CODEC, ULAW);
    ioctl(myFD, IXJCTL_REC_CODEC, ULAW);
#endif

    ioctl(myFD, IXJCTL_PLAY_START);
    ioctl(myFD, IXJCTL_REC_START);

    // Echo Cancellation doesn't work with VM.
    if (request.echoCancellation)
    {
        ioctl(myFD, IXJCTL_AEC_START, 1);
        cpLog (LOG_DEBUG, "echoCancellation is on");
    }
    else
    {
        ioctl(myFD, IXJCTL_AEC_START, 0);
        cpLog (LOG_DEBUG, "echoCancellation is off");
    }

    // Adjusts the volume...  see quicknet drivers for more info.
    ioctl(myFD, IXJCTL_MIXER, 0x0C03);   // 06 is too soft
    ioctl(myFD, IXJCTL_MIXER, 0x0F01);   // 02 is too soft


    return 0;
} // end IPJackHardware::audioStart()


int IPJackHardware::audioStop (const int channelNum)
{
    cerr << "%%%   Stopping audio   %%%\n";

    if (!audioActive) return 1;

    // make hardware calls to stop audio
    ioctl(myFD, IXJCTL_REC_STOP);
    ioctl(myFD, IXJCTL_PLAY_STOP);
    ioctl(myFD, IXJCTL_AEC_STOP);

    // TBD, need to close and reopen device to stablize ipjack
    // hardware, hopefully this can be cleaned up with new driver
    close(myFD);
    myFD = open(myDeviceName, O_RDWR);
    if (myFD < 0)
    {
        cerr << "Cannot reopen" << myDeviceName << endl;
        exit(1);
    }

    // close RTP session
    audioStack->transmitRTCPBYE();
    delete audioStack; audioStack = 0;
    if (inRtpPkt)
    {
        delete inRtpPkt;
        inRtpPkt = 0;
    }

    // mark audio as deactivated
    audioActive = false;
    audioHalfActive = true;
    hasPlayed = false;

    return 0;
} // end IPJackHardware::audioStop()


HardwareSignal
IPJackHardware::getSignalOn (MgcpPackageType package, int index) const
{
    printf("IPJackHardware::getSignalOn\n");
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return IPJackHardwareGenericSigOn[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return IPJackHardwareLineSigOn[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return IPJackHardwareDtmfSigOn[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cerr << "unrecognized package\n";
            return SignalNULL;

        }
        break;
    } // end switch

    return SignalNULL;

} // end IPJackHardware::getSignalOn()


HardwareSignal
IPJackHardware::getSignalOff (MgcpPackageType package, int index) const
{
    if (index < 0)
    {
        return SignalNULL;
    }

    switch (package)
    {
        case MgcpGenericMediaPackageType:
        {
            if (index < GenericReqSigMaxSignal)
            {
                return IPJackHardwareGenericSigOff[index];
            }
        }
        break;

        case MgcpLinePackageType:
        {
            if (index < LineReqSigMaxSignal)
            {
                return IPJackHardwareLineSigOff[index];
            }
        }
        break;

        case MgcpDTMFPackageType:
        {
            if (index < DtmfReqSigMaxSignal)
            {
                return IPJackHardwareDtmfSigOff[index];
            }
        }
        break;

        case MgcpMFPackageType:
        case MgcpTrunkPackageType:
        case MgcpHandsetPackageType:
        case MgcpRTPPackageType:
        case MgcpNetworkAccessServerPackageType:
        case MgcpAnnouncementPackageType:
        case MgcpScriptPackageType:
        case MgcpXVoiceMailPackageType:
        default:
        {
            cerr << "unrecognized package\n";
            return SignalNULL;

        }
        break;
    } // end switch

    return SignalNULL;

} // end IPJackHardware::getSignalOff()


void IPJackHardware::provideTone (IPJackTone tone)
{
    int size;
#ifdef USE_LINEAR16
    char buff1[LINEAR16_PAYLOAD];
#else
    char buff1[ULAW_PAYLOAD];
#endif

    int toneFD;
    bool provideTone = true;

    switch (tone)
    {
        case IPJackDialTone:
        {
            cerr << "%%%   Provide dial tone %%%\n";
            toneFD = open("Tone/dialtone", O_RDONLY);
        }
        break;


        case IPJackRingbackTone:
        {
            cerr << "%%%   Provide ringback tone %%%\n";
            toneFD = open("Tone/ringback", O_RDONLY);
        }
        break;

        default:
        {
            cerr << "unsupported tone requested\n";
            toneFD = -1;
        }
        break;

    } // end switch

    if (toneFD < 0)
    {
        cerr << "error opening tone file\n";
        provideTone = false;
    }
    else
    {
        bool redo = true;

        while (redo)
        {

            redo = false;

            close(myFD);

            // Used to stablize the device (?)
            vusleep(10000);


            // open device for write only
            cerr << "Open myDeviceName" << endl;
            myFD = open(myDeviceName, O_WRONLY);
            cerr << "Done Open myDeviceName" << endl;

            if (myFD < 0)
            {
                cerr << "*** cannot open *** " << myDeviceName << endl;
                provideTone = false;
            }
            else
            {
                struct timeval start;      // ctam
                struct timeval finish;     // ctam
                gettimeofday (&start, 0);    // ctam

                cerr << "Begin:  IXJCTL_PLAY_CODEC" << endl;
#ifdef USE_LINEAR16
                ioctl(myFD, IXJCTL_PLAY_CODEC, LINEAR16);
#else
                ioctl(myFD, IXJCTL_PLAY_CODEC, ULAW);
#endif
                ioctl(myFD, IXJCTL_PLAY_START);

                gettimeofday (&finish, 0);   // ctam

                long diff = finish.tv_usec -
                            start.tv_usec +
                            1000000 * (finish.tv_sec - start.tv_sec);

                if (diff > 100000)
                    redo = true;
            }
        }
    }

    // play until file ends or onhook or dtmf is ready
    while (provideTone)
    {
#ifdef USE_LINEAR16
        size = read(toneFD, buff1, LINEAR16_PAYLOAD);
#else
size = read(toneFD, buff1, ULAW_PAYLOAD);
#endif
        fflush(stdout);
        if (size > 0)
        {
            // play some tone
            write(myFD, buff1, size);

            if (ioctl(myFD, IXJCTL_DTMF_READY))
            {
                // digit is ready
                if (tone == IPJackDialTone)
                {
                    // stop tone and process digit
                    provideTone = false;
                }
                else
                {
                    // throw digits away while playing tone
                    ioctl(myFD, IXJCTL_GET_DTMF);
                }
            }
            else if (!(ioctl(myFD, IXJCTL_HOOKSTATE)))
            {
                // report change from offhook to onhook
                reportEvent(endpointQ, EventOnhook);
                hookStateOffhook = false;
                provideTone = false;
            }
            else if (myQ->size())
            {
                // need to stop tone and process endpoint msg
                provideTone = false;
            }
        }
        else
        {
            // end of file
            provideTone = false;
        }
    } // end while provideTone

    // stop playing and close file
    ioctl(myFD, IXJCTL_AEC_STOP);
    ioctl(myFD, IXJCTL_PLAY_STOP);
    close(myFD);

    myFD = open(myDeviceName, O_RDWR);

    if (myFD < 0)
    {
        cerr << "*** cannot open ***" << myDeviceName << "\n";
        exit(1);
    }
    close(toneFD);
} // end provideTone()
