
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ReqEv_cxx_Version =
    "$Id: ReqEv.cxx,v 1.16 2001/01/23 22:22:34 cktam Exp $";
#include "MgCommand.hxx"
#include "cpLog.h"
#include "ReqEv.h"

ReqEv::ReqEv (MgNotificationEvent event)
        : observedEvent(event)
{
    this->stop();
} 


ReqEv::~ReqEv (void)
{
} 



void
ReqEv::start (const string& requestId, Sptr < MgRequestedEvent > event)
{
    requested = true;
    myRequestId = requestId;
    // TBD notifiedEntity = rqnt->getNotifiedEntity();

    // map mgcp action into actions
    for (int i = 0; i < event->getNumAction(); i++)
    {
        switch (event->getItemAction(i).getActionType())
        {
            case MgcpRequestedActionTypeNotify:
            {
                action = ActionNotify;
            }
            break;

            case MgcpRequestedActionTypeAccumulate:
            {
                action = ActionAccumulate;
            }
            break;

            case MgcpRequestedActionTypeUseDigitMap:
            {
                action = ActionDigitMap;
            }
            break;

            case MgcpRequestedActionTypeSwap:
            {
                swapAudio = true;
            }
            break;

            case MgcpRequestedActionTypeIgnore:
            {
                action = ActionIgnore;
            }
            break;

            case MgcpRequestedActionTypeKeepActive:
            {
                keepSignalsActive = true;
            }
            break;

            case MgcpRequestedActionTypeEmbedNotificationRequest:
            {
                embeddedRequest = true;
            }
            break;

            default:
            {
                cpLog(LOG_ERR, "Unknown action ignored");
            }
            break;

        } 
    }
}


void
ReqEv::stop (void)
{
    requested = false;
    swapAudio = false;
    keepSignalsActive = false;
    embeddedRequest = false;
    action = ActionIgnore;
}


void
ReqEv::show (void)
{
    cout << "Requested Event:";
    cout << "\nrequested     : " << requested;
    cout << "\nrequestId     : " << myRequestId;
    cout << "\nnotifiedEntity: " << notifiedEntity;
    cout << "\nswapAudio     : " << swapAudio;
    cout << "\nkeepSignals   : " << keepSignalsActive;
    cout << "\nembedded      : " << embeddedRequest;
    cout << "\naction        : " << static_cast < int > (action);
    cout << '\n';
} 
