#ifndef REQ_SIG_H_
#define REQ_SIG_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const ReqSig_h_Version =
    "$Id: ReqSig.h,v 1.16 2001/01/23 22:22:34 cktam Exp $";

#include "MgCommand.hxx"
#include "common.h"
class Hardware;

/** Enumeration of signal types with respect to their predefined
 ** duration, note that for ReqSigTimeOut an extra duration parameter
 ** is required to specify the duration of the timer.
 */
enum SignalDurationType
{
    SignalOnOff,
    SignalTimeout,
    SignalBrief
};

/** Generic Media Package requested signals enumeration.  Provides
 ** indices with which to access array of Generic Media Package
 ** requested signals. An entry exists for each signal the call agent
 ** can ask the gateway to send.
 */
enum GenericReqSig
{
    GenericReqSigPatternDetected,
    GenericReqSigRingbackTone,
    GenericReqSigRingbackOnConnection,
    GenericReqSigConfirmTone,
    GenericReqSigNetworkCongestionTone,
    GenericReqSigInterceptTone,
    GenericReqSigPreemptionTone,
    GenericReqSigMaxSignal,
    GenericReqSigInvalidSignal
};

/** Line Package requested signals enumeration.  Provides indices
 ** with which to access array of Line Package requested signals. An
 ** entry exists for each signal the call agent can ask the gateway to
 ** send. 
 */
enum LineReqSig
{
    LineReqSigAdsiDisplay,
    LineReqSigVisualMsg,
    LineReqSigAnswerTone,
    LineReqSigBusyTone,
    LineReqSigCallerId,
    LineReqSigCallWaitTone,
    LineReqSigDialTone,
    LineReqSigMsgWaitInd,
    LineReqSigNetworkBusy,
    LineReqSigRinging,
    LineReqSigDsnctRing0,
    LineReqSigDsnctRing1,
    LineReqSigDsnctRing2,
    LineReqSigDsnctRing3,
    LineReqSigDsnctRing4,
    LineReqSigDsnctRing5,
    LineReqSigDsnctRing6,
    LineReqSigDsnctRing7,
    LineReqSigRingSplash,
    LineReqSigPromptTone,
    LineReqSigErrorTone,
    LineReqSigStutterTone,
    LineReqSigAlertTone,
    LineReqSigRecordWarnTone,
    LineReqSigCallCardSrvTone,
    LineReqSigOffhookWarnTone,
    LineReqSigDsnctTonePattern,
    LineReqSigMaxSignal,
    LineReqSigInvalidSignal
};


/** DTMF Package requested signals enumeration.  Provides indices
 ** with which to access array of DTMF Package requested signals. An
 ** entry exists for each signal the call agent can ask the gateway to
 ** send. 
 */
enum DtmfReqSig
{
    DtmfReqSig0,
    DtmfReqSig1,
    DtmfReqSig2,
    DtmfReqSig3,
    DtmfReqSig4,
    DtmfReqSig5,
    DtmfReqSig6,
    DtmfReqSig7,
    DtmfReqSig8,
    DtmfReqSig9,
    DtmfReqSigHash,
    DtmfReqSigStar,
    DtmfReqSigA,
    DtmfReqSigB,
    DtmfReqSigC,
    DtmfReqSigD,
    DtmfReqSigMaxSignal,
    DtmfReqSigInvalidSignal
};


/** Class used to store and track a call agent requested signal
 */
class ReqSig
{
    public:

        /** Creates a ReqSig and initializes all fields to hard coded
         ** values. These values are in accordance with the MGCP
         ** specification and are needed to handle the signal
         ** approriately.
         */
        ReqSig (Hardware& hardware,
                string endpointId,
                MgcpPackageType package,
                int signal,
                SignalDurationType type,
                int duration = 0);

        /** Destructor, stops the signal (if currently active)
         */
        ~ReqSig (void);

        /** Marks the signal as active and sets the hardware appropriately.
         */
        void
        start (const RQNT rqnt);

        /** Marks the signal as inactive and sets the hardware appropriately.
         */
        void
        stop (void);

        /** For debugging, dump all fields of signal to standard out
         */
        void
        show (void);

    private:

        /// true if requested by call agent
        bool active;

        /// RequestIdentifier supplied by call agent
        string requestId;

        MgcpPackageType myPackage;
        int myIndex;
        SignalDurationType myType;
        int myDuration;
        Fifo < HardwareMsg > * myHardwareQ;
        HardwareSignal myStartSignal;
        HardwareSignal myStopSignal;

        string myEndpointId;
};

#endif 
