#ifndef TFTP_CONFIG_H
#define TFTP_CONFIG_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const tftpCfg_h_Version =
    "$Id: tftpCfg.h,v 1.7 2001/01/23 22:22:34 cktam Exp $";

#include "inetLib.h"

#include <stdio.h>
#include <stdlib.h>

#include "tftpLib.h"

const int CONFIG_FILE_LEN         = 128;
const int GW_ENDPOINT_ID_MAX      = 256;
const int MAX_DEVICE_NAME_LEN     = 256;
const int MAX_AGENT_HOSTNAME_LEN  = 256;
const int MAX_DATA_LEN            = 256;


enum cnfgDataTags
{
    enumBoard,
    enumDeviceName,
    enumGatewayPort,
    enumAgentHostname,
    enumAgentPort,
    enumNumEndpoints,
    enumEndpointId,
    enumCntsPerEndpt,
    enumFirstRtpPort
} ;


typedef struct cfgData
{
    cnfgDataTags dataType;
    int dataLen;
    char dataStr[MAX_DATA_LEN];
    int numDataRead;
    int endptIdIdx;
};

typedef struct
{
    char ipAddr[INET_ADDR_LEN];
    char subnetMask[INET_ADDR_LEN];
    char tfptServerAddr[INET_ADDR_LEN];
    char configFileName[CONFIG_FILE_LEN];
}
MtaDhcpDataStruct;

typedef struct
{
    char deviceName[MAX_DEVICE_NAME_LEN];
    int gatewayPort;
    char agentHostname[MAX_AGENT_HOSTNAME_LEN];
    int agentPort;
    int numEndpoints;
    char endpointId[GW_ENDPOINT_MAX][GW_ENDPOINT_ID_MAX];
    int cntsPerEndpt;
    int firstRtpPort;  // start of range of rtp ports
}

ConfigDataStruct;

extern void tftpCfg(void);


#endif 
