#ifndef CONNECTION_H_
#define CONNECTION_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Connection_h_Version =
    "$Id: Connection.h,v 1.15 2000/12/18 23:42:25 bko Exp $";

#include "SdpSession.hxx"
#include "common.h"

class MgAgent;
class Hardware;

/** Connection States
 */
enum ConnectionState
{
    ConnectionClosed = 0,
    ConnectionHalfOpen,
    ConnectionOpen
};


/** Structure used to store and track a connection
 */
class Connection
{
    public:

        /** Creates a Connection.
         */
        Connection (MgAgent* agent, Fifo < HardwareMsg > * outputQ, int endpointId);

        /** Destructor
         */
        ~Connection (void);

        /** TBD change to MgcpResponseCode once available
         */
        MgcpResponseCode
        open (CRCX crcx);

        /** TBD change to MgcpResponseCode once available
         */
        MgcpResponseCode
        modify (MDCX mdcx);

        /** TBD change to MgcpResponseCode once available
         */
        MgcpResponseCode
        close (DLCX dlcx);

        /** member data access methods
         */
        inline ConnectionState
        getState (void)
        {
            return myState;
        };

        inline const string
        getId (void)
        {
            return myId;
        };

    private:

        /** Initializes myId.
         */
        void
        setMyId (void);

        /** Initializes myLocalSD by assigning an unused port from a range
         ** of usable ports.
         */
        void
        setMyLocalSD (void);

        /** Sends a request to the hardware for an audio channel using the
         ** existing member data to set up the paramters. Note that this
         ** does not guarantee that the hardware will be able to honor the
         ** request, in which case it may respond a failure event at some
         ** later time.  
         */
        void
        requestAudioSession (MgcpConnectionOptions options);

        /** Sends a request to the hardware to close an existing audio
         ** channel.
         */
        void
        closeAudioSession (void);

        /** Change state of Connection and display debug info
         */
        void
        changeState (ConnectionState);

        // call agent associated with the Connection
        MgAgent* myAgent;

        // location to send hardware requests
        Fifo < HardwareMsg > * myHardwareQ;

        // current state of the Connection
        ConnectionState myState;

        // connection id
        string myId;

        // the MGCP parameters associated with the Connection
        int myEndpointIndex;
        string myCallId;
        string myMode;
        SdpSession myLocalSD;
        SdpSession myRemoteSD;
};

#endif // CONNECTION_H_
