
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Gateway_cxx_Version =
    "$Id: Gateway.cxx,v 1.25 2001/01/18 02:45:17 liuhong Exp $";
#include <stdio.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <typeinfo>
#include <unistd.h>
#include "Gateway.h"
#include "Endpoint.h"
#include "NullHardware.h"
#include "TrunkHardware.h"
#include "TestHardware.h"
#include "IPJackHardware.h"
// #include "vmHardware.h"
#include "parse3tuple.h"
#include "support.hxx"

// global benchmark testing data
int numOfRevCalls;
int numOfAttCalls;
int numOfCRCX;
int numOfMDCX;
int numOfDLCX;
int numOfRQNT;
int numOfBusy;
int numOfRestart;

time_t startTime;
time_t endTime;
time_t statTime;

timeval RQNTTime[GW_ENDPOINT_MAX];
timeval CRCXTime[GW_ENDPOINT_MAX];
timeval MDCXTime[GW_ENDPOINT_MAX];
timeval DLCXTime[GW_ENDPOINT_MAX];
long RQNT_TO_CRCX[GW_ENDPOINT_MAX];
long CRCX_TO_MDCX[GW_ENDPOINT_MAX];
long MDCX_TO_DLCX[GW_ENDPOINT_MAX];
long DLCX_TO_RQNT[GW_ENDPOINT_MAX];
long RQNT_TO_DLCX[GW_ENDPOINT_MAX];

int numOfRQNT_TO_CRCX[GW_ENDPOINT_MAX];
int numOfCRCX_TO_MDCX[GW_ENDPOINT_MAX];
int numOfMDCX_TO_DLCX[GW_ENDPOINT_MAX];
int numOfDLCX_TO_RQNT[GW_ENDPOINT_MAX];
int numOfRQNT_TO_DLCX[GW_ENDPOINT_MAX];

long max_RQNT_TO_CRCX[GW_ENDPOINT_MAX];

int numOfBadStates;

const string restart = "restart";

map < string, int > gwEndpointMap;

/*********************************************************************
 
Gateway Extras
 
*********************************************************************/

// global configuration data
GatewayConfigData configData;

// global ptr to the gateway
Gateway* theGateway;

/** Callback function to start pthread for each endpoint.
 */
void* endpointCallback (void* objectPtr)
{
    Gateway* object = static_cast < Gateway* > (objectPtr);

    object->handleEndpoint(NULL);
    return NULL;
} // end endpointCallback()


/** Callback function to start pthread for MGCP stack of agent.
 */
void* agentCallback (void* objectPtr)
{
    MgAgent* object = static_cast < MgAgent* > (objectPtr);

    //    object->threadProcess();
    object->asyncProcess();
    return NULL;
} // end agentCallback()



/*********************************************************************
 
Gateway
 
*********************************************************************/

Gateway::Gateway (void)
{
    int errorcode;
    int i;

    numOfAttCalls = 0;
    numOfRevCalls = 0;
    numOfCRCX = 0;
    numOfMDCX = 0;
    numOfDLCX = 0;
    numOfRQNT = 0;
    numOfBusy = 0;
    numOfRestart = 0;
    numOfBadStates = 0;

    // configure gateway according to configuration data
    myPort = configData.gatewayPort;
    if (configData.firstRtpPort % 2)
    {
        currentRtpPort = configData.firstRtpPort + 1;
    }
    else
    {
        currentRtpPort = configData.firstRtpPort;
    }

    // set the global ptr to this gateway
    theGateway = this;

    // create mutexs
    errorcode = pthread_mutex_init(&cid_mutex, NULL);
    assert(!errorcode);
    errorcode = pthread_mutex_init(&rtp_mutex, NULL);
    assert(!errorcode);

    // create MGCP domain name from standard hostname
    char hostName[256];
    gethostname(hostName, 256);
    myDomainName = hostName;

    // create agent
    myAgent = NULL;
    setAgent(configData.agentHostname, configData.agentPort);
    assert(myAgent != NULL);

    // create queues
    // hardwareQ = new Fifo<HardwareMsg>;
    // assert(hardwareQ != NULL);

    hardwareQ = NULL;

    // use only one endpointQ to handle all the endpoints.
    endpointQ[0] = new Fifo < EndpointMsg > ;
    assert(endpointQ[0] != NULL);

    // Set the rest of them to NULL
    i = 0;
    while (i < GW_ENDPOINT_MAX)
    {
        myHardware[i] = NULL;
        i++;
    }


    // create all endpoints, this MUST be done after creating hardware
    activeEndpoints = 0;
    for (i = 0; i < configData.numEndpoints; i++)
    {
        digitString[i] = digitArray[i];

        dialNumber[i] = getDialNumber(i, configData.dialNumber, digitString[i]);

        cpLog(LOG_DEBUG, "Creating Endpoint: %d, dial num = %s", i, dialNumber[i]);

        myEndpointId[i] = configData.endpointId[i];
        myEndpoint[i] = new Endpoint(myEndpointId[i] + "@" + myDomainName,
                                     dialNumber[i],
                                     configData.runmode,
                                     i,
                                     configData.cntsPerEndpt,
                                     myAgent,
                                     *myHardware[i],
                                     endpointQ[0],
                                     hardwareQ);

        RQNT_TO_CRCX[i] = 0;
        CRCX_TO_MDCX[i] = 0;
        MDCX_TO_DLCX[i] = 0;
        DLCX_TO_RQNT[i] = 0;
        numOfRQNT_TO_CRCX[i] = 0;
        numOfCRCX_TO_MDCX[i] = 0;
        numOfMDCX_TO_DLCX[i] = 0;
        numOfDLCX_TO_RQNT[i] = 0;
        max_RQNT_TO_CRCX[i] = 0;


        gwEndpointMap.insert (pair < string, int > (string(myEndpointId[i] + "@" + myDomainName),
                              i));

        assert(myEndpoint[i] != NULL);
        activeEndpoints++;
    }

    // Set the rest of them to NULL.

    while (i < GW_ENDPOINT_MAX )
    {
        myEndpoint[i] = NULL;
        i++;
    }

    errorcode = pthread_create( &(myEndpointThread[0]), NULL,
                                endpointCallback,
                                (void *) theGateway );
    assert(!errorcode);


    // send restart in progress to agent.

    /*
        MgRestartInProgress myRestart("*", restart, "none");
        myAgent->send(myRestart);
    */

    sleep(1);

    for (i = 0; i < configData.numEndpoints; i++)
    {
        myEndpoint[i]->restartInProgress(restart);
    }

} // end Gateway::Gateway()


Gateway::~Gateway (void)
{
    int numEndpoints = activeEndpoints;
    int errorcode;

    deleteAgent();
    deleteHardware();
    for (int i = 0; i < numEndpoints; i++)
    {
        deleteEndpoint(i);
    }
    delete [] myEndpointId;

    errorcode = pthread_mutex_destroy(&cid_mutex);
    assert(errorcode != EBUSY /* some thread has cid_mutex locked */);
    assert(!errorcode);
    errorcode = pthread_mutex_destroy(&rtp_mutex);
    assert(errorcode != EBUSY /* some thread has rtp_mutex locked */);
    assert(!errorcode);

} // end Gateway::~Gateway()


MgAgent*
Gateway::setAgent (string agentHostname, int agentPort)
{
    int errorcode;

    MgAgent* newAgent = new MgAgent(agentHostname.c_str(), agentPort, myPort);
    if (newAgent == NULL)
    {
        cpLog(LOG_ERR, "Could not create new agent, keeping old agent\n");
    }
    else
    {
        deleteAgent();
        myAgent = newAgent;
    }

    // create thread for mgcp stack to communicate with agent
    errorcode = pthread_create( &myAgentThread, NULL,
                                agentCallback, (void *) myAgent );

    assert(!errorcode);
    return newAgent;
} // end Gateway::setAgent()


int
Gateway::getEndpointIndex (string id)
{
    // residential and trunk gateways should not have to deal with
    // wildcards; the use of "*" for ALL and "$" for ANY is reserved
    // for media gateways

    // locate matching endpoint id from list

    int endpointIndex;

    map < string, int > ::iterator p = gwEndpointMap.find(id);

    if (p != gwEndpointMap.end())
    {
        endpointIndex = p->second;
        return endpointIndex;
    }
    else
        return GW_INV_ENDPOINT_INDEX;

} // end Gateway::getEndpointIndex()


void
Gateway::forwardCommand (CMD cmd, MgCommandType type,
                         Fifo < EndpointMsg > * outputQ,
                         int endpointIndex)
{

    /**********************************************************************
     
        // this is comment is incorrect:
     
        // reuse same msg each time, fifo makes its own copy and there is
        // only one gateway thread
        static EndpointMsg msg;
        static EndpointAgentCommand command;
        
    **********************************************************************/

    // reuse same msg each time, fifo makes its own copy and there is
    // only one gateway thread
    static EndpointMsg msg;
    EndpointAgentCommand command;

    command.type = type;
    command.cmd = cmd;

    msg.type = EndpointAgentCommandType;
    msg.command = command;
    msg.endpointIndex = endpointIndex;

    cpLog(LOG_DEBUG, "forwardCommand,   type =  %d\n", type);

    outputQ->add(msg);
} // end Gateway::forwardCommand()


void
Gateway::deleteAgent (void)
{
    if (myAgentThread != (pthread_t)NULL)
    {
        pthread_join( myAgentThread, NULL );
        myAgentThread = (pthread_t)NULL;
    }

    if (myAgent)
    {
        delete myAgent;
        myAgent = NULL;
    }
}


void
Gateway::deleteHardware (void)
{
    if (myHardwareThread != (pthread_t)NULL)
    {
        pthread_join( myHardwareThread, NULL );
        myHardwareThread = (pthread_t)NULL;
    }
    /*
        for (int i = 0; i < GW_ENDPOINT_MAX; i++)
        {
            if (myHardware[i])
            {
    	    delete myHardware[i];
    	    myHardware[i] = NULL;
            }
        };
        // flush hardware queue, then delete it
     
        while (hardwareQ->size() != 0)
        {
    	hardwareQ->getNext();
        }
        delete hardwareQ;
        hardwareQ = NULL;
    */
} // end Gateway::deleteHardware()


void
Gateway::deleteEndpoint (int index)
{
    if (index < activeEndpoints)
    {
        if (myEndpointThread[index] != (pthread_t)NULL)
        {
            pthread_join( myEndpointThread[index], NULL );
            myEndpointThread[index] = (pthread_t)NULL;
        }

        if (myEndpoint[index] != NULL)
        {
            delete myEndpoint[index];
            myEndpoint[index] = NULL;
        }
        activeEndpoints--;
    }

    // flush endpoint queue, then delete it
    while (endpointQ[index]->size() != 0)
    {
        endpointQ[index]->getNext();
    }
    delete endpointQ[index];
    endpointQ[index] = NULL;

} // end Gateway::deleteEndpoint()


void
Gateway::handleHardware(void* params)
{
    // process forever on behalf of test hardware
    fd_set readfds;
    bool operational = true;
    int dummy = 0;
    struct timeval tv;

    while (operational)
    {
        // reset file descriptor
        FD_ZERO(&readfds);
        // addToFdSet(&readfds);
        //if (maxFd <= gateway.myAgent->getFd())
        //maxFd =  gateway.myAgent->getFd() + 1;
        tv.tv_usec = 300;
        tv.tv_sec = 0;

        select(0, NULL, NULL, NULL, &tv);

        // block on select but timeout needed to service myQ

        if (dummy == 1)
        {
            cpLog(LOG_ERR, "Select returned with an error - ignore");
        }
        else // if (retval > 0)
        {
            for (int i = 0; i < GW_ENDPOINT_MAX; i++)
            {
                if (myHardware[i])
                {
                    // hardware input received
                    if (myHardware[i]->process(&readfds) < 0)
                    {
                        cpLog(LOG_ERR, "Hardware failed - shutting down");
                        operational = false;
                    }
                }
            }

        }

    } // end while

}
;


void
Gateway::handleEndpoint(void* params)
{
    EndpointMsg msg;

    while (1)
    {
        msg = endpointQ[0]->getNext();

        cpLog (LOG_DEBUG, "EP Q:    msg.type = %d    msg.endpointIndex = %d",
               msg.type, msg.endpointIndex);

        if (msg.endpointIndex < GW_ENDPOINT_MAX && msg.endpointIndex >= 0)
        {
            switch (msg.type)
            {
                case EndpointAgentCommandType:
                {
                    cpLog (LOG_DEBUG, "Got Agent msg" );
                    myEndpoint[msg.endpointIndex]->processAgentCommand(msg.command);
                }
                break;

                case EndpointHardwareEventType:
                {
                    cpLog (LOG_DEBUG, "Got hardware msg");
                    myEndpoint[msg.endpointIndex]->processHardwareEvent(msg.event);
                }
                break;

                case EndpointTimeoutType:
                {
                    cpLog (LOG_DEBUG, "Timeout");
                    myEndpoint[msg.endpointIndex]->processTimeout(msg.timeout);
                }
                break;

                default:
                {
                    cpLog (LOG_DEBUG, "Unknown Msg");
                }
                break;
            }
        }
        else
        {
            cerr << "Invalid endpointIndex!!!" << endl;
        }

    };

}

Gateway* fakeGateway;

void killGatewayGlobal(int sig)
{
    fakeGateway->killGateway(sig);
}

void
Gateway::gatewayMain (void)
{
    CMD cmd;
    MgCommandType cmdType;
    int endpointIndex = GW_INV_ENDPOINT_INDEX;

    struct timeval timeNowRSIP;
    struct timeval timeRSIP;



    // Intercept the Interrupt signal from user and send to killGateway()
    fakeGateway = this;

    signal(SIGINT, killGatewayGlobal);

    fd_set fds0;
    myAgent->addToFdSet (&fds0);

    fd_set fds;
    fd_set fd_good;
    struct timeval tv;

    gettimeofday(&timeRSIP, NULL);

    const long RSIP_DELAY = 5000;
    int rsipEndpoint = 0;
    bool doRestart = false;

    myAgent->addToFdSet(&fd_good);

    bool noNonBlockFlag;
    // bool neverBlock = false;

    while (1)
    {
        if (doRestart)
        {
            gettimeofday(&timeNowRSIP, NULL);
            long x;
            if ((x = tv_diff(timeNowRSIP, timeRSIP)) > RSIP_DELAY)
            {
                //                 cout << "restarted " << rsipEndpoint << endl;
                myEndpoint[rsipEndpoint]->restartInProgress(restart);
                timeRSIP.tv_sec = timeNowRSIP.tv_sec;
                timeRSIP.tv_usec = timeNowRSIP.tv_usec;
            }
            else
            {
                //                cout << "diff: " << x << endl;
            }

            rsipEndpoint++;
            if (rsipEndpoint >= configData.numEndpoints)
            {
                doRestart = false;
            }
        }


        tv.tv_sec = 0;
        tv.tv_usec = 300;

        fds = fd_good;
        int maxFd = 32;

        noNonBlockFlag = true;

        select(maxFd, &fds, NULL, NULL, &tv);
        while (noNonBlockFlag)
        {

            // check time to print out stats.

            time(&endTime);

            if ((difftime(endTime, statTime) > configData.debugMsgInterval))
            {
                cpLog (LOG_INFO, "***  STATS  ***  Attempted = %d   Received = %d   Bad = %d" ,
                       numOfAttCalls,
                       numOfRevCalls,
                       numOfBadStates);
                memcpy(&statTime, &endTime, sizeof(endTime));
            };

            if (!doRestart || myAgent->checkMsgReady(0))
            {
                cmd = myAgent->asyncReceive();
                cmdType = cmd->getType();

                // if command is for a given endpoint, forward it
                switch (cmdType)
                {
                    case MgEndpointConfigurationType:
                    {
                        EPCF epcf;
                        epcf.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(epcf->getEndpointId());
                    }
                    break;

                    case MgCreateConnectionType:
                    {
                        CRCX crcx;
                        crcx.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(crcx->getEndpointId());

                        numOfCRCX++;
                    }
                    break;

                    case MgModifyConnectionType:
                    {
                        MDCX mdcx;
                        mdcx.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(mdcx->getEndpointId());

                        numOfMDCX++;
                    }
                    break;

                    case MgDeleteConnectionAgentType:
                    {
                        DLCX dlcx;
                        dlcx.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(dlcx->getEndpointId());

                        numOfDLCX++;
                    }
                    break;

                    case MgNotificationRequestType:
                    {
                        RQNT rqnt;
                        rqnt.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(rqnt->getEndpointId());

                        numOfRQNT++;
                    }
                    break;

                    case MgAuditEndpointType:
                    {
                        AUEP auep;
                        auep.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(auep->getEndpointId());
                    }
                    break;

                    case MgAuditConnectionType:
                    {
                        AUCX aucx;
                        aucx.dynamicCast(cmd);
                        endpointIndex = getEndpointIndex(aucx->getEndpointId());
                    }
                    break;

                    default:
                    {
                        //		cpLog(LOG_ERR, "Rcvd unknown cmd, type = %d",
                        //		      cmdType);
                    }

                } // end switch

                // send cmd to appropriate endpoint
                if (endpointIndex != GW_INV_ENDPOINT_INDEX)
                {
                    // NOTE:  Use only endpointQ[0]!!!
                    // Only one queue is created!!
                    forwardCommand(cmd, cmdType, endpointQ[0], endpointIndex);
                }
                else
                {
                    cpLog(LOG_DEBUG, "Gateway needes to handle command, type = %d",
                          cmdType);
                }
            }
            if (doRestart == true)
            {
                noNonBlockFlag = false;
            }
        }
    } // end while

} // end Gateway::gatewayMain()



int
Gateway::newRtpPort (void)
{
    int newPort;
    int errorcode;
    // lock
    errorcode = pthread_mutex_lock(&rtp_mutex);
    assert( !errorcode );

    if ( currentRtpPort >= (configData.firstRtpPort + GW_NUM_RTP_PORTS) )
    {
        currentRtpPort = configData.firstRtpPort;
    }

    // intentionally increment after assigning
    newPort = currentRtpPort;
    currentRtpPort += 2;

    // unlock
    errorcode = pthread_mutex_unlock(&rtp_mutex);
    assert( !errorcode );
    return newPort;
} // end Gateway::newRtpPort()


void
deleteRtpPort (int port)
{
    // nothing needed for this simplfied scheme
} // end Gateway::deleteRtpPort()



string
Gateway::newConnectionId (void)
{
    // cycle through numbers and assume connection id will be free
    // before wrapping around
    static unsigned long connectionNumber = 1;
    string id;
    int errorcode;
    // lock
    errorcode = pthread_mutex_lock(&cid_mutex);
    assert( !errorcode );

    id = "c" + itos(connectionNumber++);


    // unlock
    errorcode = pthread_mutex_unlock(&cid_mutex);
    assert( !errorcode );

    return id;

} // end Gateway::newConnectionId()



char*
Gateway::getDialNumber(int index, string startNum, char* digitString)
{
    int tempNum;
    /*
        for (int i = 0; i < MAX_DIGIT_BUFFER; i++)
        {
            digitArray[i] = '\0';
        }
    */
    tempNum = atoi(startNum.c_str()) + index;

    sprintf(digitString, "%d", tempNum);

    /*
        if (tempNum >=  100000)
            sprintf (digitString[myEndpointIndex], "0%d", tempNum);
        else if (tempNum >= 10000)
            sprintf (digitString[myEndpointIndex], "00%d", tempNum);
        else if (tempNum >= 1000)
            sprintf (digitString[myEndpointIndex], "000%d", tempNum);
        else if (tempNum >= 100)
            sprintf (digitString[myEndpointIndex], "0000%d", tempNum);
        else if (tempNum >= 10)
            sprintf (digitString[myEndpointIndex], "00000%d", tempNum);
        else if (tempNum >= 0)
            sprintf (digitString[myEndpointIndex], "000000%d", tempNum);
    */ 
    return digitString;
}


void
Gateway::killGateway(int sig)
{
    time(&endTime);

    double TotalRQNT_TO_CRCX = 0;
    double TotalCRCX_TO_MDCX = 0;
    double TotalMDCX_TO_DLCX = 0;
    double TotalDLCX_TO_RQNT = 0;
    int TotalnumOfRQNT_TO_CRCX = 0;
    int TotalnumOfCRCX_TO_MDCX = 0;
    int TotalnumOfMDCX_TO_DLCX = 0;
    int TotalnumOfDLCX_TO_RQNT = 0;


    if (configData.runmode == "CallingGW")
    {
        for (int i = 0; i < configData.numEndpoints; i++)
        {
            TotalRQNT_TO_CRCX += RQNT_TO_CRCX[i];
            TotalCRCX_TO_MDCX += CRCX_TO_MDCX[i];
            TotalMDCX_TO_DLCX += MDCX_TO_DLCX[i];
            TotalDLCX_TO_RQNT += DLCX_TO_RQNT[i];

            RQNT_TO_DLCX[i] = RQNT_TO_CRCX[i] + CRCX_TO_MDCX[i] + MDCX_TO_DLCX[i];

            TotalnumOfRQNT_TO_CRCX += numOfRQNT_TO_CRCX[i];
            TotalnumOfCRCX_TO_MDCX += numOfCRCX_TO_MDCX[i];
            TotalnumOfMDCX_TO_DLCX += numOfMDCX_TO_DLCX[i];
            TotalnumOfDLCX_TO_RQNT += numOfDLCX_TO_RQNT[i];

            if (numOfRQNT_TO_CRCX[i] != 0)
            {
                cerr << "ep [" << i << "]:  numOfRQNT_TO_CRCX = " << numOfRQNT_TO_CRCX[i];
                cerr << ", ave (usec) = " << RQNT_TO_CRCX[i] / numOfRQNT_TO_CRCX[i];
                cerr << ", max (usec) = " << max_RQNT_TO_CRCX[i] << endl;
            }
        };
    };


    printf("\n\n\n\n******************************************************\n");

    for (int i = 0; i < 3; i++)
        printf("***************   GOT KILL SIGNAL!!!   ***************\n");

    printf("******************************************************\n");
    cerr << "      - STATS -" << endl;
    cerr << "      Attempted Calls     = " << numOfAttCalls << endl;
    cerr << "      Received Calls      = " << numOfRevCalls << endl;
    cerr << "      num of CRCX         = " << numOfCRCX << endl;
    cerr << "      num of MDCX         = " << numOfMDCX << endl;
    cerr << "      num of DLCX         = " << numOfDLCX << endl;
    cerr << "      num of RQNT         = " << numOfRQNT << endl;
    if (TotalnumOfRQNT_TO_CRCX != 0)
    {
        cerr << "      RQNT_TO_CRCX (usec) = " << TotalRQNT_TO_CRCX / TotalnumOfRQNT_TO_CRCX;
        cerr << ", " << TotalnumOfRQNT_TO_CRCX << endl;
        cerr << "      CRCX_TO_MDCX (usec) = " << TotalCRCX_TO_MDCX / TotalnumOfCRCX_TO_MDCX;
        cerr << ", " << TotalnumOfCRCX_TO_MDCX << endl;
        cerr << "      MDCX_TO_DLCX (usec) = " << TotalMDCX_TO_DLCX / TotalnumOfMDCX_TO_DLCX;
        cerr << ", " << TotalnumOfMDCX_TO_DLCX << endl;
        cerr << "      DLCX_TO_RQNT (usec) = " << TotalDLCX_TO_RQNT / TotalnumOfDLCX_TO_RQNT;
        cerr << ", " << TotalnumOfDLCX_TO_RQNT << endl;
    };
    cerr << "      Busy                = " << numOfBusy << endl;
    cerr << "      Restarts            = " << numOfRestart << endl;
    cerr << "      Bad States          = " << numOfBadStates << endl;
    cerr << "      Total time (sec)    = " << difftime(endTime, startTime) << endl;
    printf("******************************************************\n");
    printf("******************************************************\n");
    exit(0);
}


/*********************************************************************
 
Configuration File Handling
 
*********************************************************************/

/// Enumeration of valid tags in gateway configuration file
enum
{
    GatewayConfigTagHardware = 0,
    GatewayConfigTagGatewayPort,
    GatewayConfigTagAgentHostname,
    GatewayConfigTagAgentPort,
    GatewayConfigTagNumEndpoints,
    GatewayConfigTagEndpoint,
    GatewayConfigTagCntsPerEndpt,
    GatewayConfigTagFirstRtpPort,
    GatewayConfigTagRunMode,
    GatewayConfigTagCallDuration,
    GatewayConfigTagCallDelay,
    GatewayConfigTagAnswerDelay,
    GatewayConfigTagDialNumber,
    GatewayConfigTagDebugMsgInt,
    GatewayConfigTagEndpointDelay,
    GatewayConfigTagReceiveGWHU,
    GatewayConfigTagMax,
    GatewayConfigInvalid
} GatewayConfigTag;

/// Valid tag strings for gateway configuration file
static char* gatewayConfigTagString[GatewayConfigTagMax] =
    {
        "GatewayHardware",
        "GatewayPort",
        "GatewayAgentHostname",
        "GatewayAgentPort",
        "GatewayNumEndpoints",
        "GatewayEndpoint",
        "GatewayCntsPerEndpt",
        "GatewayFirstRtpPort",
        "GatewayRunMode",
        "GatewayCallDuration",
        "GatewayCallDelay",
        "GatewayAnswerDelay",
        "DialNumber",
        "DebugMsgInterval",
        "BetweenEndpointDelay",
        "ReceiveGWHU"
    };


/** Converts a tag string to an index, takes a string and returns the
 ** corresponding index, or GatewayConfigInvalid if matching tag is
 ** not found.
 */
int
gatewayConfigTag2i (char* tag)
{
    int i;

    // search until matching tag is found
    for (i = 0; i < GatewayConfigTagMax; i++)
    {
        if (strcmp(tag, gatewayConfigTagString[i]) == 0)
        {
            //cpLog (LOG_DEBUG, "Matched tag = %s", tag);
            return i;
        }
    }

    // no match found, report error
    cpLog (LOG_ERR, "Unknown tag = %s", tag);
    return GatewayConfigInvalid;

} // end gatewayConfigTag2i()


/** Initializes gateway configuration data based on valid tags
 ** extracted from configuration file. Takes a valid tag/type/value
 ** and sets the corresponding field in the configuration data.
 */
void
setGatewayConfigParameter (char* tag, char* type, char* value)
{
    // static int index;

    switch (gatewayConfigTag2i(tag))
    {
        case GatewayConfigTagHardware:
        {
            configData.deviceName = value;
        }
        break;

        case GatewayConfigTagGatewayPort:
        {
            configData.gatewayPort = atoi(value);
        }
        break;

        case GatewayConfigTagAgentHostname:
        {
            configData.agentHostname = value;
        }
        break;

        case GatewayConfigTagAgentPort:
        {
            configData.agentPort = atoi(value);
        }
        break;

        case GatewayConfigTagNumEndpoints:
        {
            configData.numEndpoints = atoi(value);
        }
        break;

        case GatewayConfigTagEndpointDelay:
        {
            configData.betweenEndpointDelay = atoi(value);
        }
        break;

        case GatewayConfigTagEndpoint:
        {
            configData.startEndpoint = atoi(value);
        }
        break;

        case GatewayConfigTagCntsPerEndpt:
        {
            configData.cntsPerEndpt = atoi(value);
        }
        break;

        case GatewayConfigTagFirstRtpPort:
        {
            configData.firstRtpPort = atoi(value);
        }
        break;

        case GatewayConfigTagRunMode:
        {
            configData.runmode = value;
        }
        break;


        case GatewayConfigTagCallDuration:
        {
            configData.callDuration = atoi(value);
        }
        break;

        case GatewayConfigTagCallDelay:
        {
            configData.callDelay = atoi(value);

        }
        break;

        case GatewayConfigTagAnswerDelay:
        {
            configData.answerDelay = atoi(value);
        }
        break;


        case GatewayConfigTagDebugMsgInt:
        {
            configData.debugMsgInterval = atoi(value);
        }
        break;

        case GatewayConfigTagReceiveGWHU:
        {
            configData.receiveGWHU = atoi(value);
        }
        break;

        case GatewayConfigTagDialNumber:
        {
            configData.dialNumber = value;
        }
        break;

        default:
        {
            cpLog (LOG_ERR, "Invalid tag index");
        }
        break;
    } // end switch

} // end setGatewayConfigParameter()



void
showConfigData (void)
{
    cerr << "\n-------------------------------------------------\n";
    cerr << "Gateway Configuration Data";
    cerr << "\nDevice Name:    " << configData.deviceName;
    cerr << "\nGateway Port#:  " << configData.gatewayPort;
    cerr << "\nAgent Hostname  " << configData.agentHostname;
    cerr << "\nAgent Port#:    " << configData.agentPort;
    cerr << "\nEndpoints:      " << configData.numEndpoints;
    cerr << "\nCntsPerEndpt:   " << configData.cntsPerEndpt;
    cerr << "\nRtp port range: " << configData.firstRtpPort
    << "-"
    << (configData.firstRtpPort + GW_NUM_RTP_PORTS - 1);

    cerr << "\nRun Mode:       " << configData.runmode;
    cerr << "\nCallDuration:   " << configData.callDuration;
    cerr << "\nCallDelay:      " << configData.callDelay;
    cerr << "\nAnswerDelay:    " << configData.answerDelay;
    cerr << "\nDialNumber:     " << configData.dialNumber;
    cerr << "\nDebugMsgInt:    " << configData.debugMsgInterval;
    cerr << "\nReceiveGWHU:    " << configData.receiveGWHU;
    cerr << "\nStartEndpoint   " << configData.startEndpoint;
    cerr << "\nBetween Endpt   " << configData.betweenEndpointDelay;

    cerr << "\nEndptId[0]:  " << configData.endpointId[0];

    cerr << "   to   ";
    cerr << "EndptId[" << configData.numEndpoints - 1 << "]:  ";
    cerr << configData.endpointId[configData.numEndpoints - 1];

    /*
        for (int i = 0; i < configData.numEndpoints; i++)
        {
    	if (i < 10)
    	{
    	    cerr << "\nEndpointId[" << i << "]:  " << configData.endpointId[i];
    	}
    	else
    	{
    	    cerr << "\nEndpointId[" << i << "]: " << configData.endpointId[i];
    	}
        }
    */
    cerr << "\n-------------------------------------------------\n";


} // end showConfigData()


/*********************************************************************
 
main
 
*********************************************************************/

/** Reads configuration file into global data structure and launches
 ** the gateway
 */
int
main (int argc, char* argv[])
{
    // setup logfile
    cpLogSetLabel ("gw");

    int GW_DEBUG_LEVEL = atoi(getenv("GW_DEBUG_LEVEL") ? getenv("GW_DEBUG_LEVEL") : "0");

    if (GW_DEBUG_LEVEL > 0)
    {
        cpLogSetPriority (LOG_DEBUG);
    }
    else
        cpLogSetPriority (LOG_INFO);



    // read configuration file
    cpLog(LOG_DEBUG, "Loading gateway configuration from 'gw.cfg'");

    if (parse3tuple ("gw.cfg", setGatewayConfigParameter) == false)
    {
        cpLog(LOG_ERR, "Error parsing configuration file");
    }

    /*
        // use argv[1] as number of calls 
     
        configData.numOfCalls = atoi(argv[1]);
    */
    // use argv[2] as agent hostname

    if (argc >= 2)
    {
        configData.agentHostname = argv[1];
    };

    configData.deviceName = "test";
    // create the endpoint ids

    for (int i = 0; i < configData.numEndpoints; i ++)
    {
        char* tempString;
        char tempArray[10];
        tempString = tempArray;
        sprintf(tempString, "%s%d", "ep", i + configData.startEndpoint);
        configData.endpointId[i] = tempString;
    }

    /*
        if (argc >= 3)
        {
    	configData.agentHostname = argv[2];
    	cpLog(LOG_DEBUG, "Override config file, set agent hostname = %s",
    	      configData.agentHostname.c_str());
        }
    */
    // show config data
    showConfigData();

    // create new gateway according to configuration data
    Gateway gateway;

    time(&startTime);
    statTime = startTime;
    // startup gateway, should never return
    gateway.gatewayMain();

    gateway.killGateway(SIGINT);

    return 0;
} // end main()
