
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Hardware_cxx_Version =
    "$Id: Hardware.cxx,v 1.12 2000/12/18 23:42:25 bko Exp $";
#include "Hardware.h"
#include "Gateway.h"

extern GatewayConfigData configData;

void
Hardware::processEndpointMsg (const HardwareMsg& msg)
{
    switch (msg.type)
    {
        case HardwareSignalType:
        {
            sendSignal(msg.signal);
        }
        break;

        case HardwareAudioType:
        {
            switch (msg.request.type)
            {
                case AudioStart:
                {
                    audioStart(msg.request);
                }
                break;

                case AudioStop:
                {
                    audioStop();
                }
                break;

                default:
                {
                    cpLog (LOG_ERR, "Unknown audio request = %d",
                           msg.request.type);
                }
            } // end switch (msg.request.type)
        }

        break;

        case HardwareStartNewCall:
        {

            //            rand_number = 1+(int) ((.5* configData.callDelay)*rand()/(RAND_MAX+1.0));

            /*
                        stopTimer(HWTimeoutStartCall);
                        startTimer(HWTimeoutStartCall, configData.callDelay);
            */

            //                configData.callDelay - (0.5 * configData.callDelay) + (2 * rand_number));

        }

        break;

        case HardwareTimeoutType:
        {
            if (msg.timeout == GW_FAKE_RSIP)
            {
                // do nothing.
            }

            else
                chgCallState(msg.timeout);
        }
        break;

        default:
        {
            cpLog (LOG_ERR, "Unknown endpoint msg = %d",
                   msg.type);
        }
        break;
    } // end switch


} // end Hardware::processEndpointMsg()



void
Hardware::reportEvent (Fifo < EndpointMsg > * outputQ, const HardwareEvent& event)
{

    // reuse same msg each time, fifo makes its own copy and there is
    // only one hardware thread
    static EndpointMsg msg;

    msg.type = EndpointHardwareEventType;
    msg.event = event;
    msg.endpointIndex = myEndpointIndex;
    outputQ->add(msg);
} // end Hardware::reportEvent()




// *************************************************************************
//  chgCallState
//
// *************************************************************************

void
Hardware::chgCallState (HardwareTimeout timeout)
{
    cpLog (LOG_DEBUG, "Hardware::chgCallState    ***timeout = %d  ***", timeout);

    switch (timeout)
    {
        case HWTimeoutEndCall:
        {
            myTestCallState = TestOnhook;
        }
        break;

        case HWTimeoutStartCall:
        {
            myTestCallState = TestOffhook;
        }
        break;

        case HWTimeoutDialDigits:
        {
            myTestCallState = TestSendDigits;
            //myTestCallState = getTestDigit();
        }

        break;

        case HWTimeoutAnswerCall:
        {
            myTestCallState = TestOffhook;
        }
        break;

        case HWTimeoutBadState:
        {
            myTestCallState = TestBadState;
        }
        default:
        {
            break;
        }
    };
};

// *************************************************************************
//  getTestDigit
//
// *************************************************************************


TestCallState
Hardware::getTestDigit()
{
    TestCallState tempState;

    if (myDialDigits[digitPosition] != '\0')
    {
        switch (myDialDigits[digitPosition])
        {
            case '1':
            {
                tempState = TestDigit1;
            }
            break;

            case '2':
            {
                tempState = TestDigit2;
            }
            break;

            case '3':
            {
                tempState = TestDigit3;
            }
            break;

            case '4':
            {
                tempState = TestDigit4;
            }
            break;

            case '5':
            {
                tempState = TestDigit5;
            }
            break;

            case '6':
            {
                tempState = TestDigit6;
            }
            break;

            case '7':
            {
                tempState = TestDigit7;
            }
            break;

            case '8':
            {
                tempState = TestDigit8;
            }
            break;

            case '9':
            {
                tempState = TestDigit9;
            }
            break;

            case '0':
            {
                tempState = TestDigit0;
            }
            break;

            default:
            {
                break;
            }
        }
        digitPosition++;

    }
    else
    {
        // Reached the end.  Set the position back to 0 for the next time.
        digitPosition = 0;
        tempState = TestEndDialing;
    };

    return tempState;
};



// *************************************************************************
//  startTimer
//
// *************************************************************************

void
Hardware::startTimer(HardwareTimeout timer, int msec)
{
    HardwareMsg msg;

    if (myTestTimerId[timer])
    {
        stopTimer (timer);
    }

    msg.type = HardwareTimeoutType;
    msg.timeout = timer;
    msg.endpointIndex = myEndpointIndex;

    cpLog(LOG_DEBUG, "HW startTimer:  timeout = %d  msec = %d", timer, msec);

    myTestTimerId[timer] = myQ->addDelayMs(msg, msec);
};



// *************************************************************************
//  stopTimer
//
// *************************************************************************

void
Hardware::stopTimer(HardwareTimeout timer)
{
    if (myTestTimerId[timer] != (FifoEventId) NULL)
    {
        myQ->cancel (myTestTimerId[timer]);

        cpLog(LOG_DEBUG, "HW stopTimer:  timeout = %d", timer);
        myTestTimerId[timer] = (FifoEventId) NULL;
    }
};



// *************************************************************************
//  getTestCallState
//
// *************************************************************************

TestCallState
Hardware::getTestCallState()
{
    return (myTestCallState);
};



void
Hardware::beginCall(int callDelay)
{
    stopTimer(HWTimeoutStartCall);
    startTimer(HWTimeoutStartCall, callDelay);

}
