
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ReqSig_cxx_Version =
    "$Id: ReqSig.cxx,v 1.11 2000/12/18 23:42:25 bko Exp $";
#include "ReqSig.h"
#include "Hardware.h"


ReqSig::ReqSig (const Hardware& hardware,
                MgcpPackageType package,
                int signal,
                int endpointIndex,
                SignalDurationType type, int duration)
        : requestId("")
{
    //    myHardwareQ = hardware.getQ();
    myEndpointIndex = endpointIndex;
    myPackage = package;
    myIndex = signal;
    myType = type;
    myDuration = duration;
    //    myStartSignal = hardware.getSignalOn(myPackage, myIndex);
    //    myStopSignal = hardware.getSignalOff(myPackage, myIndex);

    // make sure signal is operational but not active
    stop();

} // end ReqSig::ReqSig()


ReqSig::~ReqSig (void)
{
    // stop signal if active
    if (active)
    {
        stop();
    }

} // end ReqSig::~ReqSig()


void
ReqSig::start (const RQNT rqnt)
{
    HardwareMsg msg;

    // create and send msg to hardware to activate the signal
    msg.type = HardwareSignalType;
    msg.signal = myStartSignal;
    msg.endpointIndex = myEndpointIndex;
    //    myHardwareQ->add(msg);

    // update signal to reflect that it has been activated
    active = true;
    requestId = rqnt->getRequestIdentifier();

} // end ReqSig:start()


void
ReqSig::stop (void)
{
    HardwareMsg msg;

    // create and send msg to hardware to stop the signal
    msg.type = HardwareSignalType;
    msg.signal = myStopSignal;
    msg.endpointIndex = myEndpointIndex;
    //    myHardwareQ->add(msg);

    // update signal to reflect that it has been stopped
    active = false;

} // end ReqSig:stop()


void
ReqSig::show (void)
{
    cout << "Signal:";
    cout << "\nactive        : " << active;
    cout << "\nrequestId     : " << requestId;
    cout << '\n';
} // end ReqSig::show()
