/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpStatus_cxx_Version =
    "$Id: HttpStatus.cxx,v 1.7 2001/11/05 18:07:36 icahoon Exp $";


#include "HttpStatus.hxx"
#include "Verify.hxx"
#include <cassert>
#include <cstdlib>
#include <cstdio>


using Vocal::Http::Status;
using Vocal::Http::EXTENSION;
using Vocal::Http::DIGIT;
using Vocal::Http::SP;
using Vocal::Http::Token;


const Token         Status::Token100("100");
const Token         Status::Token101("101");
const Token         Status::Token200("200");
const Token         Status::Token201("201");
const Token         Status::Token202("202");
const Token         Status::Token203("203");
const Token         Status::Token204("204");
const Token         Status::Token205("205");
const Token         Status::Token206("206");
const Token         Status::Token300("300");
const Token         Status::Token301("301");
const Token         Status::Token302("302");
const Token         Status::Token303("303");
const Token         Status::Token304("304");
const Token         Status::Token305("305");
const Token         Status::Token307("307");
const Token         Status::Token400("400");
const Token         Status::Token401("401");
const Token         Status::Token402("402");
const Token         Status::Token403("403");
const Token         Status::Token404("404");
const Token         Status::Token405("405");
const Token         Status::Token406("406");
const Token         Status::Token407("407");
const Token         Status::Token408("408");
const Token         Status::Token409("409");
const Token         Status::Token410("410");
const Token         Status::Token411("411");
const Token         Status::Token412("412");
const Token         Status::Token413("413");
const Token         Status::Token414("414");
const Token         Status::Token415("415");
const Token         Status::Token416("416");
const Token         Status::Token417("417");
const Token         Status::Token500("500");
const Token         Status::Token501("501");
const Token         Status::Token502("502");
const Token         Status::Token503("503");
const Token         Status::Token504("504");
const Token         Status::Token505("505");


const size_t    Status::myTotal = 40;


const Status::ReasonPhrase   Status::ReasonPhrase100
    = "Continue";
const Status::ReasonPhrase   Status::ReasonPhrase101
    = "Switching Protocols";
const Status::ReasonPhrase   Status::ReasonPhrase200
    = "OK";
const Status::ReasonPhrase   Status::ReasonPhrase201
    = "Created";
const Status::ReasonPhrase   Status::ReasonPhrase202
    = "Accepted";
const Status::ReasonPhrase   Status::ReasonPhrase203
    = "Non-Authoritative Information";
const Status::ReasonPhrase   Status::ReasonPhrase204
    = "No Content";
const Status::ReasonPhrase   Status::ReasonPhrase205
    = "Reset Content";
const Status::ReasonPhrase   Status::ReasonPhrase206
    = "Partial Content";
const Status::ReasonPhrase   Status::ReasonPhrase300
    = "Multiple Choices";
const Status::ReasonPhrase   Status::ReasonPhrase301
    = "Moved Permanently";
const Status::ReasonPhrase   Status::ReasonPhrase302
    = "Found";
const Status::ReasonPhrase   Status::ReasonPhrase303
    = "See Other";
const Status::ReasonPhrase   Status::ReasonPhrase304
    = "Not Modified";
const Status::ReasonPhrase   Status::ReasonPhrase305
    = "Use Proxy";
const Status::ReasonPhrase   Status::ReasonPhrase307
    = "Temporary Redirect";
const Status::ReasonPhrase   Status::ReasonPhrase400
    = "Bad Request";
const Status::ReasonPhrase   Status::ReasonPhrase401
    = "Unauthorized";
const Status::ReasonPhrase   Status::ReasonPhrase402
    = "Payment Required";
const Status::ReasonPhrase   Status::ReasonPhrase403
    = "Forbidden";
const Status::ReasonPhrase   Status::ReasonPhrase404
    = "Not Found";
const Status::ReasonPhrase   Status::ReasonPhrase405
    = "Method Not Allowed";
const Status::ReasonPhrase   Status::ReasonPhrase406
    = "Not Acceptable";
const Status::ReasonPhrase   Status::ReasonPhrase407
    = "Proxy Authentication Required";
const Status::ReasonPhrase   Status::ReasonPhrase408
    = "Request Time-out";
const Status::ReasonPhrase   Status::ReasonPhrase409
    = "Conflict";
const Status::ReasonPhrase   Status::ReasonPhrase410
    = "Gone";
const Status::ReasonPhrase   Status::ReasonPhrase411
    = "Length Required";
const Status::ReasonPhrase   Status::ReasonPhrase412
    = "Precondition Failed";
const Status::ReasonPhrase   Status::ReasonPhrase413
    = "Request Entity Too Large";
const Status::ReasonPhrase   Status::ReasonPhrase414
    = "Request-URI Too Large";
const Status::ReasonPhrase   Status::ReasonPhrase415
    = "Unsupported Media Type";
const Status::ReasonPhrase   Status::ReasonPhrase416
    = "Requested range not satisfiable";
const Status::ReasonPhrase   Status::ReasonPhrase417
    = "Expectation Failed";
const Status::ReasonPhrase   Status::ReasonPhrase500
    = "Internal Server Error";
const Status::ReasonPhrase   Status::ReasonPhrase501
    = "Not Implemented";
const Status::ReasonPhrase   Status::ReasonPhrase502
    = "Bad Gateway";
const Status::ReasonPhrase   Status::ReasonPhrase503
    = "Service Unavailable";
const Status::ReasonPhrase   Status::ReasonPhrase504
    = "Gateway Time-out";
const Status::ReasonPhrase   Status::ReasonPhrase505
    = "HTTP Version not supported";


Status::Table           Status::myTable;
Status::DecodeTable     Status::myDecodeTable;


Status::Status(Code pCode)
    :   myExtension(true)
{
    init();
    
    code(pCode);

    test_covered("0");
};


Status::Status(Code pCode, const ReasonPhrase & pReason)
    :   myExtension(true)
{
    init();
    
    code(pCode);
    reasonPhrase(pReason);

    test_covered("1");
};


Status::Status(const Token & pToken)
    :   myCode(EXTENSION),
        myExtension(true)
{
    init();
    
    if ( pToken.size() )
    {
        code(pToken);
    }

    test_covered("2");
};


Status::Status(const Token & pToken, const ReasonPhrase & pReason)
    :   myCode(EXTENSION),
        myExtension(true)
{
    init();
    
    if ( pToken.size() )
    {
        code(pToken);
        reasonPhrase(pReason);
    }

    test_covered("3");
};


Status::Status(const Status & src)
    :   myCode(src.myCode),
        myExtension(src.myExtension),
        myToken(src.myToken),
        myReasonPhrase(src.myReasonPhrase)
{
    test_covered("4");
}

    

Status::~Status()
{
    test_covered("5");
}


const Status &
Status::operator=(const Status & src)
{
    if ( this != &src )
    {
        myCode = src.myCode;        
        myExtension = src.myExtension;
        myToken = src.myToken;
        myReasonPhrase = src.myReasonPhrase;

        test_covered("6");
    }

    test_covered("7");

    return ( *this );
}


bool
Status::operator==(const Status & src) const
{
    test_covered("8");

    return ( myCode == src.myCode );
}


bool
Status::operator<(const Status & src) const
{
    test_covered("9");

    return ( myCode < src.myCode );
}


Status::Code
Status::code() const
{
    test_covered("10");

    return ( myCode );
}


void
Status::code(Code pCode)
{
    myExtension = false;
    myToken.erase();
    myReasonPhrase.erase();

    Table::const_iterator i = myTable.find(pCode);
    if ( i == myTable.end() )
    {
        myExtension = true;
        
        if ( pCode < 1000 )
        {
            char buf[5];
            snprintf(buf, 4, "%03u", pCode);
            myToken = buf;
            
            test_covered("11");
        }
        else
        {
            pCode = EXTENSION;
            
            test_covered("12");
        }
    }
    myCode = pCode;

    test_covered("13");
}


void
Status::code(const Token & pToken)
throw ( Vocal::Http::Exception )
{   
    myCode = EXTENSION;
    myExtension = true;
    myToken.erase();
    myReasonPhrase.erase();
    
    DecodeTable::const_iterator i = myDecodeTable.find(pToken);
    if ( i != myDecodeTable.end() )
    {
        myCode = i->second;
        myExtension = false;

        test_covered("14");
        
        return;
    }

    // This is an extension
    //
    if ( pToken.size() == 3 )
    {
        myCode = strtoul(pToken.c_str(), 0, 10);
        myToken = pToken;

        test_covered("15");
        
        return;
    }

    test_covered("16");

    throw ( Vocal::Http::Exception(Exception::BadStatus) );
}


bool    
Status::isExtension() const
{
    test_covered("17");

    return ( myExtension );
}


const Token &  
Status::token() const
{
    if ( isExtension() )
    {
        test_covered("18");

        return ( myToken );
    }

    test_covered("19");

    return ( *myTable[myCode].token );
}


const Status::ReasonPhrase &  
Status::reasonPhrase() const
{
    if ( isExtension() || myReasonPhrase.size() != 0 )
    {
        test_covered("20");

        return ( myReasonPhrase );
    }

    test_covered("21");

    return ( *myTable[myCode].reasonPhrase );
}


void
Status::reasonPhrase(const ReasonPhrase & reason)
{
    test_covered("22");

    myReasonPhrase = reason;
}


bool    
Status::isInformational() const
{
    test_covered("23");
    
    return ( 100 <= myCode && myCode < 200 );
}


bool    
Status::isSuccess() const
{
    test_covered("24");
    
    return ( 200 <= myCode && myCode < 300 );
}


bool    
Status::isRedirection() const
{
    test_covered("25");
    
    return ( 300 <= myCode && myCode < 400 );
}


bool    
Status::isClientError() const
{
    test_covered("26");
    
    return ( 400 <= myCode && myCode < 500 );
}


bool    
Status::isServerError() const
{
    test_covered("27");
    
    return ( 500 <= myCode && myCode < 600 );
}


size_t
Status::totalCodes()
{
    test_covered("28");

    return ( myTotal );
}


ostream &   
Status::writeTo(ostream & out) const
{
    test_covered("29");

    return ( out << token() << SP << reasonPhrase() );
}


void
Status::init()
{
    if ( myTable.size() == 0 )
    {
        initTable();
        initDecodeTable();
        
        test_covered("30");
    }
}


void
Status::initTable()
{
    myTable[100] = TableEntry(Token100, ReasonPhrase100);
    myTable[101] = TableEntry(Token101, ReasonPhrase101);
    myTable[200] = TableEntry(Token200, ReasonPhrase200);
    myTable[201] = TableEntry(Token201, ReasonPhrase201);
    myTable[202] = TableEntry(Token202, ReasonPhrase202);
    myTable[203] = TableEntry(Token203, ReasonPhrase203);
    myTable[204] = TableEntry(Token204, ReasonPhrase204);
    myTable[205] = TableEntry(Token205, ReasonPhrase205);
    myTable[206] = TableEntry(Token206, ReasonPhrase206);
    myTable[300] = TableEntry(Token300, ReasonPhrase300);
    myTable[301] = TableEntry(Token301, ReasonPhrase301);
    myTable[302] = TableEntry(Token302, ReasonPhrase302);
    myTable[303] = TableEntry(Token303, ReasonPhrase303);
    myTable[304] = TableEntry(Token304, ReasonPhrase304);
    myTable[305] = TableEntry(Token305, ReasonPhrase305);
    myTable[307] = TableEntry(Token307, ReasonPhrase307);
    myTable[400] = TableEntry(Token400, ReasonPhrase400);
    myTable[401] = TableEntry(Token401, ReasonPhrase401);
    myTable[402] = TableEntry(Token402, ReasonPhrase402);
    myTable[403] = TableEntry(Token403, ReasonPhrase403);
    myTable[404] = TableEntry(Token404, ReasonPhrase404);
    myTable[405] = TableEntry(Token405, ReasonPhrase405);
    myTable[406] = TableEntry(Token406, ReasonPhrase406);
    myTable[407] = TableEntry(Token407, ReasonPhrase407);
    myTable[408] = TableEntry(Token408, ReasonPhrase408);
    myTable[409] = TableEntry(Token409, ReasonPhrase409);
    myTable[410] = TableEntry(Token410, ReasonPhrase410);
    myTable[411] = TableEntry(Token411, ReasonPhrase411);
    myTable[412] = TableEntry(Token412, ReasonPhrase412);
    myTable[413] = TableEntry(Token413, ReasonPhrase413);
    myTable[414] = TableEntry(Token414, ReasonPhrase414);
    myTable[415] = TableEntry(Token415, ReasonPhrase415);
    myTable[416] = TableEntry(Token416, ReasonPhrase416);
    myTable[417] = TableEntry(Token417, ReasonPhrase417);
    myTable[500] = TableEntry(Token500, ReasonPhrase500);
    myTable[501] = TableEntry(Token501, ReasonPhrase501);
    myTable[502] = TableEntry(Token502, ReasonPhrase502);
    myTable[503] = TableEntry(Token503, ReasonPhrase503);
    myTable[504] = TableEntry(Token504, ReasonPhrase504);
    myTable[505] = TableEntry(Token505, ReasonPhrase505);

    assert( totalCodes() == myTable.size() );

    test_covered("31");
}


void
Status::initDecodeTable()
{
    myDecodeTable[Token100] = 100;
    myDecodeTable[Token101] = 101;
    myDecodeTable[Token200] = 200;
    myDecodeTable[Token201] = 201;
    myDecodeTable[Token202] = 202;
    myDecodeTable[Token203] = 203;
    myDecodeTable[Token204] = 204;
    myDecodeTable[Token205] = 205;
    myDecodeTable[Token206] = 206;
    myDecodeTable[Token300] = 300;
    myDecodeTable[Token301] = 301;
    myDecodeTable[Token302] = 302;
    myDecodeTable[Token303] = 303;
    myDecodeTable[Token304] = 304;
    myDecodeTable[Token305] = 305;
    myDecodeTable[Token307] = 307;
    myDecodeTable[Token400] = 400;
    myDecodeTable[Token401] = 401;
    myDecodeTable[Token402] = 402;
    myDecodeTable[Token403] = 403;
    myDecodeTable[Token404] = 404;
    myDecodeTable[Token405] = 405;
    myDecodeTable[Token406] = 406;
    myDecodeTable[Token407] = 407;
    myDecodeTable[Token408] = 408;
    myDecodeTable[Token409] = 409;
    myDecodeTable[Token410] = 410;
    myDecodeTable[Token411] = 411;
    myDecodeTable[Token412] = 412;
    myDecodeTable[Token413] = 413;
    myDecodeTable[Token414] = 414;
    myDecodeTable[Token415] = 415;
    myDecodeTable[Token416] = 416;
    myDecodeTable[Token417] = 417;
    myDecodeTable[Token500] = 500;
    myDecodeTable[Token501] = 501;
    myDecodeTable[Token502] = 502;
    myDecodeTable[Token503] = 503;
    myDecodeTable[Token504] = 504;
    myDecodeTable[Token505] = 505;

    assert( totalCodes() == myDecodeTable.size() );

    test_covered("32");
}
