#if !defined(VOCAL_HTTP_STATUS_HXX)
#define VOCAL_HTTP_STATUS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpStatus_hxx_Version =
    "$Id: HttpStatus.hxx,v 1.7 2001/11/05 18:07:36 icahoon Exp $";


#include "Http.hxx"
#include "Writer.hxx"
#include <map>


/** Infrastructure common to VOCAL.<br><br>
 */
namespace Vocal 
{


/** Http Protocol Libraries<br><br>
 */
namespace Http
{


/** Implementation of RFC 2616, Section 6-1.<br><br>
 */
class Status : public Vocal::IO::Writer
{
    public:

        ///
        typedef Type        Code;
        ///
        typedef Token       ReasonPhrase;


        // Token values
        //
        static const Token  Token100;
        static const Token  Token101;
        static const Token  Token200;
        static const Token  Token201;
        static const Token  Token202;
        static const Token  Token203;
        static const Token  Token204;
        static const Token  Token205;
        static const Token  Token206;
        static const Token  Token300;
        static const Token  Token301;
        static const Token  Token302;
        static const Token  Token303;
        static const Token  Token304;
        static const Token  Token305;
        static const Token  Token307;
        static const Token  Token400;
        static const Token  Token401;
        static const Token  Token402;
        static const Token  Token403;
        static const Token  Token404;
        static const Token  Token405;
        static const Token  Token406;
        static const Token  Token407;
        static const Token  Token408;
        static const Token  Token409;
        static const Token  Token410;
        static const Token  Token411;
        static const Token  Token412;
        static const Token  Token413;
        static const Token  Token414;
        static const Token  Token415;
        static const Token  Token416;
        static const Token  Token417;
        static const Token  Token500;
        static const Token  Token501;
        static const Token  Token502;
        static const Token  Token503;
        static const Token  Token504;
        static const Token  Token505;


        // Reason phrases
        //
        static const ReasonPhrase   ReasonPhrase100;
        static const ReasonPhrase   ReasonPhrase101;
        static const ReasonPhrase   ReasonPhrase200;
        static const ReasonPhrase   ReasonPhrase201;
        static const ReasonPhrase   ReasonPhrase202;
        static const ReasonPhrase   ReasonPhrase203;
        static const ReasonPhrase   ReasonPhrase204;
        static const ReasonPhrase   ReasonPhrase205;
        static const ReasonPhrase   ReasonPhrase206;
        static const ReasonPhrase   ReasonPhrase300;
        static const ReasonPhrase   ReasonPhrase301;
        static const ReasonPhrase   ReasonPhrase302;
        static const ReasonPhrase   ReasonPhrase303;
        static const ReasonPhrase   ReasonPhrase304;
        static const ReasonPhrase   ReasonPhrase305;
        static const ReasonPhrase   ReasonPhrase307;
        static const ReasonPhrase   ReasonPhrase400;
        static const ReasonPhrase   ReasonPhrase401;
        static const ReasonPhrase   ReasonPhrase402;
        static const ReasonPhrase   ReasonPhrase403;
        static const ReasonPhrase   ReasonPhrase404;
        static const ReasonPhrase   ReasonPhrase405;
        static const ReasonPhrase   ReasonPhrase406;
        static const ReasonPhrase   ReasonPhrase407;
        static const ReasonPhrase   ReasonPhrase408;
        static const ReasonPhrase   ReasonPhrase409;
        static const ReasonPhrase   ReasonPhrase410;
        static const ReasonPhrase   ReasonPhrase411;
        static const ReasonPhrase   ReasonPhrase412;
        static const ReasonPhrase   ReasonPhrase413;
        static const ReasonPhrase   ReasonPhrase414;
        static const ReasonPhrase   ReasonPhrase415;
        static const ReasonPhrase   ReasonPhrase416;
        static const ReasonPhrase   ReasonPhrase417;
        static const ReasonPhrase   ReasonPhrase500;
        static const ReasonPhrase   ReasonPhrase501;
        static const ReasonPhrase   ReasonPhrase502;
        static const ReasonPhrase   ReasonPhrase503;
        static const ReasonPhrase   ReasonPhrase504;
        static const ReasonPhrase   ReasonPhrase505;


        /** Create the status given the code
         */
        Status(Code = Http::EXTENSION);
        
        
        /** Create the status given the code and the reason phrase
         */
        Status(Code, const ReasonPhrase &);
        
        
        /** Create the status given the token
         */
        Status(const Token &);
        

        /** Create the status given the token and the reason phrase
         */
        Status(const Token &, const ReasonPhrase &);
        

        /** Copy constructor
         */
        Status(const Status &);


        /** Virtual destructor
         */
        virtual ~Status();


        /** Assignment operator
         */
        const Status &  operator=(const Status &);
        

        /** Equality operator
         */
        bool            operator==(const Status &) const;


        /** Less than relational operator
         */
        bool            operator<(const Status &) const;


        /** Code accessor
         */
        Code       code() const;    
        
        
        /** Code mutator. The Code can be an extension. If so you
         *  may also want to set the reasonPhrase manually.
         */
        void       code(Code);    
        

        /** Set the code given the token text
         */
        void        code(const Token &) throw ( Vocal::Http::Exception );
        

        /** Returns true if the status code is an extended code.
         */
        bool    isExtension() const;
        

        /** Provide the token of this status code
         */
        const Token &  token() const;
        
        
        /** Provide the reason phrase
         */
        const ReasonPhrase &    reasonPhrase() const;


        /** Set the reason phrase.
         */
        void    reasonPhrase(const ReasonPhrase &);


        /** Returns true if the status code is in the 100s.
         */
        bool    isInformational() const;
        
        
        /** Returns true if the status code is in the 200s.
         */
        bool    isSuccess() const;
        

        /** Returns true if the status code is in the 300s.
         */
        bool    isRedirection() const;
        

        /** Returns true if the status code is in the 400s.
         */
        bool    isClientError() const;
        

        /** Returns true if the status code is in the 500s.
         */
        bool    isServerError() const;
        

        /** The total number of known status codes
         */
        static size_t   totalCodes();


        /** Write this status to a stream
         */
        ostream &       writeTo(ostream &) const;
        

    protected:


        void    init();
        
            
        /** Create the status code to token text / reason table
         */
        void    initTable();
            
        class TableEntry
        {
            public:
                
                TableEntry() {}
                
                TableEntry(const Token & tkn, const ReasonPhrase & reason)
                    : token(&tkn), reasonPhrase(&reason) {}

                const Token         *   token;
                const ReasonPhrase  *   reasonPhrase;
        };

        typedef map<Code, TableEntry>   Table;

        static  Table                   myTable;



        /** Create the token text to status code table
         */
        void    initDecodeTable();

        typedef map<Token, Code>    DecodeTable;
        
        static  DecodeTable         myDecodeTable;


        static const size_t myTotal;

        Code                myCode;
        bool                myExtension;
        Token               myToken;
        ReasonPhrase        myReasonPhrase;
};


} // namespace Http
} // namespace Vocal


#endif // !defined(VOCAL_HTTP_STATUS_HXX)
