/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpMessageTest_cxx_Version = 
    "$Id: HttpMessageTest.cxx,v 1.7 2002/07/11 06:01:57 icahoon Exp $";


#include "Application.hxx"
#include "HttpMessage.hxx"
#include "HttpVersion.hxx"
#include "HttpException.hxx"
#include "IPAddress.hxx"
#include "TCPClientSocket.hxx"
#include "Verify.hxx"


using namespace Vocal;
using namespace Vocal::Transport;


class TestApplication : public Application
{
    public:
        TestApplication() {}
        virtual ~TestApplication() {}
        
    	ReturnCode  	run();
};


class TestApplicationCreator : public ApplicationCreator
{
    Application * create() { return ( new TestApplication ); }
};


int main(int argc, char ** argv, char ** arge)
{
    TestApplicationCreator creator;
    Application::creator(&creator);
    return ( Application::main(argc, argv, arge) );
}


ReturnCode TestApplication::run()
{
    // GET /foo.txt HTTP/1.0
    // Connection: Keep-Alive
    // User-Agent: Mozilla/4.72 [en] (X11; U; Linux 2.2.16-23.uid32 i686)
    // Host: localhost:8080
    // Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*
    // Accept-Encoding: gzip
    // Accept-Language: en
    // Accept-Charset: iso-8859-1,*,utf-8
    //
    Http::Method    method(Http::Method::GET);
    Http::Url       url("/index.html");

    list<Http::Header>  headers;

    Http::Header   connection(Http::Header::Connection, "Keep-Alive");
    Http::Header   userAgent(Http::Header::UserAgent, "Mozilla/4.72 [en] (X11; U; Linux 2.2.16-23.uid32 i686)");
    Http::Header   host(Http::Header::Host, "localhost");
    Http::Header   accept(Http::Header::Accept, "image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, image/png, */*");
    Http::Header   acceptEncoding(Http::Header::AcceptEncoding, "gzip");
    Http::Header   acceptLanguage(Http::Header::AcceptLanguage, "en");
    Http::Header   acceptCharset(Http::Header::AcceptCharset, "iso-8859-1,*,utf-8");

    headers.push_back(connection);
    headers.push_back(userAgent);
    headers.push_back(host);
    headers.push_back(accept);
    headers.push_back(acceptEncoding);
    headers.push_back(acceptLanguage);
    headers.push_back(acceptCharset);
    
    string          body;
    
    Http::Message   request(method, url, headers, body);

    cout << request;

    Http::Message   decodedRequest;
    
    try
    {
        decodedRequest.decode(request.encode());
        
        // Force it to rebuild.
        //
        //Http::Version     newVersion(1,1);
        //decodedRequest.version(newVersion);
        test_verify(true);
        
        cout << decodedRequest;
    }
    catch ( Http::Exception & e )
    {
        cout << e.message() << endl;
        test_verify(false);
    }

    
    Http::Status    status(200);
    headers.clear();
    
    assert( headers.size() == 0 );

    Http::Message   response(status, headers, body);
    Http::Message   decodedResponse;
    
    try
    {
        decodedResponse.decode(response.encode());
        test_verify(true);
    }
    catch ( Http::Exception & e )
    {
        cout << e.message() << endl;
        test_verify(false);
    }
    
    cout << "Request = \n{\n"<< request << "}" << endl;
    cout << "Decoded Request = \n{\n"<< decodedRequest << "}" << endl;
    
    cout << "Response = {\n" << response << "}" << endl;
    cout << "Decoded Response = \n{\n" << decodedResponse << "}" << endl;

    IPAddress           vorgIP("127.0.0.1", 80);
    TCPClientSocket     socket;

    socket.connect(vorgIP);
    
    string  encodedRequest = request.encode();
    
    int bytesSent = socket.send(encodedRequest);
    
    assert( (size_t)bytesSent == encodedRequest.size() );
    
    bool    done = false;

    string  buffer("", 8192);
    string  encodedResponse("");

    Http::Message   resp;
    
    while ( !done )
    {
        socket.receive(buffer);
        encodedResponse += buffer;

        try
        {
            resp.decode(encodedResponse);
            
            cout << "{\n" << encodedResponse << "}\n";
            
            done = true;
        }
        catch ( Http::Exception & e )
        {
        }
    }    


    // Force it to rebuild.
    //
    Http::Version     newVersion(1,1);
    resp.version(newVersion);
    
    cout << "\n\n----\n{\n" << resp << "}\n" << endl;


    cout << "This is my message body: \n{\n" << resp.body() << "}\n" << endl;
    
    return ( test_return_code(2) );
}
