/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HttpTest_cxx_Version = 
    "$Id: HttpTest.cxx,v 1.3 2002/07/11 06:01:57 icahoon Exp $";


#include "Application.hxx"
#include "Http.hxx"
#include "Verify.hxx"


using namespace Vocal;


class TestApplication : public Application
{
    public:
        TestApplication() {}
        virtual ~TestApplication() {}
        
    	ReturnCode  	run();
};


class TestApplicationCreator : public ApplicationCreator
{
    Application * create() { return ( new TestApplication ); }
};


int main(int argc, char ** argv, char ** arge)
{
    TestApplicationCreator creator;
    Application::creator(&creator);
    ReturnCode  count = Application::main(argc, argv, arge);
    return ( test_return_code(count) );
}


ReturnCode TestApplication::run()
{
    int test_count = 0;
    
    size_t  byteCount = 0;
    string  current;

    // Make sure CRLF is stripped off.
    //    
    string  test = "This is a test";

    current = test + Http::CRLF;
    
    string  token = Http::getToken(current, byteCount, Http::CRLF, true);
    
    test_verify( token == test );               ++test_count;
    test_verify( current.size() == 0 );         ++test_count;
    test_verify( byteCount == test.size() + Http::CRLF.size() ); ++test_count;


    // Make sure only the first CRLF is stripped off.
    //
    byteCount = 0;
    current = test + Http::CRLF + Http::CRLF;

    token = Http::getToken(current, byteCount, Http::CRLF, true);
    
    test_verify( token == test );               ++test_count;
    test_verify( current == Http::CRLF );       ++test_count;
    test_verify( byteCount == test.size() + Http::CRLF.size() ); ++test_count;


    // Make sure that the search string in the middle of a line is found.
    // Check remove leading for something other than white space.
    //
    byteCount = 0;
    string  name("Name"), value("Value");
    test = name + Http::COLON + Http::SP + value;
    
    current = test + Http::CRLF;
    
    token = Http::getToken(current, byteCount, Http::COLON);
    
    test_verify( token == name );               ++test_count;
    test_verify( byteCount == name.size() );    ++test_count;

    byteCount += Http::removeLeading(current, Http::COLON);
    
    test_verify( byteCount == name.size() + 1 ); ++test_count;

    token = Http::getToken(current, byteCount, Http::CRLF, true);
    
    test_verify( token == value );              ++test_count;
    test_verify( current.size() == 0 );         ++test_count;
    test_verify( byteCount == test.size() + Http::CRLF.size() ); ++test_count;


    // Check for reading the entire string and expecting the end.
    //
    byteCount = 0;
    current = test;
    
    token = Http::getToken(current, byteCount, Http::CRLF, true);
    
    test_verify( token == test );               ++test_count;
    test_verify( byteCount == test.size() );    ++test_count;
    
    
    // Check for reading the entire string and not expecting the end.
    //
    byteCount = 0;
    current = test;
    
    try
    {
        token = Http::getToken(current, byteCount, Http::CRLF);
        
        test_verify(false);     ++test_count;
    }
    catch( Http::Exception & e )
    {
        test_verify( e.error() == Http::Exception::Incomplete ); ++test_count;
    }
    

    // Check for hitting the end of a line when we weren't expecting it.
    //
    byteCount = 0;
    current = test + Http::CRLF;
    
    try
    {
        token = Http::getToken(current, byteCount, string("#"));
        
        test_verify(false);     ++test_count;
    }
    catch ( Http::Exception & e )
    {
        test_verify( e.error() == Http::Exception::Incomplete ); ++test_count;
    }
    
    // Check removeCRLF.
    //
    current = "\n";
    test_verify( Http::removeCRLF(current) == 1 );  ++test_count;
    test_verify( current.size() == 0 );             ++test_count;

    current = "\r\n";
    test_verify( Http::removeCRLF(current) == 2 );  ++test_count;
    test_verify( current.size() == 0 );             ++test_count;


    // Check removeLeading
    //
    current = Http::WS + Http::CRLF + Http::WS + Http::CRLF;

    test_verify( Http::removeLeading(current, Http::WS) 
                    == Http::WS.size() );           ++test_count;


    // getToken with only whitespace
    //
    byteCount = 0;
    current = Http::WS + Http::CRLF + Http::WS + Http::CRLF;

    token = Http::getToken(current, byteCount, Http::CRLF, true);
    
    test_verify( token.size() == 0 );               ++test_count;
    test_verify( byteCount = Http::WS.size() );     ++test_count;
    
    return ( test_count );
}
