
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "LGOperator.hxx"
#include "OpRing.hxx"
#include "SipProxyEvent.hxx"
#include "LGListening.hxx"
#include "InviteMsg.hxx"
#include "SipEvent.hxx"
#include "CallInfo.hxx"

LGListening::LGListening()
    : opRing(0)
{
    cpLog(LOG_CRIT,"Using more debugs for err reporting...");
}

LGListening::~LGListening()
{
    delete opRing;
}

bool
LGListening::addOp(LGOperator* nextOp)
{
    opRing = dynamic_cast<OpRing*>(nextOp);

    assert(opRing);
    return true;
}

LGState*
LGListening::process(const Sptr<SipProxyEvent>& event)
{
    assert(opRing);

    LGState * nextstate = 0;

    Sptr<SipEvent> sipEvent;
    if(sipEvent.dynamicCast(event)!=0)
    {
	// process sip event...
	Sptr<InviteMsg> invite;
	if(invite.dynamicCast(sipEvent->getSipMsg()) != 0)
	{
	    // send 183 for invites
	    sipEvent->getSipStack()->sendReply(new StatusMsg(*invite, 183));

	    // start the ring
	    nextstate = opRing->process(event);
	}
	//	else
	    //	    cpLog(LOG_CRIT,"ERR: not expecting %s",
	    //		  sipEvent->getSipMsg()->encode().logData());

    }
    else
	{
	    cpLog(LOG_CRIT,"ERROR: Received NON_SipEvent in Listening state!!!");
	    cpLog(LOG_CRIT,"message is %s", event->getCallInfo()->getSipCommand()->getRequestLine().encode().logData());
	}
       
    return nextstate;
}
