
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "SipProxyEvent.hxx"
#include "TimerEvent.hxx"
#include "LGOperator.hxx"
#include "LGState.hxx"
#include "CallInfo.hxx"
#include "LGParam.hxx"
#include "OpTerminate.hxx"
#include "LGInfo.hxx"
#include "ByeMsg.hxx"
#include "SipEvent.hxx"
#include "TimerEvent.hxx"
#include "SipTransceiver.hxx"
#include "LGStateMcData.hxx"

LGState *
OpTerminate::process(const Sptr<SipProxyEvent>& event)
{

    Sptr<CallInfo> callInfo = event->getCallInfo();
    Sptr<ByeMsg> bye;
    if(LGParam::Function==LGParam::ORIGINATING)
    {
	Sptr<LGStateMcData> lgData;
	if(lgData.dynamicCast(callInfo->getCallStateData())==0)
	{
	    assert(0);
	}
	bye = new ByeMsg(*lgData->response);
    }
    else
	bye = new ByeMsg(*callInfo->getSipCommand());

    SipVia via;
    via.setHost(LGParam::FromHost);
    via.setPort(LGParam::UDPPort);
    via.setRPort(LGParam::RPort);
    if(LGParam::Transport == LGParam::TCP)
    {
	bye->getMutableRequestLine().setTransportParam("TCP");
	via.setTransport("TCP");
    }
    bye->flushViaList();
    bye->setVia(via);
    callInfo->setSipCommand(bye);
    if(LGParam::RequestTimeOut)
	    callInfo->setTimer(this->setTimer(event, LGParam::RequestTimeOut));
    Sptr<SipTransceiver> sipStack;

    Sptr<TimerEvent> timerEvent;
    if(timerEvent.dynamicCast(event)!=0)
	sipStack = timerEvent->getSipStack();
    else
    {
	Sptr<SipEvent> sipEvent;
	if(sipEvent.dynamicCast(event)!=0)
	    sipStack = sipEvent->getSipStack();
	else
	    assert(0);
    }
    sipStack->sendAsync(bye);
//    sipStack->sendAsync(request, LGParam::NextHopHost, LGParam::NextHopPort);

    return nextState;
}

