
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include "global.h"
static const char* const MgcpCommand_cxx_Version  =
    "$Id: MgcpCommand.cxx,v 1.52 2002/11/22 02:26:11 bko Exp $";



/*
 
To do:
 
the copy constructors for the events do not correctly copy the
underlying items, which they should do (?)
 
*/

#include "MgcpCommand.hxx"
#include <iostream>
#include <strstream>
#include "support.hxx"
#include "Sptr.hxx"
#include "TokenRecognizer.hxx"
#include <deque>
#include "MgcpParse.hxx"

#include "MgcpDebugManager.hxx"

#define MGCP_MESSAGE_BUF_MAX 512

ostream& operator<<(ostream& s, const UniqueId& x)
{
    s << x.m_host << ":" << x.m_port << ":" << x.m_transactionID;
    return s;
}


ostream& operator<<(ostream& s, MgcpPacket& x)
{
    x.print(s);
    return s;
}

ostream& operator<<(ostream& s, MgcpCommand& x)
{
    x.printData(s);
    return s;
}

/**********************************************************************
 
		    Helper functions and templates
 
**********************************************************************/

template < class T, class U > inline bool assign_if_same_type(T* to,
        U* from)
{
    T* ptr;
    if ((ptr = dynamic_cast < T* > (from)) != 0)
    {
        *to = *ptr;
        return true;
    }
    else
    {
        return false;
    }
}

/**********************************************************************
 
			   UniqueId
 
*********************************************************************/
bool UniqueId::operator < ( const UniqueId& uID ) const
{
    if (this->m_host < uID.m_host)
        return true;
    else if (this->m_host > uID.m_host)
        return false;

    if ( this->m_port < uID.m_port )
        return true;
    else if (this->m_port > uID.m_port )
        return false;

    if ( this->m_transactionID < uID.m_transactionID)
        return true;
    else if (this->m_transactionID > uID.m_transactionID)
        return false;
    return false;
}


bool UniqueId::operator == ( const UniqueId& uID ) const
{
    if (this->m_host != uID.m_host)
    {
        return false;
    }
    else if (this->m_port != uID.m_port)
    {
        return false;
    }
    else if (this->m_transactionID != uID.m_transactionID)
    {
        return false;
    }

    return true;
}


/**********************************************************************
 
			   MgcpPacket
 
*********************************************************************/

MgcpPacket::MgcpPacket(int payloadSize)
{
    data = 0;
    size = 0;
    allocatedSize = 0;
    size = payloadSize;
    data = new char[size];
    assert( data );
    // initialize  buffer
    for ( int i = 0; i < size; i++)
    {
        data[i] = 0;
    }
    allocatedSize = size;
}


MgcpPacket::~MgcpPacket()
{
    delete [] data; data = 0;
    size = 0;
    allocatedSize = 0;
}


char* MgcpPacket::getMessageLocation()
{
    return data;
}


int MgcpPacket::getMessageSize()
{
    assert( size > 0 );
    return size;
}


void
MgcpPacket::setMessageSize(int s)
{
    assert( s > 0 );
    assert( s <= allocatedSize );

    size = s;
}


ostream& MgcpPacket::print(ostream& s) const
{
    s << data;
    return s;
}


/**********************************************************************
 
			   Mgcp receiver
 
 **********************************************************************/

MgcpReceiver::MgcpReceiver(int aPort, int jitterBufferMs)
{
    /// udp stack is a sendrecv stack
    myStack = new UdpStack (0, aPort) ;
    freeStack = true;
}


MgcpReceiver::~MgcpReceiver()
{
    if (freeStack)
    {
        delete myStack;
        myStack = 0;
    }
}

int
MgcpReceiver::receive( char* buffer, int bufferSize)
{
    return 0;
}


MgcpPacket*
MgcpReceiver::receive()
{
    return getPacket();
}

void MgcpReceiver::addToFdSet( fd_set* set )
{
    myStack->addToFdSet( set );
}

bool
MgcpReceiver::checkIfSet ( fd_set* set )
{
    return ( myStack->checkIfSet( set ) );
}

MgcpPacket*
MgcpReceiver::getPacket()
{
    NetworkAddress sender;

    MgcpPacket* packet = new MgcpPacket( MGCP_MESSAGE_BUF_MAX /* buffer size */ );  // fix hard code here

    int len = myStack->receiveFrom(packet->getMessageLocation(),
                                   packet->getMessageSize(),
                                   &sender);

    packet->setAddress( sender );

    if ( len <= 0 )
    {
        assert(0);

        delete packet;
        packet = 0;
        return 0;
    }

    packet->setMessageSize(len);

    MgcpDumpMessage("IN ", '*', packet);

    return packet;
}




/**********************************************************************
 
			   Mgcp transmitter
 
 **********************************************************************/
MgcpTransmitter::MgcpTransmitter( Sptr < MgcpReceiver > receiver )
        : remoteAddr("", DEFAULT_MGCP_PORT_FOR_GATEWAY)
{
    const char* remoteHost = "";
    int aPort = DEFAULT_MGCP_PORT_FOR_GATEWAY;   // fix this hard coded const

    assert( aPort >= UDP_APPL_MIN_PORT_NUM );  /* ( aPort >= 1024 ) */
    assert( aPort < UDP_APPL_MAX_PORT_NUM );   /* ( aPort < 65536 ) */

    NetworkAddress* netAddressPtr = 0;

    if ( remoteHost )
    {
        string rHostName = remoteHost;
        NetworkAddress netAddress( aPort );
        netAddressPtr = &netAddress;
        netAddressPtr->setHostName(rHostName);
    }

    if ( receiver != 0 )
    {
        myStack = receiver->getUdpStack();
        freeStack = false;
    }
    else
    {
        myStack = new UdpStack ( netAddressPtr, aPort );
        freeStack = true;
    }

}


MgcpTransmitter::MgcpTransmitter( const char* remoteHost, int aPort, Sptr < MgcpReceiver > receiver )
        : remoteAddr(remoteHost, aPort)
{
  assert( aPort >= UDP_APPL_MIN_PORT_NUM );  /* ( aPort >= 1024 ) */
  assert( aPort < UDP_APPL_MAX_PORT_NUM );   /* ( aPort < 65536 ) */

    NetworkAddress* netAddressPtr = 0;

    if ( remoteHost )
    {
        string rHostName = remoteHost;
        NetworkAddress netAddress( aPort );
        netAddressPtr = &netAddress;
        netAddressPtr->setHostName(rHostName);
    }

    if (receiver != 0)
    {
        myStack = receiver->getUdpStack();
        freeStack = false;
    }
    else
    {
        myStack = new UdpStack ( netAddressPtr, aPort );
        freeStack = true;
    }

}


string& 
MgcpTransmitter::protocolVersion()
{
    static string s(" mgcp 0.1");
    return s;
}


void
MgcpTransmitter::transmit( char* data, int length )
{
    MgcpPacket packet(length);

    memcpy(packet.getMessageLocation() , data, length);

    transmit(&packet);
}


void
MgcpTransmitter::transmit( MgcpPacket* packet )
{
    MgcpDumpMessage("OUT", '-', packet);

    myStack->transmitTo( packet->getMessageLocation(),
                         packet->getMessageSize(),
                         &remoteAddr );
}


MgcpPacket* MgcpTransmitter::allocatePacket( int payloadLength )
{
    MgcpPacket* packet = new MgcpPacket( payloadLength );
    assert( packet );

    return packet;
}


/**********************************************************************
 
			    Mgcp Singleton
 
 **********************************************************************/

int
MgcpSingleton::getTransactionId()
{
    return (instance()->myTransactionId)++;
}

MgcpSingleton* MgcpSingleton::instance__ = 0;

string& 
MgcpSingleton::protocolVersion()
{
#ifndef NCS_PROFILE
    static string s(" MGCP 0.1");
#else // NCS_PROFILE
    static string s(" MGCP 0.1 NCS 1.0");
#endif 
    return s;
}


/**********************************************************************
 
			     MgcpCommand
 
**********************************************************************/


MgcpCommand::~MgcpCommand()
{}



void MgcpCommand::insert(Sptr < MgcpParameter > parm)
{
}



void MgcpCommand::insert(MgcpParameter* parm)
{
}



void MgcpCommand::printData(ostream& output)
{
    output << "Transaction ID: " << transactionId << "\n";
    output << "EndpointId: " << endId << "\n";
}


/**********************************************************************
 
			     MgcpResponse
 
**********************************************************************/


bool MgcpResponse::readyToSend()
{
    char buf[256];

    if (message.length() == 0)
    {
        if ( myResponseCode == 0 )
            message += "000";
        else
        {
            sprintf(buf, "%d", myResponseCode);
            message += buf;
        }
        message += ' ';
        sprintf(buf, "%d", getTransactionId());
        message += buf;
        message += ' ';
        message += myComment;
        message += "\r\n";

        // automatically capture all of the relevant messages which have
        // been inserted.

	// encode all inserted parameters here except endpointId, local and remote sdp info
        map < MgcpParameterType, Sptr < MgcpParameter > > ::iterator i;
        for (i = parameters.begin(); i != parameters.end(); ++i)
        {
            if ((i->first != MgcpParameterTypeEndpointId)
                    && (i->first != MgcpParameterTypeLocalConnectionDescriptor)
                    && (i->first != MgcpParameterTypeRemoteConnectionDescriptor)
               )
            {
                if (i->second != 0)
                {
                    message += ((i->second)->encode());
                }
            }
        }

	// encode local sdp info here
        if ((i = parameters.find(MgcpParameterTypeLocalConnectionDescriptor))
                != parameters.end())
        {
            if (i->second != 0)
            {
		message += "\r\n";
                message += ((i->second)->encode());
            }
        }

	// encode remote sdp info here
        if ((i = parameters.find(MgcpParameterTypeRemoteConnectionDescriptor))
                != parameters.end())
        {
            if (i->second != 0)
            {
		message += "\r\n";
                message += ((i->second)->encode());
            }
        }
        message += '\0';

    }
    return true;
}


string MgcpResponse::encodeToSend()
{
    readyToSend();
    return message;
}


bool MgcpResponse::send(MgcpTransmitter& target)
{
    readyToSend();
    const char* p1 = message.c_str();
    int tmp = strlen(p1);
    //   const_cast<MgcpTransmitter&>(target).
    //	transmit(const_cast<char*>(message.c_str()), message.length() + 1);

    const_cast < MgcpTransmitter& > (target).
    transmit(const_cast < char* > (message.c_str()), tmp );

    return true;

}


void MgcpResponse::printData(ostream& output)
{
    output << "Type: MgcpResponse\n";
    output << "Return Code: " << (int)myResponseCode << '\n';
    output << "Transaction ID: " << getTransactionId() << '\n';
    output << "Comment: " << myComment << "\n";

    MgcpCommand::printData(output);
    for (map < MgcpParameterType, Sptr < MgcpParameter > > ::iterator
            i = parameters.begin(); i != parameters.end(); ++i)
    {
        if (i->second != 0)
        {
            output << typeid(*(i->second.getPtr())).name() << ": " << *(i->second.getPtr()) << "\n";
        }
    }

    output << "\n";
}


/**********************************************************************
 
			   DeleteConnection
 
**********************************************************************/


bool MgcpRequest::readyToSend()
{
    char buf[256];

    if (message.length() == 0)
    {
        transactionId = MgcpSingleton::getTransactionId();

        message = getVerb();

        message += ' ';
        sprintf(buf, "%d", transactionId);
        message += buf;
        message += ' ';

        message += (parameters[MgcpParameterTypeEndpointId]->encode());
        message += MgcpSingleton::protocolVersion();
        message += "\r\n";

        // encode the parameters

	map < MgcpParameterType, Sptr < MgcpParameter > > ::iterator i;

        for ( i = parameters.begin(); i != parameters.end(); ++i)
        {
            if ((i->first != MgcpParameterTypeEndpointId)
                    && (i->first != MgcpParameterTypeLocalConnectionDescriptor)
                    && (i->first != MgcpParameterTypeRemoteConnectionDescriptor))
            {
                if (i->second != 0)
                {
                    message += ((i->second)->encode());
                }
            }
        }

        message += "\r\n";

        if ((i = parameters.find(MgcpParameterTypeLocalConnectionDescriptor))
                != parameters.end())
        {
            if (i->second != 0)
            {
                message += ((i->second)->encode());
            }
        }

        if ((i = parameters.find(MgcpParameterTypeRemoteConnectionDescriptor))
                != parameters.end())
        {
            if (i->second != 0)
            {
                message += ((i->second)->encode());
            }
        }

        message += '\0';
    }

    return true;
}


string MgcpRequest::encodeToSend()
{
    readyToSend();
    return message;
}


bool MgcpRequest::send( MgcpTransmitter& target)
{
    readyToSend();
    const_cast < MgcpTransmitter& > (target).
    transmit(const_cast < char* > (message.c_str()), message.length() + 1);
    return true;
}


void MgcpRequest::printData(ostream& output)
{
    output << "Type: " << typeid(*this).name() << "\n";
    MgcpCommand::printData(output);
    for (map < MgcpParameterType, Sptr < MgcpParameter > > ::iterator
            i = parameters.begin(); i != parameters.end(); ++i)
    {
        if (i->second != 0)
        {
            output << typeid(*(i->second.getPtr())).name() << ": " << *(i->second.getPtr()) << "\n";
        }
    }

    output << "\n";
}


/**********************************************************************

               MgcpCommandEndpointConfiguration

**********************************************************************/

const string& 
MgcpCommandEndpointConfiguration::myVerb()
{
    //	static const string verb("epcf"); return verb;
    static const string verb("EPCF");
    return verb;
}


/**********************************************************************

               MgcpCommandNotificationRequest

**********************************************************************/

const string& 
MgcpCommandNotificationRequest::myVerb()
{
    //	static const string verb("rqnt"); return verb;
    static const string verb("RQNT");
    return verb;
}


/**********************************************************************

               MgcpCommandNotify

**********************************************************************/

const string& 
MgcpCommandNotify::myVerb()
{
    //	static const string verb("ntfy"); return verb;
    static const string verb("NTFY");
    return verb;
}


/**********************************************************************

               MgcpCommandCreateConnection

**********************************************************************/

const string& 
MgcpCommandCreateConnection::myVerb()
{
    //	static const string verb("crcx"); return verb;
    static const string verb("CRCX");
    return verb;
}


/**********************************************************************

               MgcpCommandModifyConnection

**********************************************************************/

const string& 
MgcpCommandModifyConnection::myVerb()
{
    //	static const string verb("mdcx"); return verb;
    static const string verb("MDCX");
    return verb;
}


/**********************************************************************

               MgcpCommandDeleteConnection

**********************************************************************/

const string& 
MgcpCommandDeleteConnection::myVerb()
{
    //	static const string verb("dlcx"); return verb;
    static const string verb("DLCX");
    return verb;
}


/**********************************************************************

               MgcpCommandAuditEndpoint

**********************************************************************/

const string& 
MgcpCommandAuditEndpoint::myVerb()
{
    //	static const string verb("auep"); return verb;
    static const string verb("AUEP");
    return verb;
}


/**********************************************************************

               MgcpCommandAuditConnection

**********************************************************************/

const string& 
MgcpCommandAuditConnection::myVerb()
{
    //	static const string verb("aucx"); return verb;
    static const string verb("AUCX");
    return verb;
}


/**********************************************************************

               MgcpCommandRestartInProgress

**********************************************************************/

const string& 
MgcpCommandRestartInProgress::myVerb()
{
    // static const string verb("rsip"); return verb;
    static const string verb("RSIP");
    return verb;
}
