#ifndef  KeepAliveAgent_hxx
#define KeepAliveAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const KeepAliveAgent_hxx_Version =
    "$Id: KeepAliveAgent.hxx,v 1.9 2001/06/29 21:02:40 bko Exp $";

#include <list>
#include <string>

#include "vtypes.h"
#include "VThread.hxx"
#include "Sptr.hxx"


class VNetworkException;
class Connection;
class VException;

/**
    Agent to maintain Keep-Alive messages to all PEP clients
    Currently keep-alive messages are sent every 10 second
*/
class KeepAliveAgent
{
    public:
        ///Interface to get ref. to singelton object.
        static KeepAliveAgent& instance(int kaTimer = 10);
        ///
        string className()
        {
            return "KeepAliveAgent";
        }

        ///Destructor
        ~KeepAliveAgent();

        ///Registers a open connection for sending keep alive message
        void registerConn(const Connection& conn);

        ///UnRegisters a connection for sending keep alive message
        void unregisterConn(const Connection& conn);

        ///
        static void* run(void* args);
    private:
        ///
        KeepAliveAgent(int kaTimer);

        /**Sends keep-alive message on a given connection
         * if failed to send the message, it closes the connection
         */
        void sendKeepAlive(Connection& conn);

        /**
           Frees the memory associated with singelton instance.
           gets register to atexit() function at the time of creation.
        */
        static void destroy(void);

	///
        static KeepAliveAgent* _instance;

	///
        Sptr < VThread > _aThread;

	///
        typedef list < Connection > ConnectionList;

	///
        int _kaTimer;

        ///Holds the list of open connections.
        static ConnectionList _connectionList;
};
#endif
