
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OSPTime_hxx_Version =
    "$Id: OSPTime.hxx,v 1.11 2001/11/05 18:07:37 icahoon Exp $";
#ifndef OSPTIME_HXX
#define OSPTIME_HXX



#include <time.h>
#include <string>

// In order to keep PolicyTransaction and PEPLib OSP-context
// free, the following code is reworked (by following the
// algorithms from the OSP SDK):

/**  class to convert OSP GMT time to calendar time and vice versa
 *   
 */
class OSPTime
{

    public:

	/** Is given year a leap year?.
           A leap year is defined as: i) multiple of 4
                                     ii) not a multiple of 100 but
                                    iii) it's a multiple of 400
          
	*/
        static bool isLeapYear( int y )
        {
            return ((y) % 4) == 0 && (!((y) % 100) == 0 || ((y) % 400) == 0);
        }

        /// get current time (in seconds) since epoch
        static time_t getCurrentTime( void );

        /**  Format of an OSP disply time stamp: YYYY-MM-DDThh:mm:ssZ
         *  convert GMT time stamp as GMT time (in seconds) since Epoch
	 */
        static time_t timeStringToTime_t( const char * ts );

        /// convert GMT broken time to time in seconds since Epoch
        static time_t timeToGMT(
            int mon, int day, int year, int hour, int minute, int second );


        /// convert calendar time to GMT time stamp string
        static int time_T2GMTString( time_t t, string & fmtStr );

    private:
	///
        static const int _DaysAtMonth[12];

	///
        static const int BASE_YEAR;            // Year 1900

	///
        static const int BASE_DATE_IN_DAYS;    // Jan 1 1970

	///
        static const string OSP_TS;

	///
        static const string OSP_TS_FORMAT;

};


#endif
