#ifndef COPSData_hxx
#define COPSData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSData_hxx_Version =
    "$Id: COPSData.hxx,v 1.9 2000/12/18 23:42:53 bko Exp $";

#include <string>
#include <iostream.h>
#include "Sptr.hxx"

///COPSData
class COPSData
{
    public:
        ///
        COPSData() : _dataBuf(NULL), _dLen(0)
        { }

        ///

        COPSData(const char* dPtr, int dLen);

        ///
        COPSData(const string& data);

        ///
        COPSData(const COPSData& other);

        ///
        ~COPSData()
        {
            delete []_dataBuf;
        };

        ///
        COPSData& operator=(const COPSData& other);

        ///
        bool operator==(const COPSData& other);

        ///
        bool operator<(const COPSData& other);

        ///
        const char* getData() const
        {
            return _dataBuf;
        };

        ///
        int length() const
        {
            return _dLen;
        };

        ///
        string str() const;

        friend ostream& operator<<(ostream& os, const COPSData& data);

        friend bool operator<(const COPSData& left, const COPSData& right);
        friend bool operator==(const COPSData& left, const COPSData& right);
    private:
        char* _dataBuf;
        int _dLen;
};

inline ostream& operator<<(ostream& os, const COPSData& data)
{
    for (int i = 0; i < data.length(); i++)
    {
        os << data._dataBuf[i];
    }
    return os;
}

inline bool operator<(const COPSData& left, const COPSData& right)
{
    if (left._dLen > right._dLen) return false ;
    if (left._dLen < right._dLen) return true ;

    if (memcmp(left._dataBuf, right._dataBuf, left._dLen) < 0)
        return true;

    return false;
}

#endif
