
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSDecision_cxx_Version =
    "$Id: COPSDecision.cxx,v 1.14 2001/05/13 07:26:01 icahoon Exp $";

#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSDecision.hxx"
#include "COPSPrObjParser.hxx"

COPSDecision::COPSDecision(DecCType cType)
{
    _dataRep._data = 0;
    _objHdr.setCNum(COPS_DEC);
    _objHdr.setCType(cType);
    if (cType == DEC_DEF) _objHdr.setDataLength(4);
}

///Initialize the decision object with values from COPSObj header
COPSDecision::COPSDecision(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    _dataRep._data = 0;
    if (_objHdr.getCType() == DEC_DEF)
    {
        u_int16_t cmdCode;
        u_int16_t cmdFlgs;
        memcpy(&cmdCode, dataPtr + 4, 2);
        memcpy(&cmdFlgs, dataPtr + 6, 2);
        _dataRep.cData._cmdCode = htons(cmdCode);
        _dataRep.cData._flags = htons(cmdFlgs);
        _objHdr.setDataLength(4);
    }
    else
    {
        u_int16_t dLen = _objHdr.getDataLength() - 4;
        COPSData d(dataPtr + 4, dLen);
        setData(d);
    }
}

COPSDecision::COPSDecision(const COPSDecision& other)
{
    _objHdr = other._objHdr;
    _padding = other._padding;
    _dataRep._data = 0;
    ;
    if (_objHdr.getCType() == 1)
    {
        _dataRep.cData._cmdCode = other._dataRep.cData._cmdCode;
        _dataRep.cData._flags = other._dataRep.cData._flags;
    }
    else
    {
        delete _dataRep._data;
        _dataRep._data = new COPSData(*other._dataRep._data);
    }
}

COPSDecision&
COPSDecision::operator=(const COPSDecision& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _padding = other._padding;
        if (_objHdr.getCType() == 1)
        {
            _dataRep.cData._cmdCode = other._dataRep.cData._cmdCode;
            _dataRep.cData._flags = other._dataRep.cData._flags;
        }
        else
        {
            delete _dataRep._data;
            _dataRep._data = new COPSData(*other._dataRep._data);
        }
    }
    return (*this);
}


COPSData
COPSDecision::getData() const
{
    if (_dataRep._data)
    {
        return (*_dataRep._data);
    }
    COPSData d;
    return (d);
}

void
COPSDecision::setData(const COPSData& data)
{
    assert(static_cast < int > (_objHdr.getCType()) >= 2);
    if (data.length() % 4)
    {
        int padLen = 4 - data.length() % 4;
        _padding = getPadding(padLen);
    }
    delete _dataRep._data;
    _dataRep._data = new COPSData(data);
    _objHdr.setDataLength(_dataRep._data->length());
}

u_int16_t
COPSDecision::getDataLength() const
{
    return (_objHdr.getDataLength() + _padding.length());
}

COPSDecision::~COPSDecision()
{
    if (_objHdr.getCType() >= 2)
    {
        delete _dataRep._data;
        _dataRep._data = 0;
    }
}

void
COPSDecision::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSDecision::writeData");
    _objHdr.writeData(id);
    if (_objHdr.getCType() >= 2)
    {
        COPSUtil::writeData(id, _dataRep._data->getData(), _dataRep._data->length());
        if (_padding.length())
        {
            COPSUtil::writeData(id, _padding.getData(), _padding.length());
        }
    }
    else
    {
        u_int16_t cmdCode = htons(_dataRep.cData._cmdCode);
        u_int16_t cmdFlgs = htons(_dataRep.cData._flags);
        COPSUtil::writeData(id, &cmdCode, 2);
        COPSUtil::writeData(id, &cmdFlgs, 2);
    }
}

void
COPSDecision::setCmdCode(DecCmdCode cCode)
{
    assert(_objHdr.getCType() == 1);
    _dataRep.cData._cmdCode = static_cast < u_int16_t > (cCode);
}

void
COPSDecision::setFlags(u_int16_t flags)
{
    assert(_objHdr.getCType() == 1);
    _dataRep.cData._flags = flags;
}

void
COPSDecision::dump(ostream& os) const
{
    os << _objHdr << endl;
    os.width(25);
    if (_objHdr.getCType() == 1)
    {
        os << "Decsion (" << getTypeStr() << ")" << endl;
        os.width(25);
        os << "Command code: " << _dataRep.cData._cmdCode << endl;
        os.width(25);
        os << "Command flags: " << _dataRep.cData._flags << endl;
    }
    else
    {
        os.width(25);
        os << "Decsion(" << getTypeStr() << ")" << endl;
        os.width(25);
        os << "Data: " << *_dataRep._data << endl;
    }
}

void
COPSDecision::dumpPrObject(ostream& os) const
{
    if (_objHdr.getCType() == 1)
    {
        dump(os);
        return ;
    }

    assert(_objHdr.getCType() == 5);
    assert(_dataRep._data);
    COPSPrObjParser parser;
    Sptr < COPSPrObjBase > prObject = parser.parse(*_dataRep._data);
    if (prObject.getPtr() != 0)
    {
        os << _objHdr << endl;
        os.width(25);
        os << *prObject << endl;
    }
    else
    {
        dump(os);
    }
}

string
COPSDecision::getTypeStr() const
{
    switch (_objHdr.getCType())
    {
        case DEC_DEF:
        return "Default";
        break;
        case DEC_STATELESS:
        return "Stateless data";
        break;
        case DEC_REPL:
        return "Replacement data";
        break;
        case DEC_CSI:
        return "Client specific decision data";
        break;
        case DEC_NAMED:
        return "Named decision data";
        break;
        default:
        return "Unknown";
    }
}
