#ifndef COPSDecision_hxx
#define COPSDecision_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const COPSDecision_hxx_Version =
    "$Id: COPSDecision.hxx,v 1.10 2000/12/18 23:42:53 bko Exp $";

#include <sys/types.h>

#include "COPSObjBase.hxx"
#include "COPSObjHeader.hxx"
#include "COPSData.hxx"
#include "Sptr.hxx"


typedef enum {
    DEC_DEF = 1,
    DEC_STATELESS,
    DEC_REPL,
    DEC_CSI,
    DEC_NAMED
} DecCType;


///COPSDecision

class COPSDecision : public COPSObjBase
{
    public:
        enum DecCmdCode
        {
            DEC_NULL = 0,
            DEC_INSTALL,
            DEC_REMOVE
    };

        /**
          Constructor to create a Decision object.By default creates
          a decision object which is of fixed length.
         */
        COPSDecision(DecCType cType = DEC_DEF);

        COPSDecision(const COPSDecision& );
        COPSDecision& operator=(const COPSDecision& );

        ///
        virtual ~COPSDecision();


        ///
        u_int16_t getDataLength() const;

        ///Accessor
        ///Get the associated data if decision object is of cType 2 or higher
        COPSData getData() const;

        ///Set the decision data if decision object is of cType 2 or higher
        void setData(const COPSData& data);

        ///retruns true if cType = 1
        bool isFlagSet() const
        {
            return ( _objHdr.getCType() == 1);
        };
        ///If cType == 1 , get the flags associated
        u_int16_t getFlags()
        {
            return ( _dataRep.cData._flags);
        };

        ///If cType == 1 ,set the cmd code
        void setCmdCode(DecCmdCode cCode);
        ///If cType == 1 ,set the cmd flags
        void setFlags(u_int16_t flags);

        ///
        bool isNullDecision()
        {
            return ( _dataRep.cData._cmdCode == 0);
        };
        ///
        bool isInstallDecision()
        {
            return ( _dataRep.cData._cmdCode == 1);
        };
        ///
        bool isRemoveDecision()
        {
            return ( _dataRep.cData._cmdCode == 2);
        };

        ///
        void dump(ostream& os)const;

        ///
        void dumpPrObject(ostream& os)const;

        ///
        string getTypeStr() const;

        ///
        bool isDecision() const
        {
            return true;
        };
        ///
        virtual bool isLocalDecision() const
        {
            return false;
        };

        ///Writes data to a given network socket
        void writeData(int id) const throw(VNetworkException&);

    protected:
        friend class COPSReqMsg;
        friend class COPSDecisionMsg;

        ///Initialize the decision object with values from COPSObj header
        COPSDecision(const char* dataPtr);

        COPSObjHeader _objHdr;
        union {
            COPSData* _data;
            struct
            {
                u_int16_t _cmdCode;
                u_int16_t _flags;
            }
            cData;
        } _dataRep;
        COPSData _padding;
};

#endif
