
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSMsg_cxx_Version =
    "$Id: COPSMsg.cxx,v 1.6 2000/12/18 23:42:54 bko Exp $";
#include <assert.h>

#include "COPSMsg.hxx"
#include "COPSObjBase.hxx"
#include "VNetworkException.hxx"
#include "cpLog.h"

void
COPSMsg::checkSanity() const throw (COPSBadDataException&)
{
    assert(0);
}

void
COPSMsg::parseHeader(const char* data)
throw (COPSBadDataException&)
{
    _dataLength = 0;
    _dataStart = 0;
    if (!_hdr)
    {
        _hdr = new COPSHeader(COPS_OP_CAT);
        memcpy(&_hdr->getDataRep(), data, 8);
        _dataStart += 8;
        _dataLength = static_cast < int32_t > (_hdr->getMsgLength());
    }
    else
    {
        //header is already read
        _dataLength = static_cast < int32_t > (_hdr->getMsgLength()) - 8;
    }

    //validate the message length
    //Should fill on the 32bit boundary
    if ((_hdr->getMsgLength() % 4))
    {
        cpLog(LOG_ALERT, "COPS message is not on 32 bit bounday");
        throw COPSBadDataException("Bad message format", __FILE__,
                                   __LINE__, 0);
    }
}

ostream& operator<<(ostream& os, const COPSMsg& msg)
{
    msg.dump(os);
    return os;
}

