
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSPrError_cxx_Version =
    "$Id: COPSPrError.cxx,v 1.4 2000/12/18 23:42:55 bko Exp $";

#include "vin.h"
#include "COPSPrError.hxx"

COPSPrError::COPSPrError(const char* data)
{
    u_int16_t a;
    memcpy(&a, data, 2);
    a = ntohs(a);
    memcpy(&_len, &a, 2);
    memcpy(&_sNum, data + 2, 2);

    u_int16_t eCode;
    memcpy(&eCode, data + 4, 2);
    _errCode = ntohs(eCode);

    u_int16_t eSubCode;
    memcpy(&eSubCode, data + 6, 2);
    _errSubCode = ntohs(eSubCode);
}

void
COPSPrError::dump(ostream& os) const
{
    COPSPrObjBase::dump(os);
    os.width(25);
    os << "Error code: " << static_cast < int > (_errCode) << " (" << strError() << ")" << endl;
    os.width(25);
    os << "Error sub code: " << static_cast < int > (_errSubCode) << endl;
}

COPSPrError&
COPSPrError::operator=(const COPSPrError& other)
{
    if (this != &other)
    {
        COPSPrObjBase::operator=(other);
        _errCode = other._errCode;
        _errSubCode = other._errSubCode;
    }
    return *this;
}

void
COPSPrError::writeData(int id) const throw(VNetworkException&)
{
    u_int16_t len = htons(_len);
    COPSUtil::writeData(id, &len, 2);
    COPSUtil::writeData(id, &_sNum, 2);

    u_int16_t eCode = htons(_errCode);
    COPSUtil::writeData(id, &eCode, 2);

    u_int16_t eSubCode = htons(_errSubCode);
    COPSUtil::writeData(id, &eSubCode, 2);
}

void*
COPSPrError::getDataRep()
{
    delete _dataRep;
    _dataRep = new char[getDataLength()];

    u_int16_t len = htons(_len);
    memcpy(_dataRep, &len, 2);
    memcpy(_dataRep + 2, &_sNum, 2);

    u_int16_t eCode = htons(_errCode);
    memcpy(_dataRep + 4, &eCode, 2);

    u_int16_t eSubCode = htons(_errSubCode);
    memcpy(_dataRep + 6, &eSubCode, 2);
    return (void*)_dataRep;
}

