
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const COPSTimer_cxx_Version =
    "$Id: COPSTimer.cxx,v 1.8 2001/05/13 07:26:02 icahoon Exp $";

#include "VocalCommon.hxx"
#include <netinet/in.h>

#include "COPSTimer.hxx"

COPSTimer::COPSTimer(u_int16_t timeVal)
{
    //Time range is 1 - 65535 seconds
    assert(timeVal);
    _dataRep._timerValue = timeVal;
    _objHdr.setDataLength(sizeof(u_int32_t));
}

///
COPSTimer::COPSTimer(const char* dataPtr)
{
    _objHdr.parse(dataPtr);
    _objHdr.checkDataLength();
    u_int16_t res;
    u_int16_t tVal;
    memcpy(&res, dataPtr + 4, sizeof(u_int16_t));
    memcpy(&tVal, dataPtr + 6, sizeof(u_int16_t));
    _dataRep._reserved = ntohs(res);
    _dataRep._timerValue = ntohs(tVal);
    _objHdr.setDataLength(sizeof(u_int32_t));
}

COPSTimer::COPSTimer(const COPSTimer& other)
{
    _objHdr = other._objHdr;
    _dataRep._reserved = other._dataRep._reserved;
    _dataRep._timerValue = other._dataRep._timerValue;
}

void
COPSTimer::dump(ostream& os) const
{

    os << _objHdr << endl;
    os.width(25);
    os << "Timer val: " << _dataRep._timerValue << endl;
}


COPSTimer&
COPSTimer::operator=(const COPSTimer& other)
{
    if (this != &other)
    {
        _objHdr = other._objHdr;
        _dataRep._reserved = other._dataRep._reserved;
        _dataRep._timerValue = other._dataRep._timerValue;
    }
    return (*this);
}

u_int16_t
COPSTimer::getDataLength()const
{
    //Add the size of the header also
    return (_objHdr.getDataLength());
}

void
COPSTimer::writeData(int id) const throw(VNetworkException&)
{
    cpLog(LOG_DEBUG, "COPSTimer::writeData");
    _objHdr.writeData(id);
    u_int16_t res = htons(_dataRep._reserved);
    u_int16_t tVal = htons(_dataRep._timerValue);
    COPSUtil::writeData(id, &res, 2);
    COPSUtil::writeData(id, &tVal, 2);
}
