
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const testOSPIntf_cxx_Version =
    "$Id: testOSPIntf.cxx,v 1.6 2000/12/18 23:43:09 bko Exp $";
/*#########################################################################
*#########################################################################
*#########################################################################
*/

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "osp.h"
#include "ospprovider.h"
#include "osppkcs1.h"
#include "ospfail.h"
#include "osptnlog.h"
#include "ospasn1.h"
#include "ospsecurity.h"
#include "osppkcs8.h"
#include "ospx509.h"
#include "ospb64.h"

#include "cpLog.h"
#include "md5String.hxx"

#include "Cops_def.hxx"
#include "COPSReqMsg.hxx"
#include "COPSReportMsg.hxx"

#include "ospostime.h"
#include "OSPTime.hxx"
#include "OSPLib.hxx"
#include "OSPIntf.hxx"
#include "PolicyTranslator.hxx"
#include "PolicyTransaction.hxx"


/*----------------------------------------------*
 *               global handles                 *
 *----------------------------------------------*/
char *callednumber = { "14048724887" };
char *callingnumber = { "14048729515" };

int testCops2OspReq( int * pid)
{
    assert( pid );

    // format a Cops Request message:
    COPSHeader hdr(COPS_OP_REQ);
    COPSContext cntxt(COPSContext::OUT, M_OSP);
    // COPSContext cntxt(COPSContext::IN_ADMIN | COPSContext::OUT, 0);

    COPSClientSI cSi(COPSClientSI::CSI_NAMED);
    COPSData csiData(COPS_key::callingNumber + "|" + callingnumber);
    cSi.setData(csiData);

    COPSClientSI cSi2(COPSClientSI::CSI_NAMED);
    COPSData csi2Data(COPS_key::calledNumber + "|" + callednumber);
    cSi2.setData(csi2Data);

    COPSClientSI cSi3(COPSClientSI::CSI_NAMED);
    COPSData csi3Data(COPS_key::user + "|" + "linux");
    cSi3.setData(csi3Data);

    COPSClientSI cSi4(COPSClientSI::CSI_NAMED);
    COPSData csi4Data(COPS_key::source + "|" + "172.16.1.101");
    cSi4.setData(csi4Data);

    COPSClientSI cSi5(COPSClientSI::CSI_NAMED);
    COPSData csi5Data(COPS_key::sourceDev + "|" + "Policy Server");
    cSi5.setData(csi5Data);

    COPSClientSI cSi6(COPSClientSI::CSI_NAMED);
    COPSData csi6Data(
        //   400 bad request
        //   COPS_key::callId+"|"+"e2782923eedff8d4c0b5680d84265530");
        // max. size of callId is 30
        //   COPS_key::callId+"|"+"E944D9A6-88C509CE-0-2888184C01");
        //   400 bad request
        COPS_key::callId + "|" + "E944D9A6-88C509CE-0-2888184C@192.168.6.210");
    // default H.323 call reference value is 3 octets
    //COPSData csi6Data(COPS_key::callId+"|"+"101");
    cSi6.setData(csi6Data);

    COPSHandle handle;
    COPSData d("1234.vovida.com");    // some unique handle
    handle.setId( d );

    COPSReqMsg msg;
    msg << hdr ;
    msg << handle ;
    msg << cntxt ;
    msg << cSi ;
    msg << cSi2 ;
    msg << cSi3 ;
    msg << cSi4 ;
    msg << cSi5 ;
    msg << cSi6 ;

    cerr << msg << endl;

    // pass the Cops request to the Policy Translator which
    // will translate the data to an OSPAuthInfo object.
    //OSPIntf & intf = OSPIntf::Instance();  // new OSPIntf & OSPProvider

    PolicyTranslator pT;
    PolicyTransaction pol;

    int err = 0;
    try
    {
        err = pT.sendReq( msg, pol );
        pol.print();
        *pid = pol.getID();
        cpLog(LOG_DEBUG, "Cops->Osp req pid = %d\n", *pid);
        string str = pol.encode();
        PolicyTransaction pol2;
        pol2.decode(str);

    }
    catch (PolicyBadDataException & e)
    {
        cpLog(LOG_DEBUG, "Cops->Osp req exception: %s",
              e.getDescription().c_str());
    }
    catch (OSPException & e)
    {
        cpLog(LOG_DEBUG,
              "Cops->Osp req - Exception occurs in libOSP: %s",
              e.getDescription().c_str());
    }
    return err;

} // testCops2OspReq( void )

int
testCops2OspUsage( int id )
{
    // format a Cops Usage message:
    COPSHeader hdr(COPS_OP_RPT);
    COPSContext cntxt(COPSContext::OUT, M_USAGE);

    COPSHandle handle;
    COPSData d("1234.vovida.com");    // some unique handle
    handle.setId( d );

    COPSReportMsg msg;
    msg << hdr ;
    msg << handle ;

    cerr << msg << endl;

    PolicyTranslator pT;
    PolicyTransaction pol;

    pol.setID( id );
    pol.setDuration( 700 );

    int err = 0;
    try
    {
        err = pT.sendUsage( msg, pol );
        pol.print();
        string str = pol.encode();
        PolicyTransaction pol2;
        pol2.decode(str);
    }
    catch (PolicyBadDataException & e)
    {
        cpLog(LOG_DEBUG, "Cops->Osp Usage exception: %s",
              e.getDescription().c_str());
    }
    catch (OSPException & e)
    {
        cpLog(LOG_DEBUG,
              "Cops->Osp usage - Exception occurs in libOSP: %s",
              e.getDescription().c_str());
    }
    return err;

}   //  int testCops2OspUsage( void )

//
int testCallAbandon( int id )
{

    PolicyTranslator pT;
    PolicyTransaction pol;
    string msg = "Abandon call";

    cpLog(LOG_DEBUG, "testCallAbandon: id %d\n", id );
    int err = 0;
    pol.setID( id );
    err = pT.abandonTrans( msg, pol );
    pol.print();

    if (err != 0)
    {
        cpLog(LOG_DEBUG, "transactionAbort id = %d, error = %d", id, err);
    }
    else
    {
        cpLog(LOG_DEBUG, "transactionAbort() completes");
    }
    return err;
}   // testCallAbandon

int
main(int argc, char *argv[])
{

    OSPTime t;
    int id1 = 0;
    int id2 = 0;

    OSPTNINITLOG("test_app.log");

    // set cpLog priority level:
    cpLogSetPriority( LOG_DEBUG );
    cpLogSetLabel("osp");

    // translate a COPS request to an OSP Auth Req:
    int errorcode = testCops2OspReq(&id1);
    printf("testCops2OspReq() trans id %d returns %d\n", id1, errorcode );

    // usleep(50000);
    OSPM_SLEEP(2);
    // sleep( 2 );

    // translate a COPS report to an OSP Usage Ind
    errorcode = testCops2OspUsage(id1);
    cpLog(LOG_DEBUG, "testCops2OspUsage() returns %d", errorcode);
    // cout << "testCops2OspUsage() returns " << errorcode << endl;

#if 0
    // send another auth request:
    printf("sending another Auth. Request\n");
    errorcode = testCops2OspReq(&id2);
    printf("testCops2OspReq() trans id %d returns %d\n", id2, errorcode );

    OSPM_SLEEP(1);

    // testing md5String object:
    md5String md5Str;
    string key = "E944D9A6-88C509CE-0-2888184C@192.168.6.210";
    string digest = md5Str.getMd5String( key );
    char output[256];
    output[0] = '\0';
    int outsize = 255;
    OSPPBase64Encode(reinterpret_cast < const unsigned char* > (digest.c_str()),
                     16,
                     output, &outsize);
    output[outsize] = '\0';

    printf("\nmd5String: key=%s \n digest=%s\n",
           key.c_str(), output);

    OSPM_SLEEP(1);
    // translate a COPS report to an OSP Usage Ind
    errorcode = testCops2OspUsage(id1);
    cout << "testCops2OspUsage() returns " << errorcode << endl;

    OSPM_SLEEP(1);
    printf("calling testCallAbandon()\n");
    errorcode = testCallAbandon( id2 );
#endif
    usleep(50000);
    // verify OSPTime class:
    time_t tm = t.timeStringToTime_t("2000-05-09T12:51:10Z");
    printf( "timeStringToTime_t = %lu\n", tm );
    printf( "ctime (from timStringToTime_t) = %s\n", ctime(&tm) );
    // tm = OSPPOSTimeConvertToGMT( 5, 9, 100, 12, 51, 10 );
    // printf( "OSPPOSTimeConvertToGMT = %lu\n", tm );
    // char tmStr[32];
    // OSPPOSTimeCalToString( tm, &tmStr[0] );
    // cout << "\nOSPPOSTimeCalToString = " << tmStr << "\n" << endl;

    // cout << "OSPC_TIMESTRINGSIZE = " << OSPC_TIMESTRINGSIZE << "\n" << endl;
    // cout << "OSP_TS_SIZE = " << OSP_TS_SIZE << "\n" << endl;
    // watch out: sizeof("....") includes the null terminator!
    // cout << "size of OSP_TS = " << sizeof(OSP_TS) << "\n" << endl;

    string fmtStr;
    t.time_T2GMTString( tm, fmtStr );

    // verify get current time:
    tm = OSPTime::getCurrentTime();
    printf( "current time = %lu\n", tm );
    printf( "ctime (from getCurrentTime) = %s\n", ctime(&tm) );

    printf("Program Over.\n");
    exit(0);
}

// end of file
