
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ReqState_cxx_Version =
    "$Id: ReqState.cxx,v 1.22 2001/04/19 04:59:03 icahoon Exp $";
#include <stdio.h>

#include "ReqStateComplete.hxx"
#include "ReqStateFail.hxx"
#include "ReqStateImplFinish.hxx"
#include "ReqStateImplStart.hxx"
#include "ReqStatePending.hxx"
#include "ReqStateReceived.hxx"
#include "ReqStateSent.hxx"
#include "ReqStateTimeout.hxx"
#include "ReqStateUninitialized.hxx"
#include "ReqStateDelete.hxx"
#include "PolRequest.hxx"
#include "PepAgent.hxx"
#include "OSPTime.hxx"
#include "PolicyCfg.hxx"


static const size_t     MY_BUFSIZE = 256;
static const u_int32_t  REQ_TIMEOUT= 20;
static const u_int32_t  REQ_WAIT   = 3*3600;   /* 3 hours */

void
ReqState::error(const string& errMsg)
{
    char buf[MY_BUFSIZE];
    sprintf(buf, "****Invalid state transition, %s", errMsg.c_str());
    cpLog(LOG_ALERT, buf);
    //throw VInvalidStateTransitionException(buf, __FILE__, __LINE__, 0);
}


void
ReqState::init(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::init");
}

void
ReqState::send(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::send");
}

void
ReqState::timeOut(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::timeOut");
}

void
ReqState::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::fail");
}

void
ReqState::received(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::received");
}

void
ReqState::implStart(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::implStart");
}

void
ReqState::implFinish(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::implFin");
}

void
ReqState::finish(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::finish");
}

void
ReqState::remove(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::remove");
}

void
ReqState::reSend(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    error(className() + "::reSend");
}


void
ReqState::changeState(PolRequest& req, ReqState* newState)
{
    assert(newState != 0);
    req.setState(newState);
}

void
ReqStateUninitialized::init(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStatePending);
    PepAgent::instance().queueForSend(req);

}

ReqStatePending::ReqStatePending()
{
    _startTime = static_cast < u_int32_t > (OSPTime::getCurrentTime());
}

void
ReqStatePending::send(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateSent);
}

void
ReqStatePending::checkIfTimedOut(PolRequest& req)
{
    u_int32_t cTime = static_cast < u_int32_t > (OSPTime::getCurrentTime());
    int noResponseTimeOut = PepAgent::instance().getResponseTime() / 1000;

    if ((int)(cTime - _startTime) > noResponseTimeOut + 10)
    {
        cpLog(LOG_DEBUG, "Did not send request for %s", req.getClientHandle().str().c_str());
        req.fail();
    }
}


ReqStateSent::ReqStateSent()
{
    _startTime = static_cast < u_int32_t > (OSPTime::getCurrentTime());
}

void
ReqStateSent::timeOut(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateTimeout);
    PepAgent::instance().queueForResponse(req);
}

void
ReqStateSent::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateFail);
    PepAgent::instance().queueForResponse(req);
}


void
ReqStateSent::checkIfTimedOut(PolRequest& req)
{
    u_int32_t cTime = static_cast < u_int32_t > (OSPTime::getCurrentTime());

    if ((cTime - _startTime) > REQ_TIMEOUT)
    {
        cpLog(LOG_DEBUG, "Did not receive response for %s", req.getClientHandle().str().c_str());
        req.timeOut();
    }
}

void
ReqStatePending::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateFail);
    PepAgent::instance().queueForResponse(req);
}

void
ReqStateSent::received(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateReceived);
    PepAgent::instance().queueForResponse(req);
}

void
ReqStateTimeout::init(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStatePending);
    req.createNewHandle();
}

void
ReqStateTimeout::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateFail);
}

void
ReqStateFail::remove(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateDelete);
    req.remove();
}

void
ReqStateFail::finish(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateDelete);
    req.remove();
}

void
ReqStateReceived::implStart(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateImplStart);
}

void
ReqStateReceived::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateFail);
    req.createUsageReport(COPSReportType::FAILURE);
    PepAgent::instance().sendUsage(&req);
}

void
ReqStateReceived::implFinish(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateComplete);
    req.createUsageReport(COPSReportType::SUCCESS, true);
    PepAgent::instance().queueForUsage(req);
}


ReqStateImplStart::ReqStateImplStart()
{
    _startTime = static_cast < u_int32_t > (OSPTime::getCurrentTime());
}

void
ReqStateImplStart::implFinish(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateImplFinish);
    req.createUsageReport(COPSReportType::SUCCESS);
    PepAgent::instance().queueForUsage(req);
}

void
ReqStateImplStart::checkIfTimedOut(PolRequest& req)
{
    u_int32_t cTime = static_cast < u_int32_t > (OSPTime::getCurrentTime());

    //Wait for 3 hours - should be long enough
    if ((cTime - _startTime) > REQ_WAIT)
    {
        cpLog(LOG_DEBUG, "Did not receive response for %s", req.getClientHandle().str().c_str());
        req.fail();
    }
}

void
ReqStateImplStart::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateFail);
    req.createUsageReport(COPSReportType::FAILURE);
    PepAgent::instance().sendUsage(&req);
}


void
ReqStateImplFinish::finish(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateComplete);
}

void
ReqStateComplete::finish(PolRequest& req)
throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateDelete);
}


void
ReqStateImplFinish::fail(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateFail);
}

void
ReqStateComplete::reSend(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStatePending);
}

void
ReqStateComplete::remove(PolRequest& req) throw (VInvalidStateTransitionException&)
{
    changeState(req, new ReqStateDelete);
}
