
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const testQoS_cxx_Version =
    "$Id: testQoS.cxx,v 1.8 2000/12/18 23:43:14 bko Exp $";
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>

#include "ProvisionInterface.hxx"
#include "PepAgent.hxx"
#include "CommandLine.hxx"


/*
    Sends a QoS request with callID 123456789 and ports 8989 to given policyServer.
    Pserver was need just to make an instance for PepAgent.
*/
int
main(int argc, const char** argv)
{
    if (argc != 6)
    {
        cerr << "Usage:" << argv[0]
             <<" <ip1> <ip2> <enable|disable> <policyServer:port> <pserver:port>"
             << endl;
        exit( -1);
    }
    try
    {
        NetworkAddress nAddr1(argv[4]);
        NetworkAddress nAddr2(argv[5]);
        ProvisionInterface::instance(nAddr2.getIpName().c_str(), nAddr2.getPort(),
                                     nAddr2.getIpName().c_str(), nAddr2.getPort());
        PepAgent::instance(nAddr1.getIpName().c_str(), nAddr1.getPort());

        cpLogSetPriority(LOG_DEBUG);
        string cmd(argv[3]);
        if (cmd == string("enable"))
        {
            //PepAgent::instance().enableQoS(argv[1], 8989, argv[2], 8989);
            PepAgent::instance().enableQoS("123456789", argv[1], 8989, argv[2], 8989);
        }
        else
        {
            //PepAgent::instance().disableQoS(argv[1], 8989, argv[2], 8989);
            PepAgent::instance().disableQoS("123456789");
        }
        while (true)
        {
            sleep(1);
        }
    }
    catch (VException& e)
    {
        cerr << "Caught exception:" << e.getDescription() << endl;
        exit( -1);
    }
}
