// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef Domain_hxx
#define Domain_hxx

#include <string>

#include "global.h"
#include "PSDBInterface.hxx"
#include "Field.hxx"
#include "FieldList.hxx"

namespace Epoisses {

/// arbitrary sorting function -- specialize for meaningful sorts
template <typename DATA>
class DataCompare
{
   public:
      bool operator()(const DATA& d1, const DATA& d2) const
      {
         return false;
      }
};

/// pairs are arbitrarily sorted by second of pair -- specialize for other sorts
template <typename T1, typename T2>
class PairDataCompare
{
   public:
      bool operator()(const pair<T1, T2>& d1, const pair<T1, T2>& d2) const
      {
         static DataCompare<T2> comparitor;
         return comparitor(d1.second, d2.second);
      }
};

/**
   Currently pretty useless. Maybe someday domain will mean more.  In
   particular, it may be true that fields will need to be associated
   with domains someday?  */
class Domain : public PSDBInterface
{
   public:
      explicit Domain(const string& _name);
      virtual ~Domain() {}

      virtual void stringify(ostream &str) const;

      const string& getName() const 
      {
         return name;
      }
      
      bool
      operator==(const Domain &rhs) const;

      bool
      operator!=(const Domain &rhs) const;

      bool
      operator<(const Domain &rhs) const;

      static const Domain None;

   protected:
      string name;
};

} // namespace Epoisses

#endif // Domain_hxx
