// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#include "Filter.hxx"
#include "Field.hxx"

using namespace Epoisses;

// don't use this operator -- it produces an empty Filter
const Filter::UnaryOperator *Filter::NoOp = new UnaryOperator("<noop>");

const Filter::UnaryOperator *Filter::IsNull = new UnaryOperator("is null");

// type safe operators as arguments to filter constructors
//const Filter::UnaryOperator *Filter::Self = new UnaryOperator("");
const Filter::UnaryOperator *Filter::Not = new UnaryOperator("not");

// !dlb! here's some implementation detail that should not be exposed..
const Filter::BinaryOperator *Filter::And = new BinaryOperator("and");

const Filter::BinaryOperator *Filter::Or = new BinaryOperator("or");

const Filter::BinaryOperator *Filter::Equal = new BinaryOperator("=");
const Filter::BinaryOperator *Filter::NotEqual = new BinaryOperator("!=");
const Filter::BinaryOperator *Filter::Less = new BinaryOperator("&lt;"); // <
const Filter::BinaryOperator *Filter::Greater = new BinaryOperator("&gt;"); // >
const Filter::BinaryOperator *Filter::LessEqual = new BinaryOperator("&lt;="); // <=
const Filter::BinaryOperator *Filter::GreaterEqual = new BinaryOperator("&gt;="); // >=

// what does this look like?
const Filter::BinaryOperator *Filter::Like = new BinaryOperator("like"); 

// empty filter
const Filter Filter::None(Sptr<FilterableInterface>(new Field<bool>("")), 
                          Filter::NoOp);

#ifdef SHOULD_WORK
Filter::Filter(const Field<bool> &field)
   : op(Filter::Self),
     arg1(field.internalField)
{
}
#endif

// For the time being, generate SQL directly
// .dlb. may want to serilize to XML and allow easy parse 
// on backend if we generalize to other DB (e.g. LDAP)
void
Filter::stringify(ostream &str) const
{
   if (op == Filter::NoOp)
   {
      return;
   }

   if (op == Filter::IsNull)
   {
      // pre-fix
      str << "(";
      arg1->stringify(str);
      str << " " << op->name << ")";
   }
   else if (op->isUnary())
   {
      str << "(" << op->name;
      arg1->stringify(str);
      str << ")";
   }
   else
   {
      // in-fix
      str << "(";
      arg1->stringify(str);
      str << " ";
      str << op->name << " ";
      arg2->stringify(str);
      str << ")";
   }
}

/// exists only for the test drivers in ../psLib/test
Filter
Filter::parse(const string &str)
{
   // I'll write this when I really have to.
   return Filter(Sptr<FilterableInterface>(new Field<string>("OK, I guess I have to write a parser")),
                 Filter::NoOp);
}

namespace Epoisses {

Filter
operator&&(const Filter &lhs,
           const Filter &rhs)
{
   // convert Filter by-value to heap Filter
   return Filter(Sptr<FilterableInterface>(new Filter(lhs)), 
                 Filter::And, 
                 Sptr<FilterableInterface>(new Filter(rhs)));
}

Filter
operator||(const Filter &lhs,
           const Filter &rhs)
{
   // convert Filter by-value to heap Filter
   return Filter(Sptr<FilterableInterface>(new Filter(lhs)), 
                 Filter::Or, 
                 Sptr<FilterableInterface>(new Filter(rhs)));
}

Filter
operator!(const Filter &arg)
{
   // convert Filter by-value to heap Filter
   return Filter(Sptr<FilterableInterface>(new Filter(arg)), 
                 Filter::Not);
}

Filter
isNull(const BaseColumnField& arg)
{
   return Filter(arg.getInternal(),
                 Filter::IsNull);
}
 
} // namespace Epoisses
