// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#ifndef IField_hxx
#define IField_hxx

#include "BaseIField.hxx"
#include <vector>
#include <loki/TypeTraits.h>

namespace Epoisses {

// !dlb! break into Constant<T>
class IConstant : public BaseIField
{
   public:
      explicit IConstant(int v)
         : BaseIField(convert(v))
      {}

      // Implement -= .dlb. generalize to operator eventually?
      IConstant(const string& fieldName, int v)
         // store the whole mess as a string
         : BaseIField(fieldName + " - ("+ convert(v) + ")")
      {}

      explicit IConstant(bool v)
         : BaseIField(convert(v))
      {}

      explicit IConstant(long v)
         : BaseIField(convert(v))
      {}

      explicit IConstant(double v)
         : BaseIField(convert(v))
      {}

      explicit IConstant(const string &v);
      explicit IConstant(const vector<string> &v);

      // need a serialization format...
      //IConstant(const DateTime &v);

   protected:
      virtual void bind(const string &s) throw (BadValueException &)
      {
         // does nothing -- can't bind a constant
      }

   private:
      virtual void stringify(ostream &s) const;
      
      /// use the encoding required by the backend
      static string encode(const string &v);

      static const string& convert(bool v);
      static string convert(int v);
      static string convert(long v);
      static string convert(double v);
      static string convert(const string & v);
      static string convert(const vector<string> & v);
};

/// all IField classes are explicit template specializations
class IField<double> : public BaseIField
{
   public:
      explicit IField(const string &_name)
         : BaseIField(_name)
      {}

      virtual ~IField() {}

      /// convert self to string
      virtual void stringify(ostream &str) const;

      /// convert bound self to double
      double value() const throw (NotBoundException&);

      /// give this IField a value
      virtual void bind(const string &s)  throw (BadValueException&);

   private:
      double theValue;
};

class IField<int> : public BaseIField
{
   public:
      explicit IField(const string &_name)
         : BaseIField(_name)
      {}

      virtual ~IField() {}

      /// convert self to string
      virtual void stringify(ostream &str) const;

      /// convert bound self to integer
      int value() const throw (NotBoundException&);

      virtual void bind(const string &s) throw (BadValueException&);

   private:
      int theValue;
};

class IField<long> : public BaseIField
{
   public:
      explicit IField(const string &_name)
         : BaseIField(_name)
      {}

      virtual ~IField() {}

      /// convert self to string
      virtual void stringify(ostream &str) const;

      /// convert bound self to long
      long value() const throw (NotBoundException&);

      virtual void bind(const string &s) throw (BadValueException&);

   private:
      long theValue;
};

class IField<bool> : public BaseIField
{
   public:
      explicit IField(const string &_name)
         : BaseIField(_name)
      {}

      virtual ~IField() {}

      /// convert self to string
      virtual void stringify(ostream &str) const;

      /// convert bound self to boolean
      bool value() const throw (NotBoundException&);

      virtual void bind(const string &s) throw (BadValueException&);

   private:
      // could compress this, but vector<bool> is a lose
      bool theValue;
};

/// all IField classes are explicit template specializations
class IField<string> : public BaseIField
{
   public:
      explicit IField(const string &_name)
         : BaseIField(_name)
      {}

      virtual ~IField() {}

      /// convert self to string
      virtual void stringify(ostream &str) const;

      /// convert bound self to string
      const string& value() const throw (NotBoundException&);

      /// give this IField a value
      virtual void bind(const string &s) throw (BadValueException&);

   private:
      string theValue;
};

/// all IField classes are explicit template specializations
class IField< vector<string> > : public BaseIField
{
   public:
      explicit IField(const string& _name)
         : BaseIField(_name)
      {}

      virtual ~IField() {}

      /// convert bound self to string
      virtual void stringify(ostream &str) const;

      /// convert self to vector<string> 
      const vector<string>& value() const throw (NotBoundException&);

      /// give this IField a value
      virtual void bind(const string &s) throw (BadValueException&);

   private:
      vector<string> theValue;
};

/*
// other IField types, each with operators that produce Filters and Assignments
class IField<DateTime>;
*/

} // namespace Epoisses

#endif // IField_hxx
