/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PSDBRequest_cxx_Version = "$Id: PSDBRequest.cxx,v 1.1 2002/09/30 04:55:12 jason Exp $";

#include <stdio.h>
#include <iostream.h>

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROV

#include "Logger.hxx"
#include "PSDBRequest.hxx"
#include "VBadDataException.hxx"
#include "Data.hxx"
#include "PSCommon.hxx"

using namespace Epoisses;

///
PSDBRequest::PSDBRequest()
   : myDomain(""), myFilter(""), myToken(""), 
     myXmlCategory(""), myXmlDescName(""),
     registerHostport(""), keywordFlag(false)
{ 
   setMessageType( PSMessage::UNKNOWN_MSG_TYPE );
   
} 

///
PSDBRequest::PSDBRequest(PSMessage::msgType type)
   : myDomain(""), myFilter(""), myToken(""), 
     myXmlCategory(""), myXmlDescName(""),
     registerHostport(""), keywordFlag(false)
{
   setMessageType( type ); 
}

///
void PSDBRequest::clear()
{
   setMessageType( PSMessage::UNKNOWN_MSG_TYPE );

   myFieldList.clear();
   // clear myTable
   myTable.clear();

   myDomain = "";
   myFilter = "";
   myToken = "";
   myXmlCategory = "";
   myXmlDescName = "";
   registerHostport = "";
   keywordFlag = false;
}

/// single table of kv pairs
void 
PSDBRequest::setFieldValue(const string& kw, const string& strVal)
   throw (VException&)
{

   if (kw.size() == 0)
   {
      DebugLog( << "setFieldValue fails, NULL keyword");
      throw VBadDataException ("NULL keyword", __FILE__, __LINE__);
   }
   Data copied(kw);
   copied.removeSpaces();
   const string copiedKw(copied.convertString());

   TableIter myTableItr = myTable.find(copiedKw);

   if (myTableItr != myTable.end() )
   {
      string strtemp = myTableItr->second;
      strtemp += strVal;
      myTable[copiedKw] = strtemp;
   }
   else
   {
      myTable[copiedKw] = strVal;
   }
}

/// get single instance of a keyword's value
void
PSDBRequest::getFieldValue( const string& kw, string& value )
   throw (VException&)
{
   //DebugLog(LOG_DEBUG, "getFieldValue " <<  kw);

   value = "";
   if (kw.size() == 0)
   {
      DebugLog (<< "getFieldValue fails, NULL keyword");
      throw VBadDataException ("NULL keyword", __FILE__, __LINE__);
   }
   if (myTable.size() == 0)
   {
      DebugLog (<< "getFieldValue fails, empty table");
      throw VBadDataException ("empty table", __FILE__, __LINE__);
   }
   TableIter i;
   i = myTable.find(kw);
   if ( i != myTable.end() )
   {
      value.assign(i->second);
   }
   else
   {   // no such keyword - throw exception
      DebugLog(<< "Failed getFieldValue: Keyword" + kw + "not found.");
      throw VBadDataException("keyword not found", __FILE__, __LINE__);
   }
}


void
PSDBRequest::print(const int loglevel)
{
   DebugLog(<< "Session ID = " << getSessionID()
            << " field size = " << myFieldList.size()
            << " table size = " << myTable.size());

   // display parent object
   // PSMessage::print(loglevel);

   DebugLog(<< "domain = " << myDomain << " filterRules = " << myFilter);

   for (FieldListtype::const_iterator flIt = myFieldList.begin();
        flIt != myFieldList.end(); flIt++)
   {
      DebugLog(<< "field list item: " << *flIt);
   }

   /// print content of myTable:
   ConstTableIter  itr;
    
   for (itr = myTable.begin(); itr != myTable.end(); itr++)
   {
      DebugLog(<< itr->first << " : " << itr->second);
   }
}

///
void
PSDBRequest::setArguments(const string& arg)
   throw (VException&)
{
   // there should not have any characters
   if (arg != "\n")
   {
      DebugLog (<<  "Duplicate keyword \"arguments\"");
      throw VBadDataException("duplicate keyword arguments", __FILE__,__LINE__);
   }
}

///
void
PSDBRequest::setDomain(const string& domain)
   throw (VException&)
{
   // there is only one time to set myDomain.
   if (! myDomain.empty())
   {
      DebugLog (<< "Duplicate keyword \"domain\"");
      throw VBadDataException("duplicate keyword domain", __FILE__,__LINE__);
   }
   else
   {
      myDomain = domain;
   }
}

///
void 
PSDBRequest::setRegisterHostport(const string& hp) 
   throw (VException&)
{ 
   if (keywordFlag)
   {
      DebugLog (<<  "Duplicate keyword \"register_hostport\"");
      throw VBadDataException("duplicate keyword register_hostport", __FILE__,__LINE__);
   }
   else
   {
      registerHostport = hp; 
   }
}

///
void 
PSDBRequest::setChangeType(const string &_changeType)
   throw (VException&)
{ 
   if (keywordFlag)
   {
      DebugLog (<< "Duplicate keyword \"changetype\"");
      throw VBadDataException("duplicate keyword changetype", __FILE__,__LINE__);
   }
   else
   {
      for (int ct = 0; ct < (int)PSCommon::MAX_CHANGE_TYPE; ct++)
      {
         if (_changeType == PSCommon::ChangeTypes[ct])
         {
            changeType = (PSCommon::ChangeType)ct;
            return;
         }
      }
      assert(0);
   }
}

///
void 
PSDBRequest::setFunctorUId(const string& hp) 
   throw (VException&)
{ 
   if (keywordFlag)
   {
      DebugLog (<< "Duplicate keyword \"update_uid\"");
      throw VBadDataException("duplicate keyword functor uid", __FILE__,__LINE__);
   }
   else
   {
      functorUID = hp; 
   }
}

///
void
PSDBRequest::setForeignDomain(const string& hp) 
   throw (VException&)
{
   if (keywordFlag)
   {
      DebugLog (<< "Duplicate keyword \"" + PSCommon::FOREIGN_DOMAIN_STR +"\"");
      throw VBadDataException("duplicate keyword foreign domain", __FILE__,__LINE__);
   }
   else
   {
      foreignDomain = hp; 
   }
}

void
PSDBRequest::setForeignField(const string& hp) 
   throw (VException&)
{
   if (keywordFlag)
   {

      DebugLog (<< "Duplicate keyword \"" + PSCommon::FOREIGN_FIELD_STR +"\"");
      throw VBadDataException("duplicate keyword foreign field", __FILE__,__LINE__);
   }
   else
   {
      foreignField = hp; 
   }
}

///
void
PSDBRequest::setFilter( const string& fr ) 
   throw (VException&)
{ 
   if (keywordFlag)
   {
      DebugLog (<< "Duplicate keyword \"filter\"");
      throw VBadDataException("duplicate keyword filter", __FILE__,__LINE__);
   }
   else
   {
      myFilter += fr; 
   }
}

///
void
PSDBRequest::setSessionID( string token )
   throw (VException&)
{
   if (keywordFlag)
   {
      DebugLog (<< "Duplicate keyword \"token\"");
      throw VBadDataException("duplicate keyword in token", __FILE__,__LINE__);
   }
   else
   {
      myToken = token; 
   }
}

///
void 
PSDBRequest::addFieldList(const string& field)
   throw (VException&)
{
   // listitem can be only appeared in GET/POST message
   // it can not be associated with a value
   if (keywordFlag)
   {
      DebugLog ("Duplicate keyword in listitem: " << field);
      throw VBadDataException("duplicate keyword in listitem", __FILE__,__LINE__);
   }
   else
   {
      Data copied(field);
      copied.removeSpaces();
      myFieldList.push_back(copied.convertString()); 
   }
}


///
void
PSDBRequest::setKeywordFlag(const string& fn, const string& val)
   throw (VException&)
{
   // the keywords are fieldlist, fieldvalue
   // it can not be associated with any characters, except '\n'
   if (val != "\n")
   {
      DebugLog ("Duplicate keyword in dbrequest: " << fn);
      throw VBadDataException("duplicate keyword", __FILE__,__LINE__);
   }
   else if (fn != "fieldlist")
   {
      //DebugLog ("Set Keyword flag");
      keywordFlag = true;
   }
}


// End of File
