/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const PSDBResult_cxx_Version ="$Id: PSDBResult.cxx,v 1.1 2002/09/30 04:55:12 jason Exp $";

#include <stdio.h>
#include <iostream.h>
#include "PSDBResult.hxx"
#include "XmlBlobHelper.hxx"
#include "VDbException.hxx"
#include "Logger.hxx"

// for PSDBResult::BindIterator
#include "FieldList.hxx"
#include "BaseField.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROV

using namespace Epoisses;

void 
PSDBResult::setError(const string& errTxt)
{
   myError = errTxt;
}

///
void PSDBResult::clear()
{
   myStatus = 0;
   myNumMatches = 0;
   setMessageType( PSMessage::UNKNOWN_MSG_TYPE );

   // clear myTables
   for (unsigned short i = 0; i < myTables.size(); i++)
   {
      myTables[i].clear();
   }

   // clear myFieldList
   myFieldList.clear();
}

/// single table of kv pairs
void 
PSDBResult::setFieldValue(const string& kw, const string& strVal)
   throw (VException&)
{
   assert(kw.size());
   (myTables[0])[kw] = strVal;
   //(myTables[0]).push_back( tableType::value_type(kw, strVal) );
}

void
PSDBResult::setFieldValue(int idx, const string& kw, const string& strVal)
   throw (VException&)
{
   assert(kw.size());
   assert(idx != -1);
   
   myTables[idx].insert( tableType::value_type(kw, strVal) );
}
 
///
void 
PSDBResult::findField( int idx, const string& kw, TableIter& itr ) const
   throw (VException&)
{
   assert(kw.size());
   assert(myTables.size());

   TableIter i;
   i = myTables[idx].find(kw); 
   if ( i != myTables[idx].end() )
   {
      itr = i;
   }
   else
   {   
      throw NoSuchFieldException ("findField failed : keyword not found", __FILE__, __LINE__);
   }
}

/// get single instance of a keyword's value
void
PSDBResult::getFieldValue( const string& kw, string& value ) const
   throw (VException&)
{
   //DebugLog(<< "getFieldValue " << kw);
   TableIter i;
   findField( 0, kw, i );
   value.assign(i->second);
}


/** given an index for  multiple matches, find the first 
 *  occurrence of a keyword, and update its value
 */
void
PSDBResult::getFieldValue( int idx, const string& kw, string& value ) const
   throw (VException&)
{
   //DebugLog(<< "getFieldValue " << kw << " for index " << idx);

   value = "";
   TableIter i;
   
   findField( idx, kw, i );
   value = i->second;
}

void
PSDBResult::getFieldValueAsBool( const string& kw, bool& boolVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( 0, kw, i );
   int x = atoi(i->second.c_str());
   boolVal = (x ? true : false);
}

void
PSDBResult::getFieldValueAsBool( int idx, const string& kw, bool& boolVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( idx, kw, i );
   int x = atoi(i->second.c_str());
   boolVal = (x ? true : false);
}

void
PSDBResult::getFieldValueAsInt( const string& kw, int& intVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( 0, kw, i );
   intVal = atoi(i->second.c_str());
}

void
PSDBResult::getFieldValueAsInt( int idx, const string& kw, int& intVal )
   throw (VException&)
{
   assert(0);
   TableIter i;

   findField( idx, kw, i );

   intVal = atoi(i->second.c_str());

}


void
PSDBResult::getFieldValueAsLong( const string& kw, long int& longVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( 0, kw, i );
   longVal = atol(i->second.c_str());
   char buf[20];
   sprintf(buf, "%ln", &longVal);
}

void
PSDBResult::getFieldValueAsLong( int idx,  const string& kw, 
                                 long int& longVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( idx, kw, i );
   longVal = atol(i->second.c_str());
   char buf[20];
   sprintf(buf, "%li", longVal);
}

void
PSDBResult::getFieldValueAsDouble( const string& kw, double& doubleVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( 0, kw, i );
   doubleVal = atof(i->second.c_str());
}

void
PSDBResult::getFieldValueAsDouble( int idx,  const string& kw, 
                                   double& doubleVal )
   throw (VException&)
{
   assert(0);
   TableIter i;
   findField( idx, kw, i );
   doubleVal = atof(i->second.c_str());
}

const tableType& 
PSDBResult::getTable(int idx) const throw (VException&)
{
   assert (idxInRange( idx ));
   return( myTables[idx]); 
}

void
PSDBResult::addFieldList(const string & val)
{
   myFieldList.push_back(val);
}

ostream& 
PSDBResult::operator<<(ostream& strm) const
{
   strm << "Domain = " << getDomain() 
        << " Status = " << getStatus() 
        << " Session ID = " << getSessionID()
        << " Error = " << getError() 
        << " Records Found = " << myTables.size()
        << endl;

   // display parent object
   // PSMessage::print(loglevel);
   for (FieldListtype::const_iterator flIt = myFieldList.begin();
        flIt != myFieldList.end(); flIt++)
   {
      strm << "  FieldList item: " << *flIt << endl;
   }

   /// print content of myTables:
   ConstTableIter  itr;
   for (unsigned int i = 0; i < myTables.size(); i++)
   {
      const tableType& record = getTable( i );
      for (itr = record.begin(); itr != record.end(); itr++)
      {
         strm << i << ") " << itr->first << " = " << itr->second << endl;
      }
   }
   strm << "compiled at " << __DATE__ " " __TIME__ << endl;
   return strm;
}

///
void 
PSDBResult::getFieldValueAsList(const string& kw,
				blobType& listResult)
   throw (VException&)
{
   assert(0);

   // get the blob string
   string blob;
   getFieldValue(kw, blob);

   XmlBlobHelper helper(blob);
   listResult = helper.getBlobTable();

}

///
void 
PSDBResult::getFieldValueAsList(int idx, 
				const string& kw, 
				blobType& listResult)
   throw (VException&)
{
   assert(0);


   // get the blob string
   string blob;
   getFieldValue(idx, kw, blob);

   XmlBlobHelper helper(blob);
   listResult = helper.getBlobTable();

}

// begin takes a FieldList to bind
PSDBResult::BindIterator
PSDBResult::begin(FieldList fieldList) const throw (BadValueException&, NoSuchFieldException&)
{
   PSDBResult_BindIterator bi(*this, fieldList);
   return bi;
}

PSDBResult::BindIterator
PSDBResult::end() const
{
   PSDBResult_BindIterator bi(*this);
   return bi;
}

// Implementation of PSDBResult_BindIterator
PSDBResult_BindIterator::PSDBResult_BindIterator(const PSDBResult& _result,
                                                 const FieldList& _fields) 
   throw (BadValueException&, NoSuchFieldException&)
   : result(_result),
     fields(_fields),
     position(0)
{
   //DebugLog(<< "BindIterator::PSDBResult_BindIterator(" << result->getNumMatches() << ")");
   bind();
   //DebugLog(<< "(after bind) BindIterator::PSDBResult_BindIterator(" << result->getNumMatches() << ")");   
}

PSDBResult_BindIterator::PSDBResult_BindIterator(const PSDBResult_BindIterator &src)
   : result(src.result),
     fields(src.fields),
     position(src.position)
{}

PSDBResult_BindIterator::~PSDBResult_BindIterator()
{
}

// constructor for end iterator
PSDBResult_BindIterator::PSDBResult_BindIterator(const PSDBResult& _result)
   : result(_result),
     fields(),
     position(result.getNumMatches())
{
}

// does all of the real work, including checking the position for validity
void
PSDBResult_BindIterator::bind() throw (BadValueException&, NoSuchFieldException&)
{
   //DebugLog(<< "PSDBResult::BindIterator::bind " << position << " of " << result.getNumMatches());

   // check the position
   const bool inRange = position > -1 && position < result.getNumMatches();

   // run through each Field in the Field list
   for (FieldList::Contents::iterator cIt = fields.begin();
        cIt != fields.end(); cIt++)
   {
      // unbind unconditionally
      (*cIt)->isBound = false;

      // bind iff the position is legal
      if (inRange)
      {
         string value;
         // might throw
         result.getFieldValue(position, (*cIt)->getName(), value);

         // might throw
         (*cIt)->bind(value);
         (*cIt)->isBound = true;
      }
   }
}

void
PSDBResult_BindIterator::operator++(int)
{
   //cerr << "PSDBResult_BindIterator::operator++(" << result->getNumMatches() << ")" << endl;
   position++;
   bind();
}

bool
PSDBResult_BindIterator::operator==(const PSDBResult_BindIterator &other)
{
   // make sure the comparison is between iterators on the same result
   assert(&result == &other.result);
   return position == other.position;
}

bool
PSDBResult_BindIterator::operator!=(const PSDBResult_BindIterator &other)
{
   //cerr << "PSDBResult_BindIterator::operator!=(" << result->getNumMatches() << ")" << endl;
   // make sure the comparison is between iterators on the same result
   assert(&result == &other.result);
   return position != other.position;
}
