#ifndef PSREQXMLCONTENTHANDLERS_HXX
#define PSREQXMLCONTENTHANDLERS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSReqXmlContentHandlersHxxVersion = "$Id: PSReqXmlContentHandlers.hxx,v 1.1 2002/09/30 04:55:12 jason Exp $";


#include <string>
#include <set>
#include <stack>
#include <xercesc/sax2/DefaultHandler.hpp>

#include "XmlContent.hxx"
#include "Sptr.hxx"
#include "VException.hxx"
#include "PSDBRequest.hxx"
#include "PSDBCommonType.h"
#include "Field.hxx"

namespace Epoisses {

class Domain;
class BaseField;
class FieldList;
class AssignmentList;
class Filter;

///
class PSReqXmlContentHandlers : public DefaultHandler
{
   public:

      /**-----------------------------------------------------------------------
       *  Constructors
       * -----------------------------------------------------------------------
       */
      PSReqXmlContentHandlers() {}
      ///
      PSReqXmlContentHandlers(Sptr<PSDBRequest> target);

      /**-----------------------------------------------------------------------
       *  Destructors
       * -----------------------------------------------------------------------
       */
      virtual ~PSReqXmlContentHandlers();


      // -----------------------------------------------------------------------
      //  Implementations of the SAX DocumentHandler interface
      // -----------------------------------------------------------------------
      ///
      void endDocument();

      ///
      void endElement( const XMLCh* const uri, 
                       const XMLCh* const localname, 
                       const XMLCh* const qname);

      ///
      void characters(const XMLCh* const chars, const unsigned int length);

      ///
      void ignorableWhitespace
      (
         const   XMLCh* const    chars
         , const unsigned int    length
         );

      ///
      void startDocument();

      ///
      void startElement(	const   XMLCh* const    uri,
                                const   XMLCh* const    localname,
                                const   XMLCh* const    qname,
                                const   Attributes&		attributes);



      // -----------------------------------------------------------------------
      //  Implementations of the SAX ErrorHandler interface
      // -----------------------------------------------------------------------
      ///
      void warning(const SAXParseException& exception);
      ///
      void error(const SAXParseException& exception);
      ///
      void fatalError(const SAXParseException& exception);



      // -----------------------------------------------------------------------
      //  Implementation of the SAX DTDHandler interface
      // -----------------------------------------------------------------------
      ///
      void notationDecl
      (
         const   XMLCh* const    name
         , const XMLCh* const    publicId
         , const XMLCh* const    systemId
         );  // not used

      ///
      void unparsedEntityDecl
      (
         const   XMLCh* const    name
         , const XMLCh* const    publicId
         , const XMLCh* const    systemId
         , const XMLCh* const    notationName
         );  // not used

      ///  set a reference to the target where parsed content will be kept
      void setParsedContentTarget( Sptr<PSDBRequest> target )
      {
         myParsedContent = target;
      }

      ///
      Sptr< PSDBRequest >& getParsedContentTarget() { return myParsedContent; }


      // -----------------------------------------------------------------------
      //  Implementation of PS Request XML protocol builder interface
      // -----------------------------------------------------------------------

      ///
      string buildGetXmlDescriptionReq(
         const string& xmlCategory,
         const string& xmlDescription)
         throw( VException& );

      ///
      string buildListXmlCatagoriesReq()
         throw( VException& );

      ///
      string buildListXmlDescriptionsReq(
         const string& xmlCategory)
         throw( VException& );


///
      string buildGetRecordsInDomainReq(
         const Domain& domain,
         const Filter& filter,
         const FieldList & fieldList)
         throw( VException& );

      string 
      buildGetRecordsInDomainReq(const Domain& domain,
                                 const Filter& filter,
                                 const FieldList& fieldList,
                                 const Field<void>& orderBy)
         throw( VException& );


      ///
      string buildGetFieldsInDomainReq( const Domain& domain )
         throw( VException& );

      /// update existing records with keyword/value pairs provided
      string buildSetRecordsInDomainReq( const Domain& domain, 
                                         const Filter& filter,
                                         const AssignmentList& assignments )
         throw (VException& );

      /// insert a new record
      string buildAddRecordInDomainReq( const Domain& domain, 
                                        const AssignmentList& assignments)
         throw( VException& );

      ///
      string buildDeleteRecordsInDomainReq( const Domain& domain,
                                            const Filter& filter )
         throw( VException& );

      ///
      string buildSessionOpenReq( int timeInSeconds ) 
         throw(VException&);

      ///
      string buildSessionCloseReq( const string& sessionID ) 
         throw( VException& );

      ///
      string buildSessionSearchDBReq( 
         const string& sessionID, const Domain& domain,
         const Filter& filter,
         const FieldList & fieldList, 
         PSCommon::PSSortType sType,
         const BaseField& sortField ) throw(VException&);

      ///
      string buildSessionGetDBResultReq( 
         const string& sessionID, int startIdx,
         int endIdx, const FieldList & fieldList)
         throw (VException&);

      ///
      string buildSessionSetDBResultReq( const string& sessionID, 
                                         const map< string, string>& fieldValPairs)
         throw (VException&);

      ///
      string buildSessionSortDBResultReq( 
         const string& sessionID, PSCommon::PSSortType sType,
         const BaseField& sortField)
         throw (VException&);

      ///
      string  buildSessionDeleteAllReq( const string& sessionID ) 
         throw (VException&);

      ///
      string buildRegisterForUpdateReq(const string& hostport,
                                       const Domain& domain,
                                       const KeySet& keySet,
                                       const string &uid)
         throw (VException&);

      ///
      string buildUpdateReq (const string& domain,
                             const FieldListtype& keyList,
                             PSCommon::ChangeType changeType)
         throw (VException&);

      ///
      string buildUnRegisterForUpdateReq(const string& hostport,
                                         const Domain& domain,
                                         const string &uid)
         throw (VException&);

      string buildRegisterForForeignUpdateReq(const string& hostport,
                                              const Domain& foreignDomain,
                                              const string& foreignKeyField,
                                              const Domain& primaryDomain,
                                              const string &uid)
         throw (VException&);

      ///
      string buildForeignUpdateReq (const string& uid,
                                    const FieldListtype& keyList,
                                    PSCommon::ChangeType changeType)
         throw (VException&);

      ///
      string buildUnRegisterForForeignUpdateReq(const string& hostport,
                                                const Domain& foreignDomain,
                                                const string& uid)
         throw (VException&);

   private :
      // -----------------------------------------------------------------------
      //  Private data members
      // -----------------------------------------------------------------------


      /// parsing helper: name of element last reported is always at the top
      stack< string > myFields;

      ///
      Sptr< PSDBRequest > myParsedContent;

      ///
      string myCharacters;

};

} // namespace Epoisses

#endif // PSREQXMLCONTENTHANDLERS_HXX
