/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <stdio.h>
#include <xercesc/util/XMLUniDefs.hpp>

#include "PSCommon.hxx"
#include "XmlContent.hxx"
#include "XmlBlobContentHandler.hxx"
#include "VNetworkException.hxx"
#include "PSDBResult.hxx"
#include "Data.hxx"
#include "Logger.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVC

using namespace Epoisses;

///
XmlBlobContentHandler::XmlBlobContentHandler(Sptr<PSBlobResult> result)
   : myChars(""), myParsedContent(result), level(0), index(-1)
{
   //DebugLog(<<  "XmlBlobContentHandler constructor invoked ");
}

///
XmlBlobContentHandler::~XmlBlobContentHandler()
{
   //DebugLog(<<  "XmlBlobContentHandler destructor invoked ");
}



///
void 
XmlBlobContentHandler::startDocument()
{
   //DebugLog( <<   "startDocument() ************ ");

}

///
void 
XmlBlobContentHandler::startElement(const XMLCh* const uri,
                                    const XMLCh* const localname,
                                    const XMLCh* const qname,
                                    const Attributes& attributes)
{
   string fn = StrX(qname).localForm();  // field name
   //DebugLog( <<  " *** startElement qname : " <<  fn.c_str() );
   myFields.push( fn );   // remember it on the stack
   myChars = "";
   level++;

   //DebugLog (<<  "Level = "<< level);
    
   // level 1 ==> table
   // level 2 ==> row
   // level 3 ==> column

   if (level == 2)
      index ++;

}

///
void 
XmlBlobContentHandler::characters(const XMLCh* const chars,
                                  const unsigned int length)
{
   //    string& fn = myFields.top();   // get current field name
   string  val= StrX( chars ).localForm();
   myChars = val;

   //DebugLog(<<  "  ****XmlBlobContentHandler::characters = " << val << " len = " << length);

   if (level > 2)
   {
      myParsedContent->setBlobValue(index, val);
   }

   //DebugLog(<<  "**** end characters ");
}

///
void 
XmlBlobContentHandler::endElement(const XMLCh* const uri, 
                                  const XMLCh* const localname, 
                                  const XMLCh* const qname)
{
   string fn = StrX( qname ).localForm();
   level--;

   if (myChars.empty())
      myParsedContent->setBlobValue(index, "");

   //DebugLog( <<  "***** endElement qname = " << fn );

   myFields.pop();      

}

///
void 
XmlBlobContentHandler::endDocument()
{
   //DebugLog( <<  "endDocument() reached" );
   assert(myFields.empty());
}

///
void 
XmlBlobContentHandler::warning(const SAXParseException& e)
{
   DebugLog(<< "SAX Handler throws Warning. " 
            << StrX(e.getSystemId()).localForm() << ":" << e.getLineNumber() << ":" << e.getColumnNumber()
            << " " << StrX(e.getMessage()).localForm());


   string err = "Catching XML Exception: ";
   err += StrX(e.getMessage()).localForm();

   throw VNetworkException (err, __FILE__, __LINE__);
}

///
void 
XmlBlobContentHandler::error(const SAXParseException& e)
{
   DebugLog(<< "SAX Handler throws Error. " 
            << StrX(e.getSystemId()).localForm() << ":" << e.getLineNumber() << ":" << e.getColumnNumber()
            << " " << StrX(e.getMessage()).localForm());

   string err = "Catching XML Exception: ";
   err += StrX(e.getMessage()).localForm();

   throw VNetworkException(err, __FILE__, __LINE__);

}

///
void 
XmlBlobContentHandler::fatalError(const SAXParseException& e)
{
   DebugLog(<< "SAX Handler throws Fatal Error. " 
            << StrX(e.getSystemId()).localForm() << ":" << e.getLineNumber() << ":" << e.getColumnNumber()
            << " " << StrX(e.getMessage()).localForm());

   string err = "Catching XML Exception: ";
   err += StrX(e.getMessage()).localForm();

   throw VNetworkException(err, __FILE__, __LINE__);
}



// end of file
