#ifndef PSINTERFACE_HXX
#define PSINTERFACE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSInterface_hxx_Version ="$Id: PSInterface.hxx,v 1.1 2002/09/30 04:55:14 jason Exp $";

#include <string>
#include <stack>
#include <ace/SOCK_Connector.h>
#include <ace/Synch.h>

#include "NetworkAddress.h"
#include "VException.hxx"

namespace Epoisses {

/**
   Essentially, this class is a wrapper for two HostInterfaces. If one
   fails, the server will fail over and try the other.
*/

class PSInterface
{
   public:
      PSInterface(const char* myHostname, const int myPort);
      ~PSInterface();

      /// Return the address of the PServer currently in use
      const NetworkAddress getNetAddress()
      {
         return _na;
      }

      /// Reads an item from provision server
      string sendRequest(const string &httpMessage)
         throw (VException&);

   private:
      class Sender
      {
         public:
            Sender(const ACE_INET_Addr& addr);
            string sendRequest(const string &httpMessage)
               throw (VException&);

            void disconnect();
               
         private:
            ACE_SOCK_Stream _server;
            ACE_SOCK_Connector _connector;

            ACE_INET_Addr _addr;
            bool _disconnected;
      };

      static const unsigned int NumSenders;
      
      Sender* checkout();
      void checkin(Sender*);
      
      ACE_INET_Addr _address;
      NetworkAddress _na;
      stack< Sender* > _senders;
      ACE_Semaphore _sema;
      ACE_Thread_Mutex _mutex;
};

} // namespace Epoisses;

#endif // PSINTERFACE_HXX
