/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSListener_cxx_Version = "$Id: PSListener.cxx,v 1.1 2002/09/30 04:55:14 jason Exp $";

#include "PSListener.hxx"

#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <boost/lexical_cast.hpp>

#include "CacheManager.hxx"
#include "Lock.hxx"
#include "Logger.hxx"
#include "Mutex.hxx"
#include "PSIFCreator.hxx"
#include "SingletonHolder.hxx"
#include "VEnvVar.hxx"
#include "VNetworkException.hxx"
#include "TimeAccumulate.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::PROVC

using namespace Epoisses;
using namespace Vocal;

const int PSListener::FirstTryPort = 6006;

void
PSListener::thread()
{        
   // reactor created in another thread...
   _reactor.owner(ACE_OS::thr_self());
   while (true)
   {
      if ( isShutdown() == true )
      {
         return;
      }
      ACE_Time_Value timeout(1);
      _reactor.handle_events(timeout);
   }
}

PSListener::~PSListener()
{
   DebugLog(<<"Shutting down PSListener");
   shutdown();
   join();
}

PSListener::PSListener()
   : _listenerPort(0)
{}

int
PSListener::getListenPort() const
{
   // check/lock/check
   if (_listenerPort == 0)
   {
      static Threads::Mutex mutex;
      Threads::Lock lock(mutex);
      if (_listenerPort == 0)
      {
         PSListener* mutableThis = const_cast<PSListener*>(this);
   
         {  
            mutableThis->_listenerPort = PSListener::FirstTryPort;
            bool foundPort = false;
            while ( !foundPort && (_listenerPort < PSListener::FirstTryPort + 100))
            {
               if (mutableThis->_acceptor.open(ACE_INET_Addr(_listenerPort), &mutableThis->_reactor) != -1)
               {
                  foundPort = true;
               }
               else
               {
                  InfoLog(<< "Tried port " << _listenerPort);
                  // could not listen here, so retry
                  mutableThis->_listenerPort++;
               }
            }
            
            if (foundPort)
            {
               DebugLog(<<  "Successfully opened listener port " << _listenerPort << ". Starting the thread");
            } 
            else 
            {
               ErrLog(<< "Failed to open any listener ports -- Fatal Error");
               throw VNetworkException("Failed to open listener port",
                                       __FILE__, __LINE__);
            }
            mutableThis->run();
         }
      }
   }
   return _listenerPort;
}

