// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
#include "ProvisioningInterface.hxx"

#include <strstream>
#include <boost/lexical_cast.hpp>

#include "cppunit/TestCase.h"
#include "cppunit/TestCaller.h"
#include "cppunit/TestSuite.h"
#include "cppunit/TextTestRunner.h"
#include "cppunit/TextTestResult.h"

#include "CommandLineParser.hxx"
#include "Logger.hxx"
#include "TimeGuard.hxx"
#include "Users.hxx"
#include "VDbException.hxx"
#include "cpLog.h"

#define VOCAL_SUBSYSTEM Subsystem::TEST

using namespace CppUnit;
using namespace Epoisses;
using namespace Vocal;


/// Base for customized test case classes.
class ResultStringTestCase : public TestCase
{
   public:
      virtual ~ResultStringTestCase()
      {}
      
      virtual void setUp()
      {
         resultString.clear();
      }

      virtual void tearDown()
      {
         resultString.freeze(0);
      }

   protected:
      strstream resultString;
      
};


class PS1 : public ResultStringTestCase
{
   public:
      
      void doit()
      {
         User::Db users;
         ProvisioningInterface& pif = ProvisioningInterface::make();
         pif.deleteRecordsInDomain(users, Filter::None);

         my_thread1.spawn(thread1, 0);
         my_thread2.spawn(thread2, 0);
         my_thread3.spawn(thread3, 0);
         my_thread4.spawn(thread4, 0);
         
         my_thread1.join();
         my_thread2.join();
         my_thread3.join();
         my_thread4.join();

         // cleanup 
         pif.deleteRecordsInDomain(users, Filter::None);
      }

      static void* thread1(void*)
      {
         ProvisioningInterface& pif = ProvisioningInterface::make();
         User::Db users;
         Sptr<PSDBResult> result;
         
         pif.addRecordInDomain(users,
                               (users.firstname = "david",
                                users.lastname = "thread1",
                                users.expired = true,
                                users.count = 7,
                                users.credits = 1000,
                                users.speed = 0.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Enlai",
                                users.lastname = "thread1",
                                users.expired = false,
                                users.count = 19,
                                users.credits = 10000,
                                users.speed = 1.0));

         // get the records associated with Shannon
         result = pif.getRecordsInDomain(users, Filter::None, users.userid);

         pif.addRecordInDomain(users,
                               (users.firstname = "Wei",
                                users.lastname = "thread1",
                                users.expired = false,
                                users.count = 27,
                                users.credits = 12000,
                                users.speed = 1.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Jason",
                                users.lastname = "thread1",
                                users.expired = true,
                                users.count = 71,
                                users.credits = 10000,
                                users.speed = 1.05));

         pif.addRecordInDomain(users,
                               (users.firstname = "Ludo",
                                users.lastname = "thread1",
                                users.expired = false,
                                users.count = 37,
                                users.credits = 10001,
                                users.speed = 0.2) );

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);

         pif.addRecordInDomain(users,
                               (users.firstname = "Speedy",
                                users.lastname = "thread1",
                                users.expired = false,
                                users.count = 0,
                                users.credits = 0,
                                users.speed = 0.00 ));

         pif.addRecordInDomain(users,
                               (users.firstname = "Belle",
                                users.lastname = "thread1",
                                users.expired = false,
                                users.count = 7,
                                users.credits = 10000,
                                users.speed = 2.00 ) );
         pif.addRecordInDomain(users,
                               (users.firstname = "Oort",
                                users.lastname = "thread1",
                                users.expired = true,
                                users.count = 17,
                                users.credits = 0,
                                users.speed = 2.02));
         pif.addRecordInDomain(users,
                               (users.firstname = "Isaac",
                                users.lastname = "thread1",
                                users.expired = true,
                                users.count = 77,
                                users.credits = 1000,
                                users.speed = 1.6));

         pif.addRecordInDomain(users,
                               (users.firstname = "Sam",
                                users.lastname = "thread1",
                                users.expired = true,
                                users.count = 777,
                                users.credits = 1000,
                                users.speed = 1.82));


         return 0;
      }
      static void* thread2(void*)
      {
         ProvisioningInterface& pif = ProvisioningInterface::make();
         User::Db users;
         Sptr<PSDBResult> result;

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "david",
                                users.lastname = "thread2",
                                users.expired = true,
                                users.count = 7,
                                users.credits = 1000,
                                users.speed = 0.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Enlai",
                                users.lastname = "thread2",
                                users.expired = false,
                                users.count = 19,
                                users.credits = 10000,
                                users.speed = 1.0));

         pif.addRecordInDomain(users,
                               (users.firstname = "Wei",
                                users.lastname = "thread2",
                                users.expired = false,
                                users.count = 27,
                                users.credits = 12000,
                                users.speed = 1.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Jason",
                                users.lastname = "thread2",
                                users.expired = true,
                                users.count = 71,
                                users.credits = 10000,
                                users.speed = 1.05));

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);

         pif.addRecordInDomain(users,
                               (users.firstname = "Ludo",
                                users.lastname = "thread2",
                                users.expired = false,
                                users.count = 37,
                                users.credits = 10001,
                                users.speed = 0.2) );

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);

         pif.addRecordInDomain(users,
                               (users.firstname = "Speedy",
                                users.lastname = "thread2",
                                users.expired = false,
                                users.count = 0,
                                users.credits = 0,
                                users.speed = 0.00 ));

         pif.addRecordInDomain(users,
                               (users.firstname = "Belle",
                                users.lastname = "thread2",
                                users.expired = false,
                                users.count = 7,
                                users.credits = 10000,
                                users.speed = 2.00 ) );
         pif.addRecordInDomain(users,
                               (users.firstname = "Oort",
                                users.lastname = "thread2",
                                users.expired = true,
                                users.count = 17,
                                users.credits = 0,
                                users.speed = 2.02));

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "Isaac",
                                users.lastname = "thread2",
                                users.expired = true,
                                users.count = 77,
                                users.credits = 1000,
                                users.speed = 1.6));

         pif.addRecordInDomain(users,
                               (users.firstname = "Sam",
                                users.lastname = "thread2",
                                users.expired = true,
                                users.count = 777,
                                users.credits = 1000,
                                users.speed = 1.82));

         return 0;
      }
      static void* thread3(void*)
      {
         User::Db users;
         ProvisioningInterface& pif = ProvisioningInterface::make();
         Sptr<PSDBResult> result;

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "david",
                                users.lastname = "thread3",
                                users.expired = true,
                                users.count = 7,
                                users.credits = 1000,
                                users.speed = 0.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Enlai",
                                users.lastname = "thread3",
                                users.expired = false,
                                users.count = 19,
                                users.credits = 10000,
                                users.speed = 1.0));

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "Wei",
                                users.lastname = "thread3",
                                users.expired = false,
                                users.count = 27,
                                users.credits = 12000,
                                users.speed = 1.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Jason",
                                users.lastname = "thread3",
                                users.expired = true,
                                users.count = 71,
                                users.credits = 10000,
                                users.speed = 1.05));

         pif.addRecordInDomain(users,
                               (users.firstname = "Ludo",
                                users.lastname = "thread3",
                                users.expired = false,
                                users.count = 37,
                                users.credits = 10001,
                                users.speed = 0.2) );

         pif.addRecordInDomain(users,
                               (users.firstname = "Speedy",
                                users.lastname = "thread3",
                                users.expired = false,
                                users.count = 0,
                                users.credits = 0,
                                users.speed = 0.00 ));

         pif.addRecordInDomain(users,
                               (users.firstname = "Belle",
                                users.lastname = "thread3",
                                users.expired = false,
                                users.count = 7,
                                users.credits = 10000,
                                users.speed = 2.00 ) );
         pif.addRecordInDomain(users,
                               (users.firstname = "Oort",
                                users.lastname = "thread3",
                                users.expired = true,
                                users.count = 17,
                                users.credits = 0,
                                users.speed = 2.02));
         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "Isaac",
                                users.lastname = "thread3",
                                users.expired = true,
                                users.count = 77,
                                users.credits = 1000,
                                users.speed = 1.6));

         pif.addRecordInDomain(users,
                               (users.firstname = "Sam",
                                users.lastname = "thread3",
                                users.expired = true,
                                users.count = 777,
                                users.credits = 1000,
                                users.speed = 1.82));

         return 0;
      }
      static void* thread4(void*)
      {
         User::Db users;
         ProvisioningInterface& pif = ProvisioningInterface::make();
         Sptr<PSDBResult> result;

         pif.addRecordInDomain(users,
                               (users.firstname = "david",
                                users.lastname = "thread4",
                                users.expired = true,
                                users.count = 7,
                                users.credits = 1000,
                                users.speed = 0.02));

         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "Enlai",
                                users.lastname = "thread4",
                                users.expired = false,
                                users.count = 19,
                                users.credits = 10000,
                                users.speed = 1.0));

         pif.addRecordInDomain(users,
                               (users.firstname = "Wei",
                                users.lastname = "thread4",
                                users.expired = false,
                                users.count = 27,
                                users.credits = 12000,
                                users.speed = 1.02));

         pif.addRecordInDomain(users,
                               (users.firstname = "Jason",
                                users.lastname = "thread4",
                                users.expired = true,
                                users.count = 71,
                                users.credits = 10000,
                                users.speed = 1.05));

         pif.addRecordInDomain(users,
                               (users.firstname = "Ludo",
                                users.lastname = "thread4",
                                users.expired = false,
                                users.count = 37,
                                users.credits = 10001,
                                users.speed = 0.2) );

         pif.addRecordInDomain(users,
                               (users.firstname = "Speedy",
                                users.lastname = "thread4",
                                users.expired = false,
                                users.count = 0,
                                users.credits = 0,
                                users.speed = 0.00 ));

         pif.addRecordInDomain(users,
                               (users.firstname = "Belle",
                                users.lastname = "thread4",
                                users.expired = false,
                                users.count = 7,
                                users.credits = 10000,
                                users.speed = 2.00 ) );
         result = pif.getRecordsInDomain(users, Filter::None, users.userid);
         pif.addRecordInDomain(users,
                               (users.firstname = "Oort",
                                users.lastname = "thread4",
                                users.expired = true,
                                users.count = 17,
                                users.credits = 0,
                                users.speed = 2.02));
         pif.addRecordInDomain(users,
                               (users.firstname = "Isaac",
                                users.lastname = "thread4",
                                users.expired = true,
                                users.count = 77,
                                users.credits = 1000,
                                users.speed = 1.6));
         result = pif.getRecordsInDomain(users, Filter::None, users.userid);

         pif.addRecordInDomain(users,
                               (users.firstname = "Sam",
                                users.lastname = "thread4",
                                users.expired = true,
                                users.count = 777,
                                users.credits = 1000,
                                users.speed = 1.82));

         return 0;
      }
      
   private:
      static ProvisioningInterface* pif;
      
      VThread my_thread1;
      VThread my_thread2;
      VThread my_thread3;
      VThread my_thread4;
};



#define psTEST(_method) suite->addTest(new TestCaller<PS1>(#_method, &PS1::_method))

int
main(int argc, char** argv)
{
   CommandLineParser args(argc, argv);
   Log::initialize(args.logType(), args.logLevel(), argv[0]);
   ProvisioningInterface::make("localhost", 6005);

   TestSuite* suite = new TestSuite;
   assert(suite->countTestCases() == 0);
   
   TextTestResult result;
   psTEST(doit);

   TextTestRunner runner;
   runner.addTest(suite);
   runner.run();

   cerr << "Finished tests" << endl;
   return 0;
}
