// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
#include "ProvisioningInterface.hxx"
#include "PSIFCreator.hxx"

#include <strstream>
#include <boost/lexical_cast.hpp>

#include "cppunit/TestCase.h"
#include "cppunit/TestCaller.h"
#include "cppunit/TestSuite.h"
#include "cppunit/TextTestRunner.h"
#include "cppunit/TextTestResult.h"
#include "cpLog.h"
#include "Users.hxx"
#include "VDbException.hxx"
#include "TimeGuard.hxx"
#include "Logger.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::TEST

using namespace CppUnit;
using namespace Vocal;
using namespace Epoisses;

const unsigned int s = 100;
const string prefix = "Oort";

class LargeCacheTest : public TestCase
{
   public:
      LargeCacheTest()
         : pif(ProvisioningInterface::make())
      {
      }
      

      virtual ~LargeCacheTest()
      {}
      
      virtual void setUp()
      {
      }

      virtual void tearDown()
      {
      }

      void testLargeString()
      {
         string foo = "base";
         for (int i = 0; i < 20; i++)
         {
            foo = foo + foo;
            cerr << foo.size() << endl;
         }
         
         CPPUNIT_ASSERT(foo.size() > 10000);
      }
      
      void empty()
      {
         pif.deleteRecordsInDomain(users, Filter::None);
      }
      
      void testPopulate()
      {
         empty();
         
         vector<string> aliases2;

         aliases2.push_back("a.1");
         aliases2.push_back("a.2");
         aliases2.push_back("a.3");

         for (int i = 0; i < s; i++)
         {
            pif.addRecordInDomain(users,
                                  (//users.userid = boost::lexical_cast<string>(i) + prefix,
                                   users.firstname = "Belle",
                                   users.lastname = "Shannon",
                                   users.expired = true,
                                   users.count = 7,
                                   users.credits = 1000,
                                   users.speed = 0.02,
                                   users.aliases = aliases2));
            cerr << ".";
         }

         InfoLog(<< "Done adding ");
         //CPPUNIT_ASSERT(UserCache::Instance().size() == s);
         //CPPUNIT_ASSERT(UserCache::Instance().exists(boost::lexical_cast<string>((int)(.7*s)) + prefix));
      }

      void testLoad()
      {
         CPPUNIT_ASSERT(UserCache::Instance().exists(boost::lexical_cast<string>((int)(.7*s)) + prefix));
      }
      
      User::Db users;
      ProvisioningInterface &pif;
};

int
main(int argc, char** argv)
{
   Log::initialize(Log::COUT, Log::INFO, "testLargeCache");

   ProvisioningInterface::make("localhost", 6005);

   UserCache::Instance();
   
   TextTestResult result;

   TestSuite* suite = new TestSuite;

   suite->addTest(new TestCaller<LargeCacheTest>("testPopulate",
                                                 &LargeCacheTest::testPopulate));
/*
   suite->addTest(new TestCaller<LargeCacheTest>("testLoad",
                                                 &LargeCacheTest::testLoad));
*/
/*
   suite->addTest(new TestCaller<LargeCacheTest>("empty",
                                                 &LargeCacheTest::empty));
*/

   TextTestRunner runner;
   runner.addTest(suite);
   runner.run();
   return 0;
}
