#ifndef CONNECTIONMANAGER_HXX
#define CONNECTIONMANAGER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "Mutex.hxx"
#include "Lock.hxx"
#include "VSemaphore.hxx"
#include <list>
#include <loki/Singleton.h>
#include "DBConnection.hxx"

namespace Epoisses 
{

/// Forward Declare DBConnection class
class DBConnection;

class ConnectionManagerImpl;

class ConnectionManager : public Loki::SingletonHolder<ConnectionManagerImpl>
{
   public:
      static DBResultset* DBSelect (const string& Domain, 
                                    const string& SearchFilter, 
                                    const FieldListtype& FieldList, 
                                    const Ordertype& Order, 
                                    const string &OrderField)
      {
         return ConnectionManager::Connection(Domain).DBSelect(Domain, SearchFilter, FieldList, Order, OrderField);
      }
            
      static DBResultset* DBSelect (const string& Domain, 
                                    const string& SearchFilter, 
                                    const FieldListtype& FieldList)
      {
         return ConnectionManager::Connection(Domain).DBSelect(Domain, SearchFilter, FieldList);
      }

      static DBResultset* DBSelect(const string& Domain, 
                                   const string& SearchFilter)
      {
         return ConnectionManager::Connection(Domain).DBSelect(Domain, SearchFilter);
      }
            
      static DBResultset* DBSelect (const string& Domain,
                                    const string& SearchFilter,
                                    const string& start,
                                    const string& end,
                                    const FieldListtype& FieldList)
      {
         return ConnectionManager::Connection(Domain).DBSelect(Domain, SearchFilter, start, end, FieldList);
      }
            
      static DBResultset* DBSelect( const string& Domain,
                                    const string& SearchFilter,
                                    const string& start,
                                    const string& end,
                                    const Ordertype& Order,
                                    const string &OrderField,
                                    const FieldListtype& FieldList)
      {
         return ConnectionManager::Connection(Domain).DBSelect(Domain, SearchFilter, start, end, Order, OrderField, FieldList);
      }
            
      static DBResultset* DBRetrieveByOid(const string& Domain, 
                                          uint oid,
                                          const FieldListtype& FieldList)
      {
         return ConnectionManager::Connection(Domain).DBRetrieveByOid(Domain, oid, FieldList);
      }  

      static string DBPrimaryKey ( const string& Domain) 

      {
         return ConnectionManager::Connection(Domain).DBPrimaryKey(Domain);
      }

      static string DBParent(const string& domain)
      {
         return ConnectionManager::Connection(domain).DBParent(domain);
      }

      static string DBRoot(const string& domain)
      {
         return ConnectionManager::Connection(domain).DBRoot(domain);
      }

      static ResultTabletype DBDescribe (const string& Domain) 
      {
         return ConnectionManager::Connection(Domain).DBDescribe(Domain);
      }

      static ResultRecordtype DBDescribe (void) 
      {
         // not implemented
         assert(0);
         ResultRecordtype foo;
         return foo;
      }

      static void DBUpdate (const string& Domain, 
                            const string& SearchFilter, 
                            const FieldValuetype& FieldValueSet) 
      {
         return ConnectionManager::Connection(Domain).DBUpdate(Domain, SearchFilter, FieldValueSet);
      }

      static void DBUpdate (const string& Domain, DBResultset* Resultset, 
                            const FieldValuetype& FieldValueSet) 
      {
         return ConnectionManager::Connection(Domain).DBUpdate(Domain, Resultset, FieldValueSet);
      }

      static int DBInsert (const string& Domain, 
                           const FieldValuetype& FieldValueSet) 
      {
         return ConnectionManager::Connection(Domain).DBInsert(Domain, FieldValueSet);
      }

      static void DBDelete (const string& Domain, 
                            const string& SearchFilter) 
      {
         return ConnectionManager::Connection(Domain).DBDelete(Domain, SearchFilter);
      }

      static void DBDelete (const string& Domain, 
                            DBResultset* Resultset) 
      {
         return ConnectionManager::Connection(Domain).DBDelete(Domain, Resultset);
      }

      static void DBCreate (const string& Domain, 
                            const FieldListtype& FieldsList, 
                            const string& PrimaryKey,
                            bool ExistenceCheck ) 
      {
         return ConnectionManager::Connection(Domain).DBCreate(Domain, FieldsList, PrimaryKey, ExistenceCheck);
      }

      static void DBCreate (const string& Domain, 
                            const FieldListtype& FieldsList, 
                            const FieldListDatatype& Datatypelist, 
                            const string& PrimaryKey,
                            bool ExistenceCheck) 
      {
         return ConnectionManager::Connection(Domain).DBCreate(Domain, FieldsList, Datatypelist, PrimaryKey, ExistenceCheck);
      }

      static void DBCreate(const string& domain, 
                           const string& parentDomain,
                           const FieldListtype& fieldsList, 
                           const FieldListDatatype& datatypelist, 
                           bool existenceCheck)
      {
         return ConnectionManager::Connection(domain).DBCreate(domain, parentDomain, fieldsList, datatypelist, existenceCheck);
      }

      static void DBAlter(const string& Domain, 
                          const string& Fieldname) 
      {
         return ConnectionManager::Connection(Domain).DBAlter(Domain, Fieldname);
      }
         
      static void DBAlter(const string& Domain, const string& Fieldname,
                          const Datatype Newtype) 
      {
         return ConnectionManager::Connection(Domain).DBAlter(Domain, Fieldname, Newtype);
      }

      static void DBDrop(const string& Domain, 
                         bool ExistenceCheck ) 
      {
         return ConnectionManager::Connection(Domain).DBDrop(Domain, ExistenceCheck);
      }

      static bool DBVerify(const string& Domain) 
      {
         return ConnectionManager::Connection(Domain).DBVerify(Domain);
      }

      class Connection : public DBConnection
      {
         public:
            Connection(const string& domain);

            ~Connection();

            // all in-line delegation
            virtual DBResultset* DBSelect (const string& Domain, 
                                           const string& SearchFilter, 
                                           const FieldListtype& FieldList, 
                                           const Ordertype& Order, 
                                           const string &OrderField)
            {
               return dbconn->DBSelect(Domain, SearchFilter, FieldList, Order, OrderField);
            }
            

            /// Return a resultset object
            virtual DBResultset* DBSelect (const string& Domain, 
                                           const string& SearchFilter, 
                                           const FieldListtype& FieldList)
            {
               return dbconn->DBSelect(Domain, SearchFilter, FieldList);
            }

            virtual DBResultset* DBSelect(const string& Domain, 
                                          const string& SearchFilter)
            {
               return dbconn->DBSelect(Domain, SearchFilter);
            }
            
            virtual DBResultset* DBSelect (const string& Domain,
                                           const string& SearchFilter,
                                           const string& start,
                                           const string& end,
                                           const FieldListtype& FieldList)
            {
               return dbconn->DBSelect(Domain, SearchFilter, start, end, FieldList);
            }
            
            /// Return a resultset object
            virtual DBResultset* DBSelect( const string& Domain,
                                           const string& SearchFilter,
                                           const string& start,
                                           const string& end,
                                           const Ordertype& Order,
                                           const string &OrderField,
                                           const FieldListtype& FieldList)
            {
               return dbconn->DBSelect(Domain, SearchFilter, start, end, Order, OrderField, FieldList);
            }
            
            virtual DBResultset* DBRetrieveByOid(const string& Domain, 
                                                 uint oid,
                                                 const FieldListtype& FieldList)
            {
               return dbconn->DBRetrieveByOid(Domain, oid, FieldList);
            }
               

            /// Return a string containing primary key of Domain
            virtual string DBPrimaryKey ( const string& Domain) 

            {
               return dbconn->DBPrimaryKey(Domain);
            }

            /// Return the parent of the domain
            virtual string DBParent(const string& domain)
            {
               return dbconn->DBParent(domain);
            }

            /// Return the root of the domain
            virtual string DBRoot(const string& domain)
            {
               return dbconn->DBRoot(domain);
            }

            /// Return a table describing fields/types/keys in Domain
            virtual ResultTabletype DBDescribe (const string& Domain) 
            {
               return dbconn->DBDescribe(Domain);
            }

            /// Return a list of domains in database
            virtual ResultRecordtype DBDescribe (void) 
            {
               return dbconn->DBDescribe();
            }

            /// Update Domain, based on searchfilter
            virtual void DBUpdate (const string& Domain, 
                                   const string& SearchFilter, 
                                   const FieldValuetype& FieldValueSet) 
            {
               return dbconn->DBUpdate(Domain, SearchFilter, FieldValueSet);
            }

            /// Update Domain, based on previous resultset
            virtual void DBUpdate (const string& Domain, DBResultset* Resultset, 
                                   const FieldValuetype& FieldValueSet) 
            {
               return dbconn->DBUpdate(Domain, Resultset, FieldValueSet);
            }

            /// Insert record
            virtual int DBInsert (const string& Domain, 
                                   const FieldValuetype& FieldValueSet) 
            {
               return dbconn->DBInsert(Domain, FieldValueSet);
            }

            /// Delete rows based on a seachfilter
            virtual void DBDelete (const string& Domain, 
                                   const string& SearchFilter) 
            {
               return dbconn->DBDelete(Domain, SearchFilter);
            }

            /// Delete rows based on a previous resultset
            virtual void DBDelete (const string& Domain, 
                                   DBResultset* Resultset) 
            {
               return dbconn->DBDelete(Domain, Resultset);
            }

            /// Create a new domain, use STRINGTYPE for all columns
            /// with or without checking current existence
            virtual void DBCreate (const string& Domain, 
                                   const FieldListtype& FieldsList, 
                                   const string& PrimaryKey,
                                   bool ExistenceCheck ) 
            {
               return dbconn->DBCreate(Domain, FieldsList, PrimaryKey, ExistenceCheck);
            }

            /// Create a new domain, specify types for each column
            /// with or without checking current existence
            virtual void DBCreate (const string& Domain, 
                                   const FieldListtype& FieldsList, 
                                   const FieldListDatatype& Datatypelist, 
                                   const string& PrimaryKey,
                                   bool ExistenceCheck) 
            {
               return dbconn->DBCreate(Domain, FieldsList, Datatypelist, PrimaryKey, ExistenceCheck);
            }

            /// Create a new domain that inherits from an existing domain,
            /// with or without checking current existence
            // Note: primary key is inherited and may not be specified here
            virtual void DBCreate(const string& domain, 
                                  const string& parentDomain,
                                  const FieldListtype& fieldsList, 
                                  const FieldListDatatype& datatypelist, 
                                  bool existenceCheck)
            {
               return dbconn->DBCreate(domain, parentDomain, fieldsList, datatypelist, existenceCheck);
            }

            /// Alter a domain, drop field
            virtual void DBAlter(const string& Domain, 
                                 const string& Fieldname) 
            {
               return dbconn->DBAlter(Domain, Fieldname);
            }
         
            /// Alter a domain, change a type of a field
            virtual void DBAlter(const string& Domain, const string& Fieldname,
                                 const Datatype Newtype) 
            {
               return dbconn->DBAlter(Domain, Fieldname, Newtype);
            }

            /// Drop a domain, with or without checking current existence
            virtual void DBDrop(const string& Domain, 
                                bool ExistenceCheck ) 
            {
               return dbconn->DBDrop(Domain, ExistenceCheck);
            }

            /// Returns true if Domain exists, false otherwise
            virtual bool DBVerify(const string& Domain) 
            {
               return dbconn->DBVerify(Domain);
            }

         private:
            // no value semantics
            Connection(const Connection&);
            Connection& operator=(const Connection&);
            DBConnection* dbconn;
      };
};

/// Connection Manager implementation -- produce connnection on demand
class ConnectionManagerImpl
{
      friend class Loki::SingletonHolder<ConnectionManagerImpl>;

   public:
      /// Constructor
      ConnectionManagerImpl();

      /// Destructor
      virtual ~ConnectionManagerImpl();

      /// loan a connection to the client
      DBConnection* 
      checkoutConnection();

      /// get the connection back
      void checkInConnection(DBConnection* dbconn);

   private:
      /// number of connections to maintain
      static const unsigned int ConnectionCount = 10;
      /// cache of connections for threaded use
      static list<DBConnection*> ConnectionCache;
      /// mutex to control cache access
      static Vocal::Threads::Mutex CacheMutex;
      /// semaphore to control cache access
      static VSemaphore CacheSemaphore;
};

} // namespace Epoisses

#endif
