// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//


#include <iostream>
#include "FastDBResultSet.hxx"
#include "VDbException.hxx"

using namespace std;
using namespace Epoisses;

// SQLResetset constructor, initialize attributes
FastDBResultset::FastDBResultset(const string& searchfilter, const FieldListtype& fieldlist, ResultTabletype* values)
   : _searchFilter(searchfilter), 
     _fieldList(fieldlist),
     _values(values)
{};

// SQLResetset destructor, free attributes 
FastDBResultset::~FastDBResultset()
{
   delete _values;
};

// GetRecords[offset] - returns the record at offset
ResultRecordtype
FastDBResultset::operator[](unsigned short offset) const
{
   return _values->at(offset);
}

// GetRecord(offset) - returns the record at offset
ResultRecordtype
FastDBResultset::GetRecord(unsigned short offset) const
{
   return _values->at(offset);
}

// GetRecords(offset1, offset2) - returns all records between offset1
// and offset2
ResultTabletype
FastDBResultset::GetRecords(unsigned short offset1, unsigned short offset2) const
{
   ResultTabletype returnResult;
   
   // Check offsets are within range of results
   if ((offset1 > offset2) || (offset1 > _values->size()) 
       || (offset2 > _values->size()))
   {
      throw VDbException("Record Offsets out of range", __FILE__, __LINE__);
   }

   for (int recordnumber = offset1; recordnumber <= offset2; recordnumber++)
   {
      returnResult.push_back(_values->at(recordnumber));
   }
   return returnResult;
};
 
// GetRecords - returns all records from a search
ResultTabletype
FastDBResultset::GetRecords() const
{
   return *_values;
}

// GetRecordCount- Return Record count
unsigned int
FastDBResultset::GetRecordCount(void) const
{
   return _values->size();
};

// GetFields - Return fields on a resultset
FieldListtype
FastDBResultset::GetFields(void) const
{
   return _fieldList;
}

const string&
FastDBResultset::GetSearchFilter(void) const
{
   return _searchFilter;
};
