#ifndef PGSQLRESULTSET_HXX
#define PGSQLRESULTSET_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



#if defined(__svr4__) || defined(__SUNPRO_CC)
    #include <libpq-fe.h>
#else
    #include <libpq-fe.h>
#endif

#include <string>
#include <vector>
#include "DBResultset.hxx"

namespace Epoisses {
   
///
class PGSQLResultset : public DBResultset
{
    private:
        /// Constructor-
        PGSQLResultset(PGresult* result, const string& searchfilter);

        /// Destructor-
        virtual ~PGSQLResultset();

        /// Return all records from a query
        virtual ResultTabletype GetRecords(void) const;

        /// Return records between offset1, and offset2
        virtual ResultTabletype GetRecords(
                unsigned short offset1, 
                unsigned short offset2) const;

        /// Return the record at [offset]
        virtual ResultRecordtype operator[](
                unsigned short offset) const;

        /// Return the record at offset
        virtual ResultRecordtype GetRecord(
                unsigned short offset) const;

        /// Return field list from query
        virtual FieldListtype GetFields(void) const;

        /// Return number of records in query
        virtual unsigned int GetRecordCount(void) const;

        /// Return serach filter from query
        virtual const string& GetSearchFilter(void) const;

        /// PgSQL API Result 
        PGresult* myPgSQLresultP;
       
        /// Store the search filter in case the resultset object is
        /// used as input to DBUpdate() or DBDelete()
        const string mySearchFilter;

        /// STL vector of strings representing fieldlist
        FieldListtype myFieldList;

        /// Number of records in resultset
        int myRecordCount;

    public:
        /// The only class that needs access
        friend class PGSQLConnection;
};
 
} // namespace Epoisses

#endif
