#ifndef ParsedDef_hxx
#define ParsedDef_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "DBConnection.hxx"
#include "VBadDataException.hxx"

namespace Epoisses {
   
class DifferenceSet;

class ParsedDef {

public:
    
    ParsedDef() {}
    ~ParsedDef() {}

    // accessors
    string getDomain() const { return myDomain; }
    const FieldListtype & getFieldList() const { return myFieldList; }
    const FieldListDatatype & getFieldTypes() const { return myFieldTypes; }
    string getPrimaryKey() const { return myPrimaryKey; } 
    string getParent() const { return myParent; }
    
    // setters
    void addField(string newField) throw (VBadDataException &); 
    void addFieldType(const Datatype newType) throw (VBadDataException &);

    void setPrimaryKey(const string &newKey) 
    { 
       myPrimaryKey = newKey; 
       transform(myPrimaryKey.begin(),myPrimaryKey.end(),myPrimaryKey.begin(), ::tolower);
    }
 
    void setDomain(const string &newDomain) 
    { 
       myDomain = newDomain; 
       transform(myDomain.begin(),myDomain.end(),myDomain.begin(), ::tolower);
    }

    void setParent(const string &newParent) 
    { 
       myParent = newParent; 
       transform(myParent.begin(),myParent.end(),myParent.begin(), ::tolower);
    }

    const DifferenceSet compareOld (const ParsedDef &oldDef);
    
protected:

    FieldListtype myFieldList;
    FieldListDatatype myFieldTypes;
    string myPrimaryKey;
    string myDomain;
    string myParent;

};


class DifferenceSet {

public:

    DifferenceSet() { myChangedPrimary = false; }
    ~DifferenceSet() {}

    FieldListtype getNewFields() const { return myNewFields; }
    FieldListtype getDelFields() const { return myDelFields; }
    bool changedPrimary() const { return myChangedPrimary; }

    void addNewField(const string &newField) 
    {
       myNewFields.push_back(newField);
       transform(myNewFields.back().begin(), myNewFields.back().end(), myNewFields.back().begin(), ::tolower);
    }      
      
    void addDelField(const string &delField) 
    {
       myDelFields.push_back(delField); 
       transform(myDelFields.back().begin(),myDelFields.back().end(),myDelFields.back().begin(), ::tolower);
    }
      
    void setChangedPrimary() { myChangedPrimary = true; }
    
protected:

    bool myChangedPrimary;
    FieldListtype myNewFields, myDelFields;

};

} // namespace Epoisses

#endif // ParsedDef_hxx
