#include <boost/lexical_cast.hpp>
#include <boost/random.hpp>

#include <cppunit/TestCaller.h>
#include <cppunit/TestCase.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TextTestResult.h>
#include <cppunit/TextTestRunner.h>
#include <strstream>
#include "CommandLineParser.hxx"
#include "ConnectionManager.hxx"
#include "DBResultset.hxx"
#include "Logger.hxx"
#include "VDbException.hxx"
#include "ThreadIf.hxx"



#define VOCAL_SUBSYSTEM Vocal::Subsystem::TEST

using namespace CppUnit;
using namespace Vocal;
using namespace Epoisses;


static const int PARALLEL = 10;

class InsertAndDelete : public ThreadIf
{
   public:
      InsertAndDelete(string groupId)
         : _groupId(groupId)
      {}
      
      virtual void thread()
      {
         FieldValuetype rec;
 
         for (int i=0; i < 500; i++)
         {
            if (_groupId == "groupid1")
            {
               InfoLog(<< "S3");
            }
            rec["groupid"] = "'" + _groupId + "'";
            rec["contact"] = "'" + _groupId + "@foo.com'";
            rec["qvalue"] = "-1";
            rec["cseq"] = "1";
            rec["expires"] = "3600";
            rec["callid"] = "'" + _groupId + "callid@foo.com'";
         
            int k = ConnectionManager::DBInsert("registrationbindings", rec);
            DebugLog( << "Inserted Record with oid: " << k);
            
            
            string foo[] = { "bindingid", "groupid", "contact", "qvalue", "callid", "cseq", "expires" };
            FieldListtype flist(foo, foo+7);
            
            DBResultset* res = ConnectionManager::DBSelect("registrationbindings", "groupid='" + _groupId + "'", flist);
         
            ResultTabletype rows = res->GetRecords();
            
            delete res;
            
            DebugLog(<< Inserter(rows));
            
            if(rows[0][1] != _groupId)
            {
               InfoLog(<< "Expected groupId:[" << _groupId << "], got [" << rows[0][1] << "]");
            }

            CPPUNIT_ASSERT(rows[0][1] == _groupId );
            CPPUNIT_ASSERT(rows[0][2] == _groupId + "@foo.com");
            CPPUNIT_ASSERT(rows[0][3] == "-1");
            CPPUNIT_ASSERT(rows[0][4] == _groupId + "callid@foo.com");
            CPPUNIT_ASSERT(rows[0][5] == "1");
            CPPUNIT_ASSERT(rows[0][6] == "3600" );
            
            ConnectionManager::DBDelete("registrationbindings", "groupid='wombat'");
            
            DBResultset* empty = ConnectionManager::DBSelect("registrationbindings", "groupid='wombat'", flist);
            
            CPPUNIT_ASSERT(empty->GetRecordCount() == 0);         
         }
      }
   private:
      string _groupId;
};


class FastDbTest : public TestCase
{
   public:

      FastDbTest()
      {}
        
      virtual ~FastDbTest()
      {}

      virtual void setUp()
      {
      }

      virtual void tearDown()
      {
      }
      
      class RowComparitor
      {
         public:
            bool operator()(const vector<string>& r1,
                            const vector<string>& r2)
            {
               return r1[0] < r2[0];
            }
      };


      void testParallelActivity()
      {
         list<InsertAndDelete*> workers;
         for (int i=0; i < PARALLEL; i++)
         {
            workers.push_back(new InsertAndDelete("groupid" + boost::lexical_cast<string>(i)));
            workers.back()->run();
         }
         while (!workers.empty())
         {
            workers.front()->join();
            delete workers.front();
            workers.pop_front();
         }
      }
};


#define TEST(_method) \
   suite->addTest(new TestCaller<FastDbTest>(#_method, \
                                             &FastDbTest::_method))


int main(int argc, char** argv)
{
   CommandLineParser args(argc, argv);
   Log::initialize(args.logType(), args.logLevel(), argv[0]);
   TestSuite* suite = new TestSuite;

   TEST(testParallelActivity);

   TextTestResult result;
   TextTestRunner runner;
   runner.addTest(suite);
   runner.run();
   DebugLog(<< "Finished");

   return 0;
}
