// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

// $Id: ClientHandler.hxx,v 1.1 2002/09/30 04:55:17 jason Exp $

#ifndef CLIENT_HANDLER_H
#define CLIENT_HANDLER_H

#include <ace/Svc_Handler.h>
#include <ace/SOCK_Stream.h>

#include "ClientAcceptor.hxx"
#include "Logger.hxx"

#define VOCAL_SUBSYSTEM Vocal::Subsystem::UTIL

namespace Epoisses
{

/* Another feature of ACE_Svc_Handler is it's ability to present the
   ACE_Task<> interface as well.  That's what the ACE_NULL_SYNCH
   parameter below is all about.  If our ClientAcceptor has chosen
   thread-per-connection then our open() method will activate us into
   a thread.  At that point, our svc() method will execute.  We still
   don't take advantage of the things ACE_NULL_SYNCH exists for but
   stick around for Tutorial 7 and pay special attention to the
   Thread_Pool object there for an explanation.  */
template <class T>
class ClientHandler : public ACE_Svc_Handler <ACE_SOCK_STREAM, ACE_NULL_SYNCH>
{
   public:
      ClientHandler (void) {};

      inline void destroy (void)
      {
         InfoLog (<< "Destroying client handler " << this);
         
         reactor()->remove_handler (this,
                                    ACE_Event_Handler::READ_MASK | ACE_Event_Handler::DONT_CALL); // don't call
         // handleClose 
         delete this;
      }

      // invoke after creating new ClientHandler when client connects
      inline int open (void* void_acceptor)
      {
         ACE_INET_Addr addr; // client 
         if (this->peer ().get_remote_addr (addr) == -1)
         {
            return -1;
         }

         DebugLog (<< "opening: " << addr.get_host_name());
         
         ClientAcceptor<T> *acceptor = (ClientAcceptor<T>*)(void_acceptor);
         assert(acceptor);
         
         return this->activate (THR_NEW_LWP | THR_DETACHED);
      }

      // call when done with task
      inline int close (u_long flags = 0)
      {
         destroy ();
         return 0;
      }

      // cleanup opportunity on error code
      inline virtual int handle_close (ACE_HANDLE handle = ACE_INVALID_HANDLE,
                                       ACE_Reactor_Mask mask = ACE_Event_Handler::ALL_EVENTS_MASK)
      {
         ACE_UNUSED_ARG (handle);
         ACE_UNUSED_ARG (mask);

         this->destroy ();
         return 0;
      }

   protected:

      // execute in thread created by open - watch for data
      inline int svc (void)
      {
         while( 1 )
         {
            if (this->process() == -1)
            {
               return -1;
            }
         }

         return 0;
      }

      /* When we register with the reactor, we're going to tell it that we
         want to be notified of READ events.  When the reactor sees that
         there is read activity for us, our handle_input() will be
         invoked. The _handle provided is the handle (file descriptor in
         Unix) of the actual connection causing the activity.  Since we're
         derived from ACE_Svc_Handler<> and it maintains it's own peer
         (ACE_SOCK_Stream) object, this is redundant for us.  However, if
         we had been derived directly from ACE_Event_Handler, we may have
         chosen not to contain the peer.  In that case, the <handle> would
         be important to us for reading the client's data.  */
      inline int handle_input (ACE_HANDLE handle)
      {
         return process();
      }

      /* This has nothing at all to do with ACE.  I've added this here as
         a worker function which I will call from handle_input().  As
         promised in Tutorial 5 I will use this now to make it easier to
         switch between our two possible concurrency strategies.  */
      virtual int process ()=0;

      /* We don't really do anything in our destructor but we've declared
         it to be protected to prevent casual deletion of this object.  As
         I said above, I really would prefer that everyone goes through the
         destroy() method to get rid of us.  */
      ~ClientHandler (void) {};
};

}

#undef VOCAL_SUBSYSTEM

#endif /* CLIENT_HANDLER_H */
