// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#if !defined(TIME_VAL_HXX)
#define TIME_VAL_HXX

static const char* const TimeHeaderVersion =
"$Id: Time.hxx,v 1.1 2002/09/30 04:55:17 jason Exp $";

#include <sys/time.h>
#include <unistd.h>
#include <iostream>

namespace Epoisses 
{

typedef unsigned long long milliseconds_t;


class Time : public timeval
{
   public:

      Time();
      Time(milliseconds_t);
      Time(long sec, long usec);
      Time(const timeval &);

      virtual ~Time();

      Time &	operator=(const timeval &);
      const Time & now();
      
      milliseconds_t	milliseconds() const;

      Time operator+(const timeval &) const;
      Time operator+(milliseconds_t) const;
      Time operator-(const timeval &) const;
      Time operator-(milliseconds_t) const;
      Time& operator+=(const timeval &);
      Time& operator+=(milliseconds_t);
      Time& operator-=(const timeval &);
      Time& operator-=(milliseconds_t);

      bool operator==(const timeval &) const;
      bool operator< (const timeval &) const;
      bool operator< (const Time &) const;

      /// Clear the timeval struct to 0.
      void clear();

      void copy(const timeval &);

      void normalize();

      static unsigned int TimeOfWeek(); // measured from Sunday at midnight in seconds
};

std::ostream& operator<<(std::ostream& strm, const Time& val);

}

#endif
