package org.vovida.prov.psLib;

import java.util.Vector;
import java.util.Hashtable;

/**
 * The return object of a ProvisioningInterface 'get' command.  If a return
 * of a call to the Provisioning Server needs to return records of data, the
 * data is contained in this structure.
 *
 * Access to the results' data is through an instance of BindIterator.
 * PSDBResult result = pif.getRecordsInDomain(...);
 * PSDBResult.BindIterator resIt = result.bindIterator(FieldList(foo, ...));
 * while (resIt.hasNext()) {
 *    int f = foo.bound();
 *    ...
 * }
 */
public class PSDBResult
{
      private String[] fieldNames;
      private Hashtable[] rows;

      /**
       * Returns the number of rows.
       */
      public int getNumRecords()
      {
         return rows.length;
      }

      public BindIterator bindIterator(FieldList fieldList)
      {
         return new BindIterator(fieldList);
      }

      public PSDBResult(String[] fieldNames, Hashtable[] rows)
      {
         this.fieldNames = fieldNames;
         this.rows = rows;
      }

      /**
       * Return the value at the supplied recordIndex and fieldName.
       * @param recordIndex the index into the internal Hashtable array
       * @param fieldName for the Hashtable at the given recordIndex get the entry
       * with this name.
       */
      public String getFieldValue(int recordIndex, String fieldName)
      {
         if (fieldNames.length == 0 || rows.length == 0)
         {
            return null;
         }
         
         return (String) (rows[recordIndex].get(fieldName));
      }

      /**
       * Client-side interface to contents of PSDBResult. */
      public class BindIterator
      {
            BindIterator(FieldList fieldList)
            {
               this.fieldList = fieldList;
            }

            public boolean hasNext()
            {
               // clear out the bindings
               for (int i = 0; i < fieldList.fields.size(); i++) {
                  Field field = (Field)fieldList.fields.get(i);
                  field.clearBound();
               }
               return counter < PSDBResult.this.getNumRecords();
            }

            public void next()
            {
               for (int i = 0; i < fieldList.fields.size(); i++) {
                  Field field = (Field)fieldList.fields.get(i);
                  field.clearBound();
                  // extract the result
                  field.bind(PSDBResult.this.getFieldValue(counter, field.name));
                  field.setBound();
               }
	    
               counter++;
            }

            private int counter;
            private FieldList fieldList;
      }
}
