<?php
// Copyright (c) 2002 Cathay Networks, Inc. All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
// 
// 3. The names "VOCAL", "Vovida Open Communication Application Library", and
// "Vovida Open Communication Application Library (VOCAL)" must not be used to
// endorse or promote products derived from this software without prior written
// permission. For written permission, please contact vocal@vovida.org.
// 
// 4. Products derived from this software may not be called "VOCAL", nor may
// "VOCAL" appear in their name, without prior written permission.
// 
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT ARE DISCLAIMED.
// IN NO EVENT SHALL CATHAY NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR
// ANY DAMAGES IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

require_once("cprov.php");
require_once("XMLParse.php");

class Response
{
  var $status;
  var $contents;

  function &Response(&$header, &$contents)
  {
    $headers = explode(" ", $header, 3);
    $this->status = $headers[1];

    $this->contents = xml_get_tree($contents);
  }
}

$ProvisioningInterface_Socket;
global $ProvisioningInterface_Socket;

class ProvisioningInterface
{
  function ProvisioningInterface($host, $port)
  {
    global $ProvisioningInterface_Socket;
    
    // open a socket if not already open
    if (!$ProvisioningInterface_Socket) {
      $ProvisioningInterface_Socket = 
        fsockopen($host, $port, $errno, $errstr, 30);
      if (!$ProvisioningInterface_Socket) {
        print "Failed to connect to $host:$port\n";
        trigger_error("Failed to connect to $host:$port -- $errstr", E_USER_ERROR);
      }
//      print "Provisioning connected to $host:$port\n";
    }
  }

  function shutdown()
  {
    global $ProvisioningInterface_Socket;

    if ($ProvisioningInterface_Socket) {
      fclose ($ProvisioningInterface_Socket);
    }
  }

  function send($httpMethod, $command, $contents)
  {
    global $ProvisioningInterface_Socket;

    $message = "$httpMethod $command HTTP/1.1\r\nContent-Length: ";
    $message .= strlen($contents);
    $message .= "\r\n\r\n$contents";

    fputs($ProvisioningInterface_Socket, $message);
  }

  // blocks
  // return a Response
  function &readHttpMessage() // === string PSConnection::readHttpMessage(Connection& conn)
  {
    global $ProvisioningInterface_Socket;

    $header = "";
    // read the header
    // read a character at a time 
    // read until just after second CR/LF 
    for ($i = 0; $i < 2; $i++) 
    { 
      while (true) {
        $c = fgets($ProvisioningInterface_Socket, 1);
        if ($c === false) {
          trigger_error("Connection to provisioning server closed during header read",
                        E_USER_ERROR);
        }
        $header .= $c;
        if ($c == "\n")
        { 
          break; 
        } 
      } 
    }

    //print "header is: |$header|\n";
    
    // determine the content-length
    $pos = strpos($header, "Content-Length: ");
    if ($pos === false) {
      trigger_error("'Content-Length: ' not found in message",
                    E_USER_ERROR);
    }
    $contentLength = (int)substr($header, $pos + strlen("Content-length: ")) + 2;
    while (strlen($contents) < $contentLength) {
      $contents .= fgets($ProvisioningInterface_Socket, $contentLength - strlen($contents));
    }
    
    if ($contents === false) {
      trigger_error("Connection to provisioning server closed during read",
                    E_USER_ERROR);
    }

    //print "Contents|$contents|\n";

    return new Response($header, $contents);
  }

  function& getRecordsInDomain(&$domain, &$filter, $fields)
  {
    //print "getRecordsInDomain(".$domain->domainName.",". $filter->toString($domain).",".Field::arrayToString($fields).")\n";
      
    $contents = 
       "<arguments>\r\n<domain>$domain->domainName</domain>\r\n<filter>".$filter->toString($domain)."</filter>\r\n<fieldlist>\r\n";
    
    // translate fields to XML
    foreach ($fields as $fieldName) {
      $field = $domain->DB->$fieldName;
      if (!$field) {
        trigger_error("Domain ".$domain->domainName." has no field '$fieldName'",
                      E_USER_ERROR);
      }
      $fieldName = strtolower($fieldName);
      $contents .= "<listitem>$fieldName</listitem>\r\n";
    }
    
    $contents = $contents."</fieldlist>\r\n</arguments>\r\n";

    // send to server
    $this->send("GET", "GetRecordsInDomain", $contents);

    // get response as string
    $response = $this->readHttpMessage();
    if ($response->status != "200") {
      trigger_error("Provisioning getRecordsInDomain Error: ".$response->contents['value'],
                    E_USER_ERROR);
    }

    return new Result($response->contents);
  }
      
  // returns value cast as type of field in domain
  // $field is a string naming the Field
  function& valueString(&$domain, &$field, &$value)
  {
    // get the field from the domain and use it to cast
    // int, float have same repesentation; e.g. 1 or 1.23
    // boolean is one of (true, false)
    // string is 'single quoted'
    // array is 'comma,seperated,list,as,string' (non-recursive!)
    return $domain->DB->$field->dbValue($value);
  }

  // returns nothing
  function setRecordsInDomain(&$domain, &$filter, $fieldValues)
  {
    // generate the XML
    $contents = 
      "<arguments>\r\n<domain>$domain->domainName</domain>\r\n<filter>".$filter->toString($domain)."</filter>\r\n<fieldvalue>\r\n";
    foreach ($fieldValues as $fieldName => $value) {
      $field = $domain->DB->$fieldName;
      if (!$field) {
        trigger_error("Domain ".$domain->domainName." has no field '$fieldName'",
                      E_USER_ERROR);
      }
    
      $fieldName = strtolower($fieldName);
      // convert to appropriate type
      $contents .= "<$fieldName>".$field->dbValue($value)."</$fieldName>\r\n";
    }
    $contents .= "</fieldvalue>\r\n</arguments>\r\n";

    // send to server
    $this->send("PUT", "SetRecordsInDomain", $contents);

    // get response as string
    $response = $this->readHttpMessage();
    if ($response->status != "200") {
      trigger_error("Provisioning setRecordsInDomain Error: ".$response->contents['value'],
                    E_USER_ERROR);
    }
  }

  // returns nothing
  function addRecordInDomain(&$domain, $fieldValues)
  {
    // generate the XML
    $contents = 
      "<arguments>\r\n<domain>".$domain->domainName."</domain>\r\n<fieldvalue>\r\n";
    foreach ($fieldValues as $fieldName => $value) {
      $field = $domain->DB->$fieldName;
      if (!$field) {
        trigger_error("Domain '".$domain->domainName."' has no field '$fieldName'",
                      E_USER_ERROR);
      }
      $fieldName = strtolower($fieldName);

      $contents .= "<$fieldName>".$field->dbValue($value)."</$fieldName>\r\n";
    }
    $contents .= "</fieldvalue>\r\n";
    $contents .= "</arguments>\r\n";

    // !dlb! this check will happen on the back end in later release
    // check if the primary key is set
    $pKey = $domain->getPrimaryKey();
    $root = $domain->getRoot();
    if (array_key_exists($pKey, $fieldValues)) {
      // query for an existing row with the same primary key value
      $r = &new $root;
      $res = $this->getRecordsInDomain($r,
                                       eq($pKey, $fieldValues[$pKey]),
                                       array($pKey));
      if ($res->count() != 0) {
        trigger_error("Duplicate primary key ".$pKey.
                      " = (".$fieldValues[$pKey].") for root domain '".$root.
                      "' from domain '".$domain->domainName."'",
                      E_USER_ERROR);
      }
    }

    // send to server
    $this->send("PUT", "AddRecordInDomain", $contents);

    // get response as string
    $response = $this->readHttpMessage();
    if ($response->status != "200") {
      trigger_error("Provisioning addRecordInDomain Error: ".$response->contents['value'],
                    E_USER_ERROR);
    }
  }

  // private
  // this functionality will be moved into the pserver
  function deleteDependentRows(&$domain, &$filter)
  {
     global $DependentDomains;
     $ddc = &$DependentDomains[$domain->getRoot()];
  
     if (count($ddc)) {
        $ids = array();

        $pk = $domain->getPrimaryKey();
                
        // id in filter?
        if (is_object($filter) &&
            $filter->name == "=" &&
            $filter->arg1 == $pk) {
           $ids[] = $filter->arg2;
        }
        else {
           // retrieve the affected ids
           $res = $this->getRecordsInDomain($domain, $filter, array($pk));
           $bindIt = $res->bindIterator($domain);
           while ($bindIt->hasNext()) {
             $bindIt->next();
             $ids[] = $domain->$pk;
           }
        }

        // invoke method on each registered dependent domain
        foreach ($ddc as $c) {
          foreach ($ids as $id) {
            $c->dependentRemove($domain->getRoot(), $id);
          }
        }
     }
  }
  
  function deleteRecordsInDomain(&$domain, &$filter)
  {
    $this->deleteDependentRows($domain, $filter);
    
    // generate the XML
    $contents = 
      "<arguments>\r\n<domain>".$domain->domainName."</domain>\r\n<filter>".$filter->toString($domain)."</filter>\r\n</arguments>\r\n";

    // send to server
    $this->send("DELETE", "DeleteRecordsInDomain", $contents);

    // get response as string
    $response = $this->readHttpMessage();
    if ($response->status != "200") {
       trigger_error("Provisioning deleteRecordsInDomain Error: ".$response->contents['value'],
                     E_USER_ERROR);
    }
  }
}

?>