
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ApplTable_cxx_Version =
    "$Id: ApplTable.cxx,v 1.19 2001/10/19 20:46:54 icahoon Exp $";
#include "ApplTable.hxx"
#include <cpLog.h>
#include <errno.h>
#include <string.h>

ApplTable::ApplTable()
{
}


ApplTable::~ApplTable()
{
}


voReturnStatus
ApplTable::deleteAppl(enum ServerType type, int port, string name )
{
    return voFailure;
}

voReturnStatus
ApplTable::deleteAppl(int index)
{
    return voFailure;

}


voReturnStatus
ApplTable::addOrUpdateAppl(enum ServerType type, int port, string name )
{
    ApplInfo appl;
    struct timezone tz;
    struct timeval tv;
    voReturnStatus retVal;

    tz.tz_minuteswest = 0;
    tz.tz_dsttime = 0;
    timerclear(&tv);
    if (gettimeofday(&tv, &tz) > 0 )
    {
        cpLog(LOG_ERR, "serverGrp:gettimeofday failed: %s", strerror(errno));
        return voFailure;
    }

    tableLock.lock();
    switch (applClash(type, port, name))
    {
        case completeExists:
        // this is the nominal case
        {
            cpLog(LOG_DEBUG, "Attempt to add existing item to application table.  item %s, port %d, not added", name.c_str(), port);
            retVal = voSuccess;
            break;
        }

        case portClash:
        {
            // This means that an application was down and has now come back on a different port.
            // we will update the ports and time of the record with the same name.
            for (apI = theTable.begin(); apI < theTable.end() && apI->name != name; apI++);
            if (apI == theTable.end())
            {
                cpLog(LOG_ERR, "portClash error: table is not mutexed correctly  item %s, port %d, not added", name.c_str(), port);
                retVal = voFailure;
                break;
            }
            if (isStackType(type))
            {
                apI->stackPort = port;
                apI->applPort = 0;
            }
            else
            {
                apI->applPort = port;
                apI->stackPort = 0;
            }
            apI->status = ApplInfo::restarting;
            apI->lastUpdateTime = tv;
            cpLog(LOG_DEBUG, "port clash when add item to application table.  item %s, port %d, not added", name.c_str(), port);
            retVal = voSuccess;
            break;
        }

        case nameClash:
        {
            // We received a register message for a port that we think is occupied by another application
            // This means that somebody has gone down and another application came up on it's old port.
            // since we don't know who is correct, we will set the status of the existing application to DOWN
            // and set all of the ports associated with the application to 0.
            // the new register message we received will be added when we receive the next register from them.
            // Having said that,
            // jtapi application will have 2 sip stacks for the same application.  It will have a similar but different
            // application name for each sipStack parent application.
            for (apI = theTable.begin(); apI < theTable.end() && apI->stackPort != port && apI->applPort != port; apI++);
            if (apI == theTable.end())
            {
                cpLog(LOG_ERR, "nameClash error: table is not mutexed correctly  item %s, port %d, not added", name.c_str(), port);
                retVal = voFailure;
                break;
            }
            apI->status = ApplInfo::down;
            apI->lastUpdateTime = tv;
            apI->applPort = 0;
            apI->stackPort = 0;
            cpLog(LOG_DEBUG, "name clash deletes item from application table. caused by new record:  item %s, port %d, not added", name.c_str(), port);
            retVal = voSuccess;
            break;
        }

        case newOne:
        {
            apRI = theTable.rbegin();
            if (apRI == theTable.rend())
                appl.index = 1;
            else
                appl.index = apRI->index + 1;
            appl.name.assign( name );
            appl.type = type;
            if (isStackType(type))
            {
                appl.stackPort = port;
                appl.applPort = 0;
            }
            else
            {
                appl.applPort = port;
                appl.stackPort = 0;
            }
            appl.status = ApplInfo::restarting;
            appl.startTime = tv;
            appl.lastUpdateTime = tv;
            theTable.push_back(appl);
            retVal = voSuccess;
            break;
        }
        case partialExists:
        {
            for (apI = theTable.begin(); apI < theTable.end() && apI->name != name; apI++);
            if (apI == theTable.end())
            {
                cpLog(LOG_ERR, "partialExists error: table is not mutexed correctly  item %s, port %d, not added", name.c_str(), port);
                retVal = voFailure;
                break;
            }

            if (isStackType(type))
            {
                if (!apI->stackPort)
                {
                    apI->stackPort = port;
                    apI->status = ApplInfo::up;
                    apI->type = type;
                    apI->lastUpdateTime = tv;
                }
            }
            else
            {
                if (!apI->applPort)
                {
                    apI->applPort = port;
                    apI->status = ApplInfo::up;
                    apI->lastUpdateTime = tv;
                }
            }
            retVal = voSuccess;
            break;
        }
        case err:
        {
            cpLog(LOG_ERR, "Error when adding item to application table.  item %s, port %d, not added", name.c_str(), port);
            retVal = voSuccess;
            break;
        }
	default :
	{
            cpLog(LOG_ERR, "Unknown error when adding item to application table.  item %s, port %d, not added", name.c_str(), port);
            retVal = voSuccess;
            break;
	}	    
    }
    tableLock.unlock();
    return retVal;
}


ApplInfo
ApplTable::getAppl(int index)
{
    ApplInfo ai;
    tableLock.lock();
    if (!index || theTable.size() < (unsigned long)index)
    {
        ai.index = 0;
    }
    else
    {
        ai = theTable[index - 1];
    }
    tableLock.unlock();
    return ai;
}


ApplInfo
ApplTable::getAppl(enum ServerType type, int port, string name )
{
    ApplInfo ai;
    ai.index = 0;
    tableLock.lock();
    for (apI = theTable.begin(); apI < theTable.end(); apI++)
    {
        if (apI->type == type && (isStackType(type) ? apI->stackPort == port : apI->applPort == port) && apI->name == name)
        {
	    ai = *apI;
            tableLock.unlock();
            return ai;
        }
    }

    tableLock.unlock();
    return ai;
}


int
ApplTable::getStackPort(int index)
{
    int retPort = 0;
    ApplInfo ai;
    tableLock.lock();
    if (!index || theTable.size() < (unsigned long)index)
    {
        ai.stackPort = 0;
    }
    else
    {
        ai = theTable[index - 1];
    }
    retPort = ai.stackPort;
    tableLock.unlock();
    return retPort;
}



int
ApplTable::getApplPort(int index)
{
    int retPort = 0;
    ApplInfo ai;
    tableLock.lock();
    if (!index || theTable.size() < (unsigned long)index)
    {
        ai.applPort = 0;
    }
    else
    {
        ai = theTable[index - 1];
    }
    retPort = ai.applPort;
    tableLock.unlock();
    return retPort;
}


int
ApplTable::getNextIndex(const int index, const enum ServerType type)
{
    int retIndex = 0;
    tableLock.lock();
    if (index >= (int)theTable.size())
    {
        tableLock.unlock();
        return retIndex;
    }
    for (apI = theTable.begin(); apI != theTable.end(); apI++)
    {
        if (apI->index > index && (type == SERVER_Unknown || apI->type == type))
        {
	    retIndex = apI->index;
            tableLock.unlock();
            return retIndex;
        }
    }
    tableLock.unlock();
    return retIndex;
}


ApplTable::RecordStatus
ApplTable::applClash(enum ServerType type, int port, string name )
{
    // make many passes through the list since it will be a short one.

    for (apI = theTable.begin();
            apI < theTable.end() && apI->name != name && apI->stackPort != port && apI->applPort != port;
            apI++);
    if (apI == theTable.end())
    {
        return ApplTable::newOne;
    }

    if (apI->name == name)
    {
        if (isStackType(type) ? (apI->stackPort == port && apI->applPort != 0) : (apI->applPort == port && apI->stackPort != 0))
        {
            return ApplTable::completeExists;
        }
        if (isStackType(type) ? (apI->stackPort == port && apI->applPort == 0) : (apI->applPort == port && apI->stackPort == 0))
        {
            return ApplTable::partialExists;
        }
        if (isStackType(type) ? (apI->stackPort == 0) : (apI->applPort == 0))
        {
            return ApplTable::partialExists;
        }
        // error case:  application was up and registerd, then down and now came up on
        // a different port
        if (isStackType(type) ? apI->stackPort != port : apI->applPort != port)
        {
            return ApplTable::portClash;
        }
    }
    else
    {
        // error case.  We received a port that we think is being used by another appl.
        // this means that one of our applications has gone down.  If the name matches any
        // of the names in the list it is a portClash, if the name is a new one, it is a
        // nameClash
        for (; apI < theTable.end(); apI++)
        {
            if (apI->name == name)
            {
                return ApplTable::portClash;
            }
        }
        return ApplTable::nameClash;
    }

    cpLog(LOG_DEBUG, "Flow control should not reach here");
    return ApplTable::err;

}


bool
ApplTable::isStackType(enum ServerType type)
{
    switch (type)
    {
        case SERVER_Unknown :
        case SERVER_FS:
        case SERVER_MS:
        case SERVER_RS:
        case SERVER_JS:
        case SERVER_Agent:
        case SERVER_NetMgnt:
        case SERVER_CDR:
        case SERVER_UA:
        case SERVER_HB:
        case SERVER_POS:
        case SERVER_CA:
        {
            return false;
        }

        case SERVER_MgcpStack:
        case SERVER_H323:
        case SERVER_SipStack:
        {
            return true;
        }

        default :
        {
            return false;
        }
    }
}
