
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpAddress_cxx_Version =
    "$Id: CPLOpAddress.cxx,v 1.13 2002/11/09 02:05:37 derekm Exp $";
#include "CPLOpAddress.hxx"
#include "CPLOpAddressSwitch.hxx"
#include "Data.hxx"

using namespace Vocal;

///Constructor
CPLOpAddress::CPLOpAddress():
        myMatchType( Is_matchType )
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "new CPLOpAddress[%x]", this );
#endif
}  //c'tor


///Destructor
CPLOpAddress::~CPLOpAddress()
{
#ifdef MEM_TEST
    cpLog( LOG_DEBUG, "~CPLOpAddress[%x]", this );
#endif
}  //d'tor


///Append an operator to this class
bool
CPLOpAddress::setNextOperator( Sptr < Operator > op, int prevNode )
{
    addOperator( op );
    return true;
}  //setNextOperator


///Initialize the Attributes member variables
bool
CPLOpAddress::setAttributes( const char* attrName, const char* attrValue)
{
    mySearchString = attrValue;

    if ( strcmp( attrName, "is" ) == 0 )
    {
        myMatchType = CPLOpAddress::Is_matchType;
    } /*if*/
    else if ( strcmp( attrName, "subdomain-of" ) == 0 )
        myMatchType = CPLOpAddress::SubDomainOf_matchType;
    else if ( strcmp( attrName, "contains" ) == 0 )
        myMatchType = CPLOpAddress::Contains_matchType;
    else  //Unknown attribute name
    {
        assert ( 0 );
    } /*else*/

    return true;

}  //setAttributes


///
const Sptr < State >
CPLOpAddress::process( Sptr < SipProxyEvent > event)
{
    return ( CPLOperator::process( event ) );
}  //process


///
bool
CPLOpAddress::compare( int subfield, Data& data )
{
    //Check to see if this operator is an <otherwise> operator.
    //The <otherwise> op has a null string.
    if ( mySearchString.size() == 0 )
        return true;
    else
    {
        //Convert Data to string
        LocalScopeAllocator lo;
        string str( data.getData(lo) );

        switch ( myMatchType )
        {
            case CPLOpAddress::Is_matchType:
            {
                if ( str == mySearchString )
                    return true;
                else
                    return false;
            } /*case*/
            break;
            case CPLOpAddress::SubDomainOf_matchType:
            {
                unsigned int idx = str.find( mySearchString );
                if ( idx == string::npos ) return false;
                if (( subfield == CPLOpAddressSwitch::Tel_SubField ) ||
                        ( subfield == CPLOpAddressSwitch::User_SubField) )
                {
                    if (idx == 0 )
                        return true;
                    else
                        return false;
                } /*if*/
                return false;
            }  /*case*/
            break;
            case CPLOpAddress::Contains_matchType:
            {
                if ( str.find( mySearchString ) != string::npos )
                    return true;
                else
                    return false;
            }  /*case*/
            break;
            default:
              ;  //No additional cases expected
        } /*switch*/

        return false;
    } /*else*/

}  //compare


///Outputs the values of the member variables
void
CPLOpAddress::logData() const
{
    switch ( myMatchType )
    {
        case CPLOpAddress::Is_matchType:
        {
            cpLog(LOG_DEBUG, "CPLOpAddress-IS(%s)", mySearchString.c_str() );
        } /*case*/
        break;
        case CPLOpAddress::SubDomainOf_matchType:
        {
            cpLog(LOG_DEBUG, "CPLOpAddress-SUBDOMAIN_OF(%s)",
                              mySearchString.c_str() );
        } /*case*/
        break;
        case CPLOpAddress::Contains_matchType:
        {
            cpLog(LOG_DEBUG, "CPLOpAddress-CONTAINS(%s)", 
                              mySearchString.c_str() );
        } /*case*/
        break;
        default:
          ;  //No additional cases expected 
    } /*switch*/

}  //logData


///Returns the name of the class
const char* const 
CPLOpAddress::name() const
{
    return "CPLOpAddress";
}  //name


//End of file

