
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const CPLOpTime_cxx_Version =
    "$Id$";
#include "CPLOpTime.hxx"

using namespace Vocal;

///
CPLOpTime::CPLOpTime()
{
}  //Constructor


///
CPLOpTime::~CPLOpTime()
{
    /// no op...
}  //Destructor


bool
CPLOpTime::setAttributes(const char* type, const char* value)
{
    if ( strcmp( type, "year" ) == 0 )
        myTimeRange.year(value);
    else if ( strcmp( type, "month" ) == 0 )
        myTimeRange.month(value);
    else if ( strcmp( type, "date" ) == 0 )
        myTimeRange.date(value);
    else if ( strcmp( type, "day" ) == 0 )
        myTimeRange.weekDay(value);
    else if ( strcmp( type, "timeofday" ) == 0 )
        myTimeRange.timeOfDay(value);
    else  //Unknown attribute name
        assert ( 0 );

    return true;
}  //setAttributes


///
const Sptr < State >
CPLOpTime::process(const Sptr < SipProxyEvent > anEvent)
{
    return CPLOperator::process(anEvent);
}  //process


///
bool
CPLOpTime::setNextOperator(Sptr < Operator > anOperator, int nodeId /*def value*/)
{
    addOperator(anOperator);
    return true;
}  //setNextOperator


///
const char* const
CPLOpTime::name() const
{
    return "CPLOpTime";
}  //name


void
CPLOpTime::logData(void) const
{
    cpLog(LOG_DEBUG, "%s[%x]: %s", name(), this, myTimeRange.toString());
}  //logData


/**
 * The otherwise implementation of time will also work fine 
 * because timeRange is always empty for an <otherwise>
 */
bool
CPLOpTime::test(const MyTime& myTime)
{
    if (myTimeRange.testYear(myTime.getYear()) == false) return false;
    if (myTimeRange.testMonth(myTime.getMonth()) == false) return false;
    if (myTimeRange.testDate(myTime.getDate()) == false) return false;
    if (myTimeRange.testWeekDay(myTime.getWeekDay()) == false) return false;
    if (myTimeRange.testTimeOfDay(myTime.getTimeOfDay()) == false) return false;
    return true;
}  //test

//End of file


