/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const CPLOpVmRedirect_cxx_Version =
    "$Id: CPLOpVmRedirect.cxx,v 1.34 2002/11/09 02:05:38 derekm Exp $";

#include <iostream.h>
#include "CPLOpVmRedirect.hxx"
#include "VmCallInfo.hxx"
#include "UavmContainer.hxx"
#include "ProvisionInterface.hxx"
#include "VException.hxx" 
#include "FsConfigFile.hxx"
#include "CommandLine.hxx"
#include "SipVia.hxx"
#include "SipRecordRoute.hxx"
#include "SipDiversion.hxx"

using namespace Vocal;

///
CPLOpVmRedirect::CPLOpVmRedirect()
{
#ifdef MEM_TEST
    cpLog (LOG_DEBUG, "CPLOpVmRedirect" );
#endif
} // CPLOpVmRedirect


///
CPLOpVmRedirect::~CPLOpVmRedirect()
{
#ifdef MEM_TEST
    cpLog (LOG_DEBUG, "~CPLOpVmRedirect" );
#endif
} // ~CPLOpVmRedirect


///
const Sptr < State >
CPLOpVmRedirect::process( const Sptr < SipProxyEvent > event )
{
    Sptr < State > returnState = myNextState;

    // Get message from event
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent != 0 )
    {
        Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
        Sptr < InviteMsg > inviteMsg;
        inviteMsg.dynamicCast( sipMsg );

        if ( inviteMsg != 0 )
        {
            processInvite( sipEvent, inviteMsg );
        } /*if*/
        else
        {
            cpLog( LOG_ERR, "FS is expecting only INVITE messages" );
            returnState = 0;
        } /*else*/
    } /*if*/
    else
    {
        cpLog( LOG_DEBUG, "Event received is not a Sip event" );
        returnState = 0;

    } /*else*/

    return returnState;
}  //process


///
const char* const
CPLOpVmRedirect::name() const
{
    return "CPLOpVmRedirect";
} //name

///
void
CPLOpVmRedirect::logData() const
{
    cpLog( LOG_DEBUG, "CPLOpVmRedirect" );

}  //logData


///
bool
CPLOpVmRedirect::setAttributes( const char* attrName, const char* attrValue)
{
    //<Vmredirect> has no attributes
    return true;
}  //setAttributes



///
bool
CPLOpVmRedirect::setNextOperator( Sptr < Operator > op, int )
{
    addOperator( op );
    return true;
}  //setNextOperator


///
void
CPLOpVmRedirect::processInvite( Sptr < SipEvent > sipEvent,
                                Sptr < InviteMsg > inviteMsg )
{
    cpLog( LOG_DEBUG, "CPLOpVmRedirect process INVITE" );

    if( CommandLine::instance()->getInt( "HEARTBEAT" ) )
    {
        // Check to make sure all vm ports are not down.
        // If they are, return a 503.
        Sptr < vector < Data > > activeVmPorts = new vector < Data > ;
        UavmContainer::instance().getListofActive(SERVER_UAVM, activeVmPorts);
        if (activeVmPorts->empty())
        {
            // all uamvs are dead.
            cpLog(LOG_ERR, "!!!!!!!******** All uavms are down ***************");
            StatusMsg statusMsg( *inviteMsg, 503);
            sipEvent->getSipStack()->sendReply( statusMsg );
            return ;
        }
    }
    else
    {
        cpLog( LOG_DEBUG, "No heartbeat" );
    }

    //Find Voice mail URL from the Voice Mail port container
    Sptr<SipUrl> vmUrl = new SipUrl;
    vmPort availableVmPort;

    if (!(VmPortContainer::instance()->getVmPort(availableVmPort)))
    {
        cpLog( LOG_WARNING, "All ports busy" );
        // No voicemail port available: return 486 and terminate the call
        StatusMsg statusMsg( *inviteMsg, 486 );
        sipEvent->getSipStack()->sendReply( statusMsg );
        return ;
    }

    //make sure port is not down.  If it is, keep trying to get next port
    //until available port is not down or no ports are available.
    if (isUavmDown(availableVmPort))
    {
        int portCounter = 1;
        int numVmPorts = VmPortContainer::instance()->getNumberOfVmPorts();
        bool foundAliveUavm = false;
        VmPortContainer::instance()->appendVmPort(availableVmPort);  //return port to queue
        while (!foundAliveUavm)
        {
            //if ports were already checked if down, return busy.
            if ( (portCounter > numVmPorts) ||
                 (!(VmPortContainer::instance()->getVmPort(availableVmPort))) )
            {
                StatusMsg statusMsg( *inviteMsg, 486 );
                sipEvent->getSipStack()->sendReply( statusMsg );
                return ;
            }

            if (!isUavmDown(availableVmPort))
            {
                foundAliveUavm = true;
            }
            else
            {
                VmPortContainer::instance()->appendVmPort(availableVmPort);
                portCounter++;
            }
        }
    }

    // return 100 to sender
    StatusMsg statusMsg( *inviteMsg, 100 );
    sipEvent->getSipStack()->sendReply( statusMsg );


    cpLog(LOG_INFO, "Redirecting to %s:%d", availableVmPort.host.c_str(), availableVmPort.port);

    // Compute the number of uavm ports used by the VM
    int usedPorts = VmPortContainer::instance()->getNumberOfConfiguredPorts() -
                    VmPortContainer::instance()->getNumberOfVmPorts();

    cpLog(LOG_INFO, "Voicemail is currently using %d uavm ports", usedPorts);

    // Store the vmPort info in CallInfo so that the port is freed at the
    // same time the call information is freed
    Sptr < VmCallInfo > vmCallInfo;
    vmCallInfo.dynamicCast( sipEvent->getCallInfo() );
    if (vmCallInfo != 0)
    {
        vmCallInfo->setVmPort(availableVmPort);
    }
    else
    {
        cpLog( LOG_ERR, "Not a vmCallInfo !" );
    }

    // Get the mailbox user to which leave a voicemail and put it in the
    // request URL. The uavm will then look at the req line user to
    // find the mailbox (this is done as we don't want to change the "To")
    // If diversion
    // get user from diversion header
    // getmailbox...(userDiversion)
    // Check if the message has the CC_Diversion header

    string mailboxNumber;                 
    cpLog( LOG_DEBUG, "Going to look for Diversion Header" );   
    SipDiversionList newdiversionList = inviteMsg->getDiversionList();
    int number = inviteMsg->getNumDiversion();
    cpLog( LOG_DEBUG, " getNumDiversion =  %d", number);
    LocalScopeAllocator lo;
//    if (newdiversionList.size() > 0) //Diversion happened
    if(number > 0)
    {
        cpLog( LOG_DEBUG, "Diversion happened");
        SipDiversionList::iterator iter;
        iter = newdiversionList.begin();
        SipDiversion* sipdiversion = new SipDiversion;
        assert(sipdiversion);
        *sipdiversion = *(*iter);
        string redirectedUserTo = sipdiversion->getUrl()->getUserValue().getData(lo);
        cpLog( LOG_DEBUG, "redirectedUserTo = %s", redirectedUserTo.c_str());
        mailboxNumber = getMailboxFromProvisioning(redirectedUserTo);
        cpLog( LOG_DEBUG, "mailboxNumber  = %s", mailboxNumber.c_str());
    }
    else 
    { 
	// No diversion has happened
        cpLog( LOG_DEBUG, "Diversion NOT happened");
        string userTo = inviteMsg->getTo().getUrl()->getUserValue().getData(lo);
        cpLog( LOG_DEBUG, "userTo =  %s", userTo.c_str());
        mailboxNumber = getMailboxFromProvisioning(userTo);
        cpLog( LOG_DEBUG, "mailboxNumber  =  %s ", mailboxNumber.c_str());
    }                                             


    // Send new INVITE to VM()
    vmUrl->setUserValue(Data(mailboxNumber));
    Data portData(availableVmPort.port);
    vmUrl->setPort(portData);
    vmUrl->setHost(availableVmPort.host);

    Sptr<BaseUrl> vmBaseUrl = vmUrl;
    SipRequestLine requestLine( SIP_INVITE, vmBaseUrl );

    inviteMsg->setRequestLine( requestLine );

    SipVia via;
    OpAddVia( via, cmdLineData.sipPort );
    via.setBranch( inviteMsg->computeBranch2() 
		   + getStateMachineData(sipEvent)->getNextBranch());
    // TODO: hardcoded for now but should distinguis busy/no answer
    // but the redirection reason is needed
    via.setComment( "Vovida: NoAnswer" );
    inviteMsg->setVia( via, 0);

    // add RecordRoute
    cpLog( LOG_DEBUG, "adding RecordRoute" );
    SipRecordRoute route;

    Sptr<SipUrl> requestSipUrl; 
    requestSipUrl.dynamicCast( inviteMsg->getRequestLine().getUrl() );

    OpAddRecordRoute( route, *requestSipUrl, cmdLineData.sipPort );
    inviteMsg->setRecordRoute( route, 0 );

    sipEvent->getSipStack()->sendAsync( *inviteMsg );

}   //processInvite


///
bool
CPLOpVmRedirect::isUavmDown(const vmPort& uavmPort)
{
    if( !CommandLine::instance()->getInt( "HEARTBEAT" ) )
    {
        // Cannot check without heartbeat
        // Assume UAVM is up
        return false;
    }

    Data uavm = uavmPort.host;
    uavm += ":";
    uavm += Data(uavmPort.port);

    if (UavmContainer::instance().getStatus(uavm) == Inactive)
        return true;
    else
        return false;
}  //isUaVmDown

// Only the master user has a mailbox: check if the user
// is a master user. If not, get the master user
// from provisioning before passing it to vmserver
const string
CPLOpVmRedirect::getMailboxFromProvisioning( const string userTo )
{
    cpLog( LOG_DEBUG, "Check mailbox for user %(%s)", userTo.c_str());
    string userProv = userTo; 
    try
    {
      // No parameters are passed as already instantiated in VmProvisioning
      ProvisionInterface& pif = ProvisionInterface::instance();     
      if (pif.isAccount( userTo) == 0) 
      {
         // If user not a master then it might be an alias: get the master  
         userProv = pif.getMasterUser( userTo );
         cpLog(LOG_DEBUG, "Converting %s to master %s",userTo.c_str(), userProv.c_str());
         return(userProv);          
      }
    } /*try*/
    catch (VException& e)
    {
       cpLog( LOG_ERR, "Network exception: %s", e.getDescription().c_str() );
       cpLog( LOG_ERR, "Could not get user alias data for (%s) from provisioning: will use the user information in the To", userTo.c_str());  
    } /*catch*/  

    return (userTo);
}


//End of File

