#ifndef FsUserData_Hxx
#define FsUserData_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const FsUserData_hxx_Version =
    "$Id: FsUserData.hxx,v 1.1 2002/05/24 23:13:22 bko Exp $";




#include <Sptr.hxx>
#include <cpLog.h>
#include "SipUrl.hxx"

#include "Feature.hxx"


namespace Vocal
{

/** Feature Server user data
 *
 */
class FsUserData
{
    public:
        ///Constructor
        FsUserData();

        ///Destructor
        ~FsUserData();

        ///Retrieve a Called/Incoming Feature
        Sptr < Feature > getCalledFeature() const;

        ///Retrieve a Calling/Outgoing Feature
        Sptr < Feature > getCallingFeature() const;

        ///Retrieve last caller
        SipUrl getLastCaller() const;

        ///Set last caller
        void setLastCaller(const SipUrl& theLastCaller);

        ///Retrieve last caller displayName
        Data getDispName() const;

        ///Set last caller
        void setDispName(const Data& theDispName);

        ///Assumes to replace Callee Feature only
        bool replaceCalleeFeature( Sptr < Feature > feature );

        ///Assumes to replace Called Feature only
        bool replaceCalledFeature( Sptr < Feature > feature );

        ///Assign feature to Called feature
        bool addCalledFeature( Sptr < Feature > feature );

        ///Assign feature to Calling feature
        bool addCalleeFeature( Sptr < Feature > feature );

        ///Print debug information
        void showUserData() const;

        ///
        bool getUserData( string& str ) const;

        ///
        void setUserFileName( const char* userFilename );

         ///
         const Data& getUserFileName() const;



    private:
        ///
        Sptr < Feature > myCalleeFeature;

        ///
        Sptr < Feature > myCalledFeature;

        /// Remember the last caller for call return feature
        SipUrl myLastCaller;

        ///
        Data myDispName;

        /** userFilename is the name of the file containing the CPL 
         * script
         **/
        Data myUserFilename;
};
 
}

#endif
