
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MailMan_cxx_Version =
    "$Id$";
#include "MailMan.hxx"
#include "VocalCommon.hxx"
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

using namespace Vocal;



MailMan::MailMan():
    mySock(-1)
{
}  //Constructor



///
MailMan::~MailMan()
{
    if (mySock != -1) shutdown(mySock, SHUT_RDWR);
    mySock = -1;
}  //Destructor


///
const MailMan&
MailMan::operator=( const MailMan& rhs )
{
    mySock = rhs.mySock; 
    return *this;

} //operator=


///
bool
MailMan::mail(const string& mailTo, const string& msg)
{
    return useSMTP(mailTo, msg);
}  //mail


///
bool
MailMan::useSMTP(const string& mailTo, const string& msg)
{
    // connect to localhost
    if (smtpSendConnect("localhost") == false) return false;

    unsigned int i = 0;

    // greet the smtp host
    if (smtpSendHello("localhost") == false) return false;


    // Get environment variables to build the "From" field.
    // NOTE:  Another way to build the "From" field is to
    // make it identical to the "To" field. 
    char *ptr = getenv("USER");
    if (ptr == 0)
    {
        ptr = getenv("LOGNAME");
    }
    if (ptr == 0) return false;  // can't construct from address!

    string addr( ptr );

    ptr = getenv("MAILDOMAIN");  // explicitly specified by administrator
    if (ptr == 0) ptr = "vovida.org";
    addr = addr + "@" + ptr;
    if (smtpSendFrom(addr.c_str()) == false) return false;

    // send "RCPT TO:", (extract recipient address from mailTo)
    ptr = (char *) mailTo.c_str();
    if (strncmp(ptr, "mailto:", 7) == 0)
    { // skip the protocol specifier from url
        ptr += 7;
        i = 7;
    }

    do
    {
        addr = "";
        for (; *ptr; i++)
        {
            if ( *ptr == ';' || *ptr == '?' || *ptr == ',') 
            {
                break;
            } /*if*/
            addr = addr + *ptr;
            ptr++;
        }
        if (smtpSendTo(addr.c_str()) == false) return false;
    }
    while (*(ptr++) == ',');


    // start data
    if (smtpSendData() == false) return false;


    // parse subject (from mailTo) and send 
    if (i < mailTo.size())
    {
        addr = "";
        for (; *ptr && *ptr != '='; ptr++);
        ptr++;
        for (; *ptr; )
        {
            if (*ptr == '&') break;  // end of subject line

            else if (*ptr == '+') addr = addr + ' ';

            else if (*ptr != '%') addr = addr + *ptr;

            else // special character
            {
                char hi = *(++ptr);
                if (hi >= '0' && hi <= '9') 
                {
                        hi -= '0';
                } /*if*/
                else if (hi >= 'a' && hi <= 'f')
                {
                    hi -= 'a' + 10;
                } /*else if*/
                else if (hi >= 'A' && hi <= 'F')
                {
                    hi -= 'A' + 10;
                } /*else if*/
                else 
                {
                    return false;
                } /*else*/

                char lo = *(++ptr);
                if (lo >= '0' && lo <= '9') 
                {
                    lo -= '0';
                } /*if*/
                else if (lo >= 'a' && lo <= 'f')
                {
                    lo -= 'a' + 10;
                } /*else if*/
                else if (lo >= 'A' && lo <= 'F')
                {
                    lo -= 'A' + 10;
                } /*else if*/
                else 
                {
                     return false;
                } /*else*/

                addr = addr + (char)( hi * 16 + lo);
            }
            ptr++;
        }
        if (smtpSendSub(addr.c_str()) == false) return false;
    }


    // send message
    if (smtpSendText(msg.c_str()) == false) return false;


    // quit 
    if (smtpSendQuit() == false) return false;

    return true;
}  //useSMTP



///
bool
MailMan::smtpSendConnect(const char * host)
{
    struct sockaddr_in addr;

    // flush any previous connection
    if (mySock != -1)
    {
        shutdown(mySock, SHUT_RDWR);
        mySock = -1;
    }


    // get smtp host's address in IPv4 format
    addr.sin_family = AF_INET;
    //Fix Network Byte Order (25)
#if __BYTE_ORDER == __LITTLE_ENDIAN
    addr.sin_port = 0x1900;
#else
    addr.sin_port = 0x0019;
#endif
    struct hostent * h = gethostbyname(host);
    if (h == 0) return false;
    addr.sin_addr.s_addr = *((long *) h->h_addr);

    // open a new socket endpoint 
    mySock = socket(PF_INET, SOCK_STREAM, 0);
    if (mySock == -1) return false;

    // connect to smtp host
    if (::connect(mySock, (struct sockaddr*)&addr, sizeof(addr)) == -1)
        return false;


    //flush out the reply
    return (flushReply("smtpPort"));
}  //smtpSendConnect



///
bool
MailMan::smtpSendHello(const char * greet)
{
    if (send(mySock, "HELO ", 5, 0) < 5) return false;
    if (send(mySock, greet, strlen(greet), 0) < 1) return false;
    if (send(mySock, "\r\n", 2, 0) < 2) return false;
    return (flushReply("250"));
}  //smtpSendHello


///
bool
MailMan::smtpSendFrom(const char * fromId)
{
    if (send(mySock, "MAIL FROM: ", 11, 0) < 11) return false;
    if (send(mySock, fromId, strlen(fromId), 0) < 1) return false;
    if (send(mySock, "\r\n", 2, 0) < 2) return false;
    return (flushReply("250"));
}  //smtpSendFrom


///
bool
MailMan::smtpSendTo(const char * toId)
{
    if (send(mySock, "RCPT TO: ", 9, 0) < 9) return false;
    if (send(mySock, toId, strlen(toId), 0) < 1) return false;
    if (send(mySock, "\r\n", 2, 0) < 2) return false;
    return (flushReply("250"));
}  //smtpSendTo


///
bool
MailMan::smtpSendData()
{
    if (send(mySock, "DATA\r\n", 6, 0) < 6) return false;
    return (flushReply("354"));
}  //smtpSendData


///
bool
MailMan::smtpSendSub(const char *txt)
{
    if (send(mySock, "Subject: ", 9, 0) < 1) return false;
    if (send(mySock, txt, strlen(txt), 0) < 1) return false;
    if (send(mySock, "\r\n", 2, 0) < 1) return false;
    return true;
} //smtpSendSub


///
bool
MailMan::smtpSendText(const char* txt)
{
    if (send(mySock, txt, strlen(txt), 0) < 1) return false;
    if (send(mySock, "\r\n.\r\n", 5, 0) < 5) return false;
    return (flushReply("250"));
}  //smtpSendText


///
bool
MailMan::smtpSendQuit()
{
    if (send(mySock, "QUIT\r\n", 6, 0) < 6) return false;
    bool retVal = flushReply("221");
    shutdown (mySock, SHUT_RDWR);
    mySock = -1;
    return retVal;
}  //smtpSendQuit


///
bool
MailMan::flushReply(const char *code)
{
    char check[4];
    bool retVal = false;

    // read the reply code
    if (recv(mySock, check, 4, 0) < 4) return retVal;

    // if reply is not as expected (code) then return false
    if (strncmp(check, code, 3) == 0) retVal = true;

    // else flush the rest of the reply text
    while (check[4] == '-')
    {
        do
        {
            if (recv(mySock, check, 1, 0) < 1) return retVal;
        }
        while (check[0] != '\n');
        if (recv(mySock, check, 4, 0) < 4) return retVal;
    }
    do
    {
        if (recv(mySock, check, 1, 0) < 1) return retVal;
    }
    while (check[0] != '\n');

    return retVal;
}  //flushReply


//End of file

