#ifndef _MAIL_MAN__HXX
#define _MAIL_MAN__HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const MailMan_hxx_Version =
    "$Id$";

#include "global.h"
#include <string>

namespace Vocal
{

/**
  *  Used to manage sendmails. 
  */

class MailMan
{
    public:
        /**@name Initialization & Deallocation Functions */
        //@{
        ///Constructor
        MailMan();
        ///Destructor
        ~MailMan();
        //@}

        ///Assignment operator 
        const MailMan& operator=( const MailMan& rhs ); 

        /** Calls SMTP to send mail
          *@param mailTo The Destination party's address
          *@param msg The Message to send
          *@return True if successful sending mail; otherwise False is returned
          *if problems contacting SMTP thread. 
          */
        bool mail(const string& mailTo, const string& msg);

        ///Constant
        static const int smtpPort = 220;

    protected:
        /** Calls SMTP to send mail
          *@param mailTo The Destination party's address
          *@param msg The Message to send
          *@return True if successful sending mail; otherwise False is returned
          *if problems contacting SMTP thread. 
          */
        bool useSMTP(const string& mailtTo, const string& msg);

        /** Create & connect to a socket
         *@param host Host Address to connect to
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendConnect(const char * host);

        /** Send a HELO command
         *@param greet Name of the user logging into SMTP
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendHello(const char *greet);

        /** Send a MAIL FROM command
         *@param fromId Originating party's address
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendFrom(const char * fromId);

        /** Send a RCPT TO command
         *@param toId Destination party's address
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendTo(const char * toId);

        /** Send DATA command
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendData();

        /** Send Subject command
         *@param txt String to be used for mail's subject line
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendSub(const char *txt);

        /** Send the body of the mail message
         *@param txt The mail message
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendText(const char *txt);

        /** Send a QUIT command
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool smtpSendQuit();

        /** Wait for the reply and flush out the socket
         *@param code The code to expect in the reply 
         *@return TRUE if successful; otherwise FALSE is returned.
         */
        bool flushReply(const char* code);

        //Socket identifier
        int mySock;

};
 
} // namespace Vocal

#endif

//End of file

