
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MyTime_cxx_Version =
    "$Id$";
#include "MyTime.hxx"
#include <string.h>
#include <time.h>
#include "cpLog.h" 

using namespace Vocal;


MyTime::MyTime()
{
    time_t junk;

    *this = time(&junk);
}  //Constructor 


MyTime::MyTime(const tm& t)
{
    *this = t;
}  //Constructor 


MyTime::MyTime(const time_t& tSecs)
{
    *this = tSecs;
}  //Constructor 


MyTime&
MyTime::operator= (const tm& t)
{
    tm_sec = t.tm_sec;
    tm_min = t.tm_min;
    tm_hour = t.tm_hour;
    tm_mday = t.tm_mday;
    tm_mon = t.tm_mon;
    tm_year = t.tm_year;
    tm_wday = t.tm_wday;
    tm_isdst = t.tm_isdst;

    return *this;
}  //operator=


MyTime&
MyTime::setYear(const TimeField aVal)
{
    tm_year = aVal;
    return *this;
}  //setYear


MyTime&
MyTime::setMonth(const TimeField aVal)
{
    tm_mon = aVal;
    return *this;
}  //setMonth


MyTime&
MyTime::setDate(const TimeField aVal)
{
    tm_mday = aVal;
    return *this;
}  //setDate


MyTime&
MyTime::setWeekDay(const TimeField aVal)
{
    tm_wday = aVal;
    return *this;
}  //setWeekDay


MyTime&
MyTime::setTimeOfDay(const TimeField aVal)
{
    //Assuming aVal is military time (e.g. 1900), dividing
    //by 100 will get the hour. 
    tm_hour = aVal / 100;
      
    //Assuming aVal is military time (e.g. 1900), modulus
    //by 100 will get the minutes. 
    tm_min = aVal % 100;

    tm_sec = 0;
    return *this;
}  //setTimeOfDay


MyTime&
MyTime::setDayLight(bool aBool)
{
    tm_isdst = (aBool == true ? + 1 : 0);
    return *this;
}  //setDaylight


TimeField
MyTime::getTimeOfDay() const
{
    return (tm_hour*100 + tm_min);
}  //getTimeOfDay


TimeField
MyTime::getWeekDay() const
{
    return tm_wday;
}  //getWeekDay


TimeField
MyTime::getDate() const
{
    return tm_mday;
}  //getDate


TimeField
MyTime::getMonth() const
{
    return tm_mon;
}  //getMonth


TimeField
MyTime::getYear() const
{
    /** Add 1900 because the tm structure defines
        tm_year as years since 1900.             **/
      
    return (tm_year + 1900);

}  //getYear


bool
MyTime::isDayLight() const
{
    if (tm_isdst < 0) 
    {
        cpLog(LOG_DEBUG, "Daylight info unavailable!");
    } /*if*/

    return (tm_isdst > 0 ? true : false);
}  //isDayLight


char *
MyTime::toString()
{
    return asctime(this);
}  //toString


MyTime&
MyTime::operator= (const time_t& tSecs)
{
    *this = *localtime(&tSecs);
    return *this;
}  //operator=

bool
MyTime::operator== (const tm& t)
{
    return (tm_sec == t.tm_sec &&
            tm_min == t.tm_min &&
            tm_hour == t.tm_hour &&
            tm_mday == t.tm_mday &&
            tm_mon == t.tm_mon &&
            tm_year == t.tm_year &&
            tm_wday == t.tm_wday);
}  //operator==


///
bool
MyTime::operator> (const tm& t)
{
    MyTime t1(*this), t2(t);
    return (mktime(&t1) > mktime(&t2));
}  //opertator >


///
bool
MyTime::operator< (const tm& t)
{
    MyTime t1(*this), t2(t);
    return (mktime(&t1) < mktime(&t2));
}  //operator <


///
bool
MyTime::operator>= (const tm& t)
{
    return (*this < t ? false : true);
}  //operator >=


///
bool
MyTime::operator<= (const tm& t)
{
    return (*this > t ? false : true);
}  //operator <=


///
bool
MyTime::operator!= (const tm& t)
{
    return (*this == t ? false : true);
}  //operator !=


//End of file

