/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony;

import javax.telephony.*;
import javax.telephony.capabilities.*;
import com.vovida.telephony.events.*;
import com.vovida.telephony.capabilities.*;
import java.util.*;

/*****************************************************/
/* VCTerminal is the Vocal Client Terminal Class    */
/* which is an implementation of the Terminal        */
/* interface from the JTapi Spec.                    */
/*****************************************************/

/**
 * VCTerminal is the Vocal Client Terminal Class which is an
 * implementation of the Terminal interface from the JTapi Spec
 */

public class VCTerminal implements Terminal
{
    final private static int LOCAL = 1;
    final private static int REMOTE = 2;
    private boolean isLocal;
    protected String name;
    protected Vector addresses;
    protected Vector terminalConnections;
    protected Vector terminalObservers;
    protected Vector callObservers;
    protected Provider provider;
    protected VCDynamicTerminalCapabilities terminalCapabilities;

    /**
     * private default constructor
     */
    private VCTerminal() {}

    /**
     * Constructor
     */
    public VCTerminal (Provider prov, boolean isLocal, String str)
    {
	this(prov, isLocal);
   	name = str;
    }

    /**
     * private constructor
     */
    private VCTerminal (Provider prov, boolean isLocal)
    {
	super();
	this.isLocal = isLocal;
	addresses = new Vector();
	terminalConnections = new Vector();
	terminalObservers = new Vector();
	callObservers = new Vector();
	provider = prov;
	terminalCapabilities = new VCDynamicTerminalCapabilities(this);
    }


/*****************************************************/
/* JTapi Implementation                              */
/*****************************************************/

    /**
     * Refer To JTapi Terminal Interface
     */
    public String getName()
    {
	return name;
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public Provider getProvider()
    {
	if (provider == null)
        {
            return null;
        }
        else
            return provider;
    }


    /**
     * Refer To JTapi Terminal Interface
     */
    public Address[] getAddresses()
    {
	Address[] addressesArray = new Address[addresses.size()];
	addresses.copyInto(addressesArray);
	return addressesArray;
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public TerminalConnection[] getTerminalConnections()
    {
	TerminalConnection[] terminalConnsArray = new
				TerminalConnection[terminalConnections.size()];
	terminalConnections.copyInto(terminalConnsArray);
	return terminalConnsArray;
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public void addObserver(TerminalObserver observer)
    {
	terminalObservers.addElement(observer);
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public TerminalObserver[] getObservers()
    {
	TerminalObserver[] observerArray = new TerminalObserver[terminalObservers.size()];
	terminalObservers.copyInto(observerArray);
	return observerArray;
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public void removeObserver(TerminalObserver observer)
    {
	terminalObservers.removeElement(observer);
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public void addCallObserver(CallObserver observer)
        throws ResourceUnavailableException,MethodNotSupportedException
    {
        callObservers.addElement(observer);
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public CallObserver[] getCallObservers()
    {
        CallObserver[] observersArray = new CallObserver[callObservers.size()];
        callObservers.copyInto(observersArray);
        return observersArray;
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public void removeCallObserver(CallObserver observer)
    {
        callObservers.removeElement(observer);
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public TerminalCapabilities getCapabilities()
    {
	return terminalCapabilities;
    }

    /**
     * Refer To JTapi Terminal Interface
     */
    public TerminalCapabilities getTerminalCapabilities(Terminal terminal, Address address)
	throws InvalidArgumentException, PlatformException
    {
	return null;
    }


/***************************************************************/
/* Additional functionality                                    */
/***************************************************************/

    /**
     * Check if Terminal is local
     */
    public boolean isLocal()
    {
	return isLocal;
    }

    /**
     * Add(associate) an address to the Terminal
     */
    protected void addAddress(Address address)
    {
	addresses.addElement(address);
    }

    /**
     * Add a TerminalConnection to Terminal
     */
    protected void addTerminalConnection(TerminalConnection termconn)
    {
	terminalConnections.addElement(termconn);
    }

    /**
     * Remove a TerminalConnection from the Terminal
     */
    protected void removeTerminalConnection(TerminalConnection termconn)
    {
	terminalConnections.removeElement(termconn);
    }

    /**
     * Check if the given address has been associated with the terminal
     */
    public boolean validAddress(Address addr)
    {
	if (addresses.contains(addr))
	    return true;
	else
	    return false;
    }

    /**
     * check that provider is in a valid state
     * if not then throw an Invalid StateException
     */
    protected void isProviderOK() throws InvalidStateException
    {
        if (this.getProvider().getState() != Provider.IN_SERVICE)
        {
            throw (new InvalidStateException(this.getProvider(),
                                        InvalidStateException.CALL_OBJECT,
                                        this.getProvider().getState()));
        }
        return;
    }

}


