/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.telephony.callcontrol;

import javax.telephony.*;
import javax.telephony.callcontrol.*;
import com.vovida.telephony.*;
import com.vovida.telephony.callcontrol.capabilities.*;
import java.util.*;


/**********************************************************/
/* VCCallControlAddress is the Vocal Client Call Control */
/* Address Class which  is an implementation              */
/* of the Call Control Address interface from the         */
/* JTapi Spec.                                            */
/**********************************************************/

/**
 * Implementation of the Jtapi CallControlAddress Interface
 * 
 */
public class VCCallControlAddress extends VCAddress implements CallControlAddress
{
    boolean doNotDisturb;
    boolean messageWaiting;
    CallControlForwarding[] callControlForwardings = { null, null, null};

    /**
     * Constructor
     */
    public VCCallControlAddress(Provider p, boolean isLocal, String number )
    {
	super(p, isLocal, number);
	doNotDisturb = false;
	messageWaiting = false;
	addressCapabilities = new VCDynamicCallControlAddressCapabilities(this);
    }


/************************************************************/
/* JTapi Implementation                                     */
/************************************************************/


    // The stored CallControlForwarding[] array consists of 3 elements
    // 1. Forward All Calls
    // 2. Forward on Busy
    // 3. Forward no Answer
    // There is no need to store more than one element for each of these 
    // elements - in fact it does not make sense

    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public void setForwarding(CallControlForwarding[] instructions)
	throws MethodNotSupportedException, InvalidStateException,
	    InvalidArgumentException
    {
	isProviderOK();
	
        // send a request to the server to reset the forwarding features
        // with a reference to this address - so forwarding features
        // can be read.

	for(int i =0; i < instructions.length; i++)
	{
	    int type = instructions[i].getType();
	    if (type == CallControlForwarding.FORWARD_UNCONDITIONALLY)
	    {
		callControlForwardings[0] = instructions[i];
	    }
            if (type == CallControlForwarding.FORWARD_ON_BUSY)
            {
                callControlForwardings[1] = instructions[i];
            }
            if (type == CallControlForwarding.FORWARD_ON_NOANSWER)
            {
                callControlForwardings[2] = instructions[i];
            }
	}

    }


    // return an array of the forwarding features for this address
    // if no features set then return null.

    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public CallControlForwarding[] getForwarding()
        throws MethodNotSupportedException
    {
	CallControlForwarding[] instructions = null;
	Vector forwardings = new Vector();
	forwardings = null;
	for (int i=0; i < callControlForwardings.length; i++)
	{
	    if (callControlForwardings[i] != null)
	    {
		forwardings.addElement(callControlForwardings[i]);
	    }
	}
	if (forwardings.size() > 0)
	{
	    instructions = new CallControlForwarding[forwardings.size()];
	    forwardings.copyInto(instructions);
	}
	return instructions;
    }
	    
	
    // cancel the forwarding feature by setting to null each of the
    // elements of the callControlForwardings array.

    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public void cancelForwarding()
	throws MethodNotSupportedException, InvalidStateException
    {
	isProviderOK();
	for (int i=0; i < callControlForwardings.length; i++)
	{
	    callControlForwardings[i] = null;
	}
    }

    // return the DoNotDisturb attribute for this address
    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public boolean getDoNotDisturb() throws MethodNotSupportedException
    {
	return doNotDisturb;
    }

    // set the DoNotDisturb attribute for this address
    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public void setDoNotDisturb(boolean enable)
	throws MethodNotSupportedException, InvalidStateException
    {
	isProviderOK();
	doNotDisturb = enable;
    }

    // return the message waiting attribute for this address
    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public boolean getMessageWaiting() throws MethodNotSupportedException
    {
	return messageWaiting;
    }

    // set the message waiting attribute for this address
    /**
     * Refer To JTapi CallControlAddress Interface
     */
    public void setMessageWaiting(boolean enable)
	throws MethodNotSupportedException, InvalidStateException
    {
	isProviderOK();
	messageWaiting = enable;
    }

/**********************************************************/
/* Additional Functionality                               */
/**********************************************************/

    // initForwarding is used to set the Forwarding features for
    // the address when the app is initiated or when these features
    // are set from the switch side during operation

    /**
     * initForwarding is used to set the Forwarding features for
     * the address when the app is initiated or when these features
     * are set from the switch side during operation
     */
    public void initForwarding(CallControlForwarding[] instructions)
        throws MethodNotSupportedException, InvalidStateException,
            InvalidArgumentException
    {
        for(int i =0; i < instructions.length; i++)
        {
            int type = instructions[i].getType();
            if (type == CallControlForwarding.FORWARD_UNCONDITIONALLY)
            {
                callControlForwardings[0] = instructions[i];
            }
            else if (type == CallControlForwarding.FORWARD_ON_BUSY)
            {
                callControlForwardings[1] = instructions[i];
            }
            else if (type == CallControlForwarding.FORWARD_ON_NOANSWER)
            {
                callControlForwardings[2] = instructions[i];
            }
        }
    }
}
