/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.vocalpad;

import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.callcontrol.*;
import javax.telephony.callcontrol.events.*;
import com.vovida.telephony.*;
import com.vovida.telephony.events.*;


// Class VCCallControlCallObserver checks for CallControl events
// for calls, connections and terminalconnections. All basic events
// are handled by the VCCallObserver class which is the superclass
// to this class.
 
/**
 * Class VCCallControlCallObserver checks for CallControl events for
 * calls, connections and terminalconnections
 */
public class VCCallControlCallObserver extends VCCallObserver implements CallControlCallObserver
{

    /**
     * Constructor
     */
    public VCCallControlCallObserver()
    {
	super();
    }


    /**
     * Refer To JTapi CallControlCallObserver Interface
     */
    public void callChangedEvent(CallEv[] eventList)
    {
	    super.callChangedEvent(eventList);
	    System.out.println("CallControlObserver - Change Event");
	    for (int i=0; i < eventList.length; i++)
	    {
	        if (eventList[i] instanceof CallCtlCallEv)
	        {
	            try
	            {
		            Call call = ((CallEv)eventList[i]).getCall();
		
	                Connection[] conns = call.getConnections();
                    int eventId = eventList[i].getID();
		
		            if (conns == null)
		            {
			            System.out.println("CtrlObserver: No connections exist");	
		            }
                    else if (eventId == CallCtlConnAlertingEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Alerting");
                    }
                    else if (eventId == CallCtlConnDialingEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Dialing");
                    }
                    else if (eventId == CallCtlConnDisconnectedEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Disc");
                    }
                    else if (eventId == CallCtlConnEstablishedEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Establish");
                    }
                    else if (eventId == CallCtlConnFailedEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Failed");
                    }
		            else if (eventId == CallCtlConnInitiatedEv.ID)
		            {
		                System.out.println("CtrlObserver:Conn Init");
		            }
                    else if (eventId == CallCtlConnNetworkAlertingEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Nwk Alrt");
                    }
                    else if (eventId == CallCtlConnNetworkReachedEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Nwk Reach");
                    }
		            else if (eventId == CallCtlConnOfferedEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Offered");
                    }
                    else if (eventId == CallCtlConnQueuedEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Queued");
                    }
                    else if (eventId == CallCtlConnUnknownEv.ID)
                    {
                        System.out.println("CtrlObserver:Conn Unknown");
                    }

                    else if (eventId == CallCtlTermConnBridgedEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Bridged");
                    }
		            else if (eventId == CallCtlTermConnDroppedEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Dropped");
                    }
                    else if (eventId == CallCtlTermConnHeldEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Held");
                    }
                    else if (eventId == CallCtlTermConnInUseEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Inuse");
                    }
                    else if (eventId == CallCtlTermConnRingingEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Ringing");
                    }
                    else if (eventId == CallCtlTermConnTalkingEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Talking");
                    }
                    else if (eventId == CallCtlTermConnUnknownEv.ID)
                    {
                        System.out.println("CtrlObserver:TermConn Unknown");
                    }
	            }
	            catch (Exception e)
	            {
		            e.printStackTrace();
		            return;
	            }
	        }

	    }
    }

}
		
