/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package com.vovida.vocalpad;

import javax.swing.*;
import javax.telephony.*;
import javax.telephony.events.*;
import com.vovida.telephony.*;
import com.vovida.telephony.events.*;


// Class VCCallObserver handles all basic telephony events for a call,
// connection and terminalconnection. Depending on the event received
// the associated application is called for update due to the 
// received event.

/**
 * Class VCCallObserver handles all basic telephony events for a call,
 * connection and terminalconnection
 */
public class VCCallObserver implements CallObserver, PadValues
{
    Japp app = null;

    /**
     * Constructor
     */
    public VCCallObserver()
    {
	super();
    }

    /**
     * Refer To JTapi CallObserver Interface
     */
    public void callChangedEvent(CallEv[] eventList)
    {
	System.out.println("Call Observer - Change Event");
	for (int i=0; i < eventList.length; i++)
	{
        int eventId = eventList[i].getID();
	    // check for a Call event
	    if (eventList[i] instanceof CallEv)
	    {
		    Japp appl = getApp();
		    System.out.println("Application State : " + appl.getState());
	        try
	        {
		        Call call = (eventList[i]).getCall();
		
	            Connection[] conns = call.getConnections();
		
		        if (conns == null)
		        {
			        System.out.println("Call Observer - Call contains no connections");	
		        }
		        else if (eventId == CallActiveEv.ID)
		        {
			        // Call becomes active - meaning that it is
			        // created and the connections are to be 
			        // connected...

			        appl.vocalPad.setPadToActiveCall(call);
		            System.out.println("Call Observer:Call Set To Active");
		        }
		        else if (eventId == CallInvalidEv.ID)
		        {
		            System.out.println("Call Observer:Call Invalid");
		        }
	        }
	        catch (Exception e)
	        {
		        e.printStackTrace();
		        return;
	        }
	    }

	    // check for a Connection event
	    if (eventList[i] instanceof ConnEv)
	    {
		    Japp appl = getApp();
		    try
		    {
		        Connection conn = ((ConnEv)eventList[i]).getConnection();
		        Call call = conn.getCall();
		        Address addr = conn.getAddress();
		    
		        if (eventId == ConnCreatedEv.ID)
		        {
			        System.out.println("Call Observer:Connection Created");
		        }
		        else if (eventId == ConnAlertingEv.ID)
		        {
			        // connection goes to alerting state
			        // if state is not set by now then no InPrpgress
			        // msgs have been received - so set accordingly
			        System.out.println("Call Observer:Connection Alerting");
			        if ((appl.getState() != MAKECALL) && 
				        (appl.getState() != ANSWERCALL))
			        {
                        if (conn.getAddress() == appl.getOrigAddress())
                        {
                            appl.vocalPad.setPadToRecvCall();
                        }
                    }
		        }
		        else if (eventId == ConnInProgressEv.ID)
		        {
			        // connection goes to inprogress state and so
			        // meaning that the destination is being contacted

			        // if the call originates at this app
			        // update the app accordingly passing the called 
			        // address name (ie. number)
			        if (appl.getState() == MAKECALL)
			        {
			            if (conn.getAddress() == appl.getOrigAddress())
			            {
				            Connection[] conns = call.getConnections();
				            String calledName = conns[1].getAddress().getName();
                            if (calledName != null)
                            {
				                appl.vocalPad.setPadToInProgress(call, calledName);
                            }
			            }
			        }
			        // if the call terminates at this app
			        // update the app accordingly to receive call
			        else
			        {
			            if (conn.getAddress() == appl.getOrigAddress())
			            {
				            appl.vocalPad.setPadToRecvCall();
			            }
			        }
			        System.out.println("Call Observer:Connection InProgress");
		        }
		        else if (eventId == ConnConnectedEv.ID)
		        {
			        // if a connection becomes active while
			        // the associated app is IDLE then tis is
			        // a call originating at the app's set
			        // but not at the app..
			        // Update the app to a state in compliance
			        // with making call.

			        System.out.println("Call Observer:Connection Connected");
			        if (appl.getState() == IDLE)
		  	        {
			            if (conn.getAddress() == appl.getOrigAddress())
			            {
				            Connection[] conns = call.getConnections();
				            String calledName = conns[1].getAddress().getName();
				            appl.vocalPad.setPadToInProgress(call, calledName);
			            }
			        }
		        }
		        else if (eventId == ConnDisconnectedEv.ID)
		        {
			        if (appl.getState() == TALKING)
			        {
			            if (conn.getAddress() == appl.getOrigAddress())
			            {
				            appl.vocalPad.setPadToIdle();
			            }
			        }
			        System.out.println("Call Observer:Connection Disconnected");
		        }
		        else if (eventId == ConnUnknownEv.ID)
		        {
			        System.out.println("Call Observer:Connection Unknown");	
		        }
		        else if (eventId == ConnFailedEv.ID)
		        {
			        if (appl.getState() == MAKECALL)
			        {
			            if (conn.getAddress() == appl.getOrigAddress())
			            {
				            appl.vocalPad.setPadToFail();
			            }
			            else
			            {
				            appl.vocalPad.setPadToBusy();
			            }
			        }
			        if (appl.getState() == ANSWERCALL)
			        {
			            if (conn.getAddress() == appl.getOrigAddress())
			            {
				            appl.vocalPad.setPadToFail();
			            }
			            else
			            {
				            appl.vocalPad.setPadToFail();
			            }
			        }
			        System.out.println("Call Observer:Connection Failed");
		        }
		    }
		    catch (Exception e)
		    {
		        e.printStackTrace();
                return;
		    }
	    }

	    // check for a terminal connection event
	    if (eventList[i] instanceof TermConnEv)
	    {
		    Japp appl = getApp();
		    try
		    {
		        TerminalConnection termConn = ((TermConnEv)eventList[i]).getTerminalConnection();
		        Connection conn = termConn.getConnection();
		        Terminal term = termConn.getTerminal();
		        Call call = conn.getCall();
		    
		        if (eventId == TermConnCreatedEv.ID)
		        {
			        System.out.println("Call Observer:TermConn Created");
		        }
		        else if (eventId == TermConnRingingEv.ID)
		        {
			        System.out.println(appl.getState());
			        if (appl.getState() == ANSWERCALL)  //ALERTING
			        {
			            if (term == appl.getOrigTerminal())
			            {
				            if (conn.getAddress() == appl.getOrigAddress())
				            {
				                Connection[] conns = call.getConnections();
				                String caller = conns[0].getAddress().getName();
				                appl.setHomeTC(termConn);
				                appl.vocalPad.setPadToRing(call, caller);
				            }
			            }
			        }
			        System.out.println("Call Observer:TermConn Ringing");
			
		        }
		        else if (eventId == TermConnActiveEv.ID)
		        {
			        System.out.println("Call Observer:TermConn Active");
			        if ((appl.getState() == ANSWERCALL) ||
			            (appl.getState() == MAKECALL))
			        {
			            if (term == appl.getOrigTerminal())
			            {
				            if (conn.getAddress() == appl.getOrigAddress())
				            {
				                if (appl.getHomeTC() != null)
				                {
				                    if (termConn == appl.getHomeTC())
				                    {
					                    appl.vocalPad.setPadToAnswered();
				                    }
				                }
				                else
				                {
					                appl.setHomeTC(termConn);
					                appl.vocalPad.setPadToAnswered();
				                }
				            }
			            }
			        }
		        }
		        else if (eventId == TermConnPassiveEv.ID)
		        {
			        System.out.println("Call Observer:TermConn Passive");
		        }
		        else if (eventId == TermConnDroppedEv.ID)
		        {
			        System.out.println("Call Observer:TermConn Dropped");
		        }
		    }
		    catch (Exception e)
		    {
		        e.printStackTrace();
                return;
		    }
			
  	    }

	}
    }

    /**
     * Set the application object
     */
    public void setApp(Japp app)
    {
	    this.app = app;
    }

    /**
     * Get the application object
     */
    public Japp getApp()
    {
	    return this.app;
    }

}
		
