
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpBusyBye_cxx_Version =
    "$Id: JSOpBusyBye.cxx,v 1.11 2002/02/25 05:51:16 sprajpat Exp $";
#include "JSOpBusyBye.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
//kevin
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"

///
JSOpBusyBye::JSOpBusyBye()
{}


///

JSOpBusyBye::~JSOpBusyBye()
{}


///

const char* const
JSOpBusyBye::name() const
{
    return "JSOpBusyBye";
}


///
const Sptr < State >
JSOpBusyBye::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive BYE from the caller
    // 2. Sned 200 for the BYE
    // 3. Change state to unknown

    cpLog( LOG_DEBUG, "JSOpBusyBye operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0);

    Sptr < ByeMsg > bye;
    bye.dynamicCast(sipMsg);
    if ( bye == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );
    JSProvider* provider = call->getProvider();

    if ( !(call->getControlled()) ) // Non-controlled call, i.e. NJS-JS
    {
        cpLog(LOG_DEBUG, "JSOpBusyBye: receive BYE message for uncontrolled call");
        SipCallId callId = sipMsg->getCallId();

        if ( (callId != call->getToCallID()) &&
                (callId != call->getFromCallID()) )
        {
            cpLog(LOG_ERR, "JSOpBusyBye: SIP call id doesnt' match to or from");
            cpLog(LOG_ERR, "JSOpBusyBye: message call id %s", callId.getData().logData() );
            cpLog(LOG_ERR, "JSOpBusyBye: from call id %s", call->getFromCallID().getData().logData() );
            cpLog(LOG_ERR, "JSOpBusyBye: to call id %s", call->getFromCallID().getData().logData() );
            // Change state to unknown to terminate this call
            // Done at the end
        }


        if ( callId == call->getToCallID() )
        {
            cpLog(LOG_ERR,
                  "JSOpBusyBye: receive bye from CALLEE which is impossible");
            // Change state to unknown to terminate this call
            // Done at the end
        }

        else if ( callId == call->getFromCallID() )
        {
            cpLog(LOG_DEBUG, "JSOpBusyBye: receive bye from CALLER");

            StatusMsg newStatus( *bye, 200 );

            Sptr <SipUrl> myUrl = new SipUrl;
            myUrl->setUserValue( provider->getConfig()->getUserName() );
            myUrl->setHost( Data( theSystem.gethostAddress() ) );
            myUrl->setPort( provider->getConfig()->getJSDSipPort() );

            SipContact me;
            me.setUrl( myUrl );
            newStatus.setNumContact( 0 );    // Clear
            newStatus.setContact( me );

            cpLog(LOG_DEBUG, "JSOpByeOk: new 200 to caller:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendReply( newStatus );
        }
    }
    else // Controlled call, i.e. JS-JS
    {
        cpLog(LOG_DEBUG, "JSOpBusyBye: receive BYE message for controlled call");

        Sptr <BaseUrl> baseUrl;
        baseUrl = bye->getRequestLine().getUrl();
        assert( baseUrl.getPtr() != 0 );
        assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

        Sptr <SipUrl> sipUrl;
        assert( sipUrl.dynamicCast( baseUrl ) != 0 );

        Data viaPort = sipUrl->getPort();
        if ( viaPort.length() == 0 )
            viaPort = 5060;

        cpLog(LOG_DEBUG, "Message received on port: %s", viaPort.logData());

        if ( (viaPort != provider->getConfig()->getJSDSipPort()) &&
                (viaPort != provider->getConfig()->getJSNSipPort()) )
        {
            cpLog(LOG_ERR, "JSOpBusyBye: BYE received on unknow port");
            // Change state to unknown to terminate this call
            // Done at the end
        }


        if ( viaPort == provider->getConfig()->getJSDSipPort() )  // JSD
        {
            cpLog(LOG_ERR, "BYE message received on JSD, impossible, move to unknown state anyway");
            // Change state to unknown to terminate this call
            // Done at the end
        }

        else // JSN
        {
            SipCallId callId = sipMsg->getCallId();

            if ( callId == call->getToCallID() )
            {
                cpLog(LOG_ERR, "BYE message from callee, impossible, move to unknown state anyway");
                // Change state to unknown to terminate this call
                // Done at the end
            }

            else
            {
                StatusMsg newStatus( *bye, 200 );

                Sptr <SipUrl> myUrl = new SipUrl;
                myUrl->setUserValue( provider->getConfig()->getUserName() );
                myUrl->setHost( Data( theSystem.gethostAddress() ) );
                myUrl->setPort( provider->getConfig()->getJSNSipPort() );

                SipContact me;
                me.setUrl( myUrl );
                newStatus.setNumContact( 0 );    // Clear
                newStatus.setContact( me );

                cpLog(LOG_DEBUG, "JSOpByeOk: new 200 to caller:");
                cpLog(LOG_DEBUG, "***************************************");
                cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
                cpLog(LOG_DEBUG, "***************************************");

                sipEvent->getSipStack()->sendReply( newStatus );
            }
        }
    }

    // Change state to unknown
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateUnknown" );
}
