#ifndef JSOPBYE_HXX
#define JSOPBYE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const JSOpBye_hxx_Version =
    "$Id$";




#include <Operator.hxx>
#include <SipProxyEvent.hxx>


/** Operator class :
 *	Here is the logig of handling BYE message:
 *	1. Check if this is a controlled call
 *	2. If NO, check which direction the message is from, call or callee,
 *		we don't have to worry about the case of the callee use the
 *		vocalpad to disconnect the call. Check how JSProvider
 *		handle the disconnect request from vocalpad.
 *		Checking direction by using the SIP call id.
 *		2.1 If from callee, means callee on-hook rather than use vocalpad
 *			2.1.1 Notify vocalpad that user on-hook
 *			2.1.2 Create ans send new BYE to caller
 *			2.1.3 Change state to "disconnected"
 *		2.2 If from caller,
 *			2.2.1 Create ans send a new BYE to callee
 *			2.2.2 Change state to "disconnected"
 *	3. If YES, check which on which port the message is received, JSD or JSN
 *	4. JSD,
 *		4.1 Add via to the message
 *		4.2 Send async, Sip stack will know where to send
 *	5. JSN, check which direction the message is from by using SIP call id
 *		5.1 If from caller
 *			5.1.1 Create and send new BYE to  callee
 *			5.1.2 Change state to "disconnected"
 *		5.2 If from callee
 *			5.1.1 Create and send new BYE to  caller
 *			5.1.2 Change state to "disconnected"  
 */
class JSOpBye : public Operator
{
    public:
        ///
        JSOpBye();
        ///
        virtual ~JSOpBye();
        ///
        const char* const name() const;
        ///
        const Sptr < State > process( const Sptr < SipProxyEvent > event );
};

#endif
