
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpByeOk_cxx_Version =
    "$Id$";
#include "JSOpByeOk.hxx"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "SipVia.hxx"
#include "JSCall.hxx"
#include "SipRequestLine.hxx"
#include "SipUrl.hxx"
#include "StatusMsg.hxx"
#include "JSStateUnknown.hxx"
//kevin
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpByeOk::JSOpByeOk()
{}


///

JSOpByeOk::~JSOpByeOk()
{}


///

const char* const
JSOpByeOk::name() const
{
    return "JSOpByeOk";
}


///
const Sptr < State >
JSOpByeOk::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 200 for BYE
    // 2. Similar to the algorithm in JSOpBye, only this time handling 200

    cpLog( LOG_DEBUG, "JSOpByeOk operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        // Not SIP message
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0);

    Sptr < StatusMsg > status;
    status.dynamicCast(sipMsg);
    if ( status == 0 )
    {
        return 0;
    }

    Data method = sipMsg->getCSeq().getMethod();

    if ( method != "BYE" )
    {
        // Only handle 200 for BYE here
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );
    JSProvider* provider = call->getProvider();

    if ( !(call->getControlled()) ) // Non-controlled call, i.e. NJS-JS
    {
        SipCallId callId = sipMsg->getCallId();

        if ( (callId != call->getToCallID()) &&
                (callId != call->getFromCallID()) )
        {
            cpLog(LOG_ERR, "JSOpByeOk: SIP call id doesnt' match tp or from");
            // Change state to unknown to terminate this call
            Sptr < JSStateMachine > stateMachine;
            stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
            assert( stateMachine != 0 );

            return stateMachine->findState( "JSStateUnknown" );
        }

        if ( callId == call->getToCallID() )
        {
            // Create and send new 200 to caller
            Sptr < ByeMsg > fromBye = call->getFromBye();
            StatusMsg newStatus( *fromBye, 200 );

            // Add contact
            Sptr <SipUrl> myUrl = new SipUrl;
            myUrl->setUserValue( provider->getConfig()->getUserName() );
            myUrl->setHost( Data( theSystem.gethostAddress() ) );
            myUrl->setPort( provider->getConfig()->getJSDSipPort() );

            SipContact me;
            me.setUrl( myUrl );
            newStatus.setNumContact( 0 );    // Clear
            newStatus.setContact( me );

            cpLog(LOG_DEBUG, "JSOpByeOk: new 200 to caller:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendReply( newStatus );
        }
        else if ( callId == call->getFromCallID() )
        {
            Sptr < ByeMsg > toBye = call->getToBye();
            StatusMsg newStatus( *toBye, 200 );

            // Add contact
            Sptr <SipUrl> myUrl = new SipUrl;
            myUrl->setUserValue( provider->getConfig()->getUserName() );
            myUrl->setHost( Data( theSystem.gethostAddress() ) );
            myUrl->setPort( provider->getConfig()->getJSDSipPort() );

            SipContact me;
            me.setUrl( myUrl );
            newStatus.setNumContact( 0 );    // Clear
            newStatus.setContact( me );

            cpLog(LOG_DEBUG, "JSOpByeOk: new 200 to callee:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendReply( newStatus );
        }

    }
    else // Controlled call, i.e. JS-JS
    {
        int viaPort = sipMsg->getVia(0).getPort();

        cpLog(LOG_DEBUG, "Message received on port: %d", viaPort);

        if ( (viaPort != provider->getConfig()->getJSDSipPort().convertInt()) &&
                (viaPort != provider->getConfig()->getJSNSipPort().convertInt()) )
        {
            cpLog(LOG_ERR, "JSOpByeOk: 200 received on unknow port");
            // Change state to unknown to terminate this call
            Sptr < JSStateMachine > stateMachine;
            stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
            assert( stateMachine != 0 );

            return stateMachine->findState( "JSStateUnknown" );
        }

        if ( viaPort == provider->getConfig()->getJSDSipPort().convertInt() )
        {
            // Remove via
            status->removeVia( 0 );

            cpLog(LOG_DEBUG, "JSOpByeOk: JSD forward 200 to next hop:");
            cpLog(LOG_DEBUG, "***************************************");
            cpLog(LOG_DEBUG, "%s", status->encode().logData());
            cpLog(LOG_DEBUG, "***************************************");

            sipEvent->getSipStack()->sendReply( *status );

            return 0;  // Remain in the disconnected state
        }

        else // JSN
        {
            SipCallId callId = sipMsg->getCallId();

            if ( (callId != call->getToCallID()) &&
                    callId != call->getFromCallID() )
            {
                cpLog(LOG_ERR, "JSOpByeOk: SIP call id doesnt' match tp or from");
                // Change state to unknown to terminate this call
                Sptr < JSStateMachine > stateMachine;
                stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
                assert( stateMachine != 0 );

                return stateMachine->findState( "JSStateUnknown" );
            }

            if ( callId == call->getToCallID() )
            {
                // Create and send new 200 to caller
                Sptr < ByeMsg > fromBye = call->getFromBye();

                // Use the constructor with SipCommand
                StatusMsg newStatus( *fromBye, 200 );

                // Add contact
                Sptr <SipUrl> myUrl = new SipUrl;
                myUrl->setUserValue( provider->getConfig()->getUserName() );
                myUrl->setHost( Data( theSystem.gethostAddress() ) );
                myUrl->setPort( provider->getConfig()->getJSNSipPort() );

                SipContact me;
                me.setUrl( myUrl );
                newStatus.setNumContact( 0 );    // Clear
                newStatus.setContact( me );

                cpLog(LOG_DEBUG, "JSOpByeOk: new 200 to caller as JSN:");
                cpLog(LOG_DEBUG, "***************************************");
                cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
                cpLog(LOG_DEBUG, "***************************************");

                sipEvent->getSipStack()->sendReply( newStatus );
            }
            else
            {
                // Create and send new 200 to callee
                Sptr < ByeMsg > toBye = call->getToBye();

                StatusMsg newStatus( *toBye, 200 );

                // Add contact
                Sptr <SipUrl> myUrl = new SipUrl;
                myUrl->setUserValue( provider->getConfig()->getUserName() );
                myUrl->setHost( Data( theSystem.gethostAddress() ) );
                myUrl->setPort( provider->getConfig()->getJSNSipPort() );
                SipContact me;
                me.setUrl( myUrl );
                newStatus.setNumContact( 0 );    // Clear
                newStatus.setContact( me );

                cpLog(LOG_DEBUG, "JSOpByeOk: new 200 to callee as JSN:");
                cpLog(LOG_DEBUG, "***************************************");
                cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
                cpLog(LOG_DEBUG, "***************************************");
                sipEvent->getSipStack()->sendReply( newStatus );
            }
        }
    }

    // Change state to unknown
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateUnknown" );
}
