
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpCalleeBusy_cxx_Version =
    "$Id: JSOpCalleeBusy.cxx,v 1.11 2002/02/25 05:51:17 sprajpat Exp $";
#include "JSOpCalleeBusy.hxx"
#include "SipEvent.hxx"
#include "SipVia.hxx"
#include "StatusMsg.hxx"
#include "JSCall.hxx"
#include "SipSdp.hxx"
#include "SipContact.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpCalleeBusy::JSOpCalleeBusy()
{}


///

JSOpCalleeBusy::~JSOpCalleeBusy()
{}


///

const char* const
JSOpCalleeBusy::name() const
{
    return "JSOpCalleeBusy";
}


///
const Sptr < State >
JSOpCalleeBusy::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 486, callee busy
    // 2. Check if received as JSD or JSN
    // 3. If JSD, then just forward the message to previous hop
    // 4. If JSN, then create new 486 and send it to caller

    cpLog( LOG_DEBUG, "JSOpCalleeBusy operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > status;
    status.dynamicCast( sipMsg );
    if ( status == 0 )
    {
        return 0;
    }

    if ( status->getStatusLine().getStatusCode() != 486 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    JSProvider* provider = call->getProvider();

    Data viaPort = sipMsg->getVia(0).getPort();
    cpLog(LOG_DEBUG, "Message received on via port :%s", viaPort.logData() );

    // need the udpStack, dest Network Address and request
    // to send a notification to client
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();

    JSRequest* req = call->getRequest();
    NetworkAddress dest;
    string callId;
    if ( viaPort == provider->getConfig()->getJSDSipPort() )
    {
        dest = call->getReceiver();
        callId = req->getRecvClientCallID();
        if (dest.getPort() == -1)
        {
            cpLog( LOG_DEBUG, "JSOpCalleeBusy port not set - Callee NJS??");
        }
    }
    else if ( viaPort == provider->getConfig()->getJSNSipPort() )
    {
        dest = call->getSender();
        callId = req->getClientCallID();
    }
    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpCalleeBusy port not set - no notifications" );
    }
    else
    {
        JSNotification notification(clientUdp, dest);
        notification.setType(CONNFAILED);
        notification.setCall(callId);
        notification.setAddr(req->getCalleeAddr());
        notification.send();
        cpLog(LOG_DEBUG, "Send CONNFAILED notif to client");
        cpLog(LOG_DEBUG, "%s", notification.getNotificationString().c_str());
        // No need to notify the callee that he is busy
    }


    if ( viaPort == provider->getConfig()->getJSDSipPort() )	// JSD
    {
        // Forward the message to previous hop
        status->removeVia( 0 );
        cpLog(LOG_DEBUG, "JSOpCalleeBusy: relay 486 to preivous hop:");
        cpLog(LOG_DEBUG, "***************************************");
        cpLog(LOG_DEBUG, "%s", status->encode().logData());
        cpLog(LOG_DEBUG, "***************************************");
        sipEvent->getSipStack()->sendReply( *status );
    }
    else if ( viaPort == provider->getConfig()->getJSNSipPort() ) // JSN
    {
        Sptr < InviteMsg > invite = call->getFromInvite();
        StatusMsg newstatus( *invite, 486/* BUSY */ );

        // Add contact to the new 486
        Sptr <SipUrl> myUrl = new SipUrl;
        myUrl->setUserValue( provider->getConfig()->getUserName() );
        myUrl->setHost( Data( theSystem.gethostAddress() ) );
        myUrl->setPort( provider->getConfig()->getJSNSipPort() );

        SipContact me;
        me.setUrl( myUrl );
        newstatus.setNumContact( 0 );    // Clear
        newstatus.setContact( me );

        cpLog(LOG_DEBUG, "JSOpCalleeBusy: new 486 to caller:");
        cpLog(LOG_DEBUG, "***************************************");
        cpLog(LOG_DEBUG, "%s", newstatus.encode().logData());
        cpLog(LOG_DEBUG, "***************************************");

        sipEvent->getSipStack()->sendReply( newstatus );

        // Change state to Callee Busy
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateCalleeBusy" );
    }
    else
    {
        cpLog(LOG_ERR, "JSOpCalleeBusy: unknow via port: %s", viaPort.logData() );
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }
    return 0;
}
