
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpClientError_cxx_Version =
    "$Id: JSOpClientError.cxx,v 1.12 2002/02/25 05:51:17 sprajpat Exp $";
#include "JSOpClientError.hxx"
#include "SipEvent.hxx"
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "StatusMsg.hxx"
#include "InviteMsg.hxx"
#include "AckMsg.hxx"
#include "SipRequestLine.hxx"
#include "SipContact.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"


///
JSOpClientError::JSOpClientError()
{}


///

JSOpClientError::~JSOpClientError()
{}


///

const char* const
JSOpClientError::name() const
{
    return "JSOpClientError";
}


///
const Sptr < State >
JSOpClientError::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 4xx
    // 2. ACK the message
    // 3. Change state to unknown

    cpLog( LOG_DEBUG, "JSOpClientError operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > statusMsg;
    statusMsg.dynamicCast( sipMsg );
    if ( statusMsg == 0 )
    {
        return 0;
    }

    if ( (statusMsg->getStatusLine().getStatusCode() < 400) ||
            (statusMsg->getStatusLine().getStatusCode() == 486) )
    {
        // Only handle error messages, i.e. >= 400
        // 486 busy here is handled in operator JSOpCalleeBusy
        return 0;
    }

    // Just change state to unknown except for INVITE , in which case
    // we need to ACK first then change state
    // In current phase, our handling of error messages, except 486,
    // is to change state to unknown, i.e. terminate the call by deleting
    // the call object.

    cpLog(LOG_ERR, "Receive error status message: %d",
          statusMsg->getStatusLine().getStatusCode());

    Data invMethod(INVITE_METHOD);
    Data method = statusMsg->getCSeq().getMethod();

    if ( method == invMethod )
    {
        Sptr < JSCall > call;
        call.dynamicCast( event->getCallInfo() );
        assert( call != 0 );

        Sptr < InviteMsg > invite = sipEvent->getInvite();
        assert( invite != 0 );

        AckMsg ack( *statusMsg );

        Sptr <BaseUrl> baseUrl;
        baseUrl = invite->getRequestLine().getUrl();
        assert( baseUrl.getPtr() != 0 );
        assert( baseUrl->getType() == SIP_URL ); // JTAPI doesn't support TEL_URL

        SipRequestLine ackRequestLine( SIP_ACK, baseUrl );

        ack.setRequestLine( ackRequestLine );
        cpLog(LOG_DEBUG, "ACK send to 404:");
        cpLog(LOG_DEBUG, "%s", ack.encode().logData());
        sipEvent->getSipStack()->sendAsync( ack );
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    JSProvider* provider = call->getProvider();
    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    JSRequest* req = call->getRequest();
    NetworkAddress dest;
    string callId;

    dest = call->getSender();
    callId = req->getClientCallID();

    JSNotification notification(clientUdp, dest);
    notification.setType(CONNFAILED);
    notification.setCall(callId);
    notification.setAddr(req->getCallerAddr());
    notification.send();
    cpLog(LOG_DEBUG, "Send CONNFAILED notif to client");
    cpLog(LOG_DEBUG, "%s", notification.getNotificationString().c_str());

    // Change state to unknown
    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateUnknown" );
}
