
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpStartCall_cxx_Version =
    "$Id$";
#include "JSOpStartCall.hxx"
#include "JSStateMachine.hxx"
#include "JSCall.hxx"
#include "JSReqEvent.hxx"
#include "JSProvider.hxx"
#include "SipRequestLine.hxx"
#include "SipAuthorization.hxx"
#include "SipUrl.hxx"
#include "SipContact.hxx"
#include "SipFrom.hxx"
#include "JSCommon.hxx"


///
JSOpStartCall::JSOpStartCall()
{}


///

JSOpStartCall::~JSOpStartCall()
{}


///

const char* const
JSOpStartCall::name() const
{
    return "JSOpStartCall";
}


///
const Sptr < State >
JSOpStartCall::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive connect request from vocalpad
    // 2. Create INVITE for "cpmtrol" leg
    // 3. Send INVITE to RS
    // 4. Change state to "InitCaller"

    cpLog( LOG_DEBUG, "JSOpStartCall operation" );

    Sptr < JSReqEvent > reqEvent;
    reqEvent.dynamicCast( event );

    if ( reqEvent == 0 )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    assert( call != 0 );
    JSProvider* provider = call->getProvider();

    JSRequest* request = call->getRequest();

    if ( request->getMsgType() != 101 /* connect request */)
    {
        // Only handle connect request
        return 0;
    }

    NetworkAddress netAddr = reqEvent->getSender();
    string ipName = netAddr.getIpName ();
    string callID = ipName + request->getMsgID();

    call->setCallID( callID );
    call->setSender( request->getSender() );
    call->setCallerAddr( request->getCallerAddr() );
    call->setCalleeAddr( request->getCalleeAddr() );
    call->setSendClientCallID( request->getClientCallID() );

    Sptr <SipUrl> callerUrl = new SipUrl;
    Data user( request->getCallerAddr() );
    callerUrl->setUserValue( user );
    callerUrl->setHost( provider->getConfig()->getRedirectHost() );
    callerUrl->setPort( provider->getConfig()->getRedirectPort() );
    callerUrl->setUserParam( "phone" );

    // This INVITE will be saved in the  call object and deleted in the
    // call object's destructor
    Sptr < InviteMsg > invite = new InviteMsg( callerUrl,
                                provider->getConfig()->getJSDSipPort().convertInt(),
                                provider->getConfig()->getRtpPort() );

    // Change the "user" of the "from" to the same as calle
    // The logic is this: js as the user of the "from" donesn't
    // provide any fuction, so why not change to the user of "to"
    // to indicate that it is the vocalpad of the "to" is initiating
    // the call.
    // This maybe a baad idea cause RS maybe using the from user to
    // compose contact list, verify with RS before using followig codes.
    /*
    	SipFrom from = inviteo->getFrom();
        SipUrl fromUrl = from.getUrl();
        fromUrl.setUser( user );
        from.setUrl( fromUrl );
        invite->setFrom( from );
    */

    // Set via branch
    SipVia via = invite->getVia();
    //via.setBranch( JS_REDIRECT );
    via.setBranch( 201 );
    invite->removeVia();
    invite->setVia( via );

    // Add contact
    Sptr <SipUrl> myUrl = new SipUrl;
    myUrl->setUserValue( provider->getConfig()->getUserName() );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    myUrl->setPort( provider->getConfig()->getJSDSipPort() );

    SipContact me;
    me.setUrl( myUrl );
    invite->setNumContact( 0 );    // Clear
    invite->setContact( me );

    // Add authorization
    //SipProxyAuthorization authorization("VovidaClassXswitch");
    //invite->setProxyAuthorization( authorization );
    // New authorization method
    // invite->setAuthBasic( provider->getConfig()->getProxyAuthKey() );
    invite->setProxyAuth( provider->getConfig()->getProxyAuthKey() );

    // Remove the SDP automatically created in the constructor
    invite->flushContentData();

    // Save the INVITE to the call object
    call->setControlInvite( invite );
    call->setRSInvite( invite );

    // Add call object to sip map
    SipCallId callId = invite->getCallId();
    Sptr < JSSipCallContainer > sipMap = provider->getSipCallContainer();
    sipMap->addCall( callId, call );

    // Get the JSD SIP stack from the provider
    Sptr < SipTransceiverFilter > sipStack = call->getProvider()->getJSDSipStack();
    cpLog(LOG_DEBUG, "JSOpStartCall: Invite sent:");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", invite->encode().logData());
    cpLog(LOG_DEBUG, "***************************************");

    sipStack->sendAsync( *invite );

    Sptr < JSStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( "JSStateInitCaller" );
}
