
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSOpUserOffhook_cxx_Version =
    "$Id$";
#include "JSOpUserOffhook.hxx"
#include "SipEvent.hxx"
#include "SipVia.hxx"
#include "StatusMsg.hxx"
#include "JSCall.hxx"
#include "SipSdp.hxx"
#include "UdpStack.hxx"
#include "Sptr.hxx"
#include "JSCommon.hxx"
#include "JSNotification.hxx"



///
JSOpUserOffhook::JSOpUserOffhook()
{}


///

JSOpUserOffhook::~JSOpUserOffhook()
{}


///

const char* const
JSOpUserOffhook::name() const
{
    return "JSOpUserOffhook";
}


///
const Sptr < State >
JSOpUserOffhook::process( const Sptr < SipProxyEvent > event )
{
    // 1. Receive 200
    // 2. If not received as JSD, throw away
    // 3. Create new 200 and send it back to caller

    cpLog( LOG_DEBUG, "JSOpUserOffhook operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > status;
    status.dynamicCast( sipMsg );
    if ( status == 0 )
    {
        return 0;
    }

    if ( status->getStatusLine().getStatusCode() != 200 )
    {
        return 0;
    }

    Data method = sipMsg->getCSeq().getMethod();

    if ( method != "INVITE" )
    {
        return 0;
    }

    Sptr < JSCall > call;
    call.dynamicCast(event->getCallInfo());
    JSProvider* provider = call->getProvider();

    Data viaPort = sipMsg->getVia(0).getPort();

    // Only handle 200 received as JSD
    if (viaPort != provider->getConfig()->getJSDSipPort() )
    {
        // Throw away
        cpLog(LOG_ERR, "receive 200 on port: %s", viaPort.logData());
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    // added notification stuff

    Sptr < UdpStack > clientUdp = provider->getClientUdpStack();
    NetworkAddress dest = call->getReceiver();
    JSRequest* req = call->getRequest();
    JSNotification notification(clientUdp, dest);

    if (dest.getPort() == -1)
    {
        cpLog( LOG_DEBUG, "JSOpUserOffhook : no notification sent" );
    }
    else
    {
        // send a termconnactive on behalf of both connections
        // send a termconnactive for caller
        notification.setType(TERMCONNACTIVE);
        notification.setCall(req->getClientCallID());
        notification.setAddr(req->getCallerAddr());
        notification.send();

        // send a termconnactive for callee
        notification.setType(TERMCONNACTIVE);
        notification.setCall(req->getClientCallID());
        notification.setAddr(req->getCalleeAddr());
        notification.send();
    }

    // Save 200 to call object
    call->setTo200( *status );

    // Create a new 200 and send it back to caller
    Sptr < InviteMsg > invite = call->getFromInvite();
    StatusMsg newStatus( *invite, 200/* OK */ );
    // Get SDP from 200
    Sptr<SipSdp> sdp;
    sdp.dynamicCast ( sipMsg->getContentData(0) );
    if (sdp == 0)
    {
        cpLog( LOG_ERR, "No remote SDP or invalid SDP in 200, terminate call" );
        //AckMsg ack( *msg );
        //sipEvent->getSipStack()->sendAsync( ack );
        //ByeMsg bye( *msg );
        //sipEvent->getSipStack()->sendAsync( bye );
        // Change state to unknown to terminate this call
        Sptr < JSStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        return stateMachine->findState( "JSStateUnknown" );
    }

    newStatus.flushContentData();
    newStatus.setContentData( &(*sdp) );

    // Add contact to the new 200
    Sptr <SipUrl> myUrl = new SipUrl;
    myUrl->setUserValue( provider->getConfig()->getUserName() );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    myUrl->setPort( provider->getConfig()->getJSDSipPort() );
    SipContact me;
    me.setUrl( myUrl );
    newStatus.setNumContact( 0 );    // Clear
    newStatus.setContact( me );

    cpLog(LOG_DEBUG, "JSOpUserOffhook: 200 sent back to caller");
    cpLog(LOG_DEBUG, "***************************************");
    cpLog(LOG_DEBUG, "%s", newStatus.encode().logData());
    cpLog(LOG_DEBUG, "***************************************");
    sipEvent->getSipStack()->sendReply( newStatus );

    return 0;
}
