
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSReqEvent_cxx_Version =
    "$Id$";
#include "cpLog.h"
#include "JSReqEvent.hxx"
#include "JSRequest.hxx"
#include "VException.hxx"


///
JSReqEvent::JSReqEvent( const Sptr < Fifo < Sptr < SipProxyEvent > > > outputFifo )
{
    assert ( outputFifo != 0 );
    SipProxyEvent::myFifo = outputFifo;
}

///
JSReqEvent::~JSReqEvent()
{}

///

void
JSReqEvent::setReqMsg( const string reqMsg )
{
    itsReqMsg = reqMsg;
}

const string
JSReqEvent::getReqMsg() const
{
    return itsReqMsg;
}

///
void
JSReqEvent::setSender( NetworkAddress& sender )
{
    itsSender = sender;
}
///
const NetworkAddress
JSReqEvent::getSender( ) const
{
    return itsSender;
}

///
//Sptr<JSRequest>
JSRequest*
JSReqEvent::decode()
{

    try
    {
	/// Parse udp packet into request object
	VXmlParser parser(itsReqMsg);

        string reqTypeStr = parser.getContent(0, "type");
        int reqType = atoi(reqTypeStr.c_str());

        JSRequest* request = new JSRequest;
        request->setMsgType(reqType);

        switch (reqType)
        {
            case 101: 		// Connect
            {
                list < string > callAddrs, callTerms;
                request->setMsgID(parser.getContent(0, "id"));
                request->setClientCallID(parser.getContent(0, "call"));
                parser.getContent("addr", "", callAddrs);
                list < string > ::iterator iterAddr = callAddrs.begin();
                request->setCallerAddr(*iterAddr);
                request->setCalleeAddr(*(++iterAddr));
                parser.getContent("term", "", callTerms);

                if ( callTerms.size() != 0 )
                {
                    list < string > ::iterator iterTerm = callTerms.begin();
                    request->setCallerTerminal(*iterTerm);
                    request->setCalleeTerminal(*(iterTerm++));
                }
                // itsRequest = request;
                return request;
            };
            case 102:
            {
                request->setClientCallID(parser.getContent(0, "call"));
                request->setMsgID(parser.getContent(0, "id"));
                list < string > callAddrs;
                parser.getContent("addr", "", callAddrs);
                list < string > ::iterator iterAddr = callAddrs.begin();
                request->setCallerAddr(*iterAddr);
                return request;
            }
            case 151:
            {
                request->setMsgID(parser.getContent(0, "id"));
                list < string > callAddrs;
                parser.getContent("addr", "", callAddrs);
                list < string > ::iterator iterAddr = callAddrs.begin();
                request->setCallerAddr(*iterAddr);
                request->setPort(parser.getContent(0, "port"));
                return request;
            }
            case 152:
            {
                return request;
            }
            case 250:
            {
                request->setClientCallID(parser.getContent(0, "call"));
                request->setMsgID(parser.getContent(0, "id"));
                list < string > callAddrs;
                parser.getContent("addr", "", callAddrs);
                list < string > ::iterator iterAddr = callAddrs.begin();
                request->setCallerAddr(*iterAddr);
                request->setCalleeAddr(*(++iterAddr));
                return request;
            }
            case 103:
            default:
            cpLog(LOG_ERR, "Unsupported request type %s", reqTypeStr.c_str());
            break;
        }

    }
    catch (VException& e)
    {
        cpLog(LOG_ERR, "Exception raised from VXmlParser: %s",
              e.getDescription().c_str());
        return 0;  // JSProvider will handle the null pointer returned
    } // end of try block


    return 0;
}
// End of File
