
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const JSSipCallContainer_cxx_Version =
    "$Id$";
#include <LockHelper.hxx>

#include "JSSipCallContainer.hxx"
#include "JSCall.hxx"


///
JSSipCallContainer::JSSipCallContainer()
{}


///

JSSipCallContainer::~JSSipCallContainer()
{}


///

Sptr < JSCall >
JSSipCallContainer::newCall( SipCallId& callId )
{
    Sptr < JSCall > call = new JSCall;
    assert( call != 0 );

    LockHelper lockHelper( myMutex );
    sipMap.insert( pair < SipCallId, Sptr < JSCall > > ( callId, call ) );

    cpLog( LOG_DEBUG, "New Call %s", callId.getData().logData() );
    return call;
}


///
Sptr < JSCall >
JSSipCallContainer::findCall( SipCallId& callId )
{
    Sptr < JSCall > call;

    LockHelper lockHelper( myMutex );

    CallMap::iterator iter = sipMap.find( callId );

    if ( iter != sipMap.end() )
    {
        cpLog( LOG_DEBUG, "Found Call %s", callId.getData().logData() );
        call = iter->second;
        assert( call != 0 );
    }
    else
    {
        cpLog( LOG_DEBUG, "Call %s not found", callId.getData().logData() );
    }
    return call;
}

///
void
JSSipCallContainer::addCall( SipCallId& callId, Sptr < JSCall > call )
{
    assert( call != 0 );

    LockHelper lockHelper( myMutex );
    sipMap.insert( pair < SipCallId, Sptr < JSCall > > ( callId, call ) );

    cpLog( LOG_DEBUG, "Add Call %s", callId.getData().logData() );
}

///
void
JSSipCallContainer::deleteCall( SipCallId& callId )
{
    Sptr < JSCall > call;

    LockHelper lockHelper( myMutex );

    CallMap::iterator iter = sipMap.find( callId );

    if ( iter != sipMap.end() )
    {
        cpLog( LOG_DEBUG, "Delete Call %s", callId.getData().logData() );
        sipMap.erase( iter );
    }
    else
    {
        cpLog( LOG_DEBUG, "Call %s not found", callId.getData().logData() );
    }
}

void
JSSipCallContainer::dump()
{
    if ( sipMap.size() > 0 )
    {
        cpLog(LOG_DEBUG, "********** Dumping SIP Map **********");
        map < SipCallId, Sptr < JSCall > > ::iterator iter;

        for (iter = sipMap.begin(); iter != sipMap.end(); iter++)
        {
            cpLog(LOG_DEBUG, "*******************");
            cpLog(LOG_DEBUG, "KEY:              %s", iter->first.getData().logData());
            cpLog(LOG_DEBUG, "Call ID:  	%s", iter->second->getCallID().c_str());
            cpLog(LOG_DEBUG, "Caller Addr:      %s", iter->second->getCallerAddr().c_str());
            cpLog(LOG_DEBUG, "Caller Addr:      %s", iter->second->getCalleeAddr().c_str());
            cpLog(LOG_DEBUG, "Controlled:       %s", (iter->second->getControlled()) ? "TRUE" : "FALSE");
            cpLog(LOG_DEBUG, "*******************");
        }
    }
}
