/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const VmBasicProxy_cxx_Version =
    "$Id: VmBasicProxy.cxx,v 1.17 2002/12/12 01:19:37 chok Exp $";
#include "VmBasicProxy.hxx"
#include "UavmContainer.hxx"
#include "VmPortContainer.hxx"

using namespace Vocal;

/// Global variables

static const char *uavmMulticastIp = "226.2.2.5";
static const int uavmMulticastPort = 6000;
static const int maxMissedHeartbeat = 4;

/// Just Overload constructor to use VmCallContainer
VmBasicProxy::VmBasicProxy( const Sptr < Builder > builder,
                            unsigned short defaultSipPort,
                            Data applName )
    : BasicProxy( builder,
		  defaultSipPort,
                  applName,
		  true,  /* Duplicated messages filtering */
		  false, /*No re-writing of top Via for NAT support */
                  APP_CONTEXT_PROXY)
{
    myCallContainer = new VmCallContainer;
    myBuilder->setCallContainer( myCallContainer );
}

///
VmBasicProxy::~VmBasicProxy()
{}


void VmBasicProxy::startUavmHeartbeatThread()
{
    deque < vmPort > listOfVmPorts = VmPortContainer::instance()->getListOfVmPorts();
    // Populate the uavmContainer: each server is considered to be down
    // after maxMissedHearthbeat heartbeats are missed (see BaseServerContainer)
    for (deque < vmPort > ::iterator iter = listOfVmPorts.begin();
            iter != listOfVmPorts.end();
            ++iter)
    {
        Data key = iter->host;
        key += ":";
        key += Data(iter->port);
        UavmContainer::instance().addServer(key, SERVER_UAVM, maxMissedHeartbeat);
    }

    itsUavmHeartbeatThread = new HeartbeatRxThread(UavmContainer::instance(), uavmMulticastIp, uavmMulticastPort );
    itsUavmHouseKeepingThread = new HouseKeepingThread(UavmContainer::instance(), 500);

    itsUavmHeartbeatThread->run();
    itsUavmHouseKeepingThread->run();

    itsUavmHeartbeatThread->join();
    itsUavmHouseKeepingThread->join();
}

// End of File
