#ifndef _MRSHL_CONF_MAP_ENTRY_HXX_
#define _MRSHL_CONF_MAP_ENTRY_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MrshlConfMapEntry_hxx_Version =
    "$Id: MrshlConfMapEntry.hxx,v 1.8 2002/03/19 21:39:47 jason Exp $";


#include <vector>
#include "MrshlConfParticipantInfo.hxx"

namespace Vocal
{

/** MrshlConfMapEntry is a class used to store/access data on
 *  an individual conference.
 */
class MrshlConfMapEntry
{
    public:

        /// Normal Constructor initializes data members
        MrshlConfMapEntry();

        /**
         * Copy constructor
         * @param rhs the entry to copy
         */
        MrshlConfMapEntry( const MrshlConfMapEntry& rhs );

        /**
         * Assignment Operator
         * @param rhs the entry to assign
         * @return *this
         */
        const MrshlConfMapEntry& operator=( const MrshlConfMapEntry& rhs );

        /// Destructor Method
        virtual ~MrshlConfMapEntry();

        /// Accessor methods
        const vector<MrshlConfParticipantInfo>&
                  getConfCallParticipantList() const;

        ///
        const Data& getBridgeAccessNumber() const
        {
            return myConfBridgeAccessNumber ;
        }

        ///
        unsigned int getNumParticipants() const
        {
            return myConfCallParticipantList.size() ;
        }

        ///
        bool getLastUserDisconnect() const
        {
            return myLastUserDisconnect ;
        }

        /**
         * Set the well-known number for ad-hoc calls
         * @param number the number to assign
         * @return void
         */
        void setBridgeAccessNumber( const Data& number )
        {
            myConfBridgeAccessNumber = number ;
        }


        /**
         * Add a participant info object to the map conference entry
         * @param number the object to add
         * @return void
         */
        void addCallToConf( const MrshlConfParticipantInfo& callInfo );

        /**
         * Method to remove particpant info object when the corresponding
         * user has left the conference
         * @param user participant leaving conference
         * @return bool true if user found, false otherwise
         */
        bool removeCallFromConf( const Data& user );

        /**
         * Method for storing the newest status message for a participant
         * @param user user
         * @param statusMsg message to store
         * @return void
         */
        void setStatusMsg( const Data& user, const StatusMsg& statusMsg );

        /**
         * Method for terminating conference when only one party remains.
         * Uses stored invite and status messages to generate byes to
         * both bridge and user
         * @param sipEvent event caused by second last participant
         * leaving conference
         * @return void
         */
        void disconnectLastCall( const Sptr < Vocal::SipEvent > sipEvent );

    private:

        /**
         * Copy method copies class members
         * @param rhs the entry to copy
         * @return void
         */
        void copyRhsToThis( const MrshlConfMapEntry& rhs );

        /// List of active participants in each conference
        vector < MrshlConfParticipantInfo > myConfCallParticipantList;

        /// Bridge access number that well-known number is mapped to
        Data myConfBridgeAccessNumber;

        /// True if the process of disconnecting last participant has begun
        bool myLastUserDisconnect;
};
 
}

#endif //  _MRSHL_CONF_MAP_ENTRY_HXX_
