/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const MrshlOpProxy_cxx_Version =
    "$Id: MrshlOpProxy.cxx,v 1.22 2002/11/13 02:39:19 veer Exp $";

#include "Marshal.hxx"
#include "MrshlOpProxy.hxx"
#include "UnknownExtensionMsg.hxx"
#include "InfoMsg.hxx"
#include "OptionsMsg.hxx"
// TODO: remove the following line - TRANSFER is deprecated
// #include "TransferMsg.hxx"
#include "ReferMsg.hxx"
#include "NotifyMsg.hxx"

using namespace Vocal;


MrshlOpProxy::MrshlOpProxy()
    : Operator()
{
}


MrshlOpProxy::~MrshlOpProxy()
{
}


const char* const
MrshlOpProxy::name() const
{
    return "MrshlOpProxy";
}



const Sptr < State >
MrshlOpProxy::process(const Sptr < SipProxyEvent > event )
{
    Marshal &mrshl = Marshal::instance();

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    // return if not a valid event
    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: SipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();

    // log reception of Proxy message
    doCpLog();
    cpLog( LOG_DEBUG_STACK, "SIP message received:\n%s,",
	   sipMsg->encode().logData() );

    Sptr < SipCommand > sipCommand;
    sipCommand.dynamicCast( sipMsg );

    // return if not a valid SipCommand
    if ( sipCommand == 0 )
    {
        cpLog( LOG_ERR, "process: SipCommand = 0" );
        return PROXY_CONTINUE;
    }

    // marshal the message:

    unsigned int mrshlStatus = mrshl.marshalIncomingCommand( sipCommand );
    Data viaBranch; // application specific portion of branch

    if ( mrshlStatus == MRSHL_STATUS_INCOMING )
    {
        viaBranch = Data(MRSHL_INCOMING_CONTACT);
    }
    else if ( mrshlStatus == MRSHL_STATUS_OUTGOING )
    {
        viaBranch = Data(MRSHL_OUTGOING_CONTACT);
    }    
    else // mrshlStatus == a sip error response code
    {
        // return appropriate error/status msg, and exit
        StatusMsg errorMsg( *sipCommand, mrshlStatus );
	cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		errorMsg.encode().logData() );
        sipEvent->getSipStack()->sendReply( errorMsg );
        return PROXY_DONE_WITH_EVENT;
    }

    // return 100 to sender
    StatusMsg statusMsg( *sipCommand, 100 );
    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		statusMsg.encode().logData() );
    sipEvent->getSipStack()->sendReply( statusMsg );

    Method sipMethod = sipMsg->getType();
    switch(sipMethod)
    {
        case SIP_INFO:
	{
            Sptr < InfoMsg > infoMsg;
            infoMsg.dynamicCast( sipMsg );
            if ( infoMsg == 0 )
            {
                return PROXY_CONTINUE;
            }

            // create new message from received message
            InfoMsg newInfoMsg( *infoMsg , InfoMsg::infoMsgForward );

            // add Via with branch, must be done AFTER setting requestLine
            Marshal::instance().addViaWithBranch( newInfoMsg, viaBranch );

            // forward according to Route (if present) otherwise to To
            // this is done automatically be the SIP stack
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		newInfoMsg.encode().logData() );
            sipEvent->getSipStack()->sendAsync( newInfoMsg );
            break;
	}
        case SIP_OPTIONS:
	{
            Sptr < OptionsMsg > optionsMsg;
            optionsMsg.dynamicCast( sipMsg );
            if ( optionsMsg == 0 )
            {
                return PROXY_CONTINUE;
            }

            // create new message from received message
            OptionsMsg newOptionsMsg( *optionsMsg,
                                      OptionsMsg::optionsMsgForward );

            // add Via with branch, must be done AFTER setting requestLine
            Marshal::instance().addViaWithBranch( newOptionsMsg, viaBranch );

            // forward according to Route (if present) otherwise to To
            // this is done automatically be the SIP stack
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		       newOptionsMsg.encode().logData() );
            sipEvent->getSipStack()->sendAsync( newOptionsMsg );
	    break;
	}
// TODO: remove the following case - TRANSFER is deprecated
//        case SIP_TRANSFER:
//	{
//            Sptr < TransferMsg > transferMsg;
//            transferMsg.dynamicCast( sipMsg );
//            if ( transferMsg == 0 )
//            {
//                return PROXY_CONTINUE;
//            }
//
//            // create new message from received message
//            TransferMsg newTransferMsg( *transferMsg , 
//                                        TransferMsg::transferMsgForward );
//
//            // add Via with branch, must be done AFTER setting requestLine
//            Marshal::instance().addViaWithBranch( newTransferMsg, viaBranch );
//
//            // forward according to Route (if present) otherwise to To
//            // this is done automatically be the SIP stack
//            sipEvent->getSipStack()->sendAsync( newTransferMsg );
//            break;
//	}
        case SIP_REFER:
        {
            Sptr < ReferMsg > referMsg;
            referMsg.dynamicCast( sipMsg );
            if ( referMsg == 0 )
            {
                return PROXY_CONTINUE;
            }

            // create new message from received message
            ReferMsg newReferMsg( *referMsg, ReferMsg::referMsgForward );

            // add Via with branch, must be done AFTER setting requestLine
            Marshal::instance().addViaWithBranch( newReferMsg, viaBranch );

            // forward according to Route (if present) otherwise to To
            // this is done automatically by the SIP stack
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		newReferMsg.encode().logData() );
            sipEvent->getSipStack()->sendAsync( newReferMsg );
            break;
        }
        case SIP_NOTIFY:
        {
            Sptr < NotifyMsg > notifyMsg;
            notifyMsg.dynamicCast( sipMsg );
            if ( notifyMsg == 0 )
            {
                return PROXY_CONTINUE;
            }

            // create new message from received message
            NotifyMsg newNotifyMsg( *notifyMsg, NotifyMsg::notifyMsgForward );

            // add Via with branch, must be done AFTER setting requestLine
            Marshal::instance().addViaWithBranch( newNotifyMsg, viaBranch );

            // forward according to Route (if present) otherwise to To
            // this is done automatically by the SIP stack
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		newNotifyMsg.encode().logData() );
            sipEvent->getSipStack()->sendAsync( newNotifyMsg );
            break;
        }
        case SIP_UNKNOWN:
//        case SIP_UNKNOWNEXTN:
	{
            Sptr < UnknownExtensionMsg > unknownMsg;
            unknownMsg.dynamicCast( sipMsg );
            if ( unknownMsg == 0 )
            {
                return PROXY_CONTINUE;
            }

            // create new message from received message
            UnknownExtensionMsg newUnknownExtensionMsg(
                *unknownMsg, 
                UnknownExtensionMsg::unknownExtensionMsgForward );

            // add Via with branch, must be done AFTER setting requestLine
            Marshal::instance().addViaWithBranch( newUnknownExtensionMsg, 
                                                  viaBranch );

            // forward according to Route (if present) otherwise to To
            // this is done automatically be the SIP stack
	    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n\n%s,",
		newUnknownExtensionMsg.encode().logData() );
            sipEvent->getSipStack()->sendAsync( newUnknownExtensionMsg );
            break;
	}
        default:
	{
            cpLog(LOG_INFO, "NOT A DEFINED SIP MESSAGE");
            break;
	}
    }//end switch

    return PROXY_DONE_WITH_EVENT;
}

// End of File
