/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const MrshlOpSubNotify_cxx_Version =
    "$Id: MrshlOpSubNotify.cxx,v 1.6.2.2 2003/03/13 03:07:27 veer Exp $";

#include "Marshal.hxx"
#include "RoundRobinServers.hxx"
#include "MrshlOpSubNotify.hxx"
#include "MrshlUserCache.hxx"
#include "SipVia.hxx"
#include "SubscribeMsg.hxx"
#include "NotifyMsg.hxx"
#include "MessageMsg.hxx"
#include "SipRecordRoute.hxx"

using namespace Vocal;

MrshlOpSubNotify::MrshlOpSubNotify()
    : Operator()
{
}


MrshlOpSubNotify::~MrshlOpSubNotify()
{
}


const char* const
MrshlOpSubNotify::name() const
{
    return "MrshlOpSubNotify";
}


const Sptr < State >
MrshlOpSubNotify::process(const Sptr < SipProxyEvent > event )
{
    Marshal &mrshl = Marshal::instance();

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );

    if ( sipEvent == 0 )
    {
        cpLog( LOG_ERR, "process: SipEvent = 0" );
        return PROXY_DONE_WITH_EVENT;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    Sptr < SubscribeMsg > subscribe;
    Sptr < NotifyMsg > notify;
    Sptr < MessageMsg > message;
    Sptr < SipCommand> request;

    subscribe.dynamicCast( sipMsg );
    if ( subscribe == 0)
    {
	if (notify.dynamicCast(sipMsg) == 0)
	{
	    if (message.dynamicCast(sipMsg) == 0)
	    {
		return PROXY_CONTINUE;
	    }
	    else request = message;
	}
	else request = notify;
    }
    else request = subscribe;

    // log reception of msg
    doCpLog();
    cpLog( LOG_DEBUG_STACK, "SIP message received:\n%s\n%s,",
	   sipMsg->encode().logData(),sipMsg->getReceivedIPName().logData() );

    // marshal the message
    unsigned int mrshlStatus = mrshl.marshalIncomingCommand( request );
    Data viaBranch; // marshal specific branch component
    bool sendBasedOnRoute = false;
    bool sendDirectlyToGateway = false;
    
    // In case the message is a Re-Invite, RS is not queried and the
    // branches need to be set as in MrshlOpStatus::process302, otherwise
    // the ACK won't match with the 200 (branches are hard-coded in the
    // ACK operator
    //
    if ( mrshlStatus == MRSHL_STATUS_INCOMING )
    {
        if ( request->getNumRoute() > 0 )
        {
            // Re-Invite
           viaBranch = Data(MRSHL_INCOMING_CONTACT);
	    sendBasedOnRoute = true;
        }
        else
        {
            viaBranch = Data(MRSHL_INCOMING_REDIRECT);
            //Set the transport of the user from the VIA
            if(request->getNumVia() == 1)
            {
                cpLog(LOG_DEBUG, "Setting user transport ");
                //const SipVia& sipVia = request->getVia(0);
                //Get the marshal user data 
                Data user = request->getFrom().getUser();
#if 0
                MrshlUserCache & myUserCache = MrshlUserCache::instance();
                try
                {
                    myUserCache.updateUserTransport( 
			user.convertString(), 
			sipVia.getTransport().convertString() 
			);
                }
                catch(VException& e)
                {
                    cpLog(LOG_DEBUG, 
			  "Failed to get user data for (%s), reason %s",
			  user.logData(), 
			  e.getDescription().c_str());
                }
#endif
            }
        }
    }
    else if ( mrshlStatus == MRSHL_STATUS_OUTGOING )
    {
        if ( request->getNumRoute() > 0 )
        {
            // Re-Invite, send to Contact without going to RS
  	    viaBranch = Data(MRSHL_OUTGOING_CONTACT);
            sendBasedOnRoute = true;
        }
        else if ( mrshl.serverType() == MRSHL_TYPE_USERAGENT )
        {
            // send to RS to get Contact
	  viaBranch = Data(MRSHL_OUTGOING_REDIRECT);
        }
	else if ( mrshl.serverType() == MRSHL_TYPE_CONFERENCE )
	{
            LocalScopeAllocator lo;
	    // check if invite uses conference well-known number
	    if ( request->getTo().getUser().getData(lo) != 
		 mrshl.bridgeNumber() )
	    {
		cpLog( LOG_ERR, "Invalid bridge well known number %s",
		       request->getTo().getUser().logData() );

		// send 404 not found
		Sptr<StatusMsg>  errorStatusMsg  
		    = new StatusMsg( *request, 404 );
		cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		       errorStatusMsg->encode().logData() , request->getTo().getHost().getData(lo));
		sipEvent->getSipStack()->sendReply( errorStatusMsg );
		return PROXY_DONE_WITH_EVENT;
	    }
	    else
	    {
		// send to RS to get Contact
		viaBranch = Data(MRSHL_OUTGOING_REDIRECT);
	    }
	}
        else
        {
            // for outgoing Invites, gateway marshals use their gateway
            // as the Contact without contacting RS
            viaBranch = Data(MRSHL_OUTGOING_CONTACT);
	    sendDirectlyToGateway = true;
        }
    }
    else // mrshlStatus == a sip error response code
    {
        // return appropriate error/status msg, and exit
        Sptr<StatusMsg> errorMsg;
	if (mrshlStatus != 407)
	{
	    errorMsg = new StatusMsg( *request, mrshlStatus );
	}
	else
	{
	    errorMsg = mrshl.generateChallenge(request,
					       407,
					       Marshal::CHALLENGE_PROXY_AUTH);
	}
	cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		errorMsg->encode().logData(), request->getTo().getHost().logData());
        sipEvent->getSipStack()->sendReply( errorMsg );
        return PROXY_DONE_WITH_EVENT;
    }

    // return 100 to sender and forward Invite to RS
    Sptr<StatusMsg> statusMsg = new StatusMsg( *request, 100 );
    cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		statusMsg->encode().logData(), request->getTo().getHost().logData() );
    sipEvent->getSipStack()->sendReply( statusMsg );

    // check if Invite already has a route, if so, forward based on
    // this route and return
    if ( sendBasedOnRoute )
    {
        // create new invite msg based on received invite msg
	Sptr<SipCommand> newReq;
	if(subscribe != 0)
	{
	    cpLog( LOG_DEBUG, "forward Subscribe based on route" );
	    newReq = new SubscribeMsg( *subscribe,
				       SubscribeMsg::subscribeMsgForward );
	}
        else
        {

	if(message != 0)
	{
	    cpLog( LOG_DEBUG, "forward Message based on route" );
	    newReq = new MessageMsg( *message,
				       MessageMsg::messageMsgForward );
	}
	else
	{
	    cpLog( LOG_DEBUG, "forward Notify based on route" );
	    newReq = new NotifyMsg( *notify, NotifyMsg::notifyMsgForward );
	}
        }

	// add Via with branch, must be done AFTER setting requestLine
	mrshl.addViaWithBranch( *newReq, viaBranch );
        newReq->setNextHopIsProxy(mrshl.isNextHopAProxy(*newReq));

        cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		newReq->encode().logData(), newReq->getTo().getHost().logData());

        sipEvent->getSipStack()->sendAsync( newReq );
        return PROXY_DONE_WITH_EVENT;
    }

    // add RecordRoute
    Sptr < SipUrl > routeSipUrl;
    Data lHost;
    int lPort;

    routeSipUrl.dynamicCast( request->getRequestLine().getUrl() );
    if ( routeSipUrl == 0 )
    {
        cpLog( LOG_ERR, "process: routeSipUrl = 0" );
        return PROXY_DONE_WITH_EVENT;
    }        

    //Set he marhsal transport
    if(mrshl.transport() == "UDP")
    {
	routeSipUrl->setTransportParam("");
    }
    else
    {
	routeSipUrl->setTransportParam("tcp");
    }
    SipRecordRoute route;

    OpAddRecordRoute( route,
		      *routeSipUrl,
		      mrshl.sipPort() );
    request->setRecordRoute( route, 0 );
#if 0
    // prep RequestUri
    if ( sendDirectlyToGateway )
    {
        // set RequestUri to gateway
        routeSipUrl->setHost( mrshl.gatewayHost() );
        routeSipUrl->setPort( Data(mrshl.gatewayPort()) );
    }
    else
    {
        // set RequestUri to redirect server
        RoundRobinServers& rServers = RoundRobinServers::instance( SERVER_RS );
        Data lHost; 
        int lPort;
        rServers.getHostNPort(sipMsg->getFrom().getUrl()->encode(), lHost, lPort);
        routeSipUrl->setHost( lHost);
        routeSipUrl->setPort( Data(lPort) );
        Data tParam("");
        if(mrshl.transport() == "TCP")
        {
            tParam = "tcp";
        }
        routeSipUrl->setTransportParam(tParam);
    }

    // set RequestUri
    SipRequestLine requestLine( request->getRequestLine() );
    requestLine.setUrl( routeSipUrl );
    request->setRequestLine( requestLine );


    // add Via with branch, must be done AFTER setting requestLine
    mrshl.addViaWithBranch( *request, viaBranch );

//     // if not an internetwork marshal, send out immediately and return
//     if ( mrshl.serverType() != MRSHL_TYPE_INTERNETWORK )
//     {
        request->setNextHopIsProxy(mrshl.isNextHopAProxy(*request));
	cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s\n%s,",
		request->encode().logData(), request->getTo().getHost().logData() );
        sipEvent->getSipStack()->sendAsync( request );
	return PROXY_DONE_WITH_EVENT;
//     }
#endif
    // prep RequestUri
    if ( sendDirectlyToGateway )
    {
	lHost = mrshl.gatewayHost();
	lPort = mrshl.gatewayPort();
        // set RequestUri to gateway
	// In Case This going to be the GateWay Call then Change the
	// ReqestURI or Gatway will reject the Call
        // In case of GatEway we Need to Change the RequestURI
        routeSipUrl->setHost( lHost);
        routeSipUrl->setPort( Data(lPort));
	SipRequestLine& requestLine = request->getMutableRequestLine();
	requestLine.setUrl( routeSipUrl );
    }
    else
    {
        // set RequestUri to redirect server
        RoundRobinServers& rServers = RoundRobinServers::instance( SERVER_RS );
	rServers.getHostNPort(sipMsg->getFrom().getUrl()->encode(), lHost, lPort);
    }

    if(lHost.length() && !sendDirectlyToGateway ){
	 NetworkAddress newNetworkAddy(lHost.convertString(), lPort);
	 sipMsg->setSendAddress(newNetworkAddy);
    }
    // add Via with branch, must be done AFTER setting requestLine
    mrshl.addViaWithBranch( *request, viaBranch );

    // if not an internetwork marshal, send out immediately and return
    if ( mrshl.serverType() != MRSHL_TYPE_INTERNETWORK )
    {
        request->setNextHopIsProxy(mrshl.isNextHopAProxy(*request));
	cpLog( LOG_DEBUG_STACK, "SIP message sent:\n %s:%d\n%s,",
	       lHost.logData(), lPort, request->encode().logData());
	sipEvent->getSipStack()->sendAsync( request, lHost, Data(lPort));
	return PROXY_DONE_WITH_EVENT;
    }

// //     // handle policy
// //     bool policySuccess = false;
// //     if ( mrshlStatus == MRSHL_STATUS_OUTGOING )
// //     {
// // 	// contact policy server to authorize user and get token
// // 	policySuccess = mrshl.policy().policyAuthorize( sipEvent,
// // 							request );
// //     }
// //     else // MRSHL_STATUS_INCOMING
// //     {
// // 	// contact policy server to validate token
// // 	policySuccess = mrshl.policy().policyValidate( sipEvent,
// // 						       request );
// //     }

// //     if ( !policySuccess )
// //     {
// // 	// policy server authorization rejected
// // 	// return 401- authorized to user
// // 	cpLog( LOG_ERR, "Policy unauthorized for callId %s",
// // 	       return->getCallId().getCallIdData().logData() );
	
// // 	Sptr<StatusMsg> statusMsg = new StatusMsg ( *request, 401 );
// // 	sipEvent->getSipStack()->sendReply( statusMsg );
// //     }

     return PROXY_DONE_WITH_EVENT;
}

// End of File
