
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const ClientSession_cxx_Version =
    "$Id$";
//Standard Include
#include <signal.h>
#include <sys/types.h>
#include <errno.h>
#include <sys/wait.h>
#include <sys/shm.h>
#include <sys/ipc.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/shm.h>
#include <sys/ipc.h>
#include <strstream.h>
#include <fstream.h>
#include <unistd.h>
#include <string.h>


//User defined
#include "NetworkAddress.h"
#include "Tcp_ServerSocket.hxx"
#include "Tcp_ClientSocket.hxx"
#include "Connection.hxx"
#include "Http_def.h"
#include "cpLog.h"
#include "UiThread.hxx"
#include "SystemConfig.hxx"
#include "Sptr.hxx"
#include "AgentApi.hxx"
#include "ClientSession.hxx"
#include "CmuSnmp.hxx"

const ulong start_req_name_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 1, 1, 0
    };
const ulong start_req_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 1, 2, 0
    };
const ulong stop_req_name_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 1, 3, 0
    };
const ulong stop_req_oid[] =
    {
        1, 3, 6, 1, 4, 1, 5738, 1, 1, 1, 4, 0
    };

void
ClientSession::sigTerm(int signo)
{
    //TODO - Nothing to do at this time
    cpLog(LOG_DEBUG, "SIGTERM");
}

void
ClientSession::sigPipe(int signo)
{
    //TODO - Nothing to do at this time
    cpLog(LOG_DEBUG, "SIGPIPE");
}

extern "C" {

static void s_sigTerm(int sig)
{
    ClientSession::sigPipe(sig);
}

static void s_sigPipe(int sig)
{
    ClientSession::sigPipe(sig);
}

}

ClientSession::ClientSession() : ThreadIf()
{
    signal(SIGPIPE, s_sigPipe);
    signal(SIGTERM, s_sigTerm);
    mDelete = false;
}


ClientSession::~ClientSession()
{
    cpLog(LOG_DEBUG, "ClientSession Destructor");

}

void
ClientSession::thread()
{
    //Loop through for Incoming messages
    cpLog(LOG_DEBUG, "************* spawn client session ***************");
    char buf[MAXLINE];

    memset(buf, 0, sizeof(buf));
    int nread = 0;
    string req, arg1, arg2;

    while (myConnection.readLine(buf, MAXLINE-1, nread) > 0)
    {
        try
        {
            buf[nread] = 0;

            string fullReq = buf;

            cpLog(LOG_DEBUG, "Processing request %s", buf);

            int comma1 = fullReq.find(",");
            int comma2 = fullReq.find (",", comma1 + 1);
            int ending = fullReq.find_first_of ("\r\n", comma2 + 1);
            if (comma1 > 0 && comma2 > 0 && nread > comma2 )
            {
                req = fullReq.substr(0, comma1 );
                arg1 = fullReq.substr(comma1 + 1, comma2 - comma1 - 1);
                arg2 = fullReq.substr(comma2 + 1, ending - comma2 - 1);

                if (req.empty() || arg1.empty() || arg2.empty())
                {
                    returnStatus(false, myConnection);
                }
                else
                {
                    if (req == "CONFIG_REQ" )
                        handleConfigReq(atoi(arg1.c_str()), myConnection);
                    else if ( req == "STOP_REQ" )
                    {
                        returnStatus(true, myConnection);
                        string pType = "stop";
                        handleRequest(arg1, arg2, myConnection,pType);
                    }
                    else if (req == "START_REQ")
                    {
                        returnStatus(true, myConnection);
                        string pType = "start";
                        handleRequest(arg1, arg2, myConnection,pType);
                    }
                }
            }
            else
            {
                returnStatus(false, myConnection);
            }
        }


        catch (VException& e)
        {
            cpLog(LOG_ERR, "Failed to process user request, reason %s", e.getDescription().c_str());
            returnStatus(false, myConnection);
        }

        memset(buf, 0, sizeof(buf));
        nread = 0;
    }
    cpLog(LOG_DEBUG, "ClientSession Thread exits ");
    myConnection.close();
    mDelete = true;
}

void
ClientSession::setConnection(Connection& conn)
{
    myConnection = conn;
}

void
ClientSession::handleRequest(string& hostnamePort, string& serverType, Connection& conn,string& type)
{
    string serverName;
    string hostname;
    string portNumber;

    Data result;
    CmuSnmp::snmpTypeT resultType;
        
    vector < ulong > stopReqNameOid(stop_req_name_oid, stop_req_name_oid + (sizeof(stop_req_name_oid) / sizeof(stop_req_name_oid[0]))) ;
    vector < ulong > stopReqOid(stop_req_oid, stop_req_oid + (sizeof(stop_req_oid) / sizeof(stop_req_oid[0]))) ;
    vector < ulong > startReqNameOid(start_req_name_oid, start_req_name_oid + (sizeof(start_req_name_oid) / sizeof(start_req_name_oid[0]))) ;
    vector < ulong > startReqOid(start_req_oid, start_req_oid + (sizeof(start_req_oid) / sizeof(start_req_oid[0]))) ;

    if (serverType == "FS")
        serverName.append("fs ");
    else if (serverType == "MS")
        serverName.append("ms ");
    else if (serverType == "RS")
        serverName.append("rs ");
    else if (serverType == "JS")
        serverName.append("js ");
    else if (serverType == "POS")
        serverName.append("policy ");
    else if (serverType == "CDR")
        serverName.append("cdr ");
    else
    {
        cpLog( LOG_ERR, "Invalid ServerType %s", serverType.c_str());
        returnStatus(false, conn);
        return ;
    }
    int pos = hostnamePort.find(":");
    if (pos == 0)
    {
        cpLog( LOG_ERR, "Invalid hostnamePort. no \":\"  %s", hostnamePort.c_str());
        returnStatus(false, conn);
        return ;
    }
    else
    {
        hostname = hostnamePort.substr(0, pos);
        portNumber = hostnamePort.substr(pos + 1, hostnamePort.size());
    }
    if (serverType != "POS" && serverType != "CDR") {
	serverName.append(portNumber);
    }

    CmuSnmp snmp(hostname.c_str(), community);
    Data serverNameData(serverName);
    Data oneData = 1;
    // ignore the result for now.
    
    if( type == "stop"){
         if (!snmp.set( stopReqNameOid, 's', serverNameData,  stopReqOid, 'i', oneData, resultType, result, 10000000, 1))
        {
            cpLog(LOG_ERR, "Stop process snmp set failed");
        }
    }
    if(type == "start"){
        if (!snmp.set( startReqNameOid, 's', serverNameData,  startReqOid, 'i', oneData, resultType, result, 10000000, 1))
        {
            cpLog(LOG_ERR, "Start process snmp set failed");
        }
    }
return;
}

void
ClientSession::handleConfigReq(int uiVerNum, Connection& conn)
{
    string retData;
    int myVerNum = (SystemConfig::instance()->getVerNum());
    // is should look like this:
    // <data version="X"> <serverconfig servertype=\"MS\" ip=\"192.168.05.114\" port=\"3060\" serverstatus=\"Active\"/> </data>\n
    if (myVerNum == 0)
    {
        string reason = "<error>Network Manager Not available</error>\n";
        retData = makeHTTPHeader(0, reason.size());
        retData += reason;
    }
    else
    {
        char verStr[18 + 20];
        memset(verStr, 0, 18);
        sprintf(verStr , "<data version=\"%d\">\n", myVerNum);
        if (uiVerNum == myVerNum)
        {
            // 8 is sizeof "</data>\n"
            retData = makeHTTPHeader(1, strlen(verStr) + 8);
            retData += verStr;
            retData.append("</data>\n");
        }
        else
        {
            string sysConfig = (SystemConfig::instance()->getConfigHTML());
            if (sysConfig.empty())
            {
                string reason = "<error>Network Configuration Not available</error>\n";
                retData = makeHTTPHeader(0, reason.size());
                retData += reason;
            }
            else
            {
                retData = makeHTTPHeader(1, sysConfig.size() + strlen(verStr));
                retData += verStr;
                retData += sysConfig.c_str();
            }
        }

    }
    cpLog(LOG_DEBUG, "Sending response:\n%s", retData.c_str());
    try
    {
        conn.writeData(retData);
    }
    catch ( ... )
    {
        cpLog(LOG_ERR, "Failed to write to connection");
    }
    return ;
}

void
ClientSession::returnStatus(bool success, Connection& conn)
{
    string retData;
    retData = makeHTTPHeader(success, 0);
    if (!success)
        cpLog(LOG_DEBUG, "return error to user");
        cpLog(LOG_DEBUG, "Sending response:\n%s", retData.c_str());

    try
    {
        conn.writeData(retData);
    }
    catch ( ... )
    {
        cpLog(LOG_ERR, "Failed to write to connection");
    }
    return ;
}

string
ClientSession::makeHTTPHeader(int success, int contLength)
{
    mstring retData;
    int st_code;
    char buf[256];
    if (success)
    {
        st_code = 200;
        sprintf(buf, "%d OK\n", st_code);
    }
    else
    {
//  This is changed because the gui can't handle 400 message
        st_code = 200;
        sprintf(buf, "%d OK\n", st_code);
//        st_code = 400;
//        sprintf(buf, "%d FAILED\n", st_code);
    }
    retData += buf;
    sprintf(buf, "%s %d\n", CONT_LENGTH, contLength);
    retData += buf;
    return (retData);
}
