
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const SystemConfig_cxx_Version =
    "$Id: SystemConfig.cxx,v 1.36 2002/04/16 00:54:24 bko Exp $";

#include "SystemConfig.hxx"
#include <iostream>
#include "cpLog.h"
#include "parse3tuple.h"


Sptr < SystemConfig > SystemConfig::theSystemConfig = 0;
string SystemConfig::itsServerTag = "";

/// Return the one Server container or create it
/// if it doesn't already exist.
Sptr < SystemConfig >
SystemConfig::instance()
{
    if ( theSystemConfig == 0 )
    {
        theSystemConfig = new SystemConfig();
    }
    return theSystemConfig;
}

SystemConfig::SystemConfig()
{
    CfgVersion = 0;
}

SystemConfig::~SystemConfig( )
{}


voReturnStatus
SystemConfig::ProcessStatusPoll(const configListT& rxConfigList)
{
    ConfigData tmpcfgdata;

    tableLock.WriteLock();
    systemCfgList.clear();
    for (vector < ConfigData > ::iterator i = rxConfigList->begin();
            i != rxConfigList->end(); i++)
    {
        tmpcfgdata = *i;
        Data key (tmpcfgdata.SvrName);
        Sptr < ConfigData > cfgdata = new ConfigData;
        *cfgdata = *i;
        systemCfgList.insert(pair < Data, Sptr < ConfigData > > (key, cfgdata));
    }
    incVerNum();
    tableLock.Unlock();
    return voSuccess;
}

voReturnStatus
SystemConfig::ProcessTrap(string Name, ServerType Type, ServerStatus newStatus)
{
    ConfigData tmpcfgdata;

    Data key(Name);
    tmpcfgdata.servertype = Type;
    tmpcfgdata.status = newStatus;
    tmpcfgdata.SvrName = Name;
    Sptr < ConfigData > cfgdata = new ConfigData(tmpcfgdata);

    updateCfg(key, cfgdata) ;
    incVerNum();
    return voSuccess;
}

string
SystemConfig::getConfigHTML()
{
    ConfigData aServer;
    string typeStr, ipStr, portStr, statusStr;
    int pos;
    string retConfig;

    tableLock.ReadLock();
    for (TableIter i = systemCfgList.begin();
            i != systemCfgList.end();
            i++)
    {
        typeStr = "";
        statusStr = "";
        ipStr = "";
        portStr = "";

        aServer = *i->second;
        switch (aServer.servertype)
        {
            case SERVER_MS :
            {
                typeStr = "MS";
                break;
            }
            case SERVER_FS :
            {
                typeStr = "FS";
                break;
            }
            case SERVER_RS :
            {
                typeStr = "RS";
                break;
            }
            case SERVER_JS :
            {
                typeStr = "JS";
                break;
            }
            case SERVER_POS :
            {
                typeStr = "POS";
                break;
            }
            case SERVER_CDR :
            {
                typeStr = "CDR";
                break;
            }
            default :
            {
                cpLog( LOG_ERR, "received bad server type");
            }
        }
        switch (aServer.status)
        {
            case Active :
            {
                statusStr = "Active";
                break;
            }
            case Inactive :
            {
                statusStr = "Inactive";
                break;
            }
            case ActivePending :
            {
                statusStr = "ActivePending";
                break;
            }
            default :
            {
                cpLog( LOG_ERR, "received invalid status type");
            }
        }

        // tmpName added because string on multiprocessor system is evil
	LocalScopeAllocator lo;
        string tmpName = aServer.SvrName.getData(lo);
        pos = tmpName.find(":", 0);
        ipStr = tmpName.substr(0, pos);
        portStr = tmpName.substr(pos + 1, tmpName.length() - pos);

        if (ipStr != "")
        {
            // is should look like this:
            // <data version=X> <serverconfig servertype=\"MS\" ip=\"192.168.05.114\" port=\"3060\" serverstatus=\"Active\"/> </data>\n
            retConfig.append("<serverconfig servertype=\"");
            retConfig.append(typeStr);
            retConfig.append("\" ip=\"");
            retConfig.append(ipStr);
            retConfig.append("\" port=\"");
            retConfig.append(portStr);
            retConfig.append("\" serverstatus=\"");
            retConfig.append(statusStr);
            retConfig.append("\"/> ");
        }
    }
    retConfig.append("</data>\n");
    tableLock.Unlock();
    return retConfig;
}

list < string >
SystemConfig::getServerNameList()
{
    ConfigData aServer;
    Sptr < string > retConfig;
    list < string > retlist;

    tableLock.ReadLock();
    for (TableIter i = systemCfgList.begin();
            i != systemCfgList.end();
            i++)
    {
        aServer = *i->second;
        string tmpName = aServer.SvrName.convertString();
        tmpName = tmpName.substr(0, tmpName.find(":"));
        cpLog(LOG_DEBUG, "adding server to list *%s*", tmpName.c_str());
        retlist.push_back( tmpName);
    }


    retlist.unique();
    tableLock.Unlock();
    return retlist;
}

int
SystemConfig::getNumberOfServers()
{
    int retVal = 0;

    tableLock.ReadLock();
    retVal = systemCfgList.size();
    tableLock.Unlock();
    return retVal;
}

void
SystemConfig::updateCfg(const Data& key, const Sptr < ConfigData > cfgdata)
{
    tableLock.WriteLock();
    /// Find the Server in the database using the key
    TableIter i = systemCfgList.find(key);

    LocalScopeAllocator lo;

    cpLog( LOG_DEBUG, "update entry %s, %d", (const char *)cfgdata->SvrName.getData(lo), cfgdata->status);
    /// Verify that a Server was found
    if (i != systemCfgList.end())
    {
        systemCfgList.erase(i);
    }
    systemCfgList.insert(pair < Data, Sptr < ConfigData > > (key, cfgdata));
    tableLock.Unlock();
}


void
SystemConfig::remove( Data& key )
{
    tableLock.WriteLock();
    /// Find the Server in the database using the key
    TableIter i = systemCfgList.find(key);

    /// Verify that a Server was found
    if (i != systemCfgList.end())
    {
        /// remove the key/Server from the map
        systemCfgList.erase(i);
        cpLog( LOG_DEBUG, "remove entry");
    }
    tableLock.Unlock();

}


void
SystemConfig::incVerNum()
{
    CfgVersion++;
    cpLog ( LOG_DEBUG, "new version #: %d", CfgVersion);
}

int
SystemConfig::getVerNum()
{
    return CfgVersion;
}
// End of File
