
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OpRegister_cxx_Version =
    "$Id: OpRegister.cxx,v 1.46.2.2 2003/01/23 02:51:46 sprajpat Exp $";
#include "RegisterMsg.hxx"
#include "SipContact.hxx"
#include "SipEvent.hxx"
#include "StatusMsg.hxx"
#include "SipTransceiver.hxx"
#include "OpRegister.hxx"
#include "RSStateWait.hxx"
#include "SipContactIf.hxx"
#include "SipExpires.hxx"

#include "RSMirror.hxx"
#include "Serialize.hxx"

using namespace Vocal;

const unsigned int EXPIRETIMEDEFAULT = 3600;

extern void updateUser(const string& userName, bool deleteFlag);
extern int setUpRegistrationForUpdate(bool force);

/**
 * Constructor
 */
OpRegister::OpRegister()
{}


/**
 * Destructor
 */
OpRegister::~OpRegister()
{}


/**
 * Return the name of this object.
 * @return  String containing a name to identify object.
 */
const char* const
OpRegister::name() const
{
    return "OpRegister";
}

/**
 * Decode the event, trigger the process of registration, and 
 * return a status response message. Begin the Mirroring process if successful.
 * @param  event  Sip event containg a sip message which may be a Register message. 
 * @return The next State 
 */
const Sptr < State >
OpRegister::process(const Sptr < SipProxyEvent > event)
{
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);

    cpLog(LOG_DEBUG, "OpRegister::process");
    if (sipEvent == 0)
    {
        cpLog(LOG_ERR, "Event is not a Sip Event");
        return PROXY_CONTINUE;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    if (sipMsg == 0)
    {
        cpLog( LOG_DEBUG, "In OpRegister::process, sipEvent->getSipMsg() returned 0" );
        return PROXY_CONTINUE;
    }

    Sptr < RegisterMsg > sipRegister;
    sipRegister.dynamicCast(sipMsg);

    if ( sipRegister == 0)
    {
	cpLog( LOG_DEBUG, "OpRegister: Not a register message");
        return PROXY_CONTINUE;
    }

    doCpLog();

    SipContactIf contacts;

    Data to;
    Sptr <BaseUrl> baseToUrl( sipRegister->getTo().getUrl() );
    if (baseToUrl.getPtr() != 0)
    {
        if (baseToUrl->getType() == SIP_URL)
        {
            Sptr <SipUrl> sipUrl;
            to = (sipUrl.dynamicCast(baseToUrl))->getUserValue();
        }
        else
        {
            cpLog(LOG_DEBUG, "To field is not defined as type SIP_URL");
        }
    }
    else
    {
        cpLog(LOG_DEBUG, "Unable to obtain To field's base pointer");
    }

    Data from;
    Sptr <BaseUrl> baseFromUrl( sipRegister->getFrom().getUrl() );
    if (baseFromUrl.getPtr() != 0)
    {
        if (baseFromUrl->getType() == SIP_URL)
        {
            Sptr <SipUrl> sipUrl;
            from = (sipUrl.dynamicCast(baseFromUrl))->getNameAddr();
        }
        else
        {
            cpLog(LOG_DEBUG, "From field is not defined as type SIP_URL");
        }
    }
    else
    {
        cpLog(LOG_DEBUG, "Unable to obtain From field's base pointer");
    }

    Sptr < vector < Data > > newContacts = new vector < Data > ;

    /// get the contacts from the message and put in newContacts
    int contactLength = sipRegister->getNumContact();

    for ( int i = 0; i < contactLength; ++i )
    {
        Sptr <BaseUrl> baseContactUrl( sipRegister->getContact(i).getUrl() );
        if (baseContactUrl.getPtr() != 0)
        {
            if (baseContactUrl->getType() == SIP_URL)
            {
                Sptr <SipUrl> sipUrl;
                 sipUrl.dynamicCast(baseContactUrl);
                Data aData(sipUrl->getNameAddr());
                aData += sipUrl->getUrlParam();
                newContacts->push_back(aData);
            }
            else
            {
                cpLog(LOG_DEBUG, "Contact field is not defined as type SIP_URL");
            }
        }
        else
        {
            cpLog(LOG_DEBUG, "Unable to obtain Contact field's base pointer");
        }
    }

    Data expireTime(INT_MAX);
    /// Attempt to get the expire time from the Expire header
    if (sipRegister->getExpires().getDelta().length())
    {
        expireTime = sipRegister->getExpires().getData();
    }
    else
    {
        SipExpires contactExpire;
        /// Check for an expire parameter in the Contacts
        for ( int i = 0; i < contactLength; ++i )
        {
            contactExpire = sipRegister->getContact(i).getExpires();
            if (contactExpire.getDelta().length())
            {
                /// Take the shortest of the existing Contact expire values
                if (expireTime.convertInt() > contactExpire.getData().convertInt())
                {
                    expireTime = contactExpire.getData();
                }
            }
        }

        if (INT_MAX == expireTime.convertInt())
        {
            /// If the Expires: header is non existant and none of the contacts
            /// contain an expires field, assume one hour expiry.
            expireTime = Data(int(EXPIRETIMEDEFAULT));
            cpLog( LOG_DEBUG, "No Expiry header, assuming one hour.");
        }
    }
    cpLog( LOG_DEBUG, "In OpRegister::process, expireTime field: [%s]", expireTime.logData() );

    cpLog( LOG_DEBUG, "In OpRegister::process, to field: %s", to.logData() );

    vector < Data > oldContacts;

    /// update the Subscriber and get the correct expiry time
    int responseCode = contacts.update( to, from, expireTime, newContacts, oldContacts);

    cpLog( LOG_DEBUG, "In OpRegister::process, response code: %d", responseCode );
    StatusMsg statusMsg( *sipRegister, responseCode);

    if(responseCode == 404)
    {
        //This may be the case if we have missed the updates
        //from the provisioning server. This happens if ps goes down
        //and restarted, it will loose all the registrations for updates
        //See if we are a candidate for updating registrations
        //if we are, try to get the subscriber data ourselves
        if(setUpRegistrationForUpdate(false))
        {
            string uName(to.c_str());
            updateUser(uName, false);
        }
    }

    /// TODO Should we do something with the oldContacts?

    /// pass the correct expireTime back in the response
    SipExpires sipExpires;
    sipExpires.setDelta(expireTime);
    statusMsg.setExpires(sipExpires);

    /// Return the original contact list back with the status message
    if(contactLength == 0) 
    {
        //Fetch register message, return back the old contacts
        *newContacts = oldContacts;
    }
    LocalScopeAllocator lo;
    for ( vector < Data > ::iterator i = newContacts->begin(); i != newContacts->end(); ++i )
    {
        string url( i->getData(lo) );
        Sptr<SipUrl> sipUrl = new SipUrl(url);
        SipContact sipContact;
        sipContact.setUrl(sipUrl);
        statusMsg.setContact( sipContact );
    }

    sipEvent->getSipStack()-> sendReply( statusMsg );

    // don't bother to mirror register message if the response
    // code is not 200 after updating the contact
    ///Ignore mirroring if it is a Fetch register
    if ((responseCode == 200) && contactLength)
    {
        // construct a RSMirror object
        RSMirror rsMirror;

        if ( !rsMirror.isFromRs( sipRegister ))
        {
            // start mirroring the sipRegister
            cpLog (LOG_DEBUG, "In OpRegister::process, Start mirroring the sip register message.");
            rsMirror.mirrorSipRegister( sipEvent, sipRegister );
        }

	// persist Registrations

        #ifdef SERIALIZE    // defined in vocal/util/io/Serialize.hxx
        {

            Data sip_register = sipRegister->encode();

            //cpLog (LOG_DEBUG, "SIP  message: %s ADD|%s\n", sip_register.logData(), to.logData());


            IO::Serialize::singleton().addWrite( to, sip_register);

        }
        #endif // persists Registrations

    }

    return PROXY_CONTINUE;
}

//End of File
