#ifndef _RSMIRROR_HXX
#define _RSMIRROR_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "Data.hxx"
#include "RegisterMsg.hxx"
#include "SipEvent.hxx"
#include "Sptr.hxx"

static const char* const RSMirrorVersion = "$Id$";

/*
 * To support system redundancy, this class mirrors
 * a Register Message from the Redirect Server to all other 
 * RS's. If the Register message is from another RS, 
 * it does nothing.
 */

class RSMirror
{
    public:
        /// Constructor
        RSMirror();

        // Copy Constructor
        RSMirror(const RSMirror& rhs);

        // Overload operator assignment =
        RSMirror& operator= (const RSMirror& rhs);

        /// Destructor
        virtual ~RSMirror();

        /**
         * Clone the sip register, add self to first via. For
         * each redirect server from server container, change
         * request line to each rs, call sipstack to send async.
         * @param  sipEvent  Event used to send the Register messages.
         * @param  sipRegister  Register message to be mirrored.
         */
        void mirrorSipRegister(const Sptr < Vocal::SipEvent > event,
                               const Sptr < Vocal::RegisterMsg > sipRegister );

        /**
         * Get the first via from the sipRegister, get the list of RS 
         * from the server container to compare with the first via.
         * @param  sipRegister  Sip register message to mirror
         * @return True if via match with any rs, else false
         */
        bool isFromRs(const Sptr < Vocal::RegisterMsg > sipRegister);

   private:

        // Copy member data from specified object to this
        void copyRhsToThis(const RSMirror& rhs);


};
#endif // _RSMIRROR_HXX
