/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CapabilitySet_cxx_Version = 
    "$Id: CapabilitySet.cxx,v 1.2 2001/05/22 18:12:37 vzubarev Exp $";


#include <vector>
#include <stdexcept>
#include <algorithm>
#include "VLog.hxx"
#include "Assert.hxx"
#include "CapabilitySet.hxx"
#include "VLog.hxx"


using namespace CapabilityTranslator;
using Vocal::Logging::VLog;

int CapabilitySet::CapabilityDescrVec::getAltCapSize(int i, int j) const
{
    int vecSize = 0, descrSize = 0;
    vecSize = getCapDescrVecSize();
    
      descrSize = getCapDescrSize(i);
    
    if((descrSize == 0)||(descrSize < j + 1))return 0;
    
    return capDescrVec[i][j].size();

}


int CapabilitySet::CapabilityDescrVec::getCapDescrSize(int i) const
{
    int vecSize;
    
    vecSize = getCapDescrVecSize();

    if((vecSize == 0)||(vecSize < i + 1 ))return 0;
    
    return capDescrVec[i].size();
}

int CapabilitySet::CapabilityDescrVec::getCapDescrVecSize() const
{

    return capDescrVec.size();

}

void CapabilitySet::CapabilityDescrVec::addCapability (int cap, int i, int j) throw (XlatorException)
{
    int vecSize = getCapDescrVecSize();
    int descrSize = 0;
    
    if( i == -1 )i = (vecSize == 0 ? 0 : vecSize - 1);
    
    if( j == -1 )
    {
      descrSize = getCapDescrSize(i);
      j = (descrSize == 0 ? 0 : descrSize - 1);
    }
      
    if( i > vecSize )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    descrSize = getCapDescrSize(i);
    
    if( j > descrSize)
        throw XlatorException("Index j out of range", __FILE__, __LINE__, 489);

    if( j == descrSize ) // add new element (AltCapSet) to CapDescriptor
    {
      AltCapSet altCapSet;
      altCapSet.push_back(cap);

      if( i < vecSize ) // do NOT add new element (CapDescriptor) to
                        // capDescrVec
      {
         capDescrVec[i].push_back(altCapSet);
      }
      else // add new element (CapDescriptor) to capDescrVec
      {
         CapDescriptor capDescr;
	 capDescr.push_back(altCapSet);
	 capDescrVec.push_back(capDescr);
      }
    }
    else // do NOT add new element (AltCapSet) to CapDescriptor
    { 
       if ( i == vecSize ) // cannot add new CapDescriptor without
                           // adding new AltCapSet
          throw XlatorException("Index i2 out of range", __FILE__, __LINE__, 489);
       capDescrVec[i][j].push_back(cap);
    }
}


void CapabilitySet::CapabilityDescrVec::addAltCapSet(
                    const CapabilitySet::CapabilityDescrVec::AltCapSet &altCapSet,
		    int i, 
		    int j) throw (XlatorException)
{
    int vecSize = getCapDescrVecSize();
    int descrSize = 0;
    
    if( i == -1 )i = (vecSize == 0 ? 0 : vecSize - 1);
    
    if( j == -1 )
    {
      descrSize = getCapDescrSize(i);
      j = (descrSize == 0 ? 0 : descrSize - 1);
    }
      
    if( i > vecSize )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    descrSize = getCapDescrSize(i);
    
    if( j > descrSize)
        throw XlatorException("Index j out of range", __FILE__, __LINE__, 489);

      if( i < vecSize ) // do NOT add new element (CapDescriptor) to
                        // capDescrVec
      {
         capDescrVec[i].push_back(altCapSet);
      }
      else // add new element (CapDescriptor) to capDescrVec
      {
         CapDescriptor capDescr;
	 capDescr.push_back(altCapSet);
	 capDescrVec.push_back(capDescr);
      }
}


void CapabilitySet::CapabilityDescrVec::addCapDescriptor(
                    const CapabilitySet::CapabilityDescrVec::CapDescriptor &capDescr,
		    int i) throw (XlatorException)
{
    int vecSize = getCapDescrVecSize();
    
    if( i == -1 )i = (vecSize == 0 ? 0 : vecSize - 1);
    
    if( i > vecSize )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    capDescrVec.push_back(capDescr);
}


void CapabilitySet::CapabilityDescrVec::removeCapDescriptor(int i) throw (XlatorException)
{
    int vecSize = getCapDescrVecSize();
    
    if( i > vecSize )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    capDescrVec.erase(capDescrVec.begin() + i);
}


int CapabilitySet::CapabilityDescrVec::getCapability(int i, int j, int h) const throw (XlatorException)
{
    if(getCapDescrVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);
    
    if(getCapDescrSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if(getAltCapSize(i, j) < h + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    return capDescrVec[i][j][h];

}

int CapabilitySet::CapabilityDescrVec::getCapabilityIndex(
                                                         int cap, 
							 int index, 
							 int curDescrN) const throw (XlatorException)
{
    const string    fn("CapabilityDescrVec::getCapabilityIndex");
    VLog    log(fn);
    
    int descrSize = getCapDescrSize(curDescrN);
    int j = 0, h = 0, altSize = 0;
    
    VDEBUG(log)<< fn << ": BEGIN ------------- " << VDEBUG_END(log);
    
    VDEBUG(log) << fn << ": descrSize of descriptor # " << curDescrN 
    	    	<< " : " << descrSize << VDEBUG_END(log);
    
    for(j = index; j < descrSize; j++)
    {
      altSize = getAltCapSize(curDescrN, j);
      
      VDEBUG(log) << fn << ": altSize : " << altSize << "; j : " << j << VDEBUG_END(log);
      
      for(h = 0; h < altSize; h++)
      {
        VDEBUG(log) << fn << ": Element of capDescrVec : " << capDescrVec[curDescrN][j][h] << VDEBUG_END(log);
	
	if(cap == capDescrVec[curDescrN][j][h])
	{
	  VDEBUG(log) << fn << ": END ---------- " << VDEBUG_END(log);
	  return j;
	}
      }
    }

    VDEBUG(log) << fn << ": END with throw ------ " << VDEBUG_END(log);
    
    throw XlatorException("Could not find the specified capability", __FILE__, __LINE__);
}							 

const CapabilitySet::CapabilityDescrVec::AltCapSet &CapabilitySet::CapabilityDescrVec::getAltCapSet(int i, int j) const throw (XlatorException)
{
    if(getCapDescrVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);
    
    if(getCapDescrSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);
    
    return capDescrVec[i][j];    
}

CapabilitySet::CapabilityDescrVec &CapabilitySet::CapabilityDescrVec::operator=(const CapabilitySet::CapabilityDescrVec &fromDescrVec)
{
  if(&fromDescrVec != this)
  {
  
  capDescrVec.clear();
  
  int vecSize =0, i = 0, j = 0, h = 0, descrSize = 0, altSize = 0;
  
  vecSize = fromDescrVec.getCapDescrVecSize();
  
  for(i = 0; i < fromDescrVec.getCapDescrVecSize(); i++)
  {
    descrSize = fromDescrVec.getCapDescrSize(i);
    
    for(j = 0; j < descrSize; j++)
    {
      altSize = fromDescrVec.getAltCapSize(i, j);
      
      for(h = 0; h < altSize; h++)
      {
        addCapability(fromDescrVec.getCapability(i, j, h), i, j);
      }
    }
  }
 }
 
 return *this;

}

void CapabilitySet::CapabilityDescrVec::clear()
{
  capDescrVec.clear();
}

void CapabilitySet::CapabilityDescrVec::sortCapDescrVec() const
{
  int vecSize =0, i = 0, j = 0, descrSize = 0;
  
  vecSize = getCapDescrVecSize();
  
  for(i = 0; i < vecSize; i++)
  {
    descrSize = getCapDescrSize(i);
    
    for(j = 0; j < descrSize; j++)
    {
        std::sort(capDescrVec[i][j].begin(), capDescrVec[i][j].end());
    }
  }
}
