/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const G711Capability_cxx_Version = 
    "$Id: G711Capability.cxx,v 1.2 2001/05/22 18:12:37 vzubarev Exp $";


#include "ptlib.h"
#include "asner.h"
#include "VLog.hxx"
#include "G711Capability.hxx"


using namespace CapabilityTranslator;
using Vocal::Logging::VLog;


G711Capability::G711Capability(Mode m, Speed s)
  : AudioCapability(G711_SMPLS_PER_FRAME, 
                    G711_FRMS_PER_PACKET, 
		    G711_CLCK_RATE), mode(m), speed(s) {} // non framed codec
                                                       // 8 samples (1 ms with this clock rate) per "frame"
						       // default packet size = 30 "frames" per packet
						       // 8000 Hz - clockrate


AudioCapability * G711Capability::clone() const
{
  return new G711Capability(mode, speed);
}


const unsigned G711Capability::g711SubType[2][2] = {
  { H245_AudioCapability::e_g711Alaw64k, H245_AudioCapability::e_g711Alaw56k },
  { H245_AudioCapability::e_g711Ulaw64k, H245_AudioCapability::e_g711Ulaw56k }
};


unsigned G711Capability::getSubType() const
{
  return g711SubType[mode][speed];
}



bool G711Capability::getParamSendMsg(H245_AudioCapability & cap,
                                       unsigned packetSize) const
{
  const string	fn("G711Capability::getParamSendMsg");
  VLog	log(fn);
  
  cap.SetTag(g711SubType[mode][speed]);

  PASN_Integer & value = cap;
  value = packetSize;
  
  VDEBUG(log) << fn << ": Packet size we added : " << int(value) << VDEBUG_END(log);
  return true;
}


bool G711Capability::setParamRecvMsg(const H245_AudioCapability & cap,
                                        unsigned & packetSize)
{
  switch (cap.GetTag()) {
    case H245_AudioCapability::e_g711Alaw64k :
      mode = G711Capability::ALaw;
      speed = G711Capability::At64k;
      break;

    case H245_AudioCapability::e_g711Alaw56k :
      mode = G711Capability::ALaw;
      speed = G711Capability::At56k;
      break;

    case H245_AudioCapability::e_g711Ulaw64k :
      mode = G711Capability::muLaw;
      speed = G711Capability::At64k;
      break;

    case H245_AudioCapability::e_g711Ulaw56k :
      mode = G711Capability::muLaw;
      speed = G711Capability::At56k;
      break;

    default :
      return false;
  }

  const PASN_Integer & value = cap;

  // Get the maximum number of milliseconds
  packetSize = value;
  return true;
}
