
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const G7231Capability_cxx_Version = 
"$Id: G7231Capability.cxx,v 1.2 2001/05/22 18:12:37 vzubarev Exp $";
#include "ptlib.h"
#include "asner.h"
#include "G7231Capability.hxx"

using namespace CapabilityTranslator;

G7231Capability::G7231Capability(bool sS, unsigned int sduFr)
  : AudioCapability(G7231_SMPLS_PER_FRAME, 
                    G7231_FRMS_PER_PACKET, 
		    G7231_CLCK_RATE), 
    silenceSuppression(sS),
    maxAl_sduAudioFrames(sduFr) {} // 240 samples per Frame (30 ms with this clock rate); 
                                   // default packet size 1 Frame per Packet
                                   // 8000 Hz clockrate
				   // in this case masAl_sduAudioFrames
				   // we interpret as packetSize


AudioCapability * G7231Capability::clone() const
{
  return new G7231Capability(silenceSuppression, maxAl_sduAudioFrames);
}


bool G7231Capability::getParamSendMsg(H245_AudioCapability & cap,
                                       unsigned packetSize) const
{
  cap.SetTag(H245_AudioCapability::e_g7231);

  H245_AudioCapability_g7231 & capG7231 = cap;
  PASN_Integer & sduFr = capG7231.m_maxAl_sduAudioFrames;
  PASN_Boolean & sS = capG7231.m_silenceSuppression;
  sduFr = maxAl_sduAudioFrames;
  sS = silenceSuppression;

  return true;
}


bool G7231Capability::setParamRecvMsg(const H245_AudioCapability & cap,
                                        unsigned & packetSize)
{
  if(cap.GetTag()!=H245_AudioCapability::e_g7231)return false;

  
  const H245_AudioCapability_g7231 & capG7231 = cap;
  const PASN_Integer & sduFr = capG7231.m_maxAl_sduAudioFrames;
  const PASN_Boolean & sS = capG7231.m_silenceSuppression;
  
  maxAl_sduAudioFrames = sduFr;
  silenceSuppression = sS;

  // 
  packetSize = sduFr;

  return true;
}
