/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323_CapTable_cxx_Version = 
    "$Id: H323_CapTable.cxx,v 1.2 2001/05/22 18:12:37 vzubarev Exp $";


#include <iterator>
#include <string>
#include "H323_CapTable.hxx"
#include "Assert.hxx"
#include "VLog.hxx"

#ifndef DEBUG
#define DEBUG 1
#endif

using namespace CapabilityTranslator;
using Vocal::Logging::VLog;

H323_CapTable::~H323_CapTable()
{
    CapTable_I p = capabilityTable.begin();
    
    while(p != capabilityTable.end())
    {
      delete *p;
      p++;
    }
    
}


H323_CapTable::CapTable_I H323_CapTable::getTableIter_pt(int ptype) const
{
  CapTable_I i_capT = std::find_if(capabilityTable.begin(), capabilityTable.end(), HCapTab_pt(ptype));

  if((i_capT != capabilityTable.end())&&((*i_capT)->getCapabilityNumber() == ptype))
    return i_capT;
  else return NULL;
} 

H323_CapTable::CapTable_In H323_CapTable::getTableIterN_pt(int ptype)
{
  CapTable_In i_capT = std::find_if(capabilityTable.begin(), capabilityTable.end(), HCapTab_pt(ptype));

  if((i_capT != capabilityTable.end())&&((*i_capT)->getCapabilityNumber() == ptype))
    return i_capT;
  else return NULL;
} 

void H323_CapTable::addCapTableEntry(Capability *capability)
{
    const string    fn("H323_CapTable::addCapTableEntry");
    VLog    log(fn);
    
  // See if already added, confuses things if you add the same instance twice
  CapTable_In p = getTableIterN_pt(capability->getCapabilityNumber());

  if(p != NULL)  
  {
    // Capability already added
    delete *p;
    capabilityTable.erase(p); // - everyone knows vector is not optimized
                              // for this type of stuff, right ? :)
			      // we use this very rarely
  }
  
  capabilityTable.push_back(capability);
  
  VDEBUG(log)<< fn << "Just added the capability : " <<  
        capability->getCapabilityNumber() << VDEBUG_END(log);
}

bool H323_CapTable::isEmpty() const
{
  return (capabilityTable.size() == 0);
}
