#ifndef __SMCAPMANAGER_H_
#define __SMCAPMANAGER_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SM_CapManager_hxx_Version = 
"$Id: SM_CapManager.hxx,v 1.6 2001/05/24 21:58:33 vzubarev Exp $";

#include <map>

#include "../CapabilityTranslatorDefs.hxx"
#include "H323_CapTable.hxx"
#include "GlobalCapTable.hxx"
#include "Capability.hxx"
#include "CapabilitySetSip.hxx"
#include "CapabilitySetH323.hxx"
#include "CapabilitySetIntersect.hxx"

#include "FeatureTCSDefs.hxx"
#include "FeatureTCS.hxx"
#include "FeatureSDPDefs.hxx"
#include "FeatureSDP.hxx"

using namespace CapabilityTranslator;

namespace CapabilityManager
{
    class EntryTCS;
    class EntrySDP;
    class EvtCapMgrRcvdSDP0;
    class EvtCapMgrRcvdTCS0;
    
    /// CapabilityManager State Machine
    class SM_CapManager
    {
    private:
	/** map between the payload type (as assigned in GlobalCapTable)
	    and particular capability object
	    will be used to construct capability Table
	*/
	typedef std::map<int, Capability*> CapMap;
	
	CapMap capabilityMap;
	
	translator::CallTranslatorEM *evtMgr;

    	FeatureTCS f_tcs;
	FeatureSDP f_sdp;

	//
	GlobalCapTable globalCapTable;

	/** map between incoming CapTable numeration and our inner numeration
	    of capabilities (as assigned by payload types in GlobalCapTable)
	*/
	std::map<int, int> mapTable;

	//
	typedef std::map<int, Capability*> CapMap;
	
	/** map between the payload type (as assigned in GlobalCapTable)
	    and particular capability object
	    will be used to construct capability Table
	*/
	CapMap capabilityMap;
	//
	CapabilitySetSip capSetSip;
	//
	CapabilitySetH323 capSetH323;
	//
	CapabilitySetIntersect capSetIntersect;
	
    public:
	friend class EntryTCS;
	friend class EntrySDP;
	friend class EvtCapMgrRcvdSDP0;
	friend class EvtCapMgrRcvdTCS0;
	
	///
	SM_CapManager(translator::CallTranslatorEM *em);
	
	///
	virtual ~SM_CapManager();

	///
	translator::CallTranslatorEM *getEM() const;
	
	///
	void init();
	
	/// Return pointer to TCS Feature
	virtual FeatureTCSBase *getFeatureTCS() throw();
	///
	virtual FeatureTCS &getFeatureTCSImp() throw();
	
	/// Return pointer to SDP Feature
	virtual FeatureSDPBase *getFeatureSDP() throw();
	///
	virtual FeatureSDP &getFeatureSDPImp() throw();

	///
	void parseCapsTCS(const TCS &msgTCS) throw();
	
	///
	void parseCapsSDP(const SDP &msgSDP) throw();
	
	///
	void setControlChannel();
    };

	/// Return pointer to Event Manager (manager of call-back events)
	inline translator::CallTranslatorEM * SM_CapManager::getEM() const
	{
	    return(evtMgr);
	}

}

#endif
