#ifndef __MSMANAGER_H_
#define __MSMANAGER_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const MSManager_hxx_Version = 
"$Id: MSManager.hxx,v 1.2 2001/05/26 01:32:37 vzubarev Exp $";

#include <map>

#include "../CapabilityTranslatorDefs.hxx"
#include "EvtMSMgr.hxx"
#include "CallTranslatorEM.hxx"
#include "SM_MSManager.hxx"

using namespace CapabilityTranslator;

namespace MediaSessionManager
{
    typedef SessionID int;
    
    /// MediaSessionManager
    class MSManager
    {
    private:
	translator::CallTranslatorEM *evtMgr;

        typedef MediaSesContainer std::map<SessionID, SM_MSManager*>;
	
	// Container of state machines for all media sessions
	MediaSesContainer mediaSesContainer;
	
	/// The number of the last SessionID assigned
	SessionID maxSesID;
	
	/// The number of the last logical channel opened
	int maxLogChN;
    public:
	///
	MSManager(translator::CallTranslatorEM *em);
	
	///
	virtual ~MSManager();

	/// Return pointer to MS StateMachine by SessionID
	SM_MSManager *getSM(SessionID i) const throw();

        /// Add state machine to a container
	void addSM(SessionID i, SM_MSManager *sm);
	
	/// handle OLC
	bool handleOLC(const OLC &msgOLC);
	
	/// make OLCAck
	bool makeOLCAck(OLCAck &msgOLCAck);
	
	/// handleOLCAck
	bool handleOLCAck(const OLCAck &msgOLCAck);
	
	/// makeOLC
	void makeOLC(OLC &msgOLC) throw();
	
	/// handle SDP
	void handleSDP(const SDP &msgSDP) throw();
	
	/// make SDP
	void makeSDP(SDP &msgSDP) throw();
	
    };
}

#endif
