#if !defined(H323_REGISTRATION_EVENT_DOT_H)
#define H323_REGISTRATION_EVENT_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const H323RegistrationEvent_hxx_Version = 
    "$Id: H323RegistrationEvent.hxx,v 1.10 2002/07/17 00:21:09 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"

#include "H323Event.hxx"
#include "Sptr.hxx"
#include "FifoBase.h"
#include "EndpointInfo.hxx"
#include "GatekeeperInfo.hxx"
#include "IPAddress.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** H.323 specific internal events.<br><br>
 */
namespace H323Events
{


using Vocal::SipH323Info::EndpointInfo;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Transport::IPAddress;


/** Registration event.<br><br>
 *
 *  @see    Vocal::H323Events::H323Event
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfo
 *  @see    Vocal::Transport::IPAddress
 */
class H323RegistrationEvent : public Vocal::H323Events::H323Event
{
    public:

    	/** Create a registration event.
	 *
	 *  @param  endpoint	The registering endpoint
	 *  @param  gatekeeper	Information about the gatekeeper
	 *  @param  rasPDU     	The RAS message that generated the event.
	 *  @param  remoteAddr	The address of the remote entity that generated
	 *  	    	    	the event.
	 *  @param  registration    Set to true if this is an RRQ.
	 *  @param  returnFifo	The fifo to send the response.
    	 */
    	H323RegistrationEvent(
	    Sptr<EndpointInfo>	    	endpoint,
	    GatekeeperInfo  	    &	gatekeeper,
	    Sptr<H225_RasMessage>       rasPDU,
	    Sptr<IPAddress> 	    	remoteAddr,
	    bool    	    	    	registration,
	    FifoBase< Sptr<Event> > &	returnFifo
	);
					

    	/** Virtual destructor.
    	 */
	virtual     ~H323RegistrationEvent();


    	/** Access the information about the registering endpoint.
    	 */
    	Sptr<EndpointInfo>  	    endpointInfo() const;


    	/** Access the information about the gatekeeper.
    	 */
	GatekeeperInfo &	    gatekeeperInfo() const;


    	/** Access the RAS message.
    	 */
    	Sptr<H225_RasMessage>       rasPDU() const;


    	/** Access the address of the remote entity.
    	 */
	Sptr<IPAddress>     	    remoteAddress() const;


    	/** Returns true if this is associated with a registration message,
	 *  returns false otherwise (could be an unregistration message).
    	 */
	bool	    	    	    registration() const;
		

    	/** Name of this class of event, "H323RegistrationEvent".
    	 */
    	virtual const char*         name() const;


    	/** Write the event to an ostream.
    	 */
	virtual ostream &           writeTo(ostream &) const;

    private:

	Sptr<EndpointInfo>  	    endpointInfo_;
	GatekeeperInfo	    	&   gatekeeperInfo_;
	Sptr<H225_RasMessage>       rasPDU_;
    	Sptr<IPAddress>     	    remoteAddr_;
	bool	    	    	    registration_;
};


} // namespace H323Events
} // namespace Vocal


#endif // !defined(H323_REGISTRATION_EVENT_DOT_H)
