/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Alias_cxx_Version = 
    "$Id: Alias.cxx,v 1.8 2002/05/15 09:43:31 icahoon Exp $";


#include "Alias.hxx"
#include <cstring>

#include <iostream>


using Vocal::SipH323Info::Alias;
using Vocal::Transport::IPAddress;


Alias::Alias()
    :	myPhone("")
{
}


Alias::Alias(const H225_ArrayOf_AliasAddress & src)
    :   myAlias(src),
        myPhone("")
{
    updatePhone();
}


Alias::Alias(const Alias & src)
    :   myAlias(src.myAlias),
        myPhone(src.myPhone)
{
}


Alias::~Alias()
{
}


Alias &    	    	    	    
Alias::operator=(const Alias & src)
{
    if ( this != &src )
    {
    	myAlias = src.myAlias;
	myPhone = src.myPhone;
    }
    return ( *this );
}


bool	    	    	    	    
Alias::operator==(const Alias & src) const
{
    if ( this == &src )
    {
    	return ( true );
    }

    return ( phone() == src.phone() && user() == src.user() );
}


bool	    	    	    	    
Alias::operator<(const Alias & src) const
{
    if	( this == &src )
    {
	return ( false );
    }

    if ( hasPhone() )
    {
        if ( src.hasPhone() )
        {
            return ( phone() < src.phone() );
        }
        return ( false );
    }
    else if ( src.hasPhone() )
    {
        return ( true );
    }

    if ( hasUser() )
    {
        if ( src.hasUser() )
        {
            return ( user() < src.user() );
        }
        return ( false );
    }
    else if ( src.hasUser() )
    {
        return ( true );
    }
    
    return ( false );
}


void
Alias::alias(const H225_ArrayOf_AliasAddress & src)
{
    myAlias = src;
    updatePhone();
}


const H225_ArrayOf_AliasAddress &   
Alias::alias() const
{
    return ( myAlias );
}


void	    	    	    	    
Alias::user(const string & src)
{
    int     i = findIndex(H225_AliasAddress::e_h323_ID);

    if ( i < 0 )
    {
	i = myAlias.GetSize();
	myAlias.SetSize(i + 1);
    }

    myAlias[i].SetTag(H225_AliasAddress::e_h323_ID);
    PASN_BMPString & alias = myAlias[i];
    alias = src.c_str();
}


string	    	    	    	    
Alias::user() const
{
    string  	result;

    int     	i = findIndex(H225_AliasAddress::e_h323_ID);

    if ( i >= 0 )
    {
        const H225_AliasAddress     alias           = myAlias[i];
    	const PASN_BMPString    &   unicode_h323id  = alias;
        const PString               h323id          = unicode_h323id; 

    	result = h323id;
    }    

    return ( result );
}


bool
Alias::hasUser() const
{
    return ( findIndex(H225_AliasAddress::e_h323_ID) < 0 ? false : true );
}


void	    	    	    	    
Alias::phone(const string & src)
{
    string valid_chars("0123456789#*");

    myPhone = src;

    for( size_t i = 0; i < myPhone.size(); ++i )
    {
    	char	c = myPhone.at(i);

	if ( c == 'p' )
	{
	    myPhone.replace(i, 1, ',');
	}
	else if ( valid_chars.find(c) == string::npos )
	{
	    myPhone.erase(i, 1);
	    --i;
	}
    }
    
    if ( myPhone.find_first_of(valid_chars) == string::npos )
    {
        myPhone.erase();
        return;
    }

    int     i = findIndex(H225_AliasAddress::e_dialedDigits);

    if ( i < 0 )
    {
	i = myAlias.GetSize();
	myAlias.SetSize(i + 1);
    }

    myAlias[i].SetTag(H225_AliasAddress::e_dialedDigits);
    PASN_IA5String & dialed_digits = myAlias[i];
    dialed_digits = src.c_str();
}


string	    	    	    	    
Alias::phone() const
{
    return ( myPhone );
}


bool    	    	
Alias::hasPhone() const
{
    return ( findIndex(H225_AliasAddress::e_dialedDigits) < 0 ? false : true );
}


void
Alias::transport(const IPAddress & transport_)
{
    int     i = findIndex(H225_AliasAddress::e_transportID);

    if ( i < 0 )
    {
	i = myAlias.GetSize();
	myAlias.SetSize(i + 1);
    }

    myAlias[i].SetTag(H225_AliasAddress::e_transportID);

    H225_TransportAddress   	    &	transAddr = myAlias[i];

    transAddr.SetTag(H225_TransportAddress::e_ipAddress);
    
    H225_TransportAddress_ipAddress & 	ipAddr = transAddr;

    memcpy( &ipAddr.m_ip[0], 
    	    &(transport_.getIPAddress().sin_addr.s_addr), 
	    4);

    ipAddr.m_port = transport_.getPort();
}


IPAddress   	    	    	    
Alias::transport() const
{
    IPAddress	result;

    int     	i = findIndex(H225_AliasAddress::e_transportID);

    if ( i >= 0 )
    {
    	H225_TransportAddress	&   transAddr = myAlias[i];

	if ( transAddr.GetTag() == H225_TransportAddress::e_ipAddress )
	{
    	    H225_TransportAddress_ipAddress & ipAddr = transAddr;
            result.setIP(ipAddr.m_ip.GetPointer());
	    result.setPort(ipAddr.m_port);
	}
    }    

    return ( result );
}
	

bool    	    	
Alias::hasTransport() const
{
    return ( findIndex(H225_AliasAddress::e_transportID) < 0 ? false : true );
}


void	    	    	    	    
Alias::url(const string & src)
{
    int     i = findIndex(H225_AliasAddress::e_url_ID);

    if ( i < 0 )
    {
	i = myAlias.GetSize();
	myAlias.SetSize(i + 1);
    }

    myAlias[i].SetTag(H225_AliasAddress::e_url_ID);
    PASN_IA5String & url_alias = myAlias[i];
    url_alias = src.c_str();
}


string	    	    	    	    
Alias::url() const
{
    string  result;

    int     	i = findIndex(H225_AliasAddress::e_h323_ID);

    if ( i >= 0 )
    {
        const H225_AliasAddress     &   alias = myAlias[i];
	const PASN_IA5String        &   url_alias = alias;
        
	result = url_alias.GetTypeAsString();
    }    
    
    return ( result );
}


bool
Alias::hasUrl() const
{
    return ( findIndex(H225_AliasAddress::e_url_ID) < 0 ? false : true );
}


void	    	    	    	    
Alias::email(const string & email_)
{
    int     i = findIndex(H225_AliasAddress::e_email_ID);

    if ( i < 0 )
    {
	i = myAlias.GetSize();
	myAlias.SetSize(i + 1);
    }

    myAlias[i].SetTag(H225_AliasAddress::e_email_ID);
    PASN_IA5String & email_alias = myAlias[i];
    email_alias = email_.c_str();
}


string	    	    	    	    
Alias::email() const
{
    string  result;

    int     	i = findIndex(H225_AliasAddress::e_email_ID);

    if ( i >= 0 )
    {
        const H225_AliasAddress         &   alias = myAlias[i];
	const PASN_IA5String            &   email_alias  = alias;

	result = email_alias.GetTypeAsString();
    }    
    
    return ( result );
}
		

bool
Alias::hasEmail() const
{
    return ( findIndex(H225_AliasAddress::e_email_ID) < 0 ? false : true );
}


int 	    	    	    	    
Alias::findIndex(unsigned int choice) const
{
    int     i, aliasSize = myAlias.GetSize();
    bool    found = false;

    for ( i = 0; i < aliasSize && !found; ++i )
    {
    	if ( myAlias[i].GetTag() == choice )
	{
	    found = true;
	    break;
	}
    }

    return ( found ? i : -1 );
}


bool	    	    	
Alias::isEmpty() const
{
    return ( myAlias.GetSize() == 0 );
}


void	    	    	
Alias::clear()
{
    myAlias.SetSize(0);
    myPhone = "";
}


ostream &           	    
Alias::writeTo(ostream & out) const
{
    bool first = true;

    if ( hasPhone() )
    {
        first = false;
        out << "phone: " << phone();
    }
    if ( hasUser() )
    {
        out << ( first ? "" : ", " ) << "user: " << user();
    }
    return ( out );
}


void
Alias::updatePhone()
{
    if ( myPhone.size() == 0 )
    {
	int 	i = findIndex(H225_AliasAddress::e_dialedDigits);

	if ( i >= 0 )
	{
	    PASN_IA5String & e164 = (PASN_IA5String &)myAlias[i];
	    myPhone = string((const char*)((PString)e164));
	}    
    }
}
