#if !defined(VOCAL_ALIAS_HXX)
#define VOCAL_ALIAS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Alias_hxx_Version = 
    "$Id: Alias.hxx,v 1.8 2002/05/15 09:43:31 icahoon Exp $";


#include "ptlib.h"
#include "h323pdu.h"
#include <string>
#include "IPAddress.hxx"
#include "Writer.hxx"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Object Information<br><br>
 */
namespace SipH323Info
{


using Vocal::Transport::IPAddress;


/** Wrapper for a sequence of h.225 alias addresses.<br><br>
 */
class Alias : public Vocal::IO::Writer
{

    public:
    
    	
	/** Default construct an empty alias.
    	 */
    	Alias();


	/** Create given a sequence of h.225 alias addresses.
    	 */
    	Alias(const H225_ArrayOf_AliasAddress 	&);


	/** Copy constructor.
    	 */
	Alias(const Alias &);


	/** Virtual destructor.
    	 */
	virtual     ~Alias();


	/** Assignment operator.
    	 */
    	Alias &    	    	    	    operator=(const Alias &);


	/** Equality operator.
    	 */
    	bool	    	    	operator==(const Alias &) const;


	/** Less than relational operator.
    	 */
    	bool	    	    	operator<(const Alias &) const;


	/** Set the alias given a sequence of h.225 alias addresses.
    	 */
    	void	    	    	alias(const H225_ArrayOf_AliasAddress &);


	/** Return the native sequence of h.225 alias addresses.
    	 */
    	const H225_ArrayOf_AliasAddress & alias() const;


	/** Sets the user identifier. Mapped to the h.323 id.
    	 */
	void	    	    	user(const string &);


	/** Returns the user identifier.
    	 */
    	string	    	    	user() const;


	/** Returns true if a user identifier is present.
    	 */
	bool	    	    	hasUser() const;
	

	/** Set the phone number.
    	 */
    	void	    	    	phone(const string &);


	/** Returns the phone number.
    	 */
	string	    	    	phone() const;


	/** Returns true if a phone number is present.
    	 */
	bool	    	    	hasPhone() const;


	/** Set the transport address.
    	 */
	void	    	    	transport(const IPAddress &);


	/** Returns the transport address.
    	 */
    	IPAddress   	    	transport() const;


	/** Returns true if a transport address is present.
    	 */
	bool    	    	hasTransport() const;
	

	/** Set the url.
    	 */
    	void	    	    	url(const string &);


	/** Returns the url.
    	 */
	string	    	    	url() const;


	/** Returns true if a url is present.
    	 */
	bool    	    	hasUrl() const;
	

	/** Sets the email address.
    	 */
	void	    	    	email(const string &);


	/** Returns the email address.
    	 */
	string	    	    	email() const;


	/** Returns true if an email address is present.
    	 */
	bool    	    	hasEmail() const;


	/** Returns the alias address type at the given index.
    	 */
	int 	    	    	findIndex(unsigned int) const;


	/** Returns true if the alias has no addresses.
    	 */
    	bool	    	    	isEmpty() const;


	/** Empty the alias.
    	 */
	void	    	    	clear();
			

	/** Write the alias to an ostream.
    	 */
	virtual ostream &       writeTo(ostream &) const;

    private:

	void	    	    	updatePhone();

    	H225_ArrayOf_AliasAddress   	    myAlias;
    	string	    	    	    	    myPhone;
};


} // namespace SipH323Info
} // namespace Vocal


#endif // !defined(VOCAL_ALIAS_HXX)
