#if !defined(ROUTER_INFO_DOT_H)
#define ROUTER_INFO_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RouterInfo_hxx_Version = 
    "$Id: RouterInfo.hxx,v 1.10 2001/05/22 08:06:14 icahoon Exp $";


#include <string>
#include "IPAddress.hxx"
#include "VMutex.h"


/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Object Information<br><br>
 */
namespace SipH323Info
{


using Vocal::Transport::IPAddress;


/** Configuration information available for the Router.<br><br>
 *
 *  @see    Vocal::Transport::IPAddress
 */
class RouterInfo
{

    public:


	/** Construct given the address for the local call signaling address, 
	 *  the router media control property and the remote client 
	 *  property.<br>
	 *
    	 *  If the router is to route H.245, routeMediaCtrl should be set true.
	 *
	 *  If the remote entity is a client (rather than a server)
	 *  remoteClient should be set true.
	 */
    	RouterInfo(
	    const IPAddress 	&   callSignalAddr,
	    bool	    	    routerMediaCtrl = false,
	    bool	    	    remoteClient  = false
	);


	/** Virtual destructor
	 */
	virtual    ~RouterInfo();


	/** Set the local call signaling address.
	 */
	void	    	    callSignalAddress(const IPAddress &);


	/** Access the local call signal address.
	 */
	IPAddress   	    callSignalAddress() const;


	/** Set the remoteClient property.
	 */
    	void	    	    remoteClient(bool);


	/** Access the remoteClient property.
	 */
    	bool	    	    remoteClient() const;


	/** Set the routerMediaControl property.
	 */
    	void	    	    routerMediaControl(bool);


	/** Access the routerMediaControl property.
	 */
    	bool	    	    routerMediaControl() const;


	/** Access the h.225 protocol id.
	 */
    	string	    	    h225ProtocolID() const;
					

    private:

    	void	    	    h225ProtocolID(const string &);

	IPAddress   	    callSignalAddress_;
    	bool	    	    remoteClient_;
    	bool	    	    routerMediaControl_;
    	string	    	    h225ProtocolID_;

	mutable VMutex	    mutex_;

    	// Suppress copying
	//
	RouterInfo(const RouterInfo &);
	RouterInfo & operator=(const RouterInfo &);
};


} // namespace SipH323Info
} // namespace Vocal


#endif // !defined(ROUTER_INFO_DOT_H)
