/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const tEndpointInfoMap_cxx_Version = 
    "$Id: tEndpointInfoMap.cxx,v 1.10 2002/05/15 11:11:47 icahoon Exp $";


#include "Application.hxx"
#include "EndpointInfoMap.hxx"
#include "GatekeeperInfo.hxx"
#include "VLog.hxx"
#include "Random.hxx"
#include <iostream>
#include <cstring>
#include <cstdlib>
#include <ctime>
#include <list>
#include <string>


using namespace Vocal;
using namespace Vocal::SipH323Info;
using namespace Vocal::Transport;
using namespace Vocal::Logging;

static void
appendAddress(
    H225_ArrayOf_TransportAddress   	&   transportAddresses,
    const IPAddress 	    	    	&   ipAddress
)
{
    PINDEX size = transportAddresses.GetSize();
    transportAddresses.SetSize(size + 1);
    
    H225_TransportAddress & transAddr = transportAddresses[size];

    transAddr.SetTag(H225_TransportAddress::e_ipAddress);

    H225_TransportAddress_ipAddress & ipAddr = transAddr;
    
    memcpy( &ipAddr.m_ip[0], 
    	    &(ipAddress.getIPAddress().sin_addr.s_addr), 
	    4);

    ipAddr.m_port = ipAddress.getPort();
}


static void
appendPhone(
    H225_ArrayOf_AliasAddress       &   prefix,
    const string                    &   phone
)
{
    PINDEX size = prefix.GetSize();
    prefix.SetSize(size + 1);
    
    H225_AliasAddress & aliasAddr = prefix[size];

    aliasAddr.SetTag(H225_AliasAddress::e_dialedDigits);

    PASN_IA5String & e164 = aliasAddr;

    e164 = PString(phone.c_str());
}


static void
appendPrefix(
    H225_ArrayOf_SupportedProtocols   	&   supportedProtocols,
    const H225_ArrayOf_AliasAddress     &   prefix
)
{
    PINDEX size = supportedProtocols.GetSize();
    supportedProtocols.SetSize(size + 1);
    
    H225_SupportedProtocols & supportedProtocol = supportedProtocols[size];

    supportedProtocol.SetTag(H225_SupportedProtocols::e_voice);

    H225_VoiceCaps & voiceCaps = supportedProtocol;
    
    voiceCaps.IncludeOptionalField(H225_VoiceCaps::e_supportedPrefixes);
    
    for ( size_t i = 0; i < prefix.GetSize(); i++ )
    {
        size = voiceCaps.m_supportedPrefixes.GetSize();
        voiceCaps.m_supportedPrefixes.SetSize(size + 1);
    
        H225_SupportedPrefix & supportedPrefix = voiceCaps.m_supportedPrefixes[size];

        supportedPrefix.m_prefix = prefix[i];
    }
}


class tApplication : public Application
{
    public:
        ReturnCode      init(int, char**, char**);
        void            uninit();

    	ReturnCode  	run();
};


class tApplicationCreator : public ApplicationCreator
{
    Application * create() { return ( new tApplication ); }
};


int main(int argc, char ** argv, char ** arge)
{
    tApplicationCreator creator;
    Application::creator(&creator);
    return ( Application::main(argc, argv, arge) );
}


ReturnCode
tApplication::init(int, char**, char**)
{
    VLog::init(LOG_VERBOSE);
    
    return ( SUCCESS );
}


void
tApplication::uninit()
{
    VLog::uninit();
}


ReturnCode
tApplication::run()
{
    Random                      random;
    H225_RegistrationRequest	rrq, rrq2;
    list<Sptr<EndpointInfo> >   no_gws;
    GatekeeperInfo  	    	gkInfo(no_gws);
    Alias   	    	    	alias;
    H225_ArrayOf_AliasAddress   prefix1, prefix2;
    IPAddress	    	    	csAddr("192.168.5.23", 1720);
    IPAddress	    	    	rasAddr("192.168.5.23", 1530);

    EndpointInfoMap 	    	eptMap;

    

    rrq.m_requestSeqNum = 1;
    rrq.m_protocolIdentifier.SetValue("0.0.8.2250.0.4");
    rrq.m_discoveryComplete = false;
    appendAddress(rrq.m_callSignalAddress, csAddr);
    appendAddress(rrq.m_rasAddress, rasAddr);

    if ( random() % 2 == 0 ) // true ) 
    {
        rrq.m_terminalType.IncludeOptionalField(H225_EndpointType::e_gateway);
        if ( random() % 2 == 0 ) // false ) 
        {
            rrq.m_terminalType.m_gateway.IncludeOptionalField(H225_GatewayInfo::e_protocol);
            appendPhone(prefix1,"140852");
            appendPhone(prefix1,"140851");
            appendPhone(prefix2,"165032");
            appendPrefix(rrq.m_terminalType.m_gateway.m_protocol, prefix1);
            appendPrefix(rrq.m_terminalType.m_gateway.m_protocol, prefix2);
        }
    }
    else
    {
        rrq.m_terminalType.IncludeOptionalField(H225_EndpointType::e_terminal);
    }

    alias.clear();
    alias.user("Radon Noble");
    alias.phone("2222");

    rrq.IncludeOptionalField(H225_RegistrationRequest::e_terminalAlias);
    rrq.m_terminalAlias = alias.alias();

    rrq.m_endpointVendor.m_vendor.m_t35CountryCode = 181;
    rrq.m_endpointVendor.m_vendor.m_t35Extension = 0;
    rrq.m_endpointVendor.m_vendor.m_manufacturerCode = 21324;

    rrq.m_endpointVendor.IncludeOptionalField(H225_VendorIdentifier::e_productId);
    rrq.m_endpointVendor.m_productId.SetValue("Microsoft NetMeeting");

    rrq.m_endpointVendor.IncludeOptionalField(H225_VendorIdentifier::e_versionId);
    rrq.m_endpointVendor.m_versionId.SetValue("3.0");

    rrq.IncludeOptionalField(H225_RegistrationRequest::e_keepAlive);
    rrq.m_keepAlive = false;

    rrq.IncludeOptionalField(H225_RegistrationRequest::e_willSupplyUUIEs);
    rrq.m_willSupplyUUIEs = false;

    Sptr<EndpointInfo>	ept = new EndpointInfo(rrq);

    ept->endpointIdentifier(gkInfo.generateEndpointIdentifier());

    cout << rrq << endl << endl;

    cout << *ept << endl << endl;

    eptMap.insert(ept);

    // Create the second rrq.
    //    
    rrq2.m_requestSeqNum = 2;
    rrq2.m_protocolIdentifier.SetValue("0.0.8.2250.0.2");
    rrq2.m_discoveryComplete = false;
    appendAddress(rrq2.m_callSignalAddress, IPAddress("1.2.3.4", 1720));
    appendAddress(rrq2.m_rasAddress, IPAddress("1.2.3.4", 1719));

    rrq2.m_terminalType.IncludeOptionalField(H225_EndpointType::e_terminal);

    alias.clear();
    alias.user("Foo Bar");
    alias.phone("1111");

    rrq2.IncludeOptionalField(H225_RegistrationRequest::e_terminalAlias);
    rrq2.m_terminalAlias = alias.alias();

    rrq2.m_endpointVendor.m_vendor.m_t35CountryCode = 181;
    rrq2.m_endpointVendor.m_vendor.m_t35Extension = 0;
    rrq2.m_endpointVendor.m_vendor.m_manufacturerCode = 22899;

    rrq2.m_endpointVendor.IncludeOptionalField(H225_VendorIdentifier::e_productId);
    rrq2.m_endpointVendor.m_productId.SetValue("Vocal HaHaHa");

    rrq2.m_endpointVendor.IncludeOptionalField(H225_VendorIdentifier::e_versionId);
    rrq2.m_endpointVendor.m_versionId.SetValue("1.0");

    rrq2.IncludeOptionalField(H225_RegistrationRequest::e_keepAlive);
    rrq2.m_keepAlive = false;

    rrq2.IncludeOptionalField(H225_RegistrationRequest::e_willSupplyUUIEs);
    rrq2.m_willSupplyUUIEs = false;

    Sptr<EndpointInfo>	ept2 = new EndpointInfo(rrq2);

    if ( random() % 2 == 0 ) //true )
    {
        ept2->endpointType(EndpointInfo::GATEWAY_TYPE);
        
        if ( random() % 2 == 0 ) //false ) 
        {
            list<string>    prefixes;
            prefixes.push_back(string("155512"));
            prefixes.push_back(string("155513"));
            prefixes.push_back(string("155514"));
            ept2->prefix(prefixes);
        }
    }

    ept2->endpointIdentifier(gkInfo.generateEndpointIdentifier());

    cout << rrq2 << endl << endl;

    cout << *ept2 << endl << endl;

    eptMap.insert(ept2);

    alias.clear();
    alias.phone("1111");

    ept = eptMap.findByAlias(alias.alias());
    
    assert( ept != 0 );

    alias.clear();
    alias.phone("2222");

    ept = eptMap.findByAlias(alias.alias());
    
    assert( ept != 0 );

    string  prefixAlias1("14085299999"), prefixAlias2("14085199999"), 
            prefixAlias3("16503299999"), prefixAlias4("15551299999"), 
            prefixAlias5("15551399999"), prefixAlias6("15551499999");

    string  matched;
        
    EndpointInfoSet     eptInfoSet 
        = eptMap.findByPrefix(prefixAlias1, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias1 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias2, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias2 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias3, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias3 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias4, matched);

    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias4 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias5, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias5 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias6, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias6 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }
    
    eptMap.erase(ept2);
    
    eptInfoSet = eptMap.findByPrefix(prefixAlias1, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias1 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias2, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias2 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias3, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias3 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }
    
    eptInfoSet = eptMap.findByPrefix(prefixAlias4, matched);

    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias4 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias5, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias5 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }

    eptInfoSet = eptMap.findByPrefix(prefixAlias6, matched);
    
    for (   EndpointInfoSet::iterator it = eptInfoSet.begin();
            it != eptInfoSet.end();
            ++it
        )
    {
        cout << "alias = \"" << prefixAlias6 << "\" => endpointInfo = { "
             << *(*it) << " }" << endl;
    }
    
    return ( 0 );
}
