/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000-2001 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const tEndpointInfoMap2_cxx_Version = 
    "$Id: tEndpointInfoMap2.cxx,v 1.2 2002/05/15 09:43:31 icahoon Exp $";


#include "EndpointInfoMap.hxx"
#include "EndpointInfo.hxx"
#include "Alias.hxx"
#include "IPAddress.hxx"
#include "Verify.hxx"


using namespace Vocal;
using namespace Vocal::SipH323Info;
using namespace Vocal::Transport;


int main()
{
    int count = 0;
    
    EndpointInfoMap     emap;
    
    Sptr<EndpointInfo>  einfo = new EndpointInfo;
    
    einfo->endpointIdentifier("endpoint_1");
    einfo->endpointType(EndpointInfo::TERMINAL_TYPE);
    
    Alias   a1;
    a1.user("siph323-5300");
    a1.phone("1111");

    einfo->alias(a1.alias());
    einfo->rasAddress(IPAddress("172.19.175.229:52345"));
    einfo->callSignalAddress(IPAddress("172.19.175.229:1720"));
    einfo->recordRegistration();
    
    emap.insert(einfo);
    
    cout << emap << endl;
    
    Alias   a;
    a.user("Ian Cahoon");
    a.phone("5495");
    
    Sptr<EndpointInfo> e = emap.findByAlias(a.alias());
    
    test_verify( e == 0 ); ++count;

    a.clear();
    a.phone("1111");
    
    e = 0;
    e = emap.findByAlias(a.alias());
    
    test_verify( e != 0 ); ++count;
    
    cout << *e << endl;
    
    a.clear();
    a.phone("2222");
    
    e = 0;
    e = emap.findByAlias(a.alias());
    
    test_verify( e == 0 ); ++count;
    
    a.clear();
    a.user("me");
    
    e = 0;
    e = emap.findByAlias(a.alias());
    
    test_verify( e == 0 ); ++count;
    
    a.clear();
    a.user("siph323-5300");
    
    e = 0;
    e = emap.findByAlias(a.alias());
    
    test_verify( e != 0 ); ++count;
    test_verify( *e == *einfo ); ++count;
    
    return ( test_return_code(count) );
}
