/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtCLCAck_cxx_Version = 
    "$Id: EvtCLCAck.cxx,v 1.10 2001/10/24 21:26:38 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtCLCAck.hxx"
#include "FeatureLCO.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtCLCAck::
EvtCLCAck(RouterCallCtl* sm)
    : EvtCallCtlLCOResponse(sm)
{
}


void EvtCLCAck::
inEstablished(FeatureLCO& lco)
{
    const string    fn("EvtCLCAck::inEstablished");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    lco.setState(&PatternLCO::inReleased, "inReleased");
    lco.releaseInd_.start();
}


void EvtCLCAck::
inWaitRelease(FeatureLCO& lco)
{
    const string    fn("EvtCLCAck::inWaitRelease");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    lco.setState(&PatternLCO::inReleased, "inReleased");
    lco.releaseCfm_.start();
}


void EvtCLCAck::
inOther(FeatureLCO& lco)
{
    const string    fn("EvtCLCAck::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VWARN(log) << "Got CLC ack in wrong state for call "
               << lco.getSM() << VWARN_END(log);
}


EvtH245B* EvtCLCAck::
clone_impl()
{
    return new EvtCLCAck(*this);
}


EvtCLCAck::TagType EvtCLCAck::
getTag() const throw()
{
    // (depends from H.245 parser)
    return H245Response::e_closeLogicalChannelAck;
}


void EvtCLCAck::
updateID()
{
    setLCN(getMessage().m_forwardLogicalChannelNumber);
}


#ifndef DISABLE_WRITER

ostream& EvtCLCAck::
writeTo(ostream& out) const
{
    out << "EvtCLCAck";
    EvtCallCtlLCOResponse::writeTo(out);
    return out;
}

#endif
