/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtCallSigTimer_cxx_Version = 
    "$Id: EvtCallSigTimer.cxx,v 1.6 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "EvtCallSigTimer.hxx"
#include "H323Consts.hxx"
#include "FeatureCallSig.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtCallSigTimer::
EvtCallSigTimer(SMCall* sm)
    : EvtTimerCallSig(sm),
      timeout_(0)
{
}


void EvtCallSigTimer::
inIdleRAS(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inIdleRAS");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


void EvtCallSigTimer::
inIdleNoRAS(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inIdleNoRAS");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


void EvtCallSigTimer::
inAdmitted(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inAdmitted");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.connectionAcceptTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inAlerting(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inAlerting");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.userTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inSetup(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inSetup");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.rasTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inAdmission(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inAdmission");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.rasTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inConnecting(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inConnecting");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.connectionEstablTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inInitiated(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inInitiated");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.connectionTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inAck(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inAck");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.connectionTimeoutInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inWaitConnect(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inWaitConnect");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    callsig.noAnswerInd_.start();
    callsig.callCleanupInd_.start();
}


void EvtCallSigTimer::
inConnect(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inConnect");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


void EvtCallSigTimer::
inOther(FeatureCallSig& callsig)
{
    const string    fn("EvtCallSigTimer::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(false);
}


mte::TimeInterval EvtCallSigTimer::
getTimerValue() const throw()
{
    return timeout_;
}


void EvtCallSigTimer::
setTimeout(mte::TimeInterval timeout) throw()
{
    const string    fn("EvtCallSigTimer::setTimeout");
    VLog            log(fn);

    VDEBUG(log) << fn << ": timeout(" << timeout << ")" << VDEBUG_END(log);

    timeout_ = timeout;
}


#ifndef DISABLE_WRITER

ostream& EvtCallSigTimer::
writeTo(ostream& out) const
{
    out << "EvtCallSigTimer";
    return out;
}

#endif
