/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtOLCAck_cxx_Version = 
    "$Id: EvtOLCAck.cxx,v 1.11 2001/10/24 21:26:39 icahoon Exp $";


#include <cassert>

#include "VLog.hxx"

#include "CallCtlUnexpectedException.hxx"
#include "EvtOLCAck.hxx"
#include "FeatureLCO.hxx"

using namespace router;
using Vocal::Logging::VLog;

EvtOLCAck::
EvtOLCAck(RouterCallCtl* sm)
    : EvtCallCtlLCOResponse(sm)
{
}


void EvtOLCAck::
inWaitEstabl(FeatureLCO& lco)
{
    const string    fn("EvtOLCAck::inWaitEstabl");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    lco.setState(&PatternLCO::inEstablished, "inEstablished");
    lco.establishCfm_.start();
}


void EvtOLCAck::
inOther(FeatureLCO& lco)
{
    const string    fn("EvtOLCAck::inOther");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    // Ignore
    VWARN(log) << fn << ": Got unexpected OLC Ack for call "
               << lco.getSM() << VWARN_END(log);

    throw CallCtlUnexpectedException(
        "Got unexpected message: OLC Ack",
        __FILE__, __LINE__);
}


EvtH245B* EvtOLCAck::
clone_impl()
{
    return new EvtOLCAck(*this);
}


EvtOLCAck::TagType EvtOLCAck::
getTag() const throw()
{
    // (depends from H.245 parser)
    return H245Response::e_openLogicalChannelAck;
}


void EvtOLCAck::
updateID()
{
    const string    fn("EvtOLCAck::updateID");
    VLog            log(fn);

    VDEBUG(log) << fn << VDEBUG_END(log);

    setLCN(getMessage().m_forwardLogicalChannelNumber);

    VDEBUG(log) << fn << ": LCN=" << getLCN() << VDEBUG_END(log);
}


#ifndef DISABLE_WRITER

ostream& EvtOLCAck::
writeTo(ostream& out) const
{
    out << "EvtOLCAck";
    EvtCallCtlLCOResponse::writeTo(out);
    return out;
}

#endif
