#ifndef INCLUDED_EvtTCS_hxx
#define INCLUDED_EvtTCS_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const EvtTCS_hxx_Version = 
"$Id: EvtTCS.hxx,v 1.9 2001/03/09 06:54:42 icahoon Exp $";

#include "mte/EvtSM2.hxx"

#include "EvtCEIMessages.hxx"
#include "EvtH245U.hxx"
#include "RouterCallCtl.hxx"

namespace router
{

    typedef mte::EvtSM2<SMCall, EvtFeatureCEI, EvtFeatureH245> EvtSM2CEI;
    typedef mte::EvtRouted<EvtSM2CEI, EvtRouterCEI>
        EvtRoutedCEI2;
    typedef mte::EvtBind1<EvtH245Request, EvtRoutedCEI2>
        EvtCallCtlCEIRequest2;

    /// Capability exchange (in) TCS message event
    class EvtTCS: public EvtCallCtlCEIRequest2
    {
    public:
        EvtTCS(RouterCallCtl* sm);

	// TCS handlers

        virtual void inIdle(FeatureCEI& cei);

        virtual void inWait(FeatureCEI& cei);
        
        virtual void inOther(FeatureCEI& cei);

	// H245 handlers

        virtual void inPaused(FeatureH245& h245);

        virtual void inTPaused(FeatureH245& h245);

        virtual void inTCE(FeatureH245& h245);

        virtual void inOpen(FeatureH245& h245);

        virtual void inOther(FeatureH245& h245);

	bool isNonZero();

    protected:
        const H245TCS& getMessage() const throw();

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        void processNewCapSet(FeatureCEI& cei);

        virtual void reply(FeatureCEI& cei);
        
        virtual EvtH245B* clone_impl();
        
        virtual TagType getTag() const throw();
    };
    
    //----------------------------
    // Inline methods
    //----------------------------

    inline const H245TCS& EvtTCS::
    getMessage() const throw()
    {
        return static_cast<const H245TCS&>(
            getSelMessage());
    }
    
}

#endif
