
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF 1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const FeatureH245_cxx_Version = 
"$Id: FeatureH245.cxx,v 1.3 2001/10/19 21:37:34 icahoon Exp $";

#include "VLog.hxx"

#include "FeatureH245.hxx"
#include "SMCall.hxx"

using namespace router;
using Vocal::Logging::VLog;


FeatureH245::
FeatureH245(SMCall* sm)
    : sm_(sm)
{
    setEntry(&entry_);
    setExit(&exit_);
    reset();
}


FeatureH245::
~FeatureH245()
{
    VLog log;
    VDEBUG(log) << "~FeatureH245" << VDEBUG_END(log);

    // Call exit methods for any state
    setState(&PatternH245::inPaused, "inPaused");
}


void FeatureH245::
reset()
{
    msdStarted = false;
    msdCompleted = false;
    gotTCS = false;
    gotTCSAck = false;
    attempt = 0;
    tcsFailures = 0;
    setStateStealth(&PatternH245::inPaused, "inPaused");
}


void FeatureH245::
process_impl(PatternH245& pattern)
{
    (pattern.*(getState()))(*this);
}


#ifndef DISABLE_WRITER

ostream& FeatureH245::
writeTo(ostream& out) const
{
    out << "FeatureH245:"
        << " state=" << getStateName();
    return out;
}


char* FeatureH245::
getStateName() const
{
    if (getState() == &PatternH245::inPaused)
        return "Paused";
    else if (getState() == &PatternH245::inTPaused)
        return "TPaused";
    else if (getState() == &PatternH245::inTCE)
        return "TCE";
    else if (getState() == &PatternH245::inOpen)
        return "Open";
    else
        return "UNKNOWN";
}

#endif
