#ifndef INCLUDED_FeatureMsd_hxx
#define INCLUDED_FeatureMsd_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const FeatureMsd_hxx_Version = 
"$Id: FeatureMsd.hxx,v 1.11 2001/03/09 06:54:42 icahoon Exp $";

#include "Sptr.hxx"

#include "mte/EventPtr.hxx"
#include "mte/Feature.hxx"

#include "H323Consts.hxx"
#include "PatternMsd.hxx"
#include "EvtMsdTimer.hxx"
#include "EntryMsd.hxx"
#include "ExitMsd.hxx"
#include "EvtMSDConfirmed.hxx"

namespace router
{

    class SMCall;

    /// Msd state variables
    class MsdData
    {
    public:
        enum SV_Status
        {
            SV_Master,
            SV_Slave,
            SV_Indeterminate
        };

        SV_Status sv_STATUS;
        long sv_SDNUM;
        int sv_NCOUNT;

        // These classes are allowed to access state variables:
        friend class FeatureMsd;
        friend class EvtMsdRequest;
        friend class EvtMSDetermination;
        friend class EvtMSDeterminationAck;
        friend class EvtMSDeterminationReject;
    };
    
    /// Feature for master-slave determination procedure
    class FeatureMsd:
        public mte::Feature<PatternMsd, FeatureMsd&>,
        public MsdData
    {
    public:
        /// Construct feature and assign pointer to call state machine
        FeatureMsd(SMCall* sm);
        
        virtual ~FeatureMsd();

        /// Return pointer to the state machine
        SMCall* getSM() const;

        /// Reset feature
        virtual void reset();

        /// Start timer
        void startTimer();

        /// Stop timer
        void stopTimer();

        /// Set random value
        void set_sv_SDNUM();

        /// Determine master-slave status
        void determineStatus(int peer_TT, long peer_SDNUM);

        /// Get status
        MsdData::SV_Status getStatus() const throw();

        // Callback events

        /** MSD indication callback event
	    (deviation from H245 standard: this callback is activated when
	    MSD procedure is started by ANY side)
	*/
        mte::EventPtr<CallbackEvent> callbackIndication_;

        /// MSD reject callback event
        mte::EventPtr<CallbackEvent> callbackReject_;

        /// MSD confirm callback event
        mte::EventPtr<EvtMSDConfirmed> callbackConfirm_;

#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

    private:
        /// Process event (implementation)
        virtual void process_impl(PatternMsd& pattern);

#ifndef DISABLE_WRITER
        char* getStateName() const;
#endif
        
        SMCall* const sm_;
        Sptr<EvtMsdTimer> timer_;
        EntryMsd entry_;
        ExitMsd exit_;
    };
    
    //----------------------------
    // Inline  methods
    //----------------------------

    inline SMCall* FeatureMsd::
    getSM() const
    {
        return sm_;
    }

    inline FeatureMsd::MsdData::SV_Status FeatureMsd::
    getStatus() const throw()
    {
        return sv_STATUS;
    }
}

#endif
