#ifndef INCLUDED_H245Defs_hxx
#define INCLUDED_H245Defs_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const H245Defs_hxx_Version = 
"$Id: H245Defs.hxx,v 1.10 2001/03/09 06:54:42 icahoon Exp $";

#include "ptlib.h"
#include "h245.h"

namespace router
{
    // Definitions specific to OpenH323

    typedef H245_MultimediaSystemControlMessage H245Message;
    typedef H245_RequestMessage H245Request;
    typedef H245_ResponseMessage H245Response;
    typedef H245_CommandMessage H245Command;
    typedef H245_IndicationMessage H245Indication;

    typedef H245_RequestMessage::Choices H245RequestTag;
    typedef H245_ResponseMessage::Choices H245ResponseTag;
    typedef H245_CommandMessage::Choices H245CommandTag;
    typedef H245_IndicationMessage::Choices H245IndicationTag;

    // H.245 messages creation
    
    template<class Base, class Msg, typename Base::Choices ttag>
    class H245Base: public Base
    {
    public:
        typedef Msg MsgType;
        
    protected:
        H245Base(typename Msg::Choices ptag);
    };

    template<class Base, class Msg, typename Base::MsgType::Choices ttag>
    class H245Derive: public Base
    {
    public:
        H245Derive();
        Msg& m();
    };

    template<class Base, class Msg, typename Base::Choices ttag>
    inline H245Base<Base, Msg, ttag>::
    H245Base(typename Msg::Choices ptag)
    {
        Base::SetTag(ttag);
        H245Message& msg = *this;
        Msg& theMsg = msg;
        theMsg.SetTag(ptag);
        // TODO: when migrate to a newer c++ compiler use following line
        // operator Msg& ().SetTag(ptag);
    }

    template<class Base, class Msg, typename Base::MsgType::Choices ttag>
    inline H245Derive<Base, Msg, ttag>::
    H245Derive()
        : Base(ttag)
    {
    }

    template<class Base, class Msg, typename Base::MsgType::Choices ttag>
    inline Msg& H245Derive<Base, Msg, ttag>::
    m()
    {
        H245Message& msg = *this;
        typename Base::MsgType& base = msg;
        return base;
        // TODO: when migrate to a newer c++ compiler use following line
        // return operator typename Base::MsgType& ().operator Msg& ();
    }

    typedef H245Base<H245Message, H245Request,
        H245Message::e_request> BH245Request;

    typedef H245Base<H245Message, H245Response,
        H245Message::e_response> BH245Response;

    typedef H245Base<H245Message, H245Command,
        H245Message::e_command> BH245Command;

    typedef H245Base<H245Message, H245Indication,
        H245Message::e_indication> BH245Indication;


    typedef H245_MasterSlaveDetermination H245MSDetermination;
    typedef H245Derive<BH245Request, H245_MasterSlaveDetermination,
        H245Request::e_masterSlaveDetermination> BH245MSDetermination;

    typedef H245_RoundTripDelayRequest H245RoundTripDelayRequest;
    typedef H245Derive<BH245Request, H245_RoundTripDelayRequest,
        H245Request::e_roundTripDelayRequest> BH245RoundTripDelayRequest;

    typedef H245_TerminalCapabilitySet H245TCS;
    typedef H245Derive<BH245Request, H245_TerminalCapabilitySet,
        H245Request::e_terminalCapabilitySet> BH245TCS;

    typedef H245_OpenLogicalChannel H245OLC;
    typedef H245Derive<BH245Request, H245OLC,
        H245Request::e_openLogicalChannel> BH245OLC;

    typedef H245_RequestChannelClose H245RCC;
    typedef H245Derive<BH245Request, H245RCC,
        H245Request::e_requestChannelClose> BH245RCC;

    typedef H245_CloseLogicalChannel H245CLC;
    typedef H245Derive<BH245Request, H245CLC,
        H245Request::e_closeLogicalChannel> BH245CLC;

    typedef H245_MasterSlaveDeterminationAck H245MSDeterminationAck;
    typedef H245Derive<BH245Response, H245_MasterSlaveDeterminationAck,
        H245Response::e_masterSlaveDeterminationAck> BH245MSDeterminationAck;

    typedef H245_MasterSlaveDeterminationReject H245MSDeterminationReject;
    typedef H245Derive<BH245Response, H245_MasterSlaveDeterminationReject,
        H245Response::e_masterSlaveDeterminationReject>
        BH245MSDeterminationReject;

    typedef H245_RoundTripDelayResponse H245RoundTripDelayResponse;
    typedef H245Derive<BH245Response, H245_RoundTripDelayResponse,
        H245Response::e_roundTripDelayResponse> BH245RoundTripDelayResponse;

    typedef H245_TerminalCapabilitySetAck H245TCSAck;
    typedef H245Derive<BH245Response, H245_TerminalCapabilitySetAck,
        H245Response::e_terminalCapabilitySetAck> BH245TCSAck;

    typedef H245_TerminalCapabilitySetReject H245TCSReject;
    typedef H245Derive<BH245Response, H245_TerminalCapabilitySetReject,
        H245Response::e_terminalCapabilitySetReject> BH245TCSReject;

    typedef H245_OpenLogicalChannelAck H245OLCAck;
    typedef H245Derive<BH245Response, H245OLCAck,
        H245Response::e_openLogicalChannelAck> BH245OLCAck;

    typedef H245_OpenLogicalChannelReject H245OLCReject;
    typedef H245Derive<BH245Response, H245OLCReject,
        H245Response::e_openLogicalChannelReject> BH245OLCReject;

    typedef H245_CloseLogicalChannelAck H245CLCAck;
    typedef H245Derive<BH245Response, H245CLCAck,
        H245Response::e_closeLogicalChannelAck> BH245CLCAck;

    typedef H245_RequestChannelCloseAck H245RCCAck;
    typedef H245Derive<BH245Response, H245RCCAck,
        H245Response::e_requestChannelCloseAck> BH245RCCAck;

    typedef H245_RequestChannelCloseReject H245RCCReject;
    typedef H245Derive<BH245Response, H245RCCReject,
        H245Response::e_requestChannelCloseReject> BH245RCCReject;

    typedef H245_MasterSlaveDeterminationRelease H245MSDeterminationRelease;
    typedef H245Derive<BH245Indication, H245_MasterSlaveDeterminationRelease,
        H245Indication::e_masterSlaveDeterminationRelease>
        BH245MSDeterminationRelease;

    typedef H245_TerminalCapabilitySetRelease H245TCSRelease;
    typedef H245Derive<BH245Indication, H245_TerminalCapabilitySetRelease,
        H245Indication::e_terminalCapabilitySetRelease>
        BH245TCSRelease;

    typedef H245_RequestChannelCloseRelease H245RCCRelease;
    typedef H245Derive<BH245Indication, H245RCCRelease,
        H245Indication::e_requestChannelCloseRelease> BH245RCCRelease;

    typedef H245_EndSessionCommand H245EndSession;
    typedef H245Derive<BH245Command, H245EndSession,
        H245Command::e_endSessionCommand>
        BH245EndSession;

}

#endif
