#if !defined(Q931_PROTOCOLBASE_DOT_H)
#define Q931_PROTOCOLBASE_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Q931ProtocolBase_hxx_Version = 
"$Id$";


#include "TPKTClientProtocol.hxx"
#include "TPKTClientSocket.hxx"
#include "Sptr.hxx"
#include "SesQ931.hxx"


namespace router
{
    class ConnectionCallSig;
}

namespace Vocal
{
    namespace SipH323Info {
    	class RouterInfo;
    }
    
    namespace Services {
    	class PollFifoService;
    }
    
    using Vocal::SipH323Info::RouterInfo;
    using Vocal::Services::PollFifoService;
    using Vocal::Transport::TPKTClientSocket;
    using Vocal::Transport::ConnectionBrokenException;
    using Vocal::IO::FileDescriptor;
    using Vocal::Transport::TPKTClientProtocol;
    
    /**
     */    
    class Q931ProtocolBase : public TPKTClientProtocol
    {

    public:

    	/**
	 */
        Q931ProtocolBase(
            PollFifoService 	    &	service,
    	    Sptr<TPKTClientSocket>  	q931Socket,
            router::SesQ931         &   q931EventSrc,
            RouterInfo	    	    &	routerInfo, 
    	    const char      	    *	name = 0
            );


    	/**
	 */
    	virtual ~Q931ProtocolBase();


    	/**
	 */
        virtual const FileDescriptor& getFileDescriptor() const;


    	/**
	 */
    	virtual TPKTClientSocket& getTPKTClientSocket();


    	/**
	 */
    	virtual void onOutgoingAvailable();


    	/**
	 */
    	virtual void onTPKTArrived(vector<u_int8_t>& message);


    	/**
	 */
        virtual void onDisconnect(ConnectionBrokenException*)
            throw ( Vocal::Transport::ProtocolException );


    	/**
	 */
        virtual void onError(SystemException*)
            throw ( Vocal::Transport::ProtocolException );


    	/**
	 */
        virtual void handleUnknownException();


    	/**
	 */
        void terminate();


    	/**
	 */
        void setConnection(router::ConnectionCallSig* connection) throw();


    	/**
	 */
        virtual void makeNewCall() {};


    	/**
	 */
        virtual void onDestroyCall() {};


    private:

    	PollFifoService     	    &	service_;
        Sptr<TPKTClientSocket>	    	q931Socket_;
        router::SesQ931             &   q931EventSrc_;
        RouterInfo  	    	    &	routerInfo_;
        router::ConnectionCallSig   *   connection_;

        friend class Q931Protocol;
    };

} // namespace Vocal


#endif // !defined(Q931_PROTOCOLBASE_DOT_H)
