/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323CallSigGW_cxx_Version = 
    "$Id: SipToH323CallSigGW.cxx,v 1.5 2002/07/10 22:09:00 icahoon Exp $";


#include "SipToH323CallSigGW.hxx"
#include "SipToH323Config.hxx"
#include "SipToH323RASPolicyBuilder.hxx"
#include "SipProxyInfo.hxx"
#include "GatekeeperInfo.hxx"
#include "RouterInfo.hxx"
#include "H323Gatekeeper.hxx"
#include "H323SipProxyService.hxx"
#include "H323CallTranslator.hxx"
#include "ServiceManager.hxx"
#include "VLog.hxx"


// TEMP CODE: 	This is a temporary work around to avoid calling
//          	gethostbyname in CapabilityMap.
//
string	    SipToH323CallSigGW_HostAddress;
//
// TEMP CODE END


using Vocal::Application;
using Vocal::SipH323CSGW::SipToH323CallSigGW;
using Vocal::Configuration::Config;
using Vocal::SipH323CSGW::SipToH323Config;
using Vocal::SipH323Info::SipProxyInfo;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::RouterInfo;
using Vocal::SipH323Proxy::H323SipProxyService;
using Vocal::Transport::IPAddress;
using Vocal::Logging::VLog;
using Vocal::H323CallTranslator;
using namespace Vocal::H323GK;
using Vocal::ReturnCode;


SipToH323CallSigGW::SipToH323CallSigGW()
    :	myIncomingSipProxyInfo(0),
    	myOutgoingSipProxyInfo(0),
    	myGkInfo(0),
	myRouterInfo(0),
    	myRasPolicyBuilder(0),
    	myIncomingSipProxy(0),
    	myOutgoingSipProxy(0),
    	myH323Gk(0),
    	myH323CallTranslator(0)
{
}


SipToH323CallSigGW::~SipToH323CallSigGW()
{
}


Config *    
SipToH323CallSigGW::createConfig()
{
    return ( new SipToH323Config );
}


string  
SipToH323CallSigGW::logFile() const
{
    const SipToH323Config  & cfg 
        = dynamic_cast<const SipToH323Config &>(config());

    char    fileName[256];
    
    size_t  rc = 0;
    
    if ( cfg.underDebugger() )
    {
        time_t  now = time(0);

        rc = strftime(fileName, 255, 
                      "siph323csgw.%Y-%m-%d:%T.log", gmtime(&now));
    }

    string  logfile(cfg.outputDirectory());
    
    if ( rc == 0 )
    {
        logfile += ServiceApplication::logFile();
    }
    else
    {
        string  current(cfg.outputDirectory());
        current += "current.txt";

        unlink(current.c_str());

        logfile += fileName;

        symlink(fileName, current.c_str());
    }
        
    
    return ( logfile );
}
         

ReturnCode
SipToH323CallSigGW::userInit()
{
    ServiceApplication::userInit();
    
    const SipToH323Config  & cfg 
        = dynamic_cast<const SipToH323Config &>(config());
    
    // Create the initialization infos.
    //
    myGkInfo = new GatekeeperInfo(
                    cfg.gateways(),
                    cfg.gatekeeperId(), 
    	    	    cfg.endpointId(),
                    cfg.sipRemoteClient());

    // TEMP CODE:   This is a temporary work around to avoid calling
    //       	    gethostbyname in CapabilityMap.
    //
    SipToH323CallSigGW_HostAddress = myGkInfo->hostAddress().getIPAsString();
    //
    // TEMP CODE

    IPAddress	incomingSipProxyAddr = myGkInfo->hostAddress();
    incomingSipProxyAddr.setPort(cfg.sipPort());
    
    myIncomingSipProxyInfo = new SipProxyInfo(incomingSipProxyAddr, 
    	    	    	    	    	    	cfg.sipRemote());

    IPAddress	outgoingSipProxyAddr = myGkInfo->hostAddress();
    outgoingSipProxyAddr.setPort(cfg.sipPort() + 1);
    
    myOutgoingSipProxyInfo = new SipProxyInfo(outgoingSipProxyAddr, 
    	    	    	    	    	    	cfg.sipRemote());
					    
    myRouterInfo = new RouterInfo(myGkInfo->callSignalAddress());
    
    // Create the sip proxy.
    //
    myIncomingSipProxy = new H323SipProxyService(
    	    	    	    	    	    *myIncomingSipProxyInfo,
					    "siph323csgw");

    myOutgoingSipProxy = new H323SipProxyService(
    	    	    	    	    	    *myOutgoingSipProxyInfo,
					    "siph323csgw");

    // Create H323 gatekeeper, passing a specialized ras policy builder that
    // knows the fifo for the sip proxy.
    //
    myRasPolicyBuilder	= new SipToH323RASPolicyBuilder;
    myH323Gk = new H323Gatekeeper(*myRasPolicyBuilder, *myGkInfo);

    // Create the call translator.
    //
    myH323CallTranslator = new H323CallTranslator(*myRouterInfo);
    
    // Connect the fifo interfaces.
    //
    //   Gatekeeper needs to know about the SipProxy.
    //   CallTranslator needs to know about Gatekeeper and SipProxy.
    //   SipProxy needs to knwo about CallTranslator.
    //
    myRasPolicyBuilder->connectToSipProxy(myIncomingSipProxy->getFifo());

    myH323CallTranslator->connectToGatekeeper(myH323Gk->getFifo());
    myH323CallTranslator->connectToSipProxy(myIncomingSipProxy->getFifo(),
    	    	    	    	    	   myOutgoingSipProxy->getFifo());

    myIncomingSipProxy->connectToCallTranslator(myH323CallTranslator->getFifo());
    myOutgoingSipProxy->connectToCallTranslator(myH323CallTranslator->getFifo());

    serviceManager().manage(*myIncomingSipProxy);
    serviceManager().manage(*myOutgoingSipProxy);
    serviceManager().manage(*myH323Gk);
    serviceManager().manage(*myH323CallTranslator);

    return ( SUCCESS );
}


void	    	    
SipToH323CallSigGW::userUninit()
{
    const string	fn("SipToH323CallSigGW::uninit");
    VLog        	log(fn);

    serviceManager().unmanage();
    
    delete myH323CallTranslator;
    myH323CallTranslator = 0;
    
    delete myH323Gk;
    myH323Gk = 0;
    
    delete myRasPolicyBuilder;
    myRasPolicyBuilder = 0;
    
    delete myIncomingSipProxy;
    myIncomingSipProxy = 0;

    delete myOutgoingSipProxy;
    myOutgoingSipProxy = 0;
    
     delete myGkInfo;
    myGkInfo = 0;
    
    delete myIncomingSipProxyInfo;
    myIncomingSipProxyInfo = 0;

    delete myOutgoingSipProxyInfo;
    myOutgoingSipProxyInfo = 0;

    delete myRouterInfo;
    myRouterInfo = 0;
    
    VINFO(log) << "SipToH323CallSigGW::uninit: Program uninitialized." 
	       << VINFO_END(log);

    ServiceApplication::userUninit();
}
