/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323Config_cxx_Version = 
    "$Id: SipToH323Config.cxx,v 1.8 2002/07/10 22:09:00 icahoon Exp $";


#include "SipToH323Config.hxx"
#include "ParsePair.hxx"
#include "Lock.hxx"
#include "vstring.hxx"
#include <string>
#include <iostream>
#include <strstream>
#include <cstdlib>
#include <cerrno>
#include <sys/stat.h>
#include <unistd.h>


using namespace Vocal;
using Vocal::SipH323CSGW::SipToH323Config;
using Vocal::SipH323Info::EndpointInfo;
using Vocal::Transport::IPAddress;
using Vocal::Configuration::Config;
using Vocal::Configuration::NameValueMap;
using Vocal::Configuration::ParsePair;
using Vocal::Configuration::VALUE_PRESENT;
using Vocal::Configuration::NO_VALUE;
using Vocal::Threads::ReadLock;
using std::string;
using std::list;


SipToH323Config::SipToH323Config()
    :   outputDirectoryStr  ("output"),
        gatekeeperIdStr     ("gkid"),
        endpointIdStr       ("eptid"),
        sipPortStr          ("sipport"),
        sipRemoteStr        ("sipremote"),
        sipRemoteClientStr  ("client"),
        gatewaysStr         ("gateways"),
        outputDirectoryChar ('o'),
        gatekeeperIdChar    ('g'),
        endpointIdChar      ('e'),
        sipPortChar         ('p'),
        sipRemoteChar       ('a'),
        sipRemoteClientChar ('c'),
        myOutputDirectory   ("./"),
        myGatekeeperId      (""),
        myEndpointId        ("endpoint_"),
        mySipPort           (22400),
        mySipRemote         ("192.168.1.1", 5060),
        mySipRemoteClient   (false)
{
}


SipToH323Config::~SipToH323Config()
{
}


void        
SipToH323Config::usage() const
{
    cerr << "usage: " << myApplicationName << " [ options ]" << endl
    	 << "  [ -o ] [ --output ]    directory   Output directory, " << endl
	 << "                                       Defaults to \"./\"" << endl
    	 << "  [ -l ] [ --loglvl ]    level       Log level, " << endl
	 << "                                       Defaults to \"LOG_ERR\"" << endl
    	 << "  [ -g ] [ --gkid ]      id          Use for gatekeeperIdentifier, " << endl
	 << "                                       Defaults to hostname" << endl
    	 << "  [ -e ] [ --eptid ]     id          Use for endpointIdentifier, " << endl
	 << "                                       Defaults to \"endpoint_\"" << endl
	 << "  [ -p ] [ --sipport ]   port        Sip udp port" << endl
	 << "                                       Defaults to 22400" << endl
	 << "  [ -a ] [ --sipremote ] ipaddress   IPAddress of sip remote entity" << endl
	 << "                                       Defaults to 192.168.1.1:5060" << endl
         << "  [ -c ] [ --client ]                If set, the remote address represents" << endl
         << "                                       a SIP client and H.323 registrations" << endl
         << "                                       will not be forwarded." << endl;
}
        

void        
SipToH323Config::loadCfgFileErrorMsg(ReturnCode) const
{
    cerr << "Error: config file \"" << myConfigFile
         << "\" could not be parsed." << endl;
}
        
    
ReturnCode  
SipToH323Config::preParse()
{
    ReturnCode  rc = SUCCESS;
    
    if  (   ( rc = Config::preParse() ) != SUCCESS
    
    
        ||  ( rc = myOptions.add(outputDirectoryStr.c_str(), 
                                 outputDirectoryChar, 
                                 VALUE_PRESENT) ) 
                != SUCCESS
                
        ||  ( rc = myOptions.add(gatekeeperIdStr.c_str(), 
                                 gatekeeperIdChar, 
                                 VALUE_PRESENT) ) 
                != SUCCESS
                
        ||  ( rc = myOptions.add(endpointIdStr.c_str(), 
                                 endpointIdChar,
                                 VALUE_PRESENT) ) 
                != SUCCESS
                
        ||  ( rc = myOptions.add(sipPortStr.c_str(), 
                                 sipPortChar, 
                                 VALUE_PRESENT) ) 
                != SUCCESS
                
        ||  ( rc = myOptions.add(sipRemoteStr.c_str(), 
                                 sipRemoteChar,
                                 VALUE_PRESENT) ) 
                != SUCCESS

        ||  ( rc = myOptions.add(sipRemoteClientStr.c_str(),
                                 sipRemoteClientChar,
                                 NO_VALUE) )
                != SUCCESS
        )
    {
        // Do nothing since rc is already set on failure.
    }

    return ( rc );
}
        
        
ReturnCode  
SipToH323Config::postParse()
{
    ReturnCode  rc = Config::postParse();

    if ( rc != SUCCESS )
    {
        return ( rc );
    }
    
    string output;
    if ( myOptions.get(outputDirectoryStr, output) && output.size() )
    {
        if ( output[output.size()-1] != '/' )
        {
	    output += "/";
        }
        if  (	output.find("../") == 0
	    ||	output.find("/../") != string::npos
	    )
        {
            usage();
	    cerr << "Error: directory cannot contain \"..\"" << endl;
	    return ( !SUCCESS );
        }

        struct stat    outputStat;

        if ( stat(output.c_str(), &outputStat) < SUCCESS )
        {
            usage();
    	    cerr << "Error: " << strerror(errno) << " for " << output << endl;
    	    return ( !SUCCESS );
        }
        if ( !S_ISDIR(outputStat.st_mode) )
        {
            usage();
	    cerr << "Error: file " << output << " isn't a directory" << endl;
	    return ( !SUCCESS );
        }

        myOutputDirectory = output;
    }

    myOptions.get(gatekeeperIdStr, myGatekeeperId);
    myOptions.get(endpointIdStr, myEndpointId);

    int sip_port = mySipPort;
    myOptions.get(sipPortStr, sip_port);
    mySipPort = sip_port;
    
    string  sip_remote;
    if ( myOptions.get(sipRemoteStr, sip_remote) )
    {
        mySipRemote.setIP(sip_remote);
    }

    mySipRemoteClient = myOptions.get(sipRemoteClientStr);
        
    rc = loadGateways();

    if ( rc != SUCCESS )
    {
        loadCfgFileErrorMsg(rc);
    }
            
    return ( rc );
}



string
SipToH323Config::outputDirectory() const
{
    ReadLock    lock(myMutex);
    return ( myOutputDirectory );
}


string   
SipToH323Config::gatekeeperId() const
{
    ReadLock    lock(myMutex);
    return ( myGatekeeperId );
}


string  
SipToH323Config::endpointId() const
{
    ReadLock    lock(myMutex);
    return ( myEndpointId );
}

        
u_int16_t   
SipToH323Config::sipPort() const
{
    ReadLock    lock(myMutex);
    return ( mySipPort );
}


IPAddress   
SipToH323Config::sipRemote() const
{
    ReadLock    lock(myMutex);
    return ( mySipRemote );
}


bool
SipToH323Config::sipRemoteClient() const
{
    ReadLock    lock(myMutex);
    return ( mySipRemoteClient );
}
        
        
list< Sptr<EndpointInfo> >  
SipToH323Config::gateways() const
{
    ReadLock    lock(myMutex);
    return ( myGateways );
}
        

ostream &   
SipToH323Config::writeTo(ostream & out) const
{
    Config::writeTo(out);
    
    out << "\n  output:                " << myOutputDirectory
	<< "\n  gatekeeperId:          " << myGatekeeperId
    	<< "\n  endpointId:            " << myEndpointId
        << "\n  sip port:              " << mySipPort
        << "\n  sip remote:            " << mySipRemote;

    for ( list< Sptr<EndpointInfo> >::const_iterator i = myGateways.begin();
          i != myGateways.end();
          ++i
        )
    {
        if ( i == myGateways.begin() )
        {
            out << "\n\nStatic Registrations:";
        }
    
        out << "\n  " << *(*i);
    }
    
    return ( out );
}

                 
ReturnCode      
SipToH323Config::loadGateways()
{
    // Load gateways value
    //
    string gateways;
    if ( !myOptions.get(gatewaysStr, gateways) )
    {
        return ( SUCCESS );
    }
    
    for ( string current_gw = getnext(gateways);
          current_gw.size();
          current_gw = getnext(gateways)
        )
    {
        // Load current gateway's call signaling address
        //
        IPAddress   callSignalAddress;
        
        string      gwCsAddrStr = current_gw + ".cs_address";

        string csipaddr;

        if ( myOptions.get(gwCsAddrStr, csipaddr) && csipaddr.size() )
        {
            callSignalAddress.setIP(csipaddr.c_str());
        }
        else
        {
            usage();
            cerr << "Error: " << gwCsAddrStr << " missing." << endl;
            return ( !SUCCESS );
        }

        // Load current gateway's prefixes
        //
        list<string>    prefixes;
        bool            stripPrefix = false;
        prefixes.clear();
        
        string          gwPrefixesStr = current_gw + ".prefix";
        string          gwStripPrefixStr = current_gw + ".strip_prefix";

        string all_prefixes;

        if ( myOptions.get(gwPrefixesStr, all_prefixes) && all_prefixes.size() )
        {
            for ( string current_prefix = getnext(all_prefixes);
                  current_prefix.size();
                  current_prefix = getnext(all_prefixes)
                )
            {
                prefixes.push_back(current_prefix);
                
            }

            if ( myOptions.get(gwStripPrefixStr) )
            {
                stripPrefix = true;
            }
        }
        else
        {
            // Accepts all prefixes
            //
            prefixes.push_back(string(""));
        }
        
        // Build the endpoint info.
        //
        Sptr<EndpointInfo>  endpointInfo = new EndpointInfo(current_gw,
                                                            callSignalAddress,
                                                            prefixes,
                                                            stripPrefix);
        // Validate the endpoint info
        //
        if ( !endpointInfo->valid() )
        {
            usage();
            cerr << "Error: Invalid endpoint information for gateway \""
                 << current_gw << '"' << endl;
            return ( !SUCCESS );
        }
        
        // Add the endpoint info into the endpoint list
        //
        myGateways.push_back(endpointInfo);        
        
    }

    return ( SUCCESS );
}
