#if !defined(VOCAL_SIP_TO_H323_RAS_POLICY_BUILDER_HXX)
#define VOCAL_SIP_TO_H323_RAS_POLICY_BUILDER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipToH323RASPolicyBuilder_hxx_Version = 
    "$Id: SipToH323RASPolicyBuilder.hxx,v 1.3 2001/08/22 20:36:28 icahoon Exp $";

#include "DefaultRASPolicyBuilder.hxx"
#include "Sptr.hxx"
#include "Event.hxx"
#include "FifoBase.h"

/** Vovida Open Communication Application Library<br><br>
 */
namespace Vocal
{


/** SIP to H.323 Call Signaling Gateway<br><br>
 */
namespace SipH323CSGW
{


using Vocal::H323GK::RASGKProtocol;
using Vocal::H323GK::RegistrationPolicy;
using Vocal::H323GK::AdmissionPolicy;
using Vocal::H323GK::LocationPolicy;
using Vocal::SipH323Info::GatekeeperInfo;
using Vocal::SipH323Info::EndpointInfoMap;
using Vocal::Services::Service;
using Vocal::Services::Event;



/** Sip to H323 Call Signaling Gateway specific RAS Policy builder.<br><br>
 *
 *  Since we have specific ras policies for the siph323csgw, we need to
 *  specific factory for them.
 *
 *  @see    Vocal::H323GK::DefaultRASPolicyBuilder
 *  @see    Vocal::SipH323CSGW::SipToH323RegistrationPolicy
 *  @see    Vocal::SipH323CSGW::SipToH323AdmissionPolicy
 *  @see    Vocal::Process::Service
 *  @see    Vocal::H323GK::RASGKProtocol
 *  @see    Vocal::SipH323Info::GatekeeperInfo
 *  @see    Vocal::SipH323Info::EndpointInfoMap
 *  @see    FifoBase
 *  @see    Sptr
 */
class SipToH323RASPolicyBuilder : public Vocal::H323GK::DefaultRASPolicyBuilder
{

    public:
    	
	/** Default constructor
	 */
    	SipToH323RASPolicyBuilder();
	
	
	/** Virtual destructor
	 */
	virtual 		    	~SipToH323RASPolicyBuilder();


    	/** Create the registration policy.
	 */
    	virtual RegistrationPolicy *	    buildRegistrationPolicy(
	    	    Service &,
	    	    RASGKProtocol &,
		    GatekeeperInfo &,
		    EndpointInfoMap &);


    	/** Create the admission policy.
	 */
	virtual AdmissionPolicy *	    buildAdmissionPolicy(
	    	    Service &,
	    	    RASGKProtocol &,
		    GatekeeperInfo &,
		    EndpointInfoMap &);


    	/** Create the location policy.
	 */
	virtual LocationPolicy *	    buildLocationPolicy(
	    	    Service &,
	    	    RASGKProtocol &,
		    GatekeeperInfo &,
		    EndpointInfoMap &);


    	/** Since the specialized policies may need to know about the
	 *  Sip proxy, we need to be given that information before the
	 *  policies are created.
	 */
    	void	connectToSipProxy(
		    FifoBase< Sptr<Event> > &);
					

    private:
    	
	FifoBase< Sptr<Event> >     *	sipProxyFifo_;
};


} // namespace SipH323CSGW
} // namespace Vocal


#endif // !defined(VOCAL_SIP_TO_H323_RAS_POLICY_BUILDER_HXX)
