/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Address_cxx_Version = 
    "$Id: Address.cxx,v 1.32 2002/07/10 07:10:22 icahoon Exp $";


#include <string>
#include <cctype>
#include <stdlib.h>
#include "VLog.hxx"
#include "Address.hxx"
#include "Assert.hxx"
#include "VLog.hxx"

#ifndef DEBUG
#define DEBUG 0
#endif


using namespace Translator;
using Vocal::Logging::VLog;


bool Address::isIp(const std::string &h_address) const
{
    std::string::size_type point = 0;
    std::string::size_type point1 = 0;
    std::string::size_type p_end = 0;
    std::string s_octet;
    int p_count = 0;
    unsigned int y = 0;
    bool flag = true;
    
    while(flag&&(point1!=std::string::npos)&&(p_count<=4))
    {
      point1 = h_address.find(".", point);
      
      if((point1==std::string::npos)&&(p_count!=3))
      {  flag = false; break; }
      
      if(point1 == std::string::npos)
        if(((h_address.length()-point)>3)||((h_address.length()-point)<1))
          {  flag = false; break; }

      if(point1 != std::string::npos)
        if(((point1-point)>3)||((point1-point)<1))
          {  flag = false; break; }
    
      if(point1!=std::string::npos)p_end = point1;
      else p_end = h_address.length();
      
      s_octet = "";
      for (y = point; y < p_end; y++)
      {
	    if(!isdigit(h_address[y]))
	      { flag = false; break; }
	    else s_octet += h_address[y];
      }
      if(atoi(s_octet.c_str()) > 255)
        { flag = false; break; }

      point = point1+1;
      p_count++;

    }
    return flag;
}

void Address::assign(const SipAddress &s_address_c) throw (XlatorException)
{
    int i = 0;
    
    SipAddress &s_address = const_cast<SipAddress &> (s_address_c);
    
    std::string nameSip = s_address.getNameAddr().convertString();
    std::string paramurlSip = s_address.getUrlParam().convertString();
    std::string validChars = "0123456789#*";
    
    
    if((nameSip.size()+paramurlSip.size())>=256)
        throw XlatorException("SIP-URL too long", __FILE__, __LINE__, 414);
	
    address.SetSize(0); // clear the thing out
    address.SetSize(alias_size);
    
    address[i].SetTag(H225_AliasAddress::e_h323_ID);
    (PASN_BMPString &)address[i] = (PString)nameSip.c_str();
    
    ++i;

    address[i].SetTag(H225_AliasAddress::e_url_ID);
    PASN_IA5String &ia5url = (PASN_IA5String &)address[i];
    ia5url = (PString)nameSip.c_str();
    
    ++i;

    if((PString)(nameSip.c_str())!=(PString)ia5url)--i;
    
    address[i].SetTag(H225_AliasAddress::e_email_ID);
    PASN_IA5String &ia5email = (PASN_IA5String &)address[i];
    std::string userH = s_address.getUserValue().convertString();
    std::string hostH = s_address.getHost().convertString();
    
    if((userH.length())&&(hostH.length()))
    {
       std::string emailH = userH + "@" + hostH;
    
       ia5email = (PString)emailH.c_str();
    
       ++i;

       if((PString)(emailH.c_str())!=(PString)ia5email)--i;
    }

    std::string paramusrSip = s_address.getUserParam().convertString();
    std::string nameusrSip = s_address.getUserValue().convertString();
    
    //if((paramusrSip == "phone")&&(i1==std::string::npos))
    // Had to get rid of this check since some UAs, such as
    // Cisco's Telecaster does not include user=phone in SipUrl
    // which is BAD, NASTY, UGLY thing :(
    //
    
    bool validPhone = true;
    
        address[i].SetTag(H225_AliasAddress::e_dialedDigits);
        PASN_IA5String &ia5phone = (PASN_IA5String &)address[i];
        ++i;

	char elem;
	for(unsigned int j = 0; j < nameusrSip.size(); j++)
	{
	    elem = nameusrSip.at(j);
	    
	    if(elem == 'p')nameusrSip.replace(j, 1, ",");
	    else
	    if(validChars.find(elem) == std::string::npos)
	    {
	        // see explanation above
		// nameusrSip.erase(j, 1);
		// j--;
		validPhone = false;
		break;
	    }
	}
	        
	if(validPhone)
	{
	    ia5phone = (PString)nameusrSip.c_str();
            if((PString)(nameusrSip.c_str())!=(PString)ia5phone)--i;
	}
	else
	{
	    --i;
	}
    
    std::string hostSip = s_address.getHost().convertString();
    std::string portSip = s_address.getPort().convertString();
    
    // looks like most of endpoints do not like
    // transport ID in H225 Array of Alias Addresses
    
    /*if(isIp(hostSip))
    {
       // fill out transport ID
       PIPSocket::Address p_address;
       p_address = (PString)hostSip.c_str();
       address[i].SetTag(H225_AliasAddress::e_transportID);
       // 
       H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i];
       trAddress.SetTag(H225_TransportAddress::e_ipAddress);
       H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;
       ipAddress.m_ip.SetSize(4);
       ipAddress.m_ip[0] = p_address.Byte1();
       ipAddress.m_ip[1] = p_address.Byte2();
       ipAddress.m_ip[2] = p_address.Byte3();
       ipAddress.m_ip[3] = p_address.Byte4();
       ipAddress.m_port = atoi(portSip.c_str());
       ++i;
    }*/
    
    // cut not assigned alias addresses
    address.SetSize(i);
    
}

void Address::assign(const IPAddress &ip_address) throw (XlatorException)
{
    const string    fn("Address::assign");
    VLog    	    log(fn);
    
    std::string strIP;
    int portIP;
    
    address.SetSize(0); // clear the thing out
    address.SetSize(1);
    
    strIP = ip_address.getIPAsString();
    portIP = ip_address.getPort();
    
    VDEBUG(log) << fn << ": Host part: " << strIP << VDEBUG_END(log);
    VDEBUG(log) << fn << ": Port part: " << portIP << VDEBUG_END(log);
    
    if(strIP.length() != 0)
    if(isIp(strIP.c_str()))
    {
       // fill out transport ID
       PIPSocket::Address p_address;
       p_address = (PString)strIP.c_str();
       address[0].SetTag(H225_AliasAddress::e_transportID);
       // 
       H225_TransportAddress &trAddress = (H225_TransportAddress &)address[0];
       trAddress.SetTag(H225_TransportAddress::e_ipAddress);
       H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;
       ipAddress.m_ip.SetSize(4);
       ipAddress.m_ip[0] = p_address.Byte1();
       ipAddress.m_ip[1] = p_address.Byte2();
       ipAddress.m_ip[2] = p_address.Byte3();
       ipAddress.m_ip[3] = p_address.Byte4();
       ipAddress.m_port = portIP;
    }
    else throw XlatorException("Invalid IPAddress", __FILE__, __LINE__, 415);
}

void Address::assign(const H323Address &h_address)
{

    address = h_address;

}

void Address::assign(const Address &a_address)
{

    address = a_address.address;
    
}

bool Address::checkSyntax(const char *c_str_url) const
{
    const string    fn("Address::checkSyntax");
    VLog    	    log(fn);
    
    bool r_val = true;
    Data data (c_str_url);
    
    // flag for SipStack mode, no in the stict mode for now
    // maybe needed later on
    //putenv("SIP_PARSER_STRICT_MODE=12");

    try
    {
      VDEBUG(log) << fn << ": Check syntax : " << endl;
      SipAddress url(data);
    }

    catch (SipUrlParserException &aS)
    {
      r_val = false;
      VWARN(log) << fn << ": " << aS.what() << VWARN_END(log);
    }
    return r_val;
}


SipAddress Address::retreiveSip() const throw (XlatorException)
{
    const string    fn("Address::retreiveSip");
    VLog    	    log(fn);
    
    bool a_found = false;
    int a_size = address.GetSize();
    std::string ret_str;
    int i = 0;
    
    // flag for SipStack mode, no in the stict mode for now
    // maybe needed later on
    // putenv("SIP_PARSER_STRICT_MODE=12");

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);
    
    if(a_found)
    {
      H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i-1];
      if(trAddress.GetTag() == H225_TransportAddress::e_ipAddress)
      {
	 H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;

	 char tHost[200];
	 std::string psHost;
	 int j;
	 
	 for(j = 0; j < 4; j++)
	 {
	    sprintf(tHost, "%d", (int)ipAddress.m_ip[j]);
	    psHost += tHost;
	    if(j < 3)psHost += ".";
	 }

	 int iPort = ipAddress.m_port;
	 
	 Data dHost(psHost), dPort(iPort);
	 
	 SipAddress sipAddresst;
	 sipAddresst.setHost(dHost);
	 sipAddresst.setPort(dPort);

	 VDEBUG(log) << fn << ": host & port : " 
	    	     << sipAddresst.getNameAddr().logData() << VDEBUG_END(log);

         a_found = false;
	 i = 0;
         while((i < a_size)&&(!a_found))
           a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);

	 if(a_found)
         {
           PASN_IA5String &ia5phone = (PASN_IA5String &)address[i-1];
            
      	   ret_str = std::string((const char*)((PString)ia5phone));
	
	   // do not require preceding "sip:" for now
	   // might be needed later
	   /*if(ret_str.find("sip:") == std::string::npos)
	      ret_str = "sip:" + ret_str;*/
	
	   sipAddresst.setUserValue(ret_str);
	   sipAddresst.setUserParam("phone");

	   VDEBUG(log) << fn << ": retreived from e_dialedDigits" << VDEBUG_END(log);

         }
	 
    	 VDEBUG(log) << fn << ": retreived from e_transportID"
	    	     << ": user & passwd : " 
		     << sipAddresst.getNameAddr().logData()
		     << VDEBUG_END(log);

	 return sipAddresst;

      } else a_found = false;
    }

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_url_ID);
    
    if(a_found)
    {
      PASN_IA5String &ia5url = (PASN_IA5String &)address[i-1];
            
      if(checkSyntax((PString)ia5url))
      {
      	ret_str = std::string((const char*)((PString)ia5url));
	
	if(ret_str.find("sip:") == std::string::npos)
	    ret_str = "sip:" + ret_str;
	
	SipAddress sipAddress(ret_str);

	VDEBUG(log) << fn << ": retreived from e_url_ID" << VDEBUG_END(log);

	return sipAddress;
      } else a_found = false;
    }

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_h323_ID);
    
    if(a_found)
    {
      PASN_BMPString &bmph323 = (PASN_BMPString &)address[i-1];
            
      if(checkSyntax((PString)bmph323))
      {
      	ret_str = std::string((const char*)((PString)bmph323));
	
	if(ret_str.find("sip:") == std::string::npos)
	    ret_str = "sip:" + ret_str;

	SipAddress sipAddressh(ret_str);

	VDEBUG(log) << fn << ": retreived from e_h323_ID" << VDEBUG_END(log);

	return sipAddressh;
      } else a_found = false;
    } 

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_email_ID);

    if(a_found)
    {
      PASN_IA5String &ia5email = (PASN_IA5String &)address[i-1];
            
      if(checkSyntax((PString)ia5email))
      {
	std::string str_email = (const char*)((PString)ia5email);
        
	VDEBUG(log) << fn << ": e-mail : " << str_email << VDEBUG_END(log);

	if(str_email.find("sip:") == std::string::npos)
	   str_email = "sip:" + str_email;
	
	SipAddress sipAddresse(str_email);
	
	VDEBUG(log) << fn <<  ": retreived from e_email_ID : "
	    	    << sipAddresse.getNameAddr().logData() << VDEBUG_END(log);

	return sipAddresse;
      } else a_found = false;
    }

    throw XlatorException("No valid address to convert to Sip address", __FILE__, __LINE__, 201);
}

IPAddress Address::retreiveIP() const throw (XlatorException)
{
    const string    fn("Address::retreiveIP");
    VLog    	    log(fn);
    
    int a_size = address.GetSize();
    bool a_found = false;
    int i = 0;
    
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);
    
    if(a_found)
    {
      H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i-1];
      if(trAddress.GetTag() == H225_TransportAddress::e_ipAddress)
      {
	 H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;

	 char tHost[200];
	 char tPort[50];
	 std::string psHost;
	 int j;
	 
	 for(j = 0; j < 4; j++)
	 {
	    sprintf(tHost, "%d", (int)ipAddress.m_ip[j]);
	    psHost += tHost;
	    if(j < 3)psHost += ".";
	 }

	 int iPort = ipAddress.m_port;
	 tHost[0]='\0';
	 strcpy(tHost, psHost.c_str());
	 
	 sprintf(tPort, "%d", iPort);
	 
	 psHost = tHost;
	 psHost += ":";
	 psHost += tPort;

	 VDEBUG(log) << fn << ": host & port : " << psHost << VDEBUG_END(log);

         IPAddress ip_Address(tHost, iPort);
	 
	 VDEBUG(log) << fn << ": retreived from e_transportID" << VDEBUG_END(log);

	 return ip_Address;

      }
    }
    throw XlatorException("No valid address to convert to IPAddress", __FILE__, __LINE__, 221);
}


H323Address Address::retreiveH323() const
{
    return address;
}


void Address::setPhone(const H323Address &h_address) throw (XlatorException)
{
    const string    fn("Address::setPhone");
    VLog    	    log(fn);
    
    int i = 0, a_size = h_address.GetSize();
    std::string c_phone;
    bool a_found = false;
    std::string validChars = "0123456789#*";
    char elem;

    while((i < a_size)&&(!a_found))
      a_found = (h_address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);
    
    if(a_found)
    {
      PASN_IA5String &ia5phone_in = (PASN_IA5String &)h_address[i-1];
            
      c_phone = std::string((const char*)((PString)ia5phone_in));
      VDEBUG(log) << fn << ": Retreived from e_dialedDigits : " << c_phone.c_str() << VDEBUG_END(log);

      for(unsigned int j = 0; j < c_phone.size(); j++)
      {
	elem = c_phone.at(j);
	    
	if(elem == 'p')c_phone.replace(j, 1, ",");
	else
	if(validChars.find(elem) == std::string::npos)
	{
	    c_phone.erase(j, 1);
	    j--;
	}
      }
    
    if(c_phone.size() == 0)a_found = false;
    
    if(!a_found)
      throw XlatorException("No valid e_dialedDigits in Alias Address", __FILE__, __LINE__);

    a_size = address.GetSize();
    i = 0; a_found = false;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);

    if(a_found)
    {
      VDEBUG(log) << fn << ": Found e_dialedDigits in Alias Address, will be replaced with a new one"
      	    	  << VDEBUG_END(log);
      i--;
    }
    else
    {
      address.SetSize(a_size + 1);
      i = a_size;
    }

    address[i].SetTag(H225_AliasAddress::e_dialedDigits);
    PASN_IA5String &ia5phone = (PASN_IA5String &)address[i];
    ia5phone = ia5phone_in;

    if((PString)(c_phone.c_str())!=(PString)ia5phone)
      throw XlatorException("Could not assign, probably phone number is incorrect", __FILE__, __LINE__);
  }
  else throw XlatorException("No valid e_dialedDigits in address to assign", __FILE__, __LINE__);
}

void Address::setPhone(const std::string &phone) throw (XlatorException)
{
    const string    fn("Address::setPhone");
    VLog    	    log(fn);
    
    int i = 0, a_size = address.GetSize();
    std::string validChars = "0123456789#*";
    bool a_found = false;
    std::string c_phone = phone;

    char elem;
    for(unsigned int j = 0; j < c_phone.size(); j++)
    {
	elem = c_phone.at(j);
	    
	if(elem == 'p')c_phone.replace(j, 1, ",");
	else
	if(validChars.find(elem) == std::string::npos)
	{
	    c_phone.erase(j, 1);
	    j--;
	}
    }
    
    if(c_phone.size() == 0)
      throw XlatorException("Phone number is not valid", __FILE__, __LINE__);

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);

    if(a_found)
    {
      VDEBUG(log) << fn << ": Found e_dialedDigits in Alias Address, will be replaced with a new one"
      	    	  << VDEBUG_END(log);
      i--;
    }
    else
    {
      address.SetSize(a_size + 1);
      i = a_size;
    }

    address[i].SetTag(H225_AliasAddress::e_dialedDigits);
    PASN_IA5String &ia5phone = (PASN_IA5String &)address[i];

    ia5phone = (PString)c_phone.c_str();

    if((PString)(c_phone.c_str())!=(PString)ia5phone)
      throw XlatorException("Could not assign, probably phone number is incorrect", __FILE__, __LINE__);
}

void Address::setTransport(const IPAddress &ipAddress_in) throw (XlatorException)
{
    const string    fn("Address::setTransport");
    VLog    	    log(fn);
    
    std::string strIP;
    int portIP;
    
    int a_size = address.GetSize();
    bool a_found = false;
    int i = 0;
    
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);
    
    if(a_found)
    {
      VDEBUG(log) << fn << ": Found TransportAddress in Alias Address, will be replaced with a new one"
      	    	  << VDEBUG_END(log);
      i--;
    }
    else
    {
      address.SetSize(a_size + 1);
      i = a_size;
    }

    strIP = ipAddress_in.getIPAsString();
    portIP = ipAddress_in.getPort();
    
    VDEBUG(log) << fn << ": Host part: " <<  strIP.c_str() << VDEBUG_END(log);
    VDEBUG(log) << fn << ": Port part: " <<  portIP << VDEBUG_END(log);
    
    if(strIP.length() != 0)
    if(!isIp(strIP.c_str()))
      throw XlatorException("Invalid IPAddress", __FILE__, __LINE__, 415);

    // fill out transport ID
    PIPSocket::Address p_address;
    p_address = (PString)strIP.c_str();
    address[i].SetTag(H225_AliasAddress::e_transportID);

    H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i];
    trAddress.SetTag(H225_TransportAddress::e_ipAddress);
    H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;
    ipAddress.m_ip.SetSize(4);
    ipAddress.m_ip[0] = p_address.Byte1();
    ipAddress.m_ip[1] = p_address.Byte2();
    ipAddress.m_ip[2] = p_address.Byte3();
    ipAddress.m_ip[3] = p_address.Byte4();
    ipAddress.m_port = portIP;
}

void Address::setH225_Transport(const H225_TransportAddress_ipAddress &ipAddress)
{
    const string    fn("Address::setTransport");
    VLog    	    log(fn);
    
    int i = 0;    
    int a_size = address.GetSize();
    bool a_found = false;
    
    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);

    if(a_found)
    {
      VDEBUG(log) << fn << ": Found TransportAddress in Alias Address, will be replaced with a new one"
      	    	  << VDEBUG_END(log);
      i--;
    }
    else
    {
      address.SetSize(a_size + 1);
      i = a_size;
    }
    
    address[i].SetTag(H225_AliasAddress::e_transportID);
 
    H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i];
    trAddress.SetTag(H225_TransportAddress::e_ipAddress);
    H225_TransportAddress_ipAddress &ip_address = (H225_TransportAddress_ipAddress &)trAddress;
    ip_address = ipAddress;
}

Address &Address::operator=(const H225_TransportAddress_ipAddress &ipAddress)
{
    clear();
    setH225_Transport(ipAddress);
    
    return *this;
}

H225_TransportAddress_ipAddress Address::getH225_Transport() const throw (XlatorException)
{
    int a_size = address.GetSize();
    bool a_found = false;
    int i = 0;
    
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);
    
    if(!a_found)
      throw XlatorException("No Transport Address...", __FILE__, __LINE__);
    
    i--;

    H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i];

    if(trAddress.GetTag() == H225_TransportAddress::e_ipAddress)
    {
	 H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;

	 return ipAddress;
    }
    else
      throw XlatorException("No TransportAddress_ipAddress found", __FILE__, __LINE__);
}

void Address::setH245_Transport(const H245_UnicastAddress_iPAddress &ipAddress)
{
    int i = 0;    

    address.SetSize(0); // clear the thing out
    address.SetSize(1);
    
    address[i].SetTag(H225_AliasAddress::e_transportID);
 
    H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i];
    trAddress.SetTag(H225_TransportAddress::e_ipAddress);
    H225_TransportAddress_ipAddress &ip_address = (H225_TransportAddress_ipAddress &)trAddress;
    ip_address.m_ip = ipAddress.m_network;
    ip_address.m_port = ipAddress.m_tsapIdentifier;
}

Address &Address::operator=(const H245_UnicastAddress_iPAddress &ipAddress)
{
    setH245_Transport(ipAddress);
    
    return *this;
}

H245_UnicastAddress_iPAddress Address::getH245_Transport() const throw (XlatorException)
{
    int a_size = address.GetSize();
    bool a_found = false;
    int i = 0;
    H245_UnicastAddress_iPAddress ip_address;
    
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);
    
    if(!a_found)
      throw XlatorException("No Transport Address...", __FILE__, __LINE__);
    
    i--;

    H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i];

    if(trAddress.GetTag() == H225_TransportAddress::e_ipAddress)
    {
	 H225_TransportAddress_ipAddress &ipAddress = (H225_TransportAddress_ipAddress &)trAddress;

	 ip_address.m_network = ipAddress.m_ip;
         ip_address.m_tsapIdentifier = ipAddress.m_port;
         return ip_address;
    }
    else
      throw XlatorException("No TransportAddress_ipAddress found", __FILE__, __LINE__);
    
}

bool Address::operator==(const Address &a_address) const
{
    int i = 0, j = 0, a_size = address.GetSize(), aa_size = a_address.address.GetSize();
    bool a_found = false;
    std::string phone, phone_in;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);
      
    if(!a_found)return false;
    else 
    {
      PASN_IA5String &ia5phone = (PASN_IA5String &)address[i-1];
      phone = std::string((const char*)((PString)ia5phone));
    }

    j = 0;
    a_found = false;
    while((j < aa_size)&&(!a_found))
      a_found = (a_address.address[j++].GetTag() == H225_AliasAddress::e_dialedDigits);

    if(!a_found)return false;
    else
    {
      PASN_IA5String &ia5phone_in = (PASN_IA5String &)a_address.address[j-1];
      phone_in = std::string((const char*)((PString)ia5phone_in));
    }

    if(phone != phone_in)return false;
    else return true;
}

bool Address::operator<(const Address &a_address) const
{
    int i = 0, j = 0, a_size = address.GetSize(), aa_size = a_address.address.GetSize();
    bool a_found = false;
    std::string phone, phone_in;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);
      
    if(!a_found)return true;
    else 
    {
      PASN_IA5String &ia5phone = (PASN_IA5String &)address[i-1];
      phone = std::string((const char*)((PString)ia5phone));
    }

    j = 0;
    a_found = false;
    while((j < aa_size)&&(!a_found))
      a_found = (a_address.address[j++].GetTag() == H225_AliasAddress::e_dialedDigits);

    if(!a_found)return false;
    else
    {
      PASN_IA5String &ia5phone_in = (PASN_IA5String &)a_address.address[j-1];
      phone_in = std::string((const char*)((PString)ia5phone_in));
    }

    if(phone < phone_in)return true;
    else return false;
}

bool Address::operator>(const Address &a_address) const
{
    int i = 0, j = 0, a_size = address.GetSize(), aa_size = a_address.address.GetSize();
    bool a_found = false;
    std::string phone, phone_in;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);
      
    if(!a_found)return false;
    else 
    {
      PASN_IA5String &ia5phone = (PASN_IA5String &)address[i-1];
      phone = std::string((const char*)((PString)ia5phone));
    }

    j = 0;
    a_found = false;
    while((j < aa_size)&&(!a_found))
      a_found = (a_address.address[j++].GetTag() == H225_AliasAddress::e_dialedDigits);

    if(!a_found)return true;
    else
    {
      PASN_IA5String &ia5phone_in = (PASN_IA5String &)a_address.address[j-1];
      phone_in = std::string((const char*)((PString)ia5phone_in));
    }

    if(phone > phone_in)return true;
    else return false;
}

void Address::setUser(const std::string &user)
{
    const string    fn("Address::setUser");
    VLog    	    log(fn);
    
    int i = 0, a_size = address.GetSize();
    bool a_found = false;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_h323_ID);

    if(a_found)
    {
      VDEBUG(log) << fn << ": Found e_h323_ID in Alias Address, will be replaced with a new one"
      	    	  << VDEBUG_END(log);
      i--;
    }
    else
    {
      address.SetSize(a_size + 1);
      i = a_size;
    }

    address[i].SetTag(H225_AliasAddress::e_h323_ID);
    (PASN_BMPString &)address[i] = (PString)user.c_str();
}

void Address::setUser(const H323Address &user) throw (XlatorException)
{
    const string    fn("Address::setUser");
    VLog    	    log(fn);
    
    int i = 0, j = 0, a_size = address.GetSize(), u_size = user.GetSize();
    bool a_found = false;

    j = 0;
    while((j < u_size)&&(!a_found))
      a_found = (address[j++].GetTag() == H225_AliasAddress::e_h323_ID);

    if(!a_found)
      throw XlatorException("No e_h323_ID in Alias Address to assign", __FILE__, __LINE__);
    else j--;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_h323_ID);

    if(a_found)
    {
      VDEBUG(log) << fn << ": Found e_h323_ID in Alias Address, will be replaced with a new one"
      	    	  << VDEBUG_END(log);
      i--;
    }
    else
    {
      address.SetSize(a_size + 1);
      i = a_size;
    }

    address[i].SetTag(H225_AliasAddress::e_h323_ID);
    (PASN_BMPString &)address[i] = (PASN_BMPString &)user[j];
}

std::string Address::getPhone() const throw (XlatorException)
{
    int i = 0, a_size = address.GetSize();
    bool a_found = false;
    std::string ret_str;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);

    if(!a_found)
    {
      throw XlatorException("No valid e_dialedDigits in address", __FILE__, __LINE__);
    }
    else
    {
      i--;
    }

    PASN_IA5String &ia5phone = (PASN_IA5String &)address[i];
    ret_str = std::string((const char*)((PString)ia5phone));
    
    return ret_str;
}

std::string Address::getUser() const throw (XlatorException)
{
    int i = 0, a_size = address.GetSize();
    bool a_found = false;
    std::string ret_str;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_h323_ID);

    if(!a_found)
    {
      throw XlatorException("No valid e_h323_ID in address", __FILE__, __LINE__);
    }
    else
    {
      i--;
    }

    PASN_BMPString &bmph323 = (PASN_BMPString &)address[i];
    ret_str = std::string((const char*)((PString)bmph323));
    
    return ret_str;
}

bool Address::hasPhone() const
{
    int i = 0, a_size = address.GetSize();
    bool a_found = false;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_dialedDigits);
      
    return a_found;
}

bool Address::hasIPAddress() const
{
    int i = 0, a_size = address.GetSize();
    bool a_found = false;
    bool ret = false;

    i = 0;
    while((i < a_size)&&(!a_found))
      a_found = (address[i++].GetTag() == H225_AliasAddress::e_transportID);
      
    if(a_found)
    {
      const H225_TransportAddress &trAddress = (H225_TransportAddress &)address[i-1];
      ret = (trAddress.GetTag() == H225_TransportAddress::e_ipAddress);
      return ret;
    }
    else return false;
}

bool Address::isEmpty() const
{
    int a_size = address.GetSize();
    bool ret = false;
    
    ret = (a_size > 0);
    return ret;
}

void Address::clear()
{
    address.SetSize(0);
}


ostream & Address::writeTo(ostream & out) const
{
    size_t  	size = address.GetSize();
    bool    	first = true;

    for ( size_t i = 0; i < size; i++ )
    {
    	switch ( address[i].GetTag() )
	{
    	    case H225_AliasAddress::e_dialedDigits:
	    {
	    	out << ( first ? "" : ", " ) 
		    << "e164 = " << ((PASN_IA5String &)address[i]).GetValue();
		    
		first = false;
    			
	    	break;
	    }
    	    case H225_AliasAddress::e_h323_ID:
	    {
	    	out << ( first ? "" : ", " ) 
	    	    << ((PASN_BMPString &)address[i]).GetValue();

    	    	first = false;
		
	    	break;
	    }
    	    case H225_AliasAddress::e_url_ID:
	    {
	    	out << ( first ? "" : ", ")
		    << "url = " << ((PASN_IA5String &)address[i]).GetValue();
		    
		first = false;

	    	break;
	    }
    	    case H225_AliasAddress::e_email_ID:
	    {
	    	out << ( first ? "" : ", ")
		    << "email = " << ((PASN_IA5String &)address[i]).GetValue();
		    
		first = false;

	    	break;
	    }
    	    case H225_AliasAddress::e_transportID:
	    {
	    	H225_TransportAddress & transAddr = address[i];

    	    	if ( transAddr.GetTag() != H225_TransportAddress::e_ipAddress )
		{
		    break;
		}

    	    	H225_TransportAddress_ipAddress & ipaddr = transAddr;

    	    	IPAddress addr(ipaddr.m_ip.GetPointer(), ipaddr.m_port);

		out << ( first ? "" : ", " )
		    << "transport address = " << addr;

	    	break;
	    }
	    default:
	    {
	    	break;
    	    }
    	}
    }
    return ( out );
}
