
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const AudioCapability_cxx_Version = 
    "$Id: AudioCapability.cxx,v 1.10 2000/12/18 23:48:23 bko Exp $";

#include "VLog.hxx"
#include "AudioCapability.hxx"

using namespace Translator;
using Vocal::Logging::VLog;

AudioCapability::AudioCapability(
                                 unsigned int sPF,
				 unsigned int pSize,
				 unsigned int cRate)
 : samplesPerFrame(sPF), packetSize(pSize), clockRate(cRate)
{
  VLog log;
  VDEBUG(log) << "AudioCapability::AudioCapability: Packet size : "
    	      << packetSize << VDEBUG_END(log);
}


Capability::MainTypes AudioCapability::getMainType() const
{
  return e_Audio;
}

unsigned int AudioCapability::getSamplesPerFrame() const
{
  return samplesPerFrame;
}


void AudioCapability::setPacketSizeFrames(unsigned int pSize)
{
      packetSize = pSize; 
}


float AudioCapability::getPacketSizeMilSec() const
{
    unsigned mClR = clockRate/1000;
    if(mClR == 0)return 0;
    return  (samplesPerFrame * packetSize)/mClR;
}

unsigned int AudioCapability::getPacketSizeSamples() const
{
    return packetSize * samplesPerFrame;
}

int AudioCapability::getPacketSizeFrames() const
{
    return packetSize;
}

void AudioCapability::setPacketSizeMilSec(float pSmS)
{
    if(samplesPerFrame == 0)packetSize = 0;
    return;

    unsigned mClR = clockRate/1000;
    packetSize = static_cast<int> ((pSmS * mClR)/samplesPerFrame);
}

bool AudioCapability::getParamSendMsg(H245_Capability & cap) const
{
  cap.SetTag(H245_Capability::e_receiveAudioCapability);
  return getParamSendMsg((H245_AudioCapability &)cap, packetSize);
}


bool AudioCapability::getParamSendMsg(H245_DataType & dataType) const
{
  dataType.SetTag(H245_DataType::e_audioData);
  return getParamSendMsg((H245_AudioCapability &)dataType, packetSize);
}


bool AudioCapability::setParamRecvMsg(const H245_Capability & cap)
{
  if (cap.GetTag() != H245_Capability::e_receiveAudioCapability &&
    cap.GetTag() != H245_Capability::e_receiveAndTransmitAudioCapability)
    return false;

  if (!setParamRecvMsg((const H245_AudioCapability &)cap, packetSize))
    return false;

  return true;
}


bool AudioCapability::setParamRecvMsg(const H245_DataType & dataType)
{
  if (dataType.GetTag() != H245_DataType::e_audioData)
    return false;

  return setParamRecvMsg((const H245_AudioCapability &)dataType, packetSize);
}

void AudioCapability::setParamRecvMsg(intSDP pSize)
{
  // we set packet size in samples
  int sPF;
  
  sPF = samplesPerFrame/1000;
  packetSize = pSize/samplesPerFrame;
}

void AudioCapability::getParamSendMsg(intSDP &pSize)
{
  // we get packet size in samples
  pSize = getPacketSizeSamples();
}

int AudioCapability::getClockRate() const
{
  return clockRate;
}
