#ifndef __CAPABILITY_H_
#define __CAPABILITY_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const Capability_hxx_Version = 
"$Id: Capability.hxx,v 1.9 2000/12/20 01:47:43 vzubarev Exp $";

#include "ptlib.h"
#include "h245.h"
#include "XlatorException.hxx"

namespace Translator
{

///
class Capability
{
  protected:
    /// Unique ID assigned to capability
    unsigned assignedCapabilityNumber;

  public:
    
    ///
    Capability();
    
    ///
    virtual ~Capability() {}
    
    typedef unsigned int intSDP;
    
    ///
    bool isEqual(const Capability&) const;

    ////
    enum MainTypes {
      /// Audio codec capability
      e_Audio,
      /// Video codec capability
      e_Video,
      /// Arbitrary data capability
      e_Data,
      /// Count of main types
      e_NumMainTypes
    };

    ///
    virtual unsigned int getSamplesPerFrame() const = 0;
    
    ///
    virtual MainTypes getMainType() const = 0;
    
    ///
    virtual unsigned  getSubType()  const = 0;
    
    ///
    virtual void setPacketSizeMilSec(
      float mSec   /// Packet Size in milliseconds
    ) {}

    ///
    virtual void setPacketSizeFrames(
      unsigned int pSize   /// Packet Size in frames
    ) {}

    ///   Get size (in samples) of the packet.
    virtual unsigned getPacketSizeSamples() const;
    
    ///
    virtual int getClockRate() const;
    
    ///
    virtual int getPacketSizeFrames() const;

    /// Get the size (in frames) of the packet.
    virtual float getPacketSizeMilSec() const;

    /**
       is called on construction of the TerminalCapabilitySet message.
       Sets the corresponding message fields from the class members.
       The default behaviour is pure - to overloaded by descendents.
     */
    virtual bool getParamSendMsg(
      H245_Capability & pdu  /// PDU to set information on
    ) const = 0;

    /**
       is called on construction of the outgoing OpenLogicalChannel message.
       Sets the corresponding message fields from the class members.
       The default behaviour is pure - to be overloaded by descendents.
     */
    virtual bool getParamSendMsg(
      H245_DataType & pdu  /// PDU to set information on
    ) const = 0;

    /**
       is called on receiving of TerminalCapabilitySet message.
       Sets class members from the corresponding message fields.
       The default behaviour is pure - to be overloaded by descendents.
     */
    virtual bool setParamRecvMsg(
      const H245_Capability & pdu  /// PDU to get information from
    ) = 0;

    /**
       is called on receiving of OpenLogicalChannel message.
       Sets class members from the corresponding message fields.
       The default behaviour is pure - to be overloaded by descendents.
     */
    virtual bool setParamRecvMsg(
      const H245_DataType & pdu  /// PDU to get information from
    ) = 0;

    /// Get unique capability number.
    int getCapabilityNumber() const { return assignedCapabilityNumber; }

    /// Set unique capability number.
    void setCapabilityNumber(unsigned num) { assignedCapabilityNumber = num; }

    /// Assign a new capability number.
    void assignCapabilityNumber(unsigned int capNumber);
    
    /// set packet size of the particular capability received in SDP message
    virtual void setParamRecvMsg(intSDP pSize) = 0;
    
    /// get packet size of the particular capability to send in SDP message
    virtual void getParamSendMsg(intSDP &pSize) = 0;
    
    ///
    virtual Capability *clone() const = 0;
    
};

}
#endif
