/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtInvite_cxx_Version = 
    "$Id: EvtInvite.cxx,v 1.25 2002/07/10 07:10:22 icahoon Exp $";


#include <cassert>
#include <cstdlib>

#include "VLog.hxx"

#include "ValAssistant.h"

#include "H323AdmissionEvent.hxx"

#include "RouterInfo.hxx"
#include "Address.hxx"

#include "EvtCallInit.hxx"

#include "Consts.hxx"
#include "EvtInvite.hxx"
#include "FeatureTrCallSig.hxx"
#include "TranslatorConsts.hxx"

#include "SipRetryAfter.hxx"

#ifdef TESTING_RESOLVER
#include "EvtResolve.hxx"
#endif

#ifdef TESTING_REDIRECT
#include "StatusMsg.hxx"
#endif

using namespace translator;
using Vocal::H323Events::H323AdmissionEvent;
using Vocal::Logging::VLog;


EvtInvite::
EvtInvite(TrRouterSIP* sm)
    : EvtSIPTrCallSigMessage(sm)
{
}


void EvtInvite::
inIdle(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtInvite::inIdle");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

#ifdef TESTING_RESOLVER
    EvtResolve er(trcallsig.getSM());
    er.hostName_ = "ek.mera.ru";
    EvtThreadBase* per = er.clone();
    per->start();
#endif

    // (depends from H.323/SIP parser)

    assert(&getMessage());
    trcallsig.invite_ = new InviteMsg(getMessage());

    // Send 'Trying' (needed only if SIP proxy didn't do it)
    //Sptr<SipMsg> sip100 = new StatusMsg(trcallsig.invite_, 100);
    //trcallsig.getSM()->getSIPConnection()->send(sip100);

    assert(trcallsig.getSM());
    router::EvtCallInit init(trcallsig.getSM()->getH323Call());

    // Fill in Call data in 'init'
    Translator::Address addressTranslator;

    // From: field
    if (!(trcallsig.getSM()->getH323Call()->getRouterInfo()->routerMediaControl()))
    {
        if (getMessage().getFrom().getTokenValue(CanRecvH245Address) ==
            CanRecvH245AddressValue)
        {
            VDEBUG(log) << fn << ": Got 'CanRecvH245Address' token" << VDEBUG_END(log);
            trcallsig.getSM()->setH245Patch();
            trcallsig.getSM()->getH323Call()->getCallSigConnection()->setOverrideH245();
        }
    }

     Sptr<BaseUrl> baseUrl = getMessage().getFrom().getUrl();
     Sptr<SipUrl> sipUrl;

     if(baseUrl.getPtr()!=0)
     {
       if(baseUrl->getType()==SIP_URL)
       {
               sipUrl.dynamicCast(baseUrl);
       }
       else
       {
               VEMERG(log) << fn << ": baseUrl is not SipUrl, cannot proceed..." << VEMERG_END(log);
       }
     }
     else
     {
       VEMERG(log) << fn << ": baseUrl is a null pointer, cannot proceed..." << VEMERG_END(log);
     }

     addressTranslator = *sipUrl;

    
    init.fromAliases_ = addressTranslator.getH323Address();

    if (addressTranslator.hasIPAddress())
    {
        init.hasFromIP_ = true;
        init.fromIP_ = addressTranslator.getIPAddress();

    }
    else
    {
        init.hasFromIP_ = false;
    }

    if (addressTranslator.hasPhone())
    {
        init.fromDisplay_ = addressTranslator.getPhone();
    }
    else
    {
        init.fromDisplay_ = "Unknown number";
    }

    // To: field
    baseUrl = getMessage().getTo().getUrl();
    //addressTranslator = sipUrl;

     if(baseUrl.getPtr()!=0)
     {
       if(baseUrl->getType()==SIP_URL)
       {
               sipUrl.dynamicCast(baseUrl);
       }
       else
       {
               VEMERG(log) << fn << ": baseUrl is not SipUrl, cannot proceed..." << VEMERG_END(log);
       }
     }
     else
     {
       VEMERG(log) << fn << ": baseUrl is a null pointer, cannot proceed..." << VEMERG_END(log);
     }

     addressTranslator = *sipUrl;


    init.toAliases_ = addressTranslator.getH323Address();

    if (addressTranslator.hasIPAddress())
    {
        init.hasToIP_ = true;
        init.toIP_ = addressTranslator.getIPAddress();

        if (addressTranslator.getIPAddress() == Vocal::Transport::IPAddress())
        {
            init.hasToIP_ = false;
        }
    }
    else
    {
        init.hasToIP_ = false;
    }

    // Translate callID
    VDEBUG(log) << fn << ": Translating callID" << VDEBUG_END(log);
    Data dcid = getMessage().getCallId().getCallIdData();
    std::string cid = dcid.convertString();
    VDEBUG(log) << fn << ": callID=" << cid << VDEBUG_END(log);

    if (cid.size() > 32 && cid[32] == '.')
    {
        VDEBUG(log) << fn << ": cid[32]=='.'" << VDEBUG_END(log);
        cid[32] = 0;
        unsigned char uc[256];

        if (convertToUnsigned(cid.c_str(), uc))
        {
            VDEBUG(log) << fn << ": confid converted" << VDEBUG_END(log);
            PASN_OctetString pos;
            pos.SetValue(uc, 16);
            init.confid_ = OpalGloballyUniqueID(pos);
            init.hasConfid_ = true;

            if (cid.size() > 65 && cid[65] == '@')
            {
                VDEBUG(log) << fn << ": cid[65]=='@'" << VDEBUG_END(log);
                cid[65] = 0;

                if (convertToUnsigned(cid.c_str() + 33, uc))
                {
                    VDEBUG(log) << fn << ": callid converted" << VDEBUG_END(log);
                    PASN_OctetString pos;
                    pos.SetValue(uc, 16);
                    init.cid_ = OpalGloballyUniqueID(pos);
                    init.hasCid_ = true;
                }
            }
        }
    }

    // Request field
    baseUrl = getMessage().getRequestLine().getUrl();
    //addressTranslator = sipUrl;

     if(baseUrl.getPtr()!=0)
     {
       if(baseUrl->getType()==SIP_URL)
       {
               sipUrl.dynamicCast(baseUrl);
       }
       else
       {
               VEMERG(log) << fn << ": baseUrl is not SipUrl, cannot proceed..." << VEMERG_END(log);
       }
     }
     else
     {
       VEMERG(log) << fn << ": baseUrl is a null pointer, cannot proceed..." << VEMERG_END(log);
     }

     addressTranslator = *sipUrl;                                        

    init.toAliases_ = addressTranslator.getH323Address();

#ifdef TESTING_REDIRECT
    SipUrl contact;
    Data user = sipUrl.getUser();
    int phone = user;
    VDEBUG(log) << fn << ": Phone=" << phone << VDEBUG_END(log);

    if (phone % 4 == 0)
    {
	phone += 1;
	contact.setUser(Data(phone));
	contact.setHost("127.0.0.1");
	contact.setPort(5070);
	Sptr<StatusMsg> sipStatus = new StatusMsg(
	    *(trcallsig.invite_), 302);
	sipStatus->setContact(SipContact(contact.encode()));
	phone += 1;
	contact.setUser(Data(phone));
	sipStatus->setContact(SipContact(contact.encode()));
	trcallsig.getSM()->getSIPConnection()->send(sipStatus);

	trcallsig.getSM()->getSIPConnection()->disable();
	trcallsig.getSM()->close();
	return;
    }
    else if ((phone % 4 == 1) || (phone % 4 == 2))
    {
	phone |= 3;
	contact.setUser(Data(phone));
	Sptr<StatusMsg> sipStatus = new StatusMsg(
	    *(trcallsig.invite_), 302);
	sipStatus->setContact(SipContact(contact.encode()));
	trcallsig.getSM()->getSIPConnection()->send(sipStatus);

	trcallsig.getSM()->getSIPConnection()->disable();
	trcallsig.getSM()->close();
	return;
    }

#endif

/* TODO**: analyze flag going from sip event to see if we need to apply to gk
    if (trcallsig.getSM()->getH323Call()->getRouterInfo()->remoteClient()
        && init.hasToIP_)
    {
        trcallsig.setState(&PatternTrCallSig::inConnectingS2H, "inConnectingS2H");
        init.start();
    }
    else
*/
    {
        trcallsig.setState(&PatternTrCallSig::inWaitOutAdmission, "inWaitOutAdmission");

        // Don't start 'init', send ARQ instead
        Sptr<H323AdmissionEvent> arq =
            new H323AdmissionEvent(
                trcallsig.getSM()->getH323Call()->getFifo()->getFifo());
        arq->destAlias(addressTranslator);
        arq->callIdentifier(trcallsig.getSM()->getH323Call()->
                            getCallSigConnection()->getCallID());
        trcallsig.getSM()->getRasConnection()->send(arq);
    }
}


void EvtInvite::
inConnected(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtInvite::inConnected");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Got 'Reinvite'
    // TODO* process fields of 'reinvite' message
    trcallsig.setState(&PatternTrCallSig::inSDPInvite, "inSDPInvite");
}


void EvtInvite::
inCollision(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtInvite::inCollision");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Got 'Reinvite'
    // TODO* process fields of 'reinvite' message
    trcallsig.setState(&PatternTrCallSig::inSDPInvite, "inSDPInvite");
}


void EvtInvite::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtInvite::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    // Got 'Reinvite' while still in call establishment

    // Cancel SDP
    trcallsig.getSM()->getTrSDPImpl()->validSdp_ = false;
    trcallsig.getSM()->getTrSDPImpl()->emptySdp_ = false;

    // Send STATUS 500 (available in 1 .. 10 sec)
    Sptr<StatusMsg> sipStatus = new StatusMsg(
        const_cast<InviteMsg&>(getMessage()),
        msip::respInternalServerError);
    Vocal::SipRetryAfter ra;
    ra.setDuration((rand() * TMR_COLLISION) / RAND_MAX + 1);
    sipStatus->setRetryAfter(ra);
    trcallsig.getSM()->getSIPConnection()->send(sipStatus);
}


EvtSIPB* EvtInvite::
clone_impl()
{
    return new EvtInvite(*this);
}


EvtInvite::TagType EvtInvite::
getTag() const throw()
{
    // (depends from SIP parser)
    return SIP_INVITE;
}


void EvtInvite::
updateID()
{
    // Ignore
}


#ifndef DISABLE_WRITER

ostream& EvtInvite::
writeTo(ostream& out) const
{
    out << "EvtInvite";
    EvtSIPTrCallSigMessage::writeTo(out);
    return out;
}

#endif
