/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF 1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const EvtResolve_cxx_Version = 
    "$Id: EvtResolve.cxx,v 1.3 2001/10/24 21:26:42 icahoon Exp $";


#include <cassert>
#include <netdb.h>
#include <netinet/in.h>
 
#include "VLog.hxx"
#include "VFunctor.hxx"
#include "VThreadPool.hxx"
#include "IPAddress.hxx"

#include "EvtResolve.hxx"
#include "FeatureTrCallSig.hxx"
#include "EvtResolveResult.hxx"

using namespace translator;
using Vocal::Logging::VLog;


EvtResolve::
EvtResolve(TrSMCall* sm)
    : EvtThread(sm)
{
}


void EvtResolve::
inOther(FeatureTrCallSig& trcallsig)
{
    const string    fn("EvtResolve::inOther");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    VThreadPool* tp = trcallsig.getSM()->getThreadPool();

    if (tp)
    {
        // Thread pool's interface requires void*, so I have no choice :(
        VFunctor functor(
            &EvtResolve::threadMain,
            reinterpret_cast<void*>(this));

        tp->addFunctor(functor);
    }
}


void EvtResolve::
threadMain(void* evtResolvePtr)
{
    assert(evtResolvePtr);

    // Thread pool's interface requires void*, so I have no choice :(
    EvtResolve* obj = reinterpret_cast<EvtResolve*>(evtResolvePtr);
    obj->doResolve();
    obj->destroy();
}


EvtThreadBase* EvtResolve::
clone_impl()
{
    return new EvtResolve(*this);
}


void EvtResolve::
doResolve()
{
    const string    fn("EvtResolve::doResolve");
    VLog            log(fn);
    
    VDEBUG(log) << fn << VDEBUG_END(log);

    assert(this);

    // Resolve address
    struct hostent* address = gethostbyname(hostName_.c_str());

    // Change network byte order to host byte order and make IPAddress
    assert(static_cast<unsigned>(address->h_length) >= sizeof(u_int32_t));
    Transport::IPAddress ipAddress(
        ntohl(*reinterpret_cast<u_int32_t*>(address->h_addr)), 0);

    VDEBUG(log) << fn <<
        ": Resolved " << hostName_ <<
        " to " << ipAddress <<
        VDEBUG_END(log);

    // TODO**: error handling...

    // Create event with the resolver results
    Sptr<EvtResolveResult> rr = new EvtResolveResult(sipRouter_);
    rr->hostName_ = hostName_;
    rr->address_ = ipAddress;
    rr->id_ = id_;
    rr->setProxy(getProxy());

    // Send event to the main thread FIFO
    fifo_->getFifo().add(rr);

}


#ifndef DISABLE_WRITER

ostream& EvtResolve::
writeTo(ostream& out) const
{
    out << "EvtResolve";
    return out;
}

#endif
