#ifndef __GSMCAPABILITY_H_
#define __GSMCAPABILITY_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const GSMCapability_hxx_Version = 
"$Id: GSMCapability.hxx,v 1.8 2000/12/20 01:48:02 vzubarev Exp $";

#include "AudioCapability.hxx"

namespace Translator
{

/**
   This class describes the GSM codec capability.

   160 samples per Unit (20 ms with this clock rate); 
   default packet size 4 Units per Packet
   8000 hz - clockrate
   in this case audioUnitSize we interpret as 
   packetSize
 */
class GSMCapability : public AudioCapability
{

#define GSM_SMPLS_PER_FRAME 160
#define GSM_FRMS_PER_PACKET 4
#define GSM_CLCK_RATE 8000

  private:
    ///
    static const unsigned gsmSubType[3];

  public:
    /// Specific GSM encoding bit rates.
    enum Rate {
      FullRate,
      HalfRate,
      EnhancedFullRate
    };

    /**Create a new GSM capability.
     */
    GSMCapability(
      Rate r = FullRate,   /// Encoding bit rate.
      bool cfNoise = false,
      bool scrmbl = false,
      unsigned int aUnitSize = 4
    );
    
    ///
    virtual ~GSMCapability() {}

    /**Create a copy of the object.
      */
    virtual AudioCapability * clone() const;

    /**Get the sub-type of the capability. This is a code dependent on the
       main type of the capability.

       This returns one of the four possible combinations of mode and speed
       using the enum values of the protocol ASN H245_AudioCapability class.
     */
    virtual unsigned getSubType() const;

    /**
       is called on construction of outgoing TerminalCapabilitySet 
       or OpenLogicalChannel message.
       Fills out message fields with corresponding class members.
     */
    virtual bool getParamSendMsg(
      H245_AudioCapability &,  /// PDU to set information on
      unsigned packetSize          /// Packet size to use in capability
    ) const;

    /**
       is called on receiving of TerminalCapabilitySet
       or OpenLogicalChannel message.
       Sets class members from corresponding message fields.
     */
    virtual bool setParamRecvMsg(
      const H245_AudioCapability &,  /// PDU to get information from
      unsigned & packetSize              /// Packet size to use in capability
    );

  protected:
    ///
    Rate    rate;
    ///
    bool comfortNoise;
    ///
    bool scrambled;
    ///
    unsigned int audioUnitSize;
    
};

}
#endif
