/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const GlobalCapTable_cxx_Version = 
    "$Id: GlobalCapTable.cxx,v 1.12 2001/01/15 21:54:22 vzubarev Exp $";


#include <iterator>
#include <algorithm>
#include <iostream>
#include "VLog.hxx"
#include "ptlib.h"
#include "h245.h"
#include "XlatorException.hxx"
#include "Assert.hxx"
#include "VLog.hxx"
#include "GlobalCapTable.hxx"


#ifndef DEBUG
#define DEBUG 1
#endif

using Vocal::Logging::VLog;

using namespace Translator;

	
    const GlobalCapTable::GlobalCapTableEntry GlobalCapTable::globalCapTable[size]={
	    { 1, H245_AudioCapability::e_g711Ulaw64k,         "PCMU", 'A', 8000, 1, "RFC1890", "" },
	    { 2, -1,                                          "G726-32", 'A', 8000, 1, "RFC1890", "" },
	    { 3, H245_AudioCapability::e_gsmFullRate,         "GSM", 'A', 8000, 1, "RFC1890", "" },
	    { 4, H245_AudioCapability::e_g7231,               "G723", 'A', 8000, 1, "Kumar", "" },
	    { 5, -1,                                          "DVI4", 'A', 8000, 1, "RFC1890", "" },
	    { 6, -1,                                          "DVI4", 'A', 16000, 1, "RFC1890", "" },
	    { 7, -1,                                          "LPC", 'A', 8000, 1, "RFC1890", "" },
	    { 8, H245_AudioCapability::e_g711Alaw64k,         "PCMA", 'A', 8000, 1, "RFC1890", "" },
	    { 9, H245_AudioCapability::e_g722_64k,            "G722", 'A', 8000, 1, "RFC1890", "" },
	    { 10, -1,                                         "L16", 'A', 44100, 2, "RFC1890", "" },
	    { 11, -1,                                         "L16", 'A', 44100, 1, "RFC1890", "" },
	    { 12, -1,                                         "QCELP", 'A', 8000, 1, "", "" },
	    { 14, -1,                                         "MPA", 'A', 90000, 1, "RFC1890, RFC2250", "" },
	    { 15, H245_AudioCapability::e_g728,               "G728", 'A', 8000, 1, "RFC1890", "" },
	    { 16, -1,                                         "DVI4", 'A', 11025, 1, "DiPol", "" },
	    { 17, -1,                                         "DVI4", 'A', 22050, 1, "DiPol", "" },
	    { 18, H245_AudioCapability::e_g729,               "G729", 'A', 8000, 1, "", "" },
	    { 25, -1,                                         "CelB", 'V', 90000, 0, "RFC2029", "" },
	    { 26, -1,                                         "JPEG", 'V', 90000, 0, "RFC2435", "" },
	    { 28, -1,                                         "nv", 'V', 90000, 0, "RFC1890", "" },
	    { 31, H245_VideoCapability::e_h261VideoCapability,"H261", 'V', 90000, 0, "RFC2032", "" },
	    { 32, -1,                                         "MPV", 'V', 90000, 0, "RFC2250", "" },
	    { 33, -1,                                         "MP2T", 'B', 90000, 0, "RFC2250", "" },
	    { 34, H245_VideoCapability::e_h263VideoCapability,"H263", 'V', 90000, 0, "Zhu", "" },
	    { 96, H245_AudioCapability::e_gsmHalfRate,        "GSM-HR", 'A', 8000, 1, "", "" },
	    { 97, H245_AudioCapability::e_gsmEnhancedFullRate,"GSM-EFR", 'A', 8000, 1, "", "" },
	    { 98, -1,                                         "L8", 'A', 0, 0, "", "" },
	    { 99, -1,                                         "RED", 'A', 0, 0, "", "" },
	    { 100, -1,                                        "VDVI", 'A', 0, 0, "", "" },
	    { 101, -1,                                        "BT656", 'V', 90000, 0, "", "" },
	    { 102, -1,                                        "H263-1998", 'V', 90000, 0, "", "" },
	    { 103, -1,                                        "MP1S", 'V', 90000, 0, "", "" },
	    { 104, -1,                                        "MP2P", 'V', 90000, 0, "", "" },
	    { 105, -1,                                        "BMPEG", 'V', 90000, 0, "", "" },
	    { 106, -1,                                        "1016", 'A', 8000, 1, "RFC1890" , "" }
    };


GlobalCapTable::GlobalCapTable_p GlobalCapTable::getTableIter_pt(int ptype) throw (XlatorException)
{
    pGlCapTable = std::find_if(globalCapTable, globalCapTable+size,GlCapTab_pt(ptype));
	    
    if (pGlCapTable == globalCapTable + size)
    {
	throw XlatorException("entry with payload type not found", __FILE__, __LINE__);
    }
    else return pGlCapTable;
}

GlobalCapTable::GlobalCapTable_p GlobalCapTable::getTableIter_ni(std::string nameI) throw (XlatorException)
{
    std::string::iterator p = nameI.begin();
    char c;
    
    while(p != nameI.end())
    {
	c = std::toupper(*p);
	*p = c;
	p++;
    }
    
    pGlCapTable = std::find_if(globalCapTable, globalCapTable+size, GlCapTab_ni(nameI));
	    
    if(pGlCapTable == globalCapTable + size)
    {
       throw XlatorException("entry with " +nameI+ " not found ", __FILE__, __LINE__);
    }
    else
    return pGlCapTable;
};

GlobalCapTable::GlobalCapTable_p GlobalCapTable::getTableIter_nh(int nameH) throw (XlatorException)
{
    pGlCapTable = std::find_if(globalCapTable, globalCapTable+size, GlCapTab_nh(nameH));
    
    if(pGlCapTable == globalCapTable + size)
    {
       throw XlatorException("entry with nameH not found ", __FILE__, __LINE__);
    }
    else
    return pGlCapTable;
};

int GlobalCapTable::findCapTableEntry(const H245_Capability & cap) throw (XlatorException)
{
  const string    fn("GlobalCapTable::findCapTableEntry");
  VLog    	  log(fn);
    
  GlobalCapTable_p pI;

  VDEBUG(log) << fn << " BEGIN ----------------------- " << VDEBUG_END(log);

  switch (cap.GetTag())
  {
    case H245_Capability::e_receiveAudioCapability :
    case H245_Capability::e_transmitAudioCapability :
    case H245_Capability::e_receiveAndTransmitAudioCapability :
    {
      const H245_AudioCapability & audio = cap;
      try
      {
	pI = getTableIter_nh(audio.GetTag());
      }
      catch (XlatorException)
      {
	VDEBUG(log) << fn << ": UNSuccessfull END --------------- " << VDEBUG_END(log);
	throw XlatorException("Capability not supported", __FILE__, __LINE__);
      }
      
      VDEBUG(log) << ": Found Capability "
                  << audio.GetTagName()
		  << "  "
		  << pI->ptypeCap
		  << " END ----------------------- "
		  << VDEBUG_END(log);
      
      return pI->ptypeCap;
    }
    
    //do not support Video & Data capability for now
    
    /*
    case H245_Capability::e_receiveVideoCapability :
    case H245_Capability::e_transmitVideoCapability :
    case H245_Capability::e_receiveAndTransmitVideoCapability :
    {
      const H245_VideoCapability & video = cap;
      try
      {
        pI = getTableIter_nh(video.GetTag());
      }
      catch (XlatorException)
      {
        throw XlatorException("Capability not supported", __FILE__, __LINE__);
      }
      
      cpLog(LOG_DEBUG, "findCapTableEntry END ----------------------- ");
      return pI->ptypeCap;
    }

    case H245_Capability::e_receiveDataApplicationCapability :
    case H245_Capability::e_transmitDataApplicationCapability :
    case H245_Capability::e_receiveAndTransmitDataApplicationCapability :
    {
      const H245_DataApplicationCapability & data = cap;
      try
      {
        pI = getTableIter_nh(data.m_application.GetTag());
      }
      catch (XlatorException)
      {
        throw XlatorException("Capability not supported", __FILE__, __LINE__);
      }
    
      cpLog(LOG_DEBUG, "findCapTableEntry END ----------------------- ");
      return pI->ptypeCap;
    }*/

    default :
      throw XlatorException("Capability not supported", __FILE__, __LINE__);
  }
  
  VDEBUG(log) << fn << ": END ----------------------- " << VDEBUG_END(log);
  return 0;
}


int GlobalCapTable::findCapTableEntry(const H245_DataType & dataType) throw (XlatorException)
{
  const string	fn("GlobalCapTable::findCapTableEntry");
  VLog log(fn);
  
  VDEBUG(log) << fn << ": dataType.GetTagName()  "
              << dataType.GetTagName()
	      << VDEBUG_END(log);

  GlobalCapTable_p pI;
  switch (dataType.GetTag())
  {
    case H245_DataType::e_audioData :
    {
      const H245_AudioCapability & audio = dataType;
      try
      {
        pI = getTableIter_nh(audio.GetTag());
      }
      catch (XlatorException)
      {
        throw XlatorException("Capability not supported", __FILE__, __LINE__);
      }
      return pI->ptypeCap;
    }
    
    // do not support Video and Data capabilities for now
    
    /*
    case H245_DataType::e_videoData :
    {
      const H245_VideoCapability & video = dataType;
      try
      {
        pI = getTableIter_nh(video.GetTag());
      }
      catch (XlatorException)
      {
        throw XlatorException("Capability not supported", __FILE__, __LINE__);
      }
      return pI->ptypeCap;
    }

    case H245_DataType::e_data :
    {
      const H245_DataApplicationCapability & data = dataType;
      try
      {
        pI = getTableIter_nh(data.GetTag());
      }
      catch (XlatorException)
      {
        throw XlatorException("Capability not supported", __FILE__, __LINE__);
      }
      return pI->ptypeCap;
    }*/

    default :
      throw XlatorException("Capability not supported", __FILE__, __LINE__);
  }
  return 0;
}
