
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const SessionDescriptorVec_cxx_Version = 
"$Id: SessionDescriptorVec.cxx,v 1.9 2000/12/18 23:48:25 bko Exp $";
#include "Assert.hxx"
#include "VLog.hxx"
#include "SessionDescriptorVec.hxx"

using namespace Translator;
using Vocal::Logging::VLog;

#ifndef DEBUG
#define DEBUG 1
#endif


int SessionDescriptorVec::TransportIDVec::getTranspIDVecSize() const
{

    return transpIDVec.size();

}

int SessionDescriptorVec::TransportIDVec::getTranspIDSetSize(int i) const throw (XlatorException)
{
    int vecSize;
    
    vecSize = getTranspIDVecSize();
    
    if((vecSize == 0)||(vecSize < i + 1))return 0;
    return transpIDVec[i].size();

}

int SessionDescriptorVec::getSessionDescrVecSize() const
{

    return sessionDescrVec.size();

}

int SessionDescriptorVec::getSessionDescrSetSize(int i) const throw (XlatorException)
{
    int vecSize;
    
    vecSize = getSessionDescrVecSize();
    if((vecSize == 0)||(vecSize < i + 1))return 0;  
    return sessionDescrVec[i].size();
}

void SessionDescriptorVec::setCapability(int cap, int i, int j) throw (XlatorException)
{
    int vecSize = getSessionDescrVecSize();
    int h = 0;

    if( i > vecSize )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    int descrSize = getSessionDescrSetSize(i);
    
    if( j >= descrSize ) // add new element (SessionDescriptor) 
                             // to SessionDescrSet
    {
      SessionDescriptor sDescr;
      SessionDescriptor sDescr0;
      sDescr0.capability = -1;
      sDescr.capability = cap;

      if( i < vecSize ) // do NOT add new element (SessionDescrSet) to
                            // sessionDescrVec
      {
         // in the case if j > descrSize we fill out
	 // the gap between them with blank elements
	 for(h = descrSize; h < j; h++)
	 {
	   sessionDescrVec[i].push_back(sDescr0);
	 }
         sessionDescrVec[i].push_back(sDescr);
      }
      else // add new element (SessionDescrSet) to sessionDescrVec
      {
         SessionDescrSet sDescrSet;
         // in the case if j > descrSize we fill out
	 // the gap between them with blank elements
	 for(h = descrSize; h < j; h++)
	 {
	   sDescrSet.push_back(sDescr0);
	 }
	 sDescrSet.push_back(sDescr);
	 sessionDescrVec.push_back(sDescrSet);
      }
    }
    else // do NOT add new element (SessionDescriptor) to sessionDescrSet
    { 
       if ( i == vecSize ) // cannot add new SessionDescrSet without
                               // adding new SessionDescriptor
          throw XlatorException("Index i2 out of range", __FILE__, __LINE__, 489);
       sessionDescrVec[i][j].capability = cap;
    }

}


void SessionDescriptorVec::setMediaN(int mediaN, int i, int j) throw (XlatorException)
{

    int vecSize = getSessionDescrVecSize();

    if( i > vecSize )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    int descrSize = getSessionDescrSetSize(i);
    
    if( j > descrSize )
        throw XlatorException("Index j out of range", __FILE__, __LINE__, 489);

    if( j == descrSize ) // add new element (SessionDescriptor) 
                             // to SessionDescrSet
    {
      SessionDescriptor sDescr;
      sDescr.mediaNumber = mediaN;

      if( i < vecSize ) // do NOT add new element (SessionDescrSet) to
                            // sessionDescrVec
      {
         sessionDescrVec[i].push_back(sDescr);
      }
      else // add new element (SessionDescrSet) to sessionDescrVec
      {
         SessionDescrSet sDescrSet;
	 sDescrSet.push_back(sDescr);
	 sessionDescrVec.push_back(sDescrSet);
      }
    }
    else // do NOT add new element (SessionDescriptor) to sessionDescrSet
    { 
       if ( i == vecSize ) // cannot add new SessionDescrSet without
                               // adding new SessionDescriptor
          throw XlatorException("Index i2 out of range", __FILE__, __LINE__, 489);
       sessionDescrVec[i][j].mediaNumber = mediaN;
    }
}

int SessionDescriptorVec::getMediaN(int i, int j) const throw(XlatorException)
{
    if(getSessionDescrVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if( getSessionDescrSetSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    return sessionDescrVec[i][j].mediaNumber;
}

int SessionDescriptorVec::getCapability(int i, int j) const throw(XlatorException)
{
    if( getSessionDescrVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if( getSessionDescrSetSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    return sessionDescrVec[i][j].capability;
}

Address SessionDescriptorVec::TransportIDVec::getRTPAddr(int i, int j) const throw(XlatorException)
{
    if( getTranspIDVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if( getTranspIDSetSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    return transpIDVec[i][j].addressRTP;
}

bool SessionDescriptorVec::TransportIDVec::isDone(int i, int j) const throw(XlatorException)
{
    if( getTranspIDVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if( getTranspIDSetSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    return transpIDVec[i][j].isDone;
}

void SessionDescriptorVec::TransportIDVec::setDone(int i, int j) throw(XlatorException)
{
    if( getTranspIDVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if( getTranspIDSetSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    transpIDVec[i][j].isDone = true;
}

Address SessionDescriptorVec::TransportIDVec::getRTCPAddr(int i, int j) const throw(XlatorException)
{
    if( getTranspIDVecSize() < i + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    if( getTranspIDSetSize(i) < j + 1)
      throw XlatorException("Index out of range", __FILE__, __LINE__, 489);

    return transpIDVec[i][j].addressRTCP;
}

void SessionDescriptorVec::TransportIDVec::setRTPAddr(const Address &rtpAddr, int i, int j) throw (XlatorException)
{
    const string    fn("TransportIDVec::setRTPAddr");
    VLog    	    log(fn);
    
    int vecSize = getTranspIDVecSize();
    int descrSize = 0, h = 0;

    /*cpLog(LOG_DEBUG, "add : ", i, " : ", j);*/
    VDEBUG(log) << fn << ": adding RTP address, i - j : " << i << ' ' << j
    	    	<< VDEBUG_END(log);
    
    if( i > vecSize  )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    descrSize = getTranspIDSetSize(i);
    
    if( j >= descrSize ) // add new element (TranspID) 
                             // to TranspIDSet
    {
      TransportID trID;
      trID.isDone = false;
      TransportID trID0;
      IPAddress ipAddress("0.0.0.0", 0);
      trID0.addressRTP = ipAddress;
      trID0.addressRTCP = ipAddress;
      trID.isDone = false;
      trID.addressRTP = rtpAddr;

      if( i < vecSize ) // do NOT add new element (TranspIDSet) to
                            // transpIDVec
      {
         // in the case if j > descrSize we fill out
	 // the gap between them with blank elements
	 for(h = descrSize; h < j; h++)
	 {
	   transpIDVec[i].push_back(trID0);
	 }
	 transpIDVec[i].push_back(trID);
      }
      else // add new element (TranspIDSet) to transpIDVec
      {
         TranspIDSet trIDSet;
         // in the case if j > descrSize we fill out
	 // the gap between them with blank elements
	 for(h = descrSize; h < j; h++)
	 {
	   trIDSet.push_back(trID0);
	 }

	 trIDSet.push_back(trID);
	 transpIDVec.push_back(trIDSet);
      }
    }
    else // do NOT add new element (TransportID) to TranspIDSet
    { 
       if ( i == vecSize ) // cannot add new TransportIDSet without
                               // adding new TransportID
          throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
       transpIDVec[i][j].addressRTP = rtpAddr;
    }

}

void SessionDescriptorVec::TransportIDVec::setRTCPAddr(const Address &rtcpAddr, int i, int j) throw (XlatorException)
{
    int vecSize = getTranspIDVecSize();
    int descrSize = 0, h = 0;

    if( i > vecSize  )
        throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
    
    descrSize = getTranspIDSetSize(i);
    
    if( j >= descrSize ) // add new element (TranspID) 
                             // to TranspIDSet
    {
      TransportID trID;
      trID.isDone = false;
      TransportID trID0;
      IPAddress ipAddress("0.0.0.0", 0);
      trID0.addressRTP = ipAddress;
      trID0.addressRTCP = ipAddress;
      trID0.isDone = false;
      trID.addressRTCP = rtcpAddr;

      if( i < vecSize ) // do NOT add new element (TranspIDSet) to
                            // transpIDVec
      {
         // in the case if j > descrSize we fill out
	 // the gap between them with blank elements
	 for(h = descrSize; h < j; h++)
	 {
	   transpIDVec[i].push_back(trID0);
	 }
         transpIDVec[i].push_back(trID);
      }
      else // add new element (TranspIDSet) to transpIDVec
      {
         TranspIDSet trIDSet;
         // in the case if j > descrSize we fill out
	 // the gap between them with blank elements
	 for(h = descrSize; h < j; h++)
	 {
	   trIDSet.push_back(trID0);
	 }
	 trIDSet.push_back(trID);
	 transpIDVec.push_back(trIDSet);
      }
    }
    else // do NOT add new element (TransportID) to TranspIDSet
    { 
       if ( i == vecSize ) // cannot add new TransportIDSet without
                               // adding new TransportID
          throw XlatorException("Index i out of range", __FILE__, __LINE__, 489);
       transpIDVec[i][j].addressRTCP = rtcpAddr;
    }

}
