#ifndef __SESSIONDESCR_H_
#define __SESSIONDESCR_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const SessionDescriptorVec_hxx_Version = 
"$Id: SessionDescriptorVec.hxx,v 1.8 2000/12/20 01:47:46 vzubarev Exp $";

#include <vector>
#include "cpLog.h"
#include "Address.hxx"
#include "XlatorException.hxx"

namespace Translator
{
    ///
    class SessionDescriptorVec
    {
    private:
        
	///
        class TransportIDVec
	{
	private:
	 
	   /// Transport ID: <IPaddress>:<port> for RTP & RTCP
	   struct TransportID
	   {
	    /** shows whether we have sent an OLC msg already
	        used for H323 side
	    */
	    bool isDone;
	    
	    ///
	    Address addressRTP;
	    
	    ///
	    Address addressRTCP;
	   };

	   ///
	   typedef std::vector<TransportID> TranspIDSet;
	   
	   ///
	   std::vector<TranspIDSet> transpIDVec;
	   
	public:
	   ///
	   TransportIDVec() {}
	   
	   ///
	   virtual ~TransportIDVec() {}
	   
	   ///
	   void setDone(int i, int j) throw (XlatorException);
	   
	   ///
	   bool isDone(int i, int j) const throw (XlatorException);
	   
	   /// add RTP port to transpIDVec[i][j]
	   void setRTPAddr(const Address &rtpAddr, int i, int j) throw (XlatorException);
	
	   /// add RTCP port to transpIDVec[i][j]
	   void setRTCPAddr(const Address &rtcpAddr, int i, int j) throw (XlatorException);

	   /// get RTP port  - transpIDVec[i][j]
	   Address getRTPAddr(int i, int j) const throw (XlatorException);

	   /// get RTCP port  - transpIDVec[i][j]
	   Address getRTCPAddr(int i, int j) const throw (XlatorException);

	   /// get TranspIDVec size (number of TranspIDSets)
	   int getTranspIDVecSize() const;
	
	   /// get TranspIDVec[i] size (number of TransportIDs)
	   int getTranspIDSetSize(int i) const throw (XlatorException);
	   
	};

	/** Session descriptor: 
	    & capability # (chosen for current session)
	*/
	struct SessionDescriptor
	{
	    /// Capability (chosen for current session)
	    int capability;
	    
	    /** Media descriptor # as assigned by us
	        number of particular altCapSet #
	        from which we chose the given capability
	        for this channel
	    */
	    int mediaNumber;
	};
	
	/// Session descriptor vector type
	typedef std::vector<SessionDescriptor> SessionDescrSet;
	
	/// List of session descriptor vectors
	std::vector<SessionDescrSet> sessionDescrVec;
    
    
    public:
	
	///
	TransportIDVec transportIDSip;
	
	///
	TransportIDVec transportIDH323;
	
	///
	TransportIDVec transportIDIntSip;

        ///
	SessionDescriptorVec() {}
	
	///
	virtual ~SessionDescriptorVec() {}
	
	/// get capability - element SessionDescrVec[i][j]
	int getCapability(int i, int j) const throw (XlatorException);
	
	/// set capability
	void setCapability(int cap, int i, int j) throw (XlatorException);

	/// get media # - element SessionDescrVec[i][j]
	int getMediaN(int i, int j) const throw (XlatorException);
	
	/// set media #
	void setMediaN(int mediaN, int i, int j) throw (XlatorException);

	/// get SessionDescrVector size (number of SessionDescrSets)
	int getSessionDescrVecSize() const;
	
	/// get SessionDescrVec[i] size (number of SessionDescriptors)
	int getSessionDescrSetSize(int i) const throw (XlatorException);
    };
}

#endif
