
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const SipEventObserver_cxx_Version = 
"$Id: SipEventObserver.cxx,v 1.14 2002/02/25 05:57:17 sprajpat Exp $";
#include "SipEventObserver.hxx"
#include "Service.hxx"
#include "TrCallFactory.hxx"
#include "TrRouterSIP.hxx"
#include "SesSip.hxx"
#include "VLog.hxx"
#include "IncomingSipEvent.hxx"


using namespace translator;


SipEventObserver::
SipEventObserver(
    Service 	    	&   service,
    TrCallFactory       &   callFactory,
    TrRouterSIP	    	&   trRouterSIP,
    SesSip  	    	&   sesSip
    )
    :	EventObserver< Sptr<Event> >(service.getEventSubject()),
    service_(service),
    callFactory_(callFactory),
	trRouterSIP_(trRouterSIP),
	sesSip_(sesSip),
	sipEvent_(0)
{
    service_.subscribe(*this);
}


SipEventObserver::
~SipEventObserver()
{
    service_.unsubscribe(*this);
}


bool SipEventObserver::
interestedIn(const Sptr<Event> & event)
{
    const string    	fn("SipEventObserver::interestedIn");
    VLog    	    	log(fn);

    VDEBUG(log) << fn << ": " << *event << VDEBUG_END(log);

    Sptr<IncomingSipEvent>  incomingSipEvent;
    
    if ( incomingSipEvent.dynamicCast(event) == 0 )
    {
	return ( false );
    }

    sipEvent_ = incomingSipEvent;

    if ( sipEvent_ == 0 )
    {
	VWARN(log) << fn << ": no sip event." << VWARN_END(log);

    	assert(0);
	
	return ( false );
    }

    return ( true );
}


void	    
SipEventObserver::onEvent(Sptr<Event>)
{
    const string    	fn("SipEventObserver::onEvent");
    VLog    	    	log(fn);
    VDEBUG(log) << "SipEventObserver::onEvent"
    	    	<< VDEBUG_END(log);

    assert( sipEvent_ != 0 );
        
#ifndef DISABLE_WRITER
    Data msgData = sipEvent_->sipEvent()->getSipMsg()->encode();
    VDEBUG(log) << fn << ": Received sip event =\n"
                << msgData.logData()
                << VDEBUG_END(log);
#endif

    // Send SIP event to the appropriate state machine
    sesSip_.generate(sipEvent_);
    
    sipEvent_ = 0;
    VDEBUG(log) << "exit SipEventObserver::onEvent"
    	    	<< VDEBUG_END(log);
}


ostream &           
SipEventObserver::writeTo(ostream & out) const
{
    return ( out << "SipEventObserver" );
}
