#ifndef INCLUDED_TrCall_hxx
#define INCLUDED_TrCall_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TrCall_hxx_Version = 
"$Id: TrCall.hxx,v 1.11 2001/05/09 23:15:43 vzubarev Exp $";

#include "VThreadPool.hxx"
#include "TSMMediator.hxx"
#include "TrSMCall.hxx"
#include "TrConnectionSIP.hxx"
#include "TrConnectionSDP.hxx"
#include "TrConnectionRas.hxx"
#include "FeatureTrCallSig.hxx"
#include "FeatureTrSDP.hxx"
#include "Call.hxx"
#include "EvtTrCCOpen.hxx"
#include "EvtTrMsdConfirm.hxx"
#include "CallTranslatorEM.hxx"
#include "CapabilityTranslator.hxx"
#include "EvtTrClose.hxx"
#include "EvtTrTerminate.hxx"
#include "EvtH245Hangup.hxx"
#include "EvtSetPhaseB.hxx"
#include "EvtSetPhaseC.hxx"
#include "EvtEnterFS.hxx"
#include "EvtExitFS.hxx"
#include "EvtTrFSOLC.hxx"
#include "EvtReceivedCCAddress.hxx"

namespace translator
{
    class TrRouterSIP;
    class TrRouterRas;
    class FeatureTrSDP;

    /// H.323/SIP call
    class TrCall: public TrSMCall
    {
    public:
        /// Construct call
        TrCall();

        ~TrCall();

        /// Return SIP connection
        virtual TrConnectionSIP* getSIPConnection() throw();

        /// Return SDP connection
        virtual TrConnectionSDP* getSDPConnection() throw();

        /// Return Ras connection
        virtual TrConnectionRas* getRasConnection() throw();

        /// Return pointer to Call Signalling feature
        virtual FeatureTrCallSigBase* getTrCallSig()
            throw();

        /// Return pointer to Media feature
        virtual FeatureTrSDPBase* getTrSDP() throw();

        /// Return pointer to Media feature (call-back)
        virtual FeatureTrSDP* getTrSDPImpl() throw();

        void setGatekeeperFifo(FifoBase< Sptr<Event> >* gatekeeperFifo)
            throw();

        void setProxyFifo(FifoBase< Sptr<Event> >* sipProxyFifo) throw();

        virtual FifoBase< Sptr<Event> >* getGatekeeperFifo()
            throw();

        virtual FifoBase< Sptr<Event> >* getProxyFifo() throw();

        void setSipRouter(TrRouterSIP* sipRouter) throw();

        virtual TrRouterSIP* getSipRouter() throw();

        virtual void setThreadPool(VThreadPool* threadPool) throw();

        virtual VThreadPool* getThreadPool() throw();

        void setRasRouter(TrRouterRas* rasRouter) throw();

        virtual TrRouterRas* getRasRouter() throw();

        /// Return pointer to H.323 call object
        virtual router::Call* getH323Call() throw();

        /// Return reference to capability map
        virtual CapabilityTranslator::CapabilityTranslator&
            getCapabilityTranslator() throw();

        virtual void complete();

        virtual void terminate(
            int reason = H225_ReleaseCompleteReason::e_undefinedReason);

        virtual void close();

        virtual void setH245Patch() throw();

        virtual bool needH245Patch() const throw();

        virtual void setReason(int reason) throw();

        virtual void startTimer(Sptr<mte::EvtTimer<router::TimerEvent> >);

        virtual void stopTimer(Sptr<mte::EvtTimer<router::TimerEvent> >);

    private:
#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif

        TSMMediator mediator_;
        router::Call h323Call_;
        TrConnectionSIP sipConnection_;
        TrConnectionSDP sdpConnection_;
        TrConnectionRas rasConnection_;
        FeatureTrCallSig callsig_;
        FeatureTrSDP mediaSDP_;
        EvtTrCCOpen evtTrCCOpen_;
        CapabilityTranslator::CapabilityTranslator capabilityTranslator_;
        EvtTrClose close_;
        EvtTrTerminate terminate_;
        router::EvtH245Hangup complete_;
        TrRouterSIP* routerSip_;
        TrRouterRas* routerRas_;
        FifoBase< Sptr<Event> >* gatekeeperFifo_;
        FifoBase< Sptr<Event> >* sipProxyFifo_;
        EvtTrMsdConfirm evtTrMsdConfirm_;
        bool h245Patch_;
        VThreadPool* threadPool_;
        EvtSetPhaseB ePhaseB_;
        EvtSetPhaseC ePhaseC_;
        EvtEnterFS eEnterFS_;
        EvtExitFS eExitFS_;
        EvtTrFSOLC eFSOLC_;

        // Callback event prototypes
        EvtResolve evtResolve_;
        router::EvtSendFSOLC evtSendFSOLC_;
        EvtZeroTCS evtZeroTCS_;
        EvtZeroSDP evtZeroSDP_;
        EvtSendSDP evtSendSDP_;
        router::EvtCEOTransferReq evtCEOTransferReq_;
        router::EvtLCOEstablishReq evtLCOEstablishReq_;
        router::EvtLCOReleaseReq evtLCOReleaseReq_;
        router::EvtLCIEstablishResp evtLCIEstablishResp_;
        router::EvtLCIReleaseReq evtLCIReleaseReq_;
        router::EvtRCORequest evtRCORequest_;
        router::EvtCreateInLC evtCreateInLC_;
        router::EvtCreateOutLC evtCreateOutLC_;
        EvtSDPParseDone evtSDPParseDone_;
        router::EvtReceivedCCAddress evtForceCCOpen_;
        CallTranslatorEM em_;
     };

}

#endif
