/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const TrCallFactoryImp_cxx_Version = 
    "$Id: TrCallFactoryImp.cxx,v 1.9 2001/03/09 06:55:08 icahoon Exp $";


#include "VLog.hxx"

#include "SesQ931.hxx"
#include "SesH245.hxx"
#include "RouterCallSig.hxx"
#include "RouterCallCtl.hxx"

#include "TrCallFactoryImp.hxx"
#include "TrRouterSIP.hxx"

using namespace translator;
using Vocal::Logging::VLog;


TrCallFactoryImp::
TrCallFactoryImp(
    PollFifoService 	    &	service,
    router::SesQ931         &   q931EventSrc,
    router::RouterCallSig   &   callSigRouter,
    router::SesH245         &   h245EventSrc,
    router::RouterCallCtl   &   callCtlRouter,
    TrRouterSIP             &   sipRouter,
    TrRouterRas             &   rasRouter,
    RouterInfo	    	    &	routerInfo,
    FifoBase< Sptr<Event> > *   gatekeeperFifo,
    FifoBase< Sptr<Event> > *   sipProxyFifo,
    VThreadPool             &   threadPool
    )
    : service_(service),
      q931EventSrc_(q931EventSrc),
      callSigRouter_(callSigRouter),
      h245EventSrc_(h245EventSrc),
      callCtlRouter_(callCtlRouter),
      sipRouter_(sipRouter),
      rasRouter_(rasRouter),
      routerInfo_(routerInfo),
      gatekeeperFifo_(gatekeeperFifo),
      sipProxyFifo_(sipProxyFifo),
      threadPool_(threadPool)
{
}


TrCall* TrCallFactoryImp::
createTrCall()
{
    TrCall* ret = new TrCall();
    assert(ret);
    assert(ret->getH323Call());

    VLog log;
    VDEBUG(log) << "Created Call object = "
                << *ret
                << VDEBUG_END(log);

    ret->getH323Call()->setFifo(&service_);
    ret->getH323Call()->setSesH245(&h245EventSrc_);
    ret->getH323Call()->setCallSigRouter(&callSigRouter_);
    ret->getH323Call()->setCallCtlRouter(&callCtlRouter_);
    ret->getH323Call()->setQ931Src(&q931EventSrc_);
    ret->getH323Call()->setRouterInfo(&routerInfo_);
    ret->setGatekeeperFifo(gatekeeperFifo_);
    ret->setProxyFifo(sipProxyFifo_);
    ret->setSipRouter(&sipRouter_);
    ret->setRasRouter(&rasRouter_);
    ret->setThreadPool(&threadPool_);

    return ret;
}
