#ifndef INCLUDED_TrSMCall_hxx
#define INCLUDED_TrSMCall_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TrSMCall_hxx_Version = 
"$Id: TrSMCall.hxx,v 1.15 2001/05/09 23:15:43 vzubarev Exp $";

#ifndef DISABLE_WRITER
#include "Writer.hxx"
#endif

#include "Sptr.hxx"

#include "mte/Feature.hxx"
#include "mte/EvtTimer.hxx"

#include "H323RouterTypes.hxx"

#include "FeatureTrCallSigDefs.hxx"
#include "FeatureTrSDPDefs.hxx"
#include "Call.hxx"

class VThreadPool;

namespace CapabilityTranslator
{
    class CapabilityTranslator;
}

namespace translator
{
    using Vocal::Services::Event;

    class TrConnectionSIP;
    class TrConnectionSDP;
    class TrConnectionRas;
    class TrRouterSIP;
    class TrRouterRas;

    /// State machine for H.323/SIP call
    class TrSMCall
#ifndef DISABLE_WRITER
        : public Vocal::IO::Writer
#endif
    {
    public:
        /// Construct state machine for H.323/SIP call
        TrSMCall();

        virtual ~TrSMCall();

        /// Return SIP connection (call-back)
        virtual TrConnectionSIP* getSIPConnection() throw() = 0;

        /// Return SDP connection (call-back)
        virtual TrConnectionSDP* getSDPConnection() throw() = 0;

        /// Return Ras connection
        virtual TrConnectionRas* getRasConnection() throw() = 0;

        /// Return pointer to Call Signalling feature
        virtual FeatureTrCallSigBase* getTrCallSig() throw() = 0;

        /// Return pointer to Media feature (call-back)
        virtual FeatureTrSDPBase* getTrSDP() throw() = 0;

        /// Return pointer to Media feature (call-back)
        virtual FeatureTrSDP* getTrSDPImpl() throw() = 0;

        /// Return pointer to H.323 call object (call-back)
        virtual router::Call* getH323Call() throw() = 0;

        virtual FifoBase< Sptr<Event> >* getGatekeeperFifo()
            throw() = 0;

        virtual FifoBase< Sptr<Event> >* getProxyFifo() throw() = 0;

        virtual TrRouterRas* getRasRouter() throw() = 0;

        virtual TrRouterSIP* getSipRouter() throw() = 0;

        virtual void setThreadPool(VThreadPool* threadPool) throw() = 0;

        virtual VThreadPool* getThreadPool() throw() = 0;

        /// Return reference to capability map (call-back)
        virtual CapabilityTranslator::CapabilityTranslator&
            getCapabilityTranslator() throw() = 0;

        virtual void complete() = 0;

        virtual void terminate(
            int reason = H225_ReleaseCompleteReason::e_undefinedReason) = 0;

        virtual void close() = 0;

        virtual void setH245Patch() throw() = 0;

        virtual bool needH245Patch() const throw() = 0;

        virtual void setReason(int reason) throw() = 0;

        virtual void startTimer(Sptr<mte::EvtTimer<router::TimerEvent> >) = 0;

        virtual void stopTimer(Sptr<mte::EvtTimer<router::TimerEvent> >) = 0;

    private:
#ifndef DISABLE_WRITER
        virtual ostream& writeTo(ostream& out) const;
#endif
    };

}

#endif
