#ifndef CASERVER_CONTAINER_HXX
#define CASERVER_CONTAINER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include <map>
#include "Sptr.hxx"
#include "BaseServerContainer.hxx"
#include "Data.hxx"

static const char* const CAServerContainer_hxx_Version = "$Id: CAServerContainer.hxx,v 1.14 2001/02/07 02:31:52 hlo Exp $";


/// Prototype for the Provisioning Server Callback
void updateCAHeartbeatParms( const string& data, const string& fileName, bool );


/**
 *  This class defines a container of callAgent and Marshal servers
 *  in the mgcp translator.
 */


class CAServerContainer: public BaseServerContainer
{
    public:
        static CAServerContainer& instance();

        /// dtor
        virtual ~CAServerContainer();

        /// update heartbeat data
        bool processHeartbeat( const Data& key);

        /// do housekeeping
        void doHouseKeeping();

        /// create entry
        void addServer( const Data& key, const ServerType serverType);

        /// display CA list
        void displayLists();

        /// get list of Active Servers specified by ServerType
        void getListofActive( const ServerType type,
                              Sptr < vector < Data > > & listofActiveServers);

        /// there's no other server
        void displayServer(const Server& server)
        {}
        ;

        ///
        void startStartupSequence(bool isUp);
        ///
        void setStartupFlag();

        ///
        void updateHBParmsFromPS();
    protected:
        /// ctor
        CAServerContainer();

        /// This is the single instance of ServerContainer
        static Sptr < CAServerContainer > theCAContainer;


    private:
        ///
        void CAServerContainer::copyRhsToThis( const CAServerContainer& rhs );
        ///
        bool startupFlag;
        ///
        Sptr < vector < Data > > itsMsList;
        ///
        CAServerContainer::CAServerContainer( const CAServerContainer& rhs );
        ///
        CAServerContainer& CAServerContainer::operator=( const CAServerContainer& rhs );
};

#endif // CASERVER_CONTAINER_HXX
