/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSConfig_cxx_Version = 
"$Id: LSConfig.cxx,v 1.6 2001/09/25 19:06:23 icahoon Exp $";

#include "global.h"
#include <unistd.h>
#include <cstdio>
#include <cstdlib>
#include <fstream.h>
#include "ProvisionInterface.hxx"
#include "LSConfig.hxx"
#include "NetworkAddress.h"
#include "TRIPRoutes.hxx"
#include "LSSessionManager.hxx"
#include "TRIPUpdateEvent.hxx"
#include "DecisionProxyAgent.hxx"
#include "OutgoingPolicyAgent.hxx"
#include <algorithm>
#include "cpLog.h"


LSConfig::LSConfig()
{
}

void 
LSConfig::init(const char* psHost, int psPort, const char* altHost, int altPort)
{
    char *envptr;
    if ((envptr = getenv("USE_LSCONFIG_FILE")) != 0)
    {
        int useFile = atoi(envptr);
        if (useFile == 1)
        {
            defaultConfig();
            return;
        }
    }
    if ((psHost == 0) && (psPort == 0))
    {
        defaultConfig();
        return;
    }
    ProvisionInterface& pif = ProvisionInterface::instance(psHost, psPort,                                                                  altHost, altPort);
    VLocationServerData locationData;
    VLocationRouteData locationRouteData;
    string ipName;
    string ipNameWithRoute;
    int myPort = 6069;
    const string rte = ".routes";

    // Get self's ip address for config file name

    char buf[256];
    gethostname(buf, 256);
    NetworkAddress na(buf);
    ipName = na.getIpName();

    if ( ipName.length() )
    {
        char buf[1024];
        sprintf( buf, "%s:%d", ipName.c_str(), myPort );
        ipName = buf;
        cpLog(LOG_INFO, "Location Server:  *** %s ***", ipName.c_str() );
        char buf2[1024];
        sprintf( buf2, "%s%s", ipName.c_str(), rte.c_str() );
        ipNameWithRoute = buf2;
    }
    else
    {
        cpLog(LOG_ERR, "*** Failed to get Location Server Data from Provisioning ***");
        return;
    }

    itsIpName = ipName;
    itsIpNameWithRoute = ipNameWithRoute;
    try
    {
        pif.getLocationData( itsIpName.c_str(), locationData );
    }
    catch (VBadDataException &e)
    {
        cpLog(LOG_ERR, "*** Failed to get Location Server Data from Provisioning ***");
        cpLog(LOG_ERR, "***     Location Server cannot continue     ***");
        throw;
    }
    getLocationData(locationData, false);

    try
    {
        pif.getLocationRouteData( itsIpNameWithRoute.c_str(), locationRouteData );
    }
    catch (VBadDataException &e)
    {
        cpLog(LOG_INFO, " Failed to get Location Server Route Data from Provisioning ");
        return;
    }
    getLocationRouteData(locationRouteData);
    pif.registerForUpdate( notifyLSServerUpdate, itsIpName, 
                                                ProvisionInterface::VLOCATION);
    pif.registerForUpdate( notifyLSRoutesUpdate, itsIpNameWithRoute, 
                                                ProvisionInterface::VLOCATION);
}

void
LSConfig::getLocationData(const VLocationServerData& lsData, bool update)
{
    if (!update)
    {
        string capability = lsData.getCapability();
        if (capability == "Receive")
        {
            cpLog(LOG_DEBUG, "Transport mode for LS is Receive Mode");
            itsTransMode = TRIP_TRANS_RCV;
        }
        else if (capability == "Send")
        {
            cpLog(LOG_DEBUG, "Transport mode for LS is Send Mode");
            itsTransMode = TRIP_TRANS_SND;
        }
        else if (capability == "SendReceive")
        {
            cpLog(LOG_DEBUG, "Transport mode for LS is SendReceive Mode");
            itsTransMode = TRIP_TRANS_SND_RCV;
        }

        itsItad = lsData.getItad();
        cpLog(LOG_DEBUG, "Itad for LS is %d", itsItad);
    }

    itsHoldTimer = lsData.getHoldTimer();
    cpLog(LOG_DEBUG, "HoldTimer for LS is %d seconds", itsHoldTimer);
    itsRestartTimer = lsData.getRestartTimer();
    cpLog(LOG_DEBUG, "RestartTimer for LS is %d seconds", itsRestartTimer);
    itsMaxRestartTimer = lsData.getMaxRestartTimer();
    cpLog(LOG_DEBUG, "MaxRestartTimer for LS is %d seconds", itsMaxRestartTimer);
    itsConnectRetryTimer = lsData.getConnectTimer();
    cpLog(LOG_DEBUG, "ConnectRetryTimer for LS is %d seconds", itsConnectRetryTimer);
    itsMaxConnectRetryTimer = lsData.getMaxConnectTimer();
    cpLog(LOG_DEBUG, "MaxConnectRetryTimer for LS is %d seconds", itsMaxConnectRetryTimer);
    itsAdvertiseTimer = lsData.getAdvertiseTimer();
    cpLog(LOG_DEBUG, "AdvertiseTimer for LS is %d seconds", itsAdvertiseTimer);

    if (lsData.getPeerListSize())
    {
        lsData.getPeerList(itsLsList);
    }

    if (!update)
    {
        //Retrieve the supported protocols from the PS
        vector <string> lists;
        if (lsData.getProtoListSize())
        {
            lsData.getProtoList(lists);
            if (lists.size())
            {
                itsProtoCapabilityList.clear();
                for (vector < string > ::const_iterator itr = lists.begin();
                    itr != lists.end(); itr++)
                {
                    itsProtoCapabilityList.push_back(getAppProto((*itr)));
                }
            }
        }

        if (lsData.getAddrListSize())
        {
            lsData.getAddrList(lists);
            if (lists.size())
            {
                itsAddrFamilyList.clear();
                for (vector < string > ::const_iterator itr = lists.begin();
                    itr != lists.end(); itr++)
                {
                    itsAddrFamilyList.push_back(getAddrFamily((*itr)));
                }
            }
        }
    }
}

TRIPAppProto
LSConfig::getAppProto(const string& proto)
{
    cpLog(LOG_DEBUG, "LSConfig::getAppProto being set for %s", proto.c_str());
    TRIPAppProto appProto;
    if (proto == "SIP")
    {
        appProto = TRIP_APP_PROTO_SIP;
    }
    if (proto == "H323-Q921")
    {
        appProto = TRIP_APP_PROTO_H323_Q921;
    }
    if (proto == "H323-RAS")
    {
        appProto = TRIP_APP_PROTO_H323_RAS;
    }
    if (proto == "H323-ANNEX-G")
    {
        appProto = TRIP_APP_PROTO_H323_ANNEX_G;
    }
    return appProto;
}

TRIPAddrFamily
LSConfig::getAddrFamily(const string& addr)
{
    cpLog(LOG_DEBUG, "LSConfig::getAddrFamily being set for %s", addr.c_str());
    TRIPAddrFamily addrFamily;
    if (addr == "POTS")
    {
        addrFamily = TRIP_ADDR_POTS;
    }
    if (addr == "ROUTING-NUM")
    {
        addrFamily = TRIP_ADDR_ROUTING_NUM;
    }
    return addrFamily;
}


void
LSConfig::getLocationRouteData(const VLocationRouteData& lsRouteData)
{
    if(lsRouteData.getRouteListSize())
    {
        TRIPItadKey key(0, TRIPId());
        vector < VRouteElement > routeList;
        lsRouteData.getRouteList(routeList);
        for (vector < VRouteElement > ::const_iterator itr = routeList.begin();
                                                itr != routeList.end(); itr++)
        {
            TRIPRoutes route(getAddrFamily((*itr).getAddr()), 
                            getAppProto((*itr).getProto()), TRIPData((*itr).getPrefix()));
            Data nextHop((*itr).getNextHop());
            Sptr <TRIPRouteElement> elem = new TRIPRouteElement(key, route, nextHop);
            itsLocalRouteSet.insert(elem);
        }
    }
}

void
LSConfig::updateLocalRoutes()
{
    updateRoutes(itsLocalRouteSet);
}

void
LSConfig::updateRoutes(TRIPGeneralRouteSet& updateRouteSet)
{
    Data prevNextHop("");
    
    bool readyToInject = false;
    bool routeFound = false;
    Sptr<TRIPWithdrawnRoute> withdrawn;
    Sptr<TRIPReachableRoute> reachable;
    for(TRIPGeneralRouteSet::const_iterator itr = updateRouteSet.begin();
                                        itr != updateRouteSet.end(); itr++)
    {
        Data nextHop = (*itr)->getNextHop();
        if (nextHop != prevNextHop)
        {
            if ((readyToInject) && (routeFound))
            {
                injectUpdateMsg(withdrawn, reachable, prevNextHop);
            }
                
            prevNextHop = nextHop;
            withdrawn = new TRIPWithdrawnRoute();
            reachable = new TRIPReachableRoute();
            readyToInject = true;
            routeFound = false;
        }
        if ((*itr)->isChanged(T_UPDATE_BIT))
        {
            routeFound = true;
            if ((*itr)->isWithdrawn())
            {
                *withdrawn << (*itr)->getRoute();
            }
            else
            {
                *reachable << (*itr)->getRoute();
            }
            (*itr)->unsetChanged(RouteChangeBitMask(T_UPDATE_BIT));
        }
    }
    if ((readyToInject) && (routeFound))
    {
        injectUpdateMsg(withdrawn, reachable, prevNextHop);
    }

}

void
LSConfig::injectUpdateMsg(Sptr<TRIPWithdrawnRoute> wRoute, Sptr<TRIPReachableRoute> rRoute,
                                                Data& nHop)
{
    Sptr<TRIPAdvertisePath> advPath = new TRIPAdvertisePath();
    Sptr<TRIPRoutedPath> rtdPath = new TRIPRoutedPath();

    Sptr <TRIPUpdateMsg> uMsg = new TRIPUpdateMsg();

    uMsg->addWithdrawnRoute(wRoute);
    uMsg->addReachableRoute(rRoute);
    uMsg->addAttribute(advPath);
    uMsg->addAttribute(rtdPath);
    Sptr <TRIPNextHopServer> nServer = new TRIPNextHopServer(itsItad, TRIPData(nHop.getData()));
    uMsg->addAttribute(nServer);
    uMsg->setTransactionId(LSSessionManager::instance().getNextTransactionId());

    const TRIPItadKey& key = LSSessionManager::instance().getTripItadKey();
    Sptr <TRIPUpdateEvent> update = new TRIPUpdateEvent(uMsg, key, TRIP_LOCAL_UPDATE);
    DecisionProxyAgent::instance().addToQueue(update);
}


bool 
LSConfig::isTrustedLs(const string& lsIp) const
{
    //Since port number of a LSPeer session is always fixed to 6069
    //No need to include port number in the comparision, just check the 
    //IP address only.
    LSConfig::LSList::const_iterator itr;
    for(itr = itsLsList.begin(); itr != itsLsList.end(); itr++)
    {
        NetworkAddress addr(*itr);
        if(addr.getIpName() == lsIp)
        {
            return true;
        }
    }
    return false;
}

void
LSConfig::updateLSFromPS()
{
    VLocationServerData locationData;
    int prevAdvertiseTimer;
    LSList prevLSList;

    prevAdvertiseTimer = itsAdvertiseTimer;
    prevLSList = itsLsList;
    itsLsList.clear();

    ProvisionInterface& pif = ProvisionInterface::instance();
    pif.getLocationData( itsIpName.c_str(), locationData );
    getLocationData(locationData, true);

    //if data has changed due to update then propagate through the system
    if (prevAdvertiseTimer != itsAdvertiseTimer)
    {
        if(itsAdvertiseTimer)
        {
            OutgoingPolicyAgent::instance().resetAdvertiseTimer(itsAdvertiseTimer);
        }
    }

    LSList removeList;
    getChangesInList(prevLSList, itsLsList, removeList);

    for(LSList::const_iterator itr = removeList.begin(); itr != removeList.end(); itr++)
    {
        LSSessionManager::instance().removeSession((*itr));
    }

    LSList addList;
    getChangesInList(itsLsList, prevLSList, addList);
        
    for(LSList::const_iterator itr = addList.begin(); itr != addList.end(); itr++)
    {
        LSSessionManager::instance().addSession((*itr));
    }
}

void
LSConfig::updateLSRoutesFromPS()
{
    VLocationRouteData locationRouteData;
    TRIPGeneralRouteSet prevRouteSet = itsLocalRouteSet;

    itsLocalRouteSet.clear();

    ProvisionInterface& pif = ProvisionInterface::instance();
    pif.getLocationRouteData( itsIpNameWithRoute.c_str(), locationRouteData );
    getLocationRouteData( locationRouteData );

    TRIPGeneralRouteSet removeSet;
    set_difference (prevRouteSet.begin(), prevRouteSet.end(), 
                    itsLocalRouteSet.begin(), itsLocalRouteSet.end(), 
                    inserter(removeSet, removeSet.begin()), GreaterRoute());
    
    TRIPGeneralRouteSet addSet;
    set_difference (itsLocalRouteSet.begin(), itsLocalRouteSet.end(),
                    prevRouteSet.begin(), prevRouteSet.end(),
                    inserter(addSet, addSet.begin()), GreaterRoute());

    if (removeSet.size())
    {
        for (TRIPGeneralRouteSet::const_iterator itr = removeSet.begin();
                                            itr != removeSet.end(); itr++)
        {
            (*itr)->setWithdrawn();
            (*itr)->setChanged(RouteChangeBitMask(T_UPDATE_BIT));
        }
        updateRoutes(removeSet);

        for (TRIPGeneralRouteSet::const_iterator prevItr = prevRouteSet.begin();
                                    prevItr != prevRouteSet.end(); prevItr++)
        {
            if ((*prevItr)->isWithdrawn())
            {
                prevRouteSet.erase(prevItr);
            }
        }   
    }

    if (addSet.size())
    {
        for (TRIPGeneralRouteSet::const_iterator addItr = addSet.begin();
                                            addItr != addSet.end(); addItr++)
        {
            prevRouteSet.insert((*addItr));
        }

        updateRoutes(prevRouteSet);
    }
    itsLocalRouteSet = prevRouteSet;
}


void
LSConfig::getChangesInList( const LSList& srcList, const LSList& destList, LSList& retList)
{
    for(LSList::const_iterator itr = srcList.begin(); itr != srcList.end(); itr++)
    {
        bool found = false;
        for(LSList::const_iterator itr2 = destList.begin(); itr2 != destList.end(); itr2++)
        {
            if ((*itr) == (*itr2))
            {
                found = true;
                break;
            }
        }
        if (!found)
        {
            retList.push_back((*itr));
        }
    }
}

void
LSConfig::defaultConfig()
{
    itsTransMode = TRIP_TRANS_SND_RCV;

    itsItad = 1234;

    itsHoldTimer = DEFAULT_HOLD_TIME;
    cpLog(LOG_DEBUG, "HoldTimer for LS is %d seconds", itsHoldTimer);
    itsRestartTimer = DEFAULT_RESTART_TIME;
    cpLog(LOG_DEBUG, "RestartTimer for LS is %d seconds", itsRestartTimer);
    itsMaxRestartTimer = DEFAULT_MAX_TIME;
    cpLog(LOG_DEBUG, "MaxRestartTimer for LS is %d seconds", itsMaxRestartTimer);
    itsConnectRetryTimer = DEFAULT_CONN_RETRY_TIME;
    cpLog(LOG_DEBUG, "ConnectRetryTimer for LS is %d seconds", itsConnectRetryTimer);
    itsMaxConnectRetryTimer = DEFAULT_MAX_TIME;
    cpLog(LOG_DEBUG, "MaxConnectRetryTimer for LS is %d seconds", itsMaxConnectRetryTimer);
    itsAdvertiseTimer = OUTGOING_POLICY_TIMER;
    cpLog(LOG_DEBUG, "AdvertiseTimer for LS is %d seconds", itsAdvertiseTimer);

    ifstream inF("./lsconfig.cfg");
    if(inF.bad())
    {
        ///Failed to read the config file
        cpLog(LOG_INFO, "Failed to read (./lsconfig.cfg)");
        exit(0);
    }

    char buf[80];
    while(inF.getline(buf, 80))
    {
        NetworkAddress addr(buf);
        char tBuf[256];
        sprintf(tBuf, "%s:%d", addr.getIpName().c_str(), addr.getPort());
        itsLsList.push_back(tBuf);
    }

    itsProtoCapabilityList.push_back(TRIPAppProto(TRIP_APP_PROTO_SIP));
    itsProtoCapabilityList.push_back(TRIPAppProto(TRIP_APP_PROTO_H323_Q921));
    itsProtoCapabilityList.push_back(TRIPAppProto(TRIP_APP_PROTO_H323_RAS));
    itsProtoCapabilityList.push_back(TRIPAppProto(TRIP_APP_PROTO_H323_ANNEX_G));
    
    itsAddrFamilyList.push_back(TRIPAddrFamily(TRIP_ADDR_POTS));
    itsAddrFamilyList.push_back(TRIPAddrFamily(TRIP_ADDR_ROUTING_NUM));

}


ostream&
operator<<(ostream& os, const LSConfig& lsconfig)
{
    LSConfig::LSList::const_iterator itr;
    os << "LS host list:" << endl;
    for(itr = lsconfig.itsLsList.begin(); itr != lsconfig.itsLsList.end(); itr++)
    {
        os << (*itr) << endl;
    }
    return os;
}

void
notifyLSServerUpdate(const string& data, const string& fileName, bool )
{
    LSSessionManager::instance().getLSConfig().updateLSFromPS();
}

void
notifyLSRoutesUpdate(const string& data, const string& fileName, bool )
{
    LSSessionManager::instance().getLSConfig().updateLSRoutesFromPS();
}
